/* lnux/arch/arm/mach-tcc92x/board-tcc9200s.c
 *
 * Author:  <linux@telechips.com>
 * Created: 10th Feb, 2009
 * Description:
 *
 * Copyright (C) Telechips, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/proc_fs.h>
#include <linux/delay.h>

#include <linux/spi/spi.h>
#include <linux/i2c.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/flash.h>
#include <mach/gpio.h>

#include <mach/bsp.h>
#include <mach/common.h>

#include <mach/tcc_used_mem.h>

extern void __init tcc9200_irq_init(void);
extern void __init tcc9200_map_common_io(void);

static struct spi_board_info tcc8900_spi0_board_info[] = {
	{
		.modalias = "spidev",
		.bus_num = 0,
		.chip_select = 0,
		.max_speed_hz = 2 * 1000 *1000,
	},
};

static void __init tcc9200_init_irq(void)
{
    tcc9200_irq_init();
//    tcc9200_gpio_init();
}

//#ifdef CONFIG_I2C_BOARDINFO
//static struct i2c_board_info __initdata tcc8900_i2c_board_info[] = {
//};
//#endif

static struct proc_dir_entry *phys_mem_info;

static inline int phys_mem_proc_info(char *buf, const char *name,
			    unsigned int base_addr, unsigned int size)
{
	return sprintf(buf, "0x%8.8x 0x%8.8x %s\n", base_addr, size, name);
}

static int phys_mem_read_proc(char *page, char **start, off_t off,
			      int count, int *eof, void *data)
{
	int len;

	len = sprintf(page, "%-10s %-10s %s\n", "base_addr", "size", "name");
	len += phys_mem_proc_info(page + len, "pmem",
				  PMEM_SURF_BASE, PMEM_SURF_SIZE);
	len += phys_mem_proc_info(page + len, "viqe",
				  PMEM_VIQE_BASE, PMEM_VIQE_SIZE);
	len += phys_mem_proc_info(page + len, "camera",
				  PMEM_CAM_BASE, PMEM_CAM_SIZE);
	len += phys_mem_proc_info(page + len, "overlay",
				  OVERLAY_MEM_BASE, OVERLAY_MEM_TOTAL_SIZE);
	len += phys_mem_proc_info(page + len, "video",
				  VIDEO_MEM_BASE, VIDEO_MEM_SIZE);
	len += phys_mem_proc_info(page + len, "fb",
				  FB_MEM_BASE, FB_MEM_TOTAL_SIZE);
	len += phys_mem_proc_info(page + len, "jpeg_header",
				  PA_JPEG_HEADER_BASE_ADDR,
				  TCC_JPEG_HEADER_SIZE);
	len += phys_mem_proc_info(page + len, "jpeg_raw",
				  PA_JPEG_RAW_BASE_ADDR, JPEG_RAW_MEM_SIZE);
	len += phys_mem_proc_info(page + len, "jpeg_stream",
				  PA_JPEG_STREAM_BASE_ADDR,
				  JPEG_STREAM_MEM_SIZE);
	return len;
}

static void __init tcc9200_init_machine(void)
{
	phys_mem_info = create_proc_entry("phys_mem", 0444, NULL);
	if (phys_mem_info) {
		phys_mem_info->read_proc = phys_mem_read_proc;
	}
//#ifdef CONFIG_I2C_BOARDINFO
//	i2c_register_board_info(-1, tcc8900_i2c_board_info, ARRAY_SIZE(tcc8900_i2c_board_info));
//#endif

	spi_register_board_info(tcc8900_spi0_board_info, ARRAY_SIZE(tcc8900_spi0_board_info));
}


static void __init tcc9200_map_io(void)
{
    tcc9200_map_common_io();
}

MACHINE_START(TCC8900, "Telechips TCC89/91/92XX Demo Board")
    /* Maintainer: Telechips Linux BSP Team <linux@telechips.com> */
    .phys_io        = 0xf0000000,
    .io_pg_offst    = ((0xf0000000) >> 18) & 0xfffc,
    .boot_params    = PHYS_OFFSET + 0x00000100,
    .map_io         = tcc9200_map_io,
    .init_irq       = tcc9200_init_irq,
    .init_machine   = tcc9200_init_machine,
    .timer          = &tcc9200_timer,
MACHINE_END
