#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <linux/clk.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/div64.h>
#include <asm/mach/map.h>
#ifdef CONFIG_PM
#include <linux/pm.h>
#endif

#include <mach/bsp.h>
#include <mach/tca_ckc.h>


#ifdef __KERNEL__
#	if defined(_TCC79x_)
#		include <mach/tcc79x.h>
#	elif defined(_TCC83x_)
#		include <mach/tcc83x_virt.h>
#	endif
#else
#	if defined(_TCC79x_)
#		include "TCC79x_Virtual.h"
#	elif defined(_TCC83x_)
#		include "TCC83x_Virtual.h"
#	endif
#endif
#include <mach/dev_gpsb.h>
#include <mach/TCC_LCD_Interface.h>
#include <mach/TCC_LCD_DriverCtrl.h>


#ifdef CONFIG_FB_TCC89XX_XGA_CLAA104XA01CW
#if 0
#define dprintk(msg...)	 { printk( "LMS480KF: " msg); }
#else
#define dprintk(msg...)	 
#endif

// Unit 100hz
#define LCDC_CLK		(650000 * 2)
#define LCD_DIVIDER		1

#define LCD_BUS_WIDTH	24

#define LPW		0	// line pulse width
#define LPC		1024	// line pulse count (active horizontal pixel - 1)
#define LSWC	0	// line start wait clock (the number of dummy pixel clock - 1)
#define LEWC	317	// line end wait clock (the number of dummy pixel clock - 1)
#define VDB		0	// Back porch Vsync delay
#define VDF		0 	// front porch of Vsync delay

#define FPW1	0	// TFT/TV : Frame pulse width is the pulse width of frmae clock
#define FLC1	768	// frmae line count is the number of lines in each frmae on the screen
#define FSWC1	0	// frmae start wait cycle is the number of lines to insert at the end each frame
#define FEWC1	35	// frame start wait cycle is the number of lines to insert at the begining each frame
#define FPW2	0	// TFT/TV : Frame pulse width is the pulse width of frmae clock
#define FLC2	768	// frmae line count is the number of lines in each frmae on the screen
#define FSWC2	0	// frmae start wait cycle is the number of lines to insert at the end each frame
#define FEWC2 	35	// frame start wait cycle is the number of lines to insert at the begining each frame

#define SYNC_INVET (IV_INVERT_EN |IH_INVERT_EN)
	
void DEV_LCD_MODULE_SPEC(LCD_MODULE_SPEC_TYPE* spec)
{
	spec->lcdc_clk = LCDC_CLK;
	spec->lcdc_clk_div = LCD_DIVIDER;
	spec->lcd_bus_width = LCD_BUS_WIDTH;
	spec->lcd_width = 1024;
	spec->lcd_height = 768;
	spec->lcd_LPW = LPW;		// line pulse width
	spec->lcd_LPC = LPC;		// line pulse count (active horizontal pixel - 1)
	spec->lcd_LSWC = LSWC;		// line start wait clock (the number of dummy pixel clock - 1)
	spec->lcd_LEWC = LEWC;		// line end wait clock (the number of dummy pixel clock - 1)
	spec->lcd_VDB = VDB;		// Back porch Vsync delay
	spec->lcd_VDF = VDF;		// front porch of Vsync delay

	spec->lcd_FPW1 = FPW1;		// TFT/TV : Frame pulse width is the pulse width of frmae clock
	spec->lcd_FLC1 = FLC1;		// frmae line count is the number of lines in each frmae on the screen
	spec->lcd_FSWC1 = FSWC1;		// frmae start wait cycle is the number of lines to insert at the end each frame
	spec->lcd_FEWC1 = FEWC1;		// frame start wait cycle is the number of lines to insert at the begining each frame

	spec->lcd_FPW2 = FPW2;		// TFT/TV : Frame pulse width is the pulse width of frmae clock
	spec->lcd_FLC2 = FLC2;		// frmae line count is the number of lines in each frmae on the screen
	spec->lcd_FSWC2 = FSWC2;		// frmae start wait cycle is the number of lines to insert at the end each frame
	spec->lcd_FEWC2 = FEWC2;		// frame start wait cycle is the number of lines to insert at the begining each frame
	spec->SyncSignal_Invert = SYNC_INVET;
}


 void DEV_LCD_Module (LCD_MODULE_CTRL uiControl)
{
	dprintk(KERN_INFO "%s :  uiControl: %d\n", __FUNCTION__, uiControl);
	switch(uiControl)
	{
		case LCD_M_INITIALIZE:
			break;

		case LCD_M_ON:
			break;

		case LCD_M_OFF:
			break;
			
		default:
			break;
	}

}
  
#endif//CONFIG_FB_TCC89XX_XGA_CLAA104XA01CW