// ****************************************** //
// Temporary Driver for GPS on Android
// 
// Title  : UART5 and GPIO (GPS_ON)
// Target : TCC92xx
//
// Author : Hae Jung Kim
// ****************************************** //

#include <linux/init.h> 
#include <linux/module.h> 
#include <linux/kernel.h> 
  
#include <linux/fs.h>            
#include <linux/mm.h>            
#include <linux/errno.h>         
#include <linux/types.h>         
#include <linux/fcntl.h>         
#include <linux/cdev.h>         
#include <linux/device.h>         
#include <linux/major.h>         
#include <linux/gpio.h>

#include <asm/uaccess.h>  
#include <asm/io.h>  

#include <mach/bsp.h>  
  
#define GPS_GPIO_DEV_NAME      "gps_gpio"   // ̽ ̸
#define GPS_GPIO_DEV_MAJOR     240          // Ƿ  ̽ ֹȣ
#define DEBUG_ON               0            //  ޽ ON

#define gps_dbg(fmt,arg...)     if(DEBUG_ON) printk("== gps debug == " fmt, ##arg)

static int      gps_major = GPS_GPIO_DEV_MAJOR;
static dev_t    dev;
static struct   cdev gps_cdev;
static struct   class *gps_class;

//  ************************************************************ //
//  Device Open : 
//  ½ UART5  Ȱȭ Ű ʴ´.
//  GPIO ⺻ GPIO ϰ  ϸ LOW· 
//  GPS ġ Ȱ · ϰ Ѵ.
//  ************************************************************ //
static int gps_gpio_open (struct inode *inode, struct file *filp)  
{
    // Set the Port Configure for the UART5
    // GPIO SETTING
#if defined(CONFIG_MACH_TCC9300)
    gpio_set_value(TCC_GPEXT1(6), 1);
#else
    BITCLR(HwGPIOD->GPFN2, (Hw23|Hw23|Hw21|Hw20));
    BITSET(HwGPIOD->GPEN,  Hw21);
    BITCLR(HwGPIOD->GPDAT,  Hw21);
#endif

    gps_dbg("tcc92xx : gps_gpio_open\n\n");
    return 0;  
}

//  ************************************************************ //
//  Device Release : 
//  ̽  Ҷ UART5 GPIO PORT D Pin15
//  ⺻ Ʈ   Ѵ.
//  ************************************************************ //
static int gps_gpio_release (struct inode *inode, struct file *filp)  
{  
#if defined(CONFIG_MACH_TCC9300)
    gpio_set_value(TCC_GPEXT1(6), 0);
#else
    BITCLR(HwGPIOD->GPEN,  Hw21);   // P15 
    BITCLR(HwGPIOD->GPDAT, Hw21);
#endif
    gps_dbg("tcc92xx : gps_gpio_close\n");
    return 0;  
}  
  
//  ************************************************************ //
//  Device Release : 
//  IO Control ȵ̵ HALκ gps_tcc.c ڵ忡
//  ȣϸ gpsα׷  start, stop ȴ.
//  ռ open ̸ õ ·
//  IOCTL gps On/Off ϰ ȴ.
//  ************************************************************ //
static int gps_gpio_ioctl (struct inode *inode, struct file *filp,
                           unsigned int cmd, unsigned long arg)  
{  
    switch( cmd )  
    {  
#if defined(CONFIG_MACH_TCC9300)
        case 0 : // GPS_On
            gpio_set_value(TCC_GPEXT1(6), 1);
            gps_dbg("tcc92xx : gps_gpio_on\n");
            break;   
        case 1 : // GPS_Off
            gpio_set_value(TCC_GPEXT1(6), 0);
            gps_dbg("tcc92xx : gps_gpio_off\n");
            break;
#else
        case 0 : // GPS_On
            BITSET(HwGPIOD->GPDAT, Hw21);
            gps_dbg("tcc92xx : gps_gpio_on\n");
            break;   
        case 1 : // GPS_Off
            BITCLR(HwGPIOD->GPDAT, Hw21);
            gps_dbg("tcc92xx : gps_gpio_off\n");
            break;
#endif  // #if defined(CONFIG_MACH_TCC9300)
        default :
            break;
    };
    return 0;  
}  
  
//  ************************************************************ //
//  File Operation Struct :
//  ̽   open, release, ioctl Ѵ.
//  ************************************************************ //
static struct file_operations gps_gpio_fops =  
{  
    .owner    = THIS_MODULE,  
    .ioctl    = gps_gpio_ioctl,  
    .open     = gps_gpio_open,       
    .release  = gps_gpio_release,    
};  

//  ************************************************************ //
//  Device Init :
//  ̽ ̹ ó Ͻ ڵ /dev 丮
//  gps_gpio ̽  ϰԲ 
//
//  ************************************************************ //
static int __init gps_gpio_init(void)  
{  
    int result;  

	if (0 == gps_major)
	{
		/* auto select a major */
		result = alloc_chrdev_region(&dev, 0, 1, GPS_GPIO_DEV_NAME);
		gps_major = MAJOR(dev);
	}
	else
	{
		/* use load time defined major number */
		dev = MKDEV(gps_major, 0);
		result = register_chrdev_region(dev, 1, GPS_GPIO_DEV_NAME);
	}

	memset(&gps_cdev, 0, sizeof(gps_cdev));

	/* initialize our char dev data */
	cdev_init(&gps_cdev, &gps_gpio_fops);

	/* register char dev with the kernel */
	result = cdev_add(&gps_cdev, dev, 1);
    
	if (0 != result)
	{
		unregister_chrdev_region(dev, 1);
		printk("Error registrating mali device object with the kernel\n");
	}

    gps_class = class_create(THIS_MODULE, GPS_GPIO_DEV_NAME);
    device_create(gps_class, NULL, MKDEV(gps_major, MINOR(dev)), NULL,
                  GPS_GPIO_DEV_NAME);

    if (result < 0)
        return result;  

#if defined(CONFIG_MACH_TCC9300)
    gpio_direction_output(TCC_GPEXT1(6), 1);    // GPS Power On
#endif

    gps_dbg("GPS driver loaded\n");

    return 0;  
}  

//  ************************************************************ //
//  Device Exit :
//
//  ڵ  ̽ ϰ ̽ ε .
//  ************************************************************ //
static void __exit gps_gpio_exit(void)  
{  
    device_destroy(gps_class, MKDEV(gps_major, 0));
    class_destroy(gps_class);

    cdev_del(&gps_cdev);
    unregister_chrdev_region(dev, 1);

#if defined(CONFIG_MACH_TCC9300)
    gpio_set_value(TCC_GPEXT1(6), 0);    // GPS Power On
#endif
    gps_dbg("GPS driver unloaded");
}  

module_init(gps_gpio_init);  
module_exit(gps_gpio_exit);  
  
MODULE_LICENSE("Dual BSD/GPL");  
