#include <bsp.h>
#include <i-tv/itv_common.h>
#include <i-tv/itv_platform.h>

#include "itcc89xx_tsif_p.h"
#include "itcc89xx_tsif_s.h"

/* frontend operations */
static void itv_platform_demod_reset(void)
{
	BITCSET(HwGPIOE->GPFN1, (Hw16-Hw12), 0);
	BITSET(HwGPIOE->GPEN, Hw11);
	BITSET(HwGPIOE->GPDAT, Hw11);
	msleep(50);
	BITCLR(HwGPIOE->GPDAT, Hw11);
	msleep(50);
	BITSET(HwGPIOE->GPDAT, Hw11);
}

#define ITV_TCC89XX_I2C_CH	1

static struct i2c_adapter *itv_platform_i2c_get_adapter(void)
{
	return i2c_get_adapter(ITV_TCC89XX_I2C_CH);
}

static void itv_platform_i2c_put_adapter(struct i2c_adapter *p_i2c_adap)
{
	if(!p_i2c_adap)
		return;

	i2c_put_adapter(p_i2c_adap);
}

itv_platform_frontend_operations_t *itv_platform_get_frontend_operations(void)
{
	itv_platform_frontend_operations_t *p_fe_ops;

	p_fe_ops = kzalloc(sizeof(*p_fe_ops), GFP_KERNEL);
	if(!p_fe_ops) {
		eprintk("out of memory\n");
		return NULL;
	}

	// demodulator operations
	p_fe_ops->demod_reset = itv_platform_demod_reset;

	// i2c operations
	p_fe_ops->i2c_get_adapter = itv_platform_i2c_get_adapter;
	p_fe_ops->i2c_put_adapter = itv_platform_i2c_put_adapter;

	return p_fe_ops;
}
EXPORT_SYMBOL(itv_platform_get_frontend_operations);

void itv_platform_put_frontend_operations(itv_platform_frontend_operations_t *p_fe_ops)
{
	kfree(p_fe_ops);
}
EXPORT_SYMBOL(itv_platform_put_frontend_operations);

/* ts interface operations */
itv_platform_tsif_operations_t *itv_platform_get_tsif_operations(void)
{
	itv_platform_tsif_operations_t *p_tsif_ops;

	p_tsif_ops = kzalloc(sizeof(*p_tsif_ops), GFP_KERNEL);
	if(!p_tsif_ops) {
		eprintk("out of memory\n");
		return NULL;
	}

	// TS parallel interface operations
	p_tsif_ops->tsif_p_init 	= tcc_tsif_p_init;
	p_tsif_ops->tsif_p_deinit 	= tcc_tsif_p_deinit;
	p_tsif_ops->tsif_p_get_pos 	= tcc_tsif_p_get_pos;
	p_tsif_ops->tsif_p_start 	= tcc_tsif_p_start;
	p_tsif_ops->tsif_p_stop 	= tcc_tsif_p_stop;	

	// TS serial interface operations
	p_tsif_ops->tsif_s_init_proc 	= tcc_tsif_s_init_proc;
	p_tsif_ops->tsif_s_init 		= tcc_tsif_s_init;
	p_tsif_ops->tsif_s_deinit 		= tcc_tsif_s_deinit;
	p_tsif_ops->tsif_s_get_pos 		= tcc_tsif_s_get_pos;
	p_tsif_ops->tsif_s_start 		= tcc_tsif_s_start;
	p_tsif_ops->tsif_s_stop 		= tcc_tsif_s_stop;

	return p_tsif_ops;
}
EXPORT_SYMBOL(itv_platform_get_tsif_operations);

void itv_platform_put_tsif_operations(itv_platform_tsif_operations_t *p_tsif_ops)
{
	kfree(p_tsif_ops);
}
EXPORT_SYMBOL(itv_platform_put_tsif_operations);

MODULE_AUTHOR("JP");
MODULE_LICENSE("GPL");
