/****************************************************************************
 *   FileName    : tca_i2c.c
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/
#include <linux/delay.h>
#include <mach/bsp.h>
#include "tca_i2c.h"


/*****************************************************************************
* Function Name : tca_i2c_reset()
* Description: I2C controller reset
******************************************************************************/
void tca_i2c_reset(void)
{ 
#if 0	/* sgbaek */
	tca_ckc_setswreset(RB_I2CCONTROLLER, OFF);
	tca_ckc_setswreset(RB_I2CCONTROLLER, ON);
#else
	tca_ckc_set_iobus_swreset(RB_I2CCONTROLLER, OFF);
	tca_ckc_set_iobus_swreset(RB_I2CCONTROLLER, ON);
#endif
}

/*****************************************************************************
* Function Name : tca_i2c_setgpio()
* Description: I2C port configuration
*              SCL0-GPA0, SDA0-GPA1
*              SCL1-GPA8, SDA1-GPA9
* input parameter:
*       int ch;   // I2C master channel
******************************************************************************/
void tca_i2c_setgpio(int ch)
{
	volatile struct tcc_gpio *gpio;

	gpio = (volatile struct tcc_gpio *)GPA_BASE;

	switch (ch) {
	case 0:
		BITCSET(gpio->FN0, (Hw4-Hw0), Hw0);	// GPA[0] function set 1
		BITCSET(gpio->FN0, (Hw8-Hw4), Hw4);	// GPA[1] function set 1
		break;
	case 1:
		BITCSET(gpio->FN1, (Hw4-Hw0), Hw0);	// GPA[8] function set 1
		BITCSET(gpio->FN1, (Hw8-Hw4), Hw4);	// GPA[9] function set 1
		break;
	default:
		break;
	}
}

/*****************************************************************************
* Function Name : tca_i2c_setclock()
* Description: I2C clock configuration
* input parameter:
*       int ch;   // I2C master channel
******************************************************************************/
void tca_i2c_setclock(int ch, volatile struct tcc_i2c_regs *i2c_reg, unsigned int i2c_clock)
{
	unsigned int i2c_clk_input_freq, prescale;

	/* I2C IO-bus enable */
	tca_ckc_setiobus(RB_I2CCONTROLLER, ENABLE);

	/* get I2C bus clock
	 * bootloader set I2C bus clock
	 */
	i2c_clk_input_freq = tca_ckc_getperi(PERI_I2C);

	prescale = ((i2c_clk_input_freq / 10) / (i2c_clock * 5)) - 1;
	i2c_reg->PRES = prescale;
	i2c_reg->CTRL = Hw7 | Hw6 | HwZERO;	// start enable, stop enable, 8bit mode
	BITSET(i2c_reg->CMD, Hw0);			// clear pending interrupt

	printk("i2c%d SCK(%d) <-- input clk(%dKhz), prescale(%d)\n", 
			ch, i2c_clock, i2c_clk_input_freq, prescale);
}

/* end of source */
