/****************************************************************************
 *   FileName    : tca_i2c.c
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/
#include <linux/delay.h>
#include <linux/io.h>

#include <bsp.h>
#include "tca_i2c.h"


/*****************************************************************************
* Function Name : tca_i2c_reset(struct tcc_i2c *i2c)
* Description: I2C controller reset
******************************************************************************/
void tca_i2c_reset(unsigned int iobus)
{ 
	tca_ckc_set_iobus_swreset(iobus, OFF);
	tca_ckc_set_iobus_swreset(iobus, ON);
}

/*****************************************************************************
* Function Name : tca_i2c_setgpio(int ch)
* Description: I2C port configuration
* input parameter:
* 		int core; 	// I2C Core
*       int ch;   	// I2C master channel
******************************************************************************/
int tca_i2c_setgpio(int core, int ch)
{
	int ret = 0;
	PGPIO gpio = (PGPIO)tcc_p2v(HwGPIO_BASE);

	switch (core) {
		case 0:
		{
			if (ch == 0) {
				BITCSET(gpio->GPAFN0, (Hw8-Hw0), Hw4|Hw0);			/* GPIO_A[1:0] */
				BITSET(gpio->GPAEN, Hw1|Hw0);
				BITCLR(gpio->GPADAT, Hw1|Hw0);
			} else if (ch == 1) {
				#if defined(CONFIG_MACH_TCC9300CM) || defined(CONFIG_MACH_TCC9300ST)
                                BITCSET(gpio->GPAFN1, (Hw8-Hw0), Hw4|Hw0);          /* GPIO_A[9:8] */
				BITSET(gpio->GPAEN, Hw9|Hw8);
				BITCLR(gpio->GPADAT, Hw9|Hw8);
				#else
				BITCSET(gpio->GPEFN3, (Hw32-Hw24), Hw28|Hw24);		/* GPIO_E[31:30] */
				BITSET(gpio->GPEEN, Hw31|Hw30);
				BITCLR(gpio->GPEDAT, Hw31|Hw30);
				#endif
			}
			break;
		}
		case 1:
		{
			if (ch == 0) {
				#if defined(CONFIG_MACH_TCC9300) 
				BITCSET(gpio->GPAFN0, (Hw16-Hw8), Hw13|Hw9);		/* GPIO_A[3:2] */
				BITSET(gpio->GPAEN, Hw3|Hw2);
				BITCLR(gpio->GPADAT, Hw3|Hw2);
				#elif defined(CONFIG_MACH_TCC9300ST) || defined(CONFIG_MACH_TCC9300CM)
				/* NC */
				ret = -1;
				#endif
			} else if (ch == 1) {
				BITCSET(gpio->GPAFN1, (Hw16-Hw8), Hw12|Hw8);		/* GPIO_A[11:10] */
				BITSET(gpio->GPAEN, Hw11|Hw10);
				BITCLR(gpio->GPADAT, Hw11|Hw10);
			}
			break;
		}
		case 2:
		{
			if (ch == 0) {
				BITCSET(gpio->GPAFN2, (Hw8-Hw0), Hw5|Hw4|Hw1|Hw0);	/* GPIO_A[17:16] */
				BITSET(gpio->GPAEN, Hw17|Hw16);
				BITCLR(gpio->GPADAT, Hw17|Hw16);
			} else if (ch == 1) {
				/* NC */
				ret = -1;
			}
			break;
		}
	}

	return ret;
}

/*****************************************************************************
* Function Name : tca_i2c_setclock()
* Description: I2C clock configuration
* input parameter:
*       int ch;   // I2C master channel
******************************************************************************/
void tca_i2c_setclock(int core, volatile struct tcc_i2c_regs *i2c_reg, unsigned int i2c_clock, unsigned int peri, unsigned int iobus)
{
	unsigned int i2c_clk_input_freq, prescale;

	/* I2C IO-bus enable */
	tca_ckc_setiobus(iobus, ENABLE);

	/* I2C peri clock enable */
	tca_ckc_setperi(peri, ENABLE, 40000, PCDIRECTXIN);
	i2c_clk_input_freq = tca_ckc_getperi(peri);

	prescale = ((i2c_clk_input_freq / 10) / (i2c_clock * 5)) - 1;
	writel(prescale, &i2c_reg->PRES);
	writel(Hw7 | Hw6 | HwZERO, &i2c_reg->CTRL);		// start enable, stop enable, 8bit mode
	writel(Hw0 | readl(&i2c_reg->CMD), &i2c_reg->CMD);	// clear pending interrupt

	/* XXX: need some delay? */
	mdelay(100);
}
