/*
 * linux/drivers/i2c/chips/tcc250.c
 *
 * Author:  <linux@telechips.com>
 * Created: 10th Jun, 2008 
 * Description: Tcc250 Driver 
 *
 * Copyright (c) Telechips, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/i2c.h>

#define TCC250_SLAVE_ADDR	0x49
static unsigned short probe_i2c[] = {0, TCC250_SLAVE_ADDR, I2C_CLIENT_END};	/* { i2c_num, i2c_addr } */
static unsigned short dummy[] = {I2C_CLIENT_END};

static struct i2c_client_address_data addr_data = {
	.normal_i2c = dummy,
	.probe = probe_i2c,
	.ignore = dummy,
};

static struct i2c_driver tcc250_i2c_driver;
struct i2c_client *tcc250_i2c_client;
EXPORT_SYMBOL(tcc250_i2c_client);

static int tcc250_i2c_probe(struct i2c_adapter *adap, int addr, int kind)
{
	struct i2c_client *i2c;

	i2c = kzalloc(sizeof(*i2c), GFP_KERNEL);
	if (!i2c)
		return -ENOMEM;

	strcpy(i2c->name, "TCC250");
	i2c->flags = 0;
	i2c->addr = addr;
	i2c->adapter = adap;
	i2c->driver = &tcc250_i2c_driver;

	tcc250_i2c_client = i2c;

	return i2c_attach_client(i2c);
}

static int tcc250_i2c_detach(struct i2c_client *client)
{
	i2c_detach_client(client);
	kfree(tcc250_i2c_client);
	return 0;
}

static int tcc250_i2c_attach(struct i2c_adapter *adap)
{
	return i2c_probe(adap, &addr_data, tcc250_i2c_probe);
}

static struct i2c_driver tcc250_i2c_driver = {
	.driver = {
		.name = "TCC250",
		.owner = THIS_MODULE,
	},
	.id = I2C_DRIVERID_TCC250,
	.attach_adapter = tcc250_i2c_attach,
	.detach_client = tcc250_i2c_detach,
};

static int __init tcc250_i2c_init(void)
{
	return i2c_add_driver(&tcc250_i2c_driver);
}

static void __exit tcc250_i2c_exit(void)
{
	i2c_del_driver(&tcc250_i2c_driver);
}

module_init(tcc250_i2c_init);
module_exit(tcc250_i2c_exit);

MODULE_AUTHOR("Telechips Inc. SYS4-3 linux@telechips.com");
MODULE_DESCRIPTION("TCC250 I2C driver");
MODULE_LICENSE("GPL");

