#include <linux/kernel.h>
#include <linux/delay.h>
#include <asm/io.h>
#include "tcc_remocon.h"
#include <bsp.h>

//======================================================
// REMOCON initialize
//======================================================
void    RemoconCommandOpen (void)
{
#if defined(CONFIG_ARCH_TCC92X)
        HwREMOCON->uCOMMAND.bCOMMAND.FF = 0;
        HwREMOCON->uCOMMAND.bCOMMAND.EN = 1;
        HwREMOCON->uCOMMAND.bCOMMAND.CLEAR = 1;
        HwREMOCON->uCOMMAND.bCOMMAND.TH = 0x14 ; // fifo threthold 20;
        HwREMOCON->uCOMMAND.bCOMMAND.WS = 1;
        HwREMOCON->uCOMMAND.bCOMMAND.DEN = 1;
        HwREMOCON->uCOMMAND.bCOMMAND.FWEN = 1;
#elif defined(CONFIG_ARCH_TCC93XX) 
        PREMOTECON      pRcu = (volatile PREMOTECON)tcc_p2v(Hw_REMOCON_BASE);
        BITCLR(pRcu->CMD, Hw0);
        BITSET(pRcu->CMD, Hw1|Hw2|(20*Hw3)|Hw12|Hw13|Hw14);
#endif /* CONFIG_ARCH_TCC92X */
}

void    RemoconCommandReset (void)
{
#if defined(CONFIG_ARCH_TCC92X)
        HwREMOCON->uCOMMAND.bCOMMAND.FF = 0;
        HwREMOCON->uCOMMAND.bCOMMAND.EN = 1;
        HwREMOCON->uCOMMAND.bCOMMAND.CLEAR = 1;
        HwREMOCON->uCOMMAND.bCOMMAND.TH = 0x14 ; // fifo threthold 20;
        HwREMOCON->uCOMMAND.bCOMMAND.WS = 0;
        HwREMOCON->uCOMMAND.bCOMMAND.DEN = 1;
        HwREMOCON->uCOMMAND.bCOMMAND.FWEN = 1;
#elif defined(CONFIG_ARCH_TCC93XX) 
        PREMOTECON      pRcu = (volatile PREMOTECON)tcc_p2v(Hw_REMOCON_BASE);
        BITCLR(pRcu->CMD, Hw0);
        BITCLR(pRcu->CMD, Hw12);
        BITSET(pRcu->CMD, Hw1|Hw2|(20*Hw3)|Hw13|Hw14);
#endif /* CONFIG_ARCH_TCC92X */
}

//======================================================
// configure address
//======================================================
void    RemoconConfigure (void)
{
#if defined(CONFIG_ARCH_TCC92X)
        HwREMOCON->uCONFIGURE.bCONFIGURE.CA = 0x02;             // polarity enable
#elif defined(CONFIG_ARCH_TCC93XX) 
        PREMOTECON      pRcu = (volatile PREMOTECON)tcc_p2v(Hw_REMOCON_BASE);
        BITSET(pRcu->INPOL, Hw0|Hw8);
        BITCLR(pRcu->INPOL, (Hw9|Hw10|Hw11));
#endif /* CONFIG_ARCH_TCC92X */
}

//======================================================
// REMOCON STATUS : irq enacle and fifo overflow (active low)
//======================================================
void    RemoconStatus (void)
{
#if defined(CONFIG_ARCH_TCC92X)
        HwREMOCON->uSTATUS.bSTATUS.ICF = 0x000;
        HwREMOCON->uSTATUS.bSTATUS.OF  = 1;
#elif defined(CONFIG_ARCH_TCC93XX) 
        PREMOTECON      pRcu = (volatile PREMOTECON)tcc_p2v(Hw_REMOCON_BASE);
        BITCSET(pRcu->STA, 0x0, Hw12);
#endif /* CONFIG_ARCH_TCC92X */
}

//======================================================
// REMOCON DIVIDE enable & ir clk & end count setting
// (end count use remocon clk)
//======================================================
void    RemoconDivide (void)
{
#if defined(CONFIG_ARCH_TCC92X)
        HwREMOCON->uCLKDIVIDE.bCLKDIVIDE.END_CNT  = 1;

        //HwREMOCON->uCLKDIVIDE.bCLKDIVIDE.CLK_DIV  = 500*26;
        HwREMOCON->uCLKDIVIDE.bCLKDIVIDE.CLK_DIV  = 500*24;
        //HwREMOCON->uCLKDIVIDE.bCLKDIVIDE.CLK_DIV  = 500*22;
        //HwREMOCON->uCLKDIVIDE.bCLKDIVIDE.CLK_DIV  = 500*21;
        //HwREMOCON->uCLKDIVIDE.bCLKDIVIDE.CLK_DIV  = 500*20;
        //HwREMOCON->uCLKDIVIDE.bCLKDIVIDE.CLK_DIV  = 500*19;
#elif defined(CONFIG_ARCH_TCC93XX) 
        PREMOTECON      pRcu = (volatile PREMOTECON)tcc_p2v(Hw_REMOCON_BASE);
        BITCSET(pRcu->CLKDIV, 0x0, ((500*24)*Hw16)|(1*Hw0));
#endif /* CONFIG_ARCH_TCC92X */
}

//======================================================
// REMOCON diable 
//======================================================
void    DisableRemocon (void)
{
#if defined(CONFIG_ARCH_TCC92X)
        HwREMOCON->uCOMMAND.bCOMMAND.FF = 1;
        HwREMOCON->uCOMMAND.bCOMMAND.EN = 0;
        HwREMOCON->uCOMMAND.bCOMMAND.CLEAR = 0;
        HwREMOCON->uCOMMAND.bCOMMAND.DEN = 0;
        HwREMOCON->uCOMMAND.bCOMMAND.FWEN = 0;
#elif defined(CONFIG_ARCH_TCC93XX) 
        PREMOTECON      pRcu = (volatile PREMOTECON)tcc_p2v(Hw_REMOCON_BASE);
        BITSET(pRcu->CMD, Hw0);
        BITCLR(pRcu->CMD, Hw1|Hw2|Hw13|Hw14);
#endif /* CONFIG_ARCH_TCC92X */
}

//======================================================
// REMOCON TEST
//======================================================
void    RemoconReset (void)
{
#if defined(CONFIG_ARCH_TCC92X)
        *(volatile uint *)0xf05f5018 &= ~Hw29;
        *(volatile uint *)0xf05f5018 |= Hw29;
#endif /* 0 */
}

void    RemoconInit (void)
{
#if defined(CONFIG_ARCH_TCC92X)
        RemoconReset ();
        RemoconConfigure ();
        RemoconStatus();
        RemoconDivide();
//      RemoconIntClear ();
        RemoconCommandOpen();
        RemoconIrqClear ();     
        RemoconIntClear();

         HwREMOCON->uCLKDIVIDE.bCLKDIVIDE.END_CNT  = 100; // data end count value        
#elif defined(CONFIG_ARCH_TCC93XX)
        RemoconIrqClear ();
        RemoconIntClear();
#endif
}

extern void rem_getrawdata(void);

void    RemoconIntClear ()
{
#if defined(CONFIG_ARCH_TCC92X)
        HwREMOCON->uCOMMAND.bCOMMAND.CLEAR = 0;
        HwREMOCON->uCOMMAND.bCOMMAND.FF = 1;
        HwREMOCON->uCOMMAND.bCOMMAND.CLEAR = 1;
        HwREMOCON->uCOMMAND.bCOMMAND.FF = 0;
#elif defined(CONFIG_ARCH_TCC93XX) 
        PREMOTECON      pRcu = (volatile PREMOTECON)tcc_p2v(Hw_REMOCON_BASE);   
        BITCLR(pRcu->CMD, Hw2);
        BITSET(pRcu->CMD, Hw0);

        udelay(120);

        BITSET(pRcu->CMD, Hw2);
        BITCLR(pRcu->CMD, Hw0);
#endif /* CONFIG_ARCH_TCC92X */
}

void    RemoconIntWait ()
{
#if defined(CONFIG_ARCH_TCC92X) 
        uint rx_data;
        while (1)
        {
                rx_data = *((volatile uint *)0xf0401014);       // VIC Interrupt Status Register
                if ((rx_data&0x400) == 0x400)
                        break;
        }
        *((volatile uint *)0xf040100c) = 0x400;                 // Clear VIC Interrupt Clear Register
#elif defined(CONFIG_ARCH_TCC93XX)
        PPIC pPic = (volatile PPIC)tcc_p2v(HwVPIC_BASE);
        uint rx_data;
        while(1)
        {
                rx_data = pPic->STS1;
                if((rx_data&0x400) == 0x400)
                        break;
        }
        pPic->CLR1 = 0x400;     //RMT bit set to 1 which means rmt interrupt clear      
#endif /* CONFIG_ARCH_TCC92X */         
}

void    RemoconIrqClear ()
{
        uint idx;

#if defined(CONFIG_ARCH_TCC92X)
        for(idx=1; idx<3000; idx ++)
                *((volatile uint *)0xf040100c) = 0x400;                 // Clear VIC Interrupt Clear Register 
#elif defined(CONFIG_ARCH_TCC93XX)
        PPIC pPic = (volatile PPIC)tcc_p2v(HwVPIC_BASE);
        for(idx=1; idx<3000; idx ++)
                pPic->CLR1 = 0x400;                     // Clear VIC Interrupt Clear Register 
#endif /* CONFIG_ARCH_TCC93XX */
}

#if 0
void    TestRemocon (void)   // Incremental modeling...
{
        uint    idx;
//        uint    tmp;
//        uint    *rx_data;
        volatile int i;
                

        uint    nReadData[20];
        //HwGPIO->pGPA.uFNC.bReg.P5 = 1;                //GPIOA for remocon pin
        HwGPIOA_FN0 = (HwGPIOA_FN0 & ~(0xF<<20)) | (1<<20);

        RemoconConfigure ();
        RemoconStatus();

        printk("TestRemocon:1\n");
        RemoconDivide();                //remocon clk divide and end_cout
        
        printk("TestRemocon:2\n");
        RemoconCommandOpen();                   

        RemoconIrqClear ();     
        RemoconIntClear();
 
        printk("TestRemocon:3\n");
        RemoconConfigure ();            
        
        printk("TestRemocon:4\n");
        RemoconStatus();
                
        printk("TestRemocon:5\n");
        while (1) 
        {
                RemoconIntWait();
                for ( i = 0 ; i < 1000 ; i++ ) 
                {
                        printk("K");
                }
                
                for ( i = 0 ; i < 20 ; i++ ) 
                {
                        nReadData[i] = HwREMOCON->uPORT.bPORT.PORT;
                }

                printk("\n");
                for ( i = 0 ; i < 20 ; i++ ) 
                {
                        printk("%x\n", nReadData[i]);
                }
                
                RemoconIntClear();
                
                RemoconInit ();
        }
        


        for ( idx = 0 ; idx < 3000 ; idx ++ );
        HwREMOCON->uSTATUS.bSTATUS.ICF = 0xfff;

        DisableRemocon ();

}
#endif