
/****************************************************************************
 *   FileName    : tca_remocon.h
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/
#ifndef __TCC_REMOCON_H__
#define __TCC_REMOCON_H__

#include <linux/types.h>

/*******************************************************************************
*	 Remote Control Interface Register Define   (Base Addr = 0xF05F3000)
********************************************************************************/
typedef	struct	_REMOCON {
	union	{
		uint	nPORT;
		struct  _REMOCON_PORT	{
			uint	PORT	:  32;	// PORT SELCT
			//uint	PORT	:  8;	// PORT SELCT
			//uint		: 24;
		}	bPORT;
	}	uPORT;
	union	{
		uint	nCOMMAND;
		struct	_REMOCON_COMMAND {
			uint	FF	:  1;	// interrupt acknowledge ( writing '1' for clear )
			uint	EN	:  1;
			uint	CLEAR	:  1;	// sent acknowledge ( 1 for disabled )
			uint	TH	:  9;	// FIFO THRESHOLD
			uint	WS	:  1;	// ir clk wait control
			uint	DEN	:  1;	// duration count enable
			uint	FWEN	:  1;	// FIFO write enable
			uint		: 17;
		}	bCOMMAND;
	}	uCOMMAND;
	union	{
		uint	nCONFIGURE;
		struct	_REMOCON_CONTROL {
#if defined(CONFIG_ARCH_TCC92X)		
			uint	CA	: 12;	// configure address
			uint		: 20;
#elif defined(CONFIG_ARCH_TCC93XX)
			uint INV : 1;
			uint       : 7;
			uint SCLK : 1;
			uint FIL : 1;
			uint FT : 2;
			uint      : 20;
#endif
		}	bCONFIGURE;
	}	uCONFIGURE;
	union	{
		uint	nSTATUS;
		struct	_REMOCON_STATUS {
			uint	ICF	: 12;	// interrupt configure ( 1 for pending )
			uint	OF	:  1;	// FIFO over flow
			uint		: 19;
		}	bSTATUS;
	}	uSTATUS;
//	union	{
//		uint	nEMPTY0;
//		struct	_REMOCON_EMPTY0	{
//			uint	EMPTY0	: 32;
//		}	bEMPTY0;
//	}	uEMPTY0;
	uint	EMPTY0;
	uint	EMPTY1;
	uint	EMPTY2;
	uint	EMPTY3;
	uint	EMPTY4;
	uint	EMPTY5;
	uint	EMPTY6;
	uint	EMPTY7;
	uint	EMPTY8;
	uint	EMPTY9;
	uint	EMPTY10;
	uint	EMPTY11;
	union	{
		uint	nCLKDIVIDE;
		struct	_REMOCON_CLKDIVED {
			uint	END_CNT	: 14;	// transfer end zero count 
			uint	CLK_DIV	: 18;	// clk clock divide for ir clk
		}	bCLKDIVIDE;
	}	uCLKDIVIDE;
}	REMOCON, *PREMOCON;

#if defined(CONFIG_ARCH_TCC92X)
#define HwREMOCON			((volatile REMOCON  *)(0xF05F3000))
#define Hw_REMOCON_BASE			*(volatile unsigned long *)0xF05F3000

#define Hw_REMOCON_RDATA	*(volatile unsigned long *)0xF05F3000
#define Hw_REMOCON_CMD		*(volatile unsigned long *)0xF05F3004
#define Hw_REMOCON_INPOL	*(volatile unsigned long *)0xF05F3008
#define Hw_REMOCON_STA		*(volatile unsigned long *)0xF05F3008
#define Hw_REMOCON_CLKDIV	*(volatile unsigned long *)0xF05F3040

#define	HwGPIOA_BASE  				*(volatile unsigned long *)0xF0102000  	// 
#define	HwGPIOA_FN0  				*(volatile unsigned long *)0xF0102024  	// 
#elif defined(CONFIG_ARCH_TCC93XX)
#define HwREMOCON					((volatile REMOCON  *)(0xB0101000))
#define Hw_REMOCON_BASE			*(volatile unsigned long *)0xB0101000

#define Hw_REMOCON_RDATA			*(volatile unsigned long *)0xB0101000
#define Hw_REMOCON_CMD			*(volatile unsigned long *)0xB0101004
#define Hw_REMOCON_INPOL			*(volatile unsigned long *)0xB0101008
#define Hw_REMOCON_STA			*(volatile unsigned long *)0xB010100c
#define Hw_REMOCON_CLKDIV			*(volatile unsigned long *)0xB0101040

//#define	HwGPIOA_BASE  				*(volatile unsigned long *)0xB010A000
//#define	HwGPIOA_FN0  				*(volatile unsigned long *)0xB010A024

#endif /* 0 */


//*******************************************
//	Remote KEY value
//*******************************************
#define REM_PWR			1	
#define REM_LOAD		2	
#define REM_SUBTITLE	3	
#define REM_AUDIO		4	
#define REM_ANGLE		5	
#define REM_INFO		6	

#define REM_KEY1		7	
#define REM_KEY2		8	
#define REM_KEY3		9	
#define REM_KEY4		10	
#define REM_KEY5		11	
#define REM_KEY6		12	
#define REM_KEY7		13	
#define REM_KEY8		14	
#define REM_KEY9		15	
#define REM_KEY0		16	
#define REM_KEY10		17	

#define REM_GOTO		18	
#define REM_TITLE		19	
#define REM_MENU		20	
#define REM_UP			21	
#define REM_DOWN		22	
#define REM_LEFT		23	
#define REM_RIGHT		24	
#define REM_ENTER		25	
#define REM_SETUP		26	
#define REM_RETURN		27	
                    	
#define REM_STOP		28	
#define REM_PLAY		29	
#define REM_PAUSE		30	
#define REM_FB			31	
#define REM_FF			32	
#define REM_LAST		33	
#define REM_NEXT		34	

#define REM_AB			35	
#define REM_REPEAT		36	
#define REM_STEP		37	
#define REM_PBC			38	
#define REM_ZOOM		39	
#define REM_MUTE		40	
#define REM_VOLDN		41	
#define REM_VOLUP		42	
#define REM_SLOW		43	
#define REM_RANDOM		44	
#define REM_KEYDEC		45	
#define REM_KEYADD		46	

#define REM_PROGRAM		47	
#define REM_CLEAR		48	
#define REM_RESUME		49	
#define REM_VIDEO		50	
#define REM_PN			51	
#define REM_DIGEST		52	
#define REM_HDMI		53	
#define REM_FUNCTION	54	
#define REM_EPG			55	
#define REM_TELETEXT	56	
#define REM_SERVICE		57	
#define REM_EXIT		58	

#define REM_GUIDE		59	
#define REM_LIST		60	
#define REM_BROWSE		61	
#define REM_RADIO		62	
#define REM_FADEC		63	
#define REM_FAADD		64	

#define REM_MODE		65	
#define REM_LANG		66	
#define REM_CHAR		67	


//*******************************************
//	Remote controller define
//*******************************************

#define STATUS0		0
#define STATUS1		1
#define STATUS2		2
#define STATUS3		3

/* 1 unit = 1 / [IOBUS/(HwREMOCON->uCLKDIVIDE.bCLKDIVIDE.CLK_DIV)]
              = 1 / [156/(500*20)]  us
              = 64.1us
*/
/* 1 unit = 1 / [IOBUS/(HwREMOCON->uCLKDIVIDE.bCLKDIVIDE.CLK_DIV)]
              = 1 / [165/(500*21)]  us
              = 63.63us
*/
/* 1 unit = 1 / [IOBUS/(HwREMOCON->uCLKDIVIDE.bCLKDIVIDE.CLK_DIV)]
			  = 1 / [165/(500*26)]	us
			  = 78.78us
*/

// Scan-code mapping for keypad
typedef struct {
	unsigned short	rcode;
	unsigned char	vkcode;
}SCANCODE_MAPPING;


/*****************************************************************************
*
* External Variables
*
******************************************************************************/

 void    RemoconCommandOpen(void);
 void    RemoconCommandReset(void);
 void	RemoconConfigure(void);
 void    RemoconStatus(void);
 void	RemoconDivide(void);
 void    DisableRemocon(void);
 void	RemoconReset(void);
 void	RemoconInit(void);
 void	RemoconIrqClear(void);
 void	RemoconIntClear(void);
 void	RemoconIntWait(void);


/*******************************************************************************
 Decide the remote type
 if you don't select any one, it will be the default remote controller
********************************************************************************/ 
//#define DARE_IR
//#define BUFFALO_IR
//#define LG_MKJ36998118
//#define CS_2000_IR_X_CANVAS
//#define SUNNIWELL_IR
//#define NOVATEK_IR
#ifdef CONFIG_DARE_IR
	#include "tcc_remocon_DARE_IR.h"

#elif defined(CONFIG_BUFFALO_IR)
	#include "tcc_remocon_BUFFALO_IR.h"

#elif defined(CONFIG_LG_MKJ36998118)
	#include "tcc_remocon_LG_MKJ36998118.h"

#elif defined(CONFIG_CS_2000_IR_X_CANVAS)
	#include "tcc_remocon_CS_2000_IR_X_CANVAS.h"

#elif defined(CONFIG_SUNNIWELL_IR)
	#include "tcc_remocon_SUNNIWELL_IR.h"

#elif defined(CONFIG_NOVATEK_IR)
	#include "tcc_remocon_NOVATEK_IR.h"
#else
	#include "tcc_remocon_default.h"
#endif
 
#endif	//__TCC_REMOCON_H__

