/* 
 * linux/drivers/char/sensor_cma3000.c
 *
 * Author:  <linux@telechips.com>
 * Created: 10th Jun, 2008 
 * Description: Telechips Linux BACK-LIGHT DRIVER
 *
 * Copyright (c) Telechips, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/moduleparam.h>
#include <linux/device.h>
#include <linux/fs.h>
#include <linux/errno.h>
#include <asm/uaccess.h>

#include <asm/mach-types.h>


//#if defined(CONFIG_ARCH_TCC92X) || defined (CONFIG_ARCH_TCC93XX)
#include <mach/bsp.h>
//#endif

#ifdef CONFIG_I2C
#   include <linux/i2c.h>
#endif
#include <asm/gpio.h>
#include <linux/i2c/sensor_i2c.h>
#include <linux/input.h>

#include <mach/sensor_ioctl.h>

#define CMA3000_DEBUG  0

#if CMA3000_DEBUG
#define sensor_dbg(fmt, arg...)     printk(fmt, ##arg)
#else
#define sensor_dbg(arg...)
#endif

#define SENSOR_DEV_NAME			"sensor"
#define SENSOR_DEV_MAJOR		249
#define SENSOR_DEV_MINOR		1



typedef struct {
	int x;
	int y;
	int z;
} tcc_sensor_accel_t;

#ifdef CONFIG_I2C
#if defined (CONFIG_ARCH_TCC93XX)
static unsigned short probe[] = { 2, 0x1C, I2C_CLIENT_END };
#elif defined (CONFIG_ARCH_TCC92X)
static unsigned short probe[] = { 1, 0x1C, I2C_CLIENT_END };
#endif
static unsigned short dummy[] = {I2C_CLIENT_END};

static struct i2c_client_address_data addr_data = {
	.normal_i2c = dummy,
	.probe      = probe,
	.ignore     = dummy,
};

static struct i2c_driver sensor_i2c_driver;
static struct i2c_client *sensor_i2c_client;

static struct timer_list *sensor_timer;
static int sensor_used_count=0;
volatile static unsigned int sensor_duration=200;
volatile static unsigned int sensor_state_flag=0;
volatile static tcc_sensor_accel_t tcc_sensor_accel;
static struct work_struct sensor_work_q;

#if defined(CONFIG_ARCH_TCC93XX)
static const struct i2c_device_id sensor_i2c_id[] = {
	{ "tcc-accel-sensor", 0, },
	{ }
};

struct sensor_i2c_chip_info {
	unsigned gpio_start;
	uint16_t reg_output;
	uint16_t reg_direction;

	struct i2c_client *client;
	struct gpio_chip gpio_chip;
};
#endif

static void sensor_timer_handler(unsigned long data)
{
		sensor_dbg("%s\n", __func__);
	if (schedule_work(&sensor_work_q) == 0) {
		sensor_dbg("cannot schedule work !!!\n");
	}
}

static void sensor_timer_registertimer(struct timer_list* ptimer, unsigned int timeover )
{
		sensor_dbg("%s\n", __func__);
	//del_timer_sync(ptimer);
	init_timer(ptimer);
	ptimer->expires = jiffies+msecs_to_jiffies(timeover);
	ptimer->data = (unsigned long)NULL;
	ptimer->function = sensor_timer_handler;

	add_timer(ptimer);
}

static void SENSOR_SEND_CMD(unsigned char reg, unsigned char val)
{
		sensor_dbg("%s\n", __func__);
	unsigned char cmd[2];
	cmd[0] = reg;
	cmd[1] = val;	
	i2c_master_send(sensor_i2c_client, cmd, 2);
}


static unsigned char SENSOR_READ_DAT(unsigned char reg)
{
		sensor_dbg("%s\n", __func__);
	unsigned char buf;

	i2c_master_send(sensor_i2c_client, &reg, 1);
	i2c_master_recv(sensor_i2c_client, &buf, 1);

	return 	buf;
}


#if defined(CONFIG_ARCH_TCC93XX)
static int sensor_i2c_probe(struct i2c_client *client, const struct i2c_device_id *id)
{
	struct sensor_i2c_platform_data 	*pdata;
	struct sensor_i2c_chip_info 	*chip;

	sensor_dbg("\n sensor_i2c_probe  :  %s \n", client->name);
	pdata = client->dev.platform_data;
	if(pdata == NULL)
	{
		dev_dbg(&client->dev, "no platform data\n");
		sensor_dbg("\n tcc_sensor_i2c  :  no platform data. \n");
		return -EINVAL;
	}

	chip = kzalloc(sizeof(struct sensor_i2c_chip_info), GFP_KERNEL);
	if(chip == NULL)
	{
		sensor_dbg("\n tcc_sensor_i2c  :  no chip info. \n");
		return -ENOMEM;
	}

	chip->client = client;
	i2c_set_clientdata(client, chip);
	sensor_i2c_client = client;

	return 0;
}

static int sensor_i2c_remove(struct i2c_client *client)
{
	struct sensor_i2c_platform_data 	*pdata = client->dev.platform_data;
	struct sensor_i2c_chip_info 		*chip  = i2c_get_clientdata(client);
	int ret = 0;

	if(pdata->teardown)
	{
		ret = pdata->teardown(client, chip->gpio_chip.base, chip->gpio_chip.ngpio, pdata->context);
		if(ret < 0)
		{
			dev_err(&client->dev, "%s failed, %d\n", "teardown", ret);
			sensor_dbg("\n tcc_sensor_i2c  :  teardown error. \n");
			return ret;
		}
	}

	kfree(chip);
	sensor_i2c_client = NULL;
	
	return 0;
}
#else // defined(CONFIG_ARCH_TCC93XX)
static int sensor_i2c_probe(struct i2c_adapter *adap, int addr, int kind)
{
    struct i2c_client *i2c;
    int ret;
	unsigned char old_ctrl, new_ctrl;

		sensor_dbg("%s\n", __func__);
//    sensor_dbg("########  %s addr : %d #########\n", __func__, addr);

    i2c = kzalloc(sizeof(*i2c), GFP_KERNEL);
    if (!i2c)
   	{		
        return -ENOMEM;
   	}

    strcpy(i2c->name, "ACCEL_SENSOR");
    i2c->flags = 0;
    i2c->addr = addr;
    i2c->adapter = adap;
    i2c->driver = &sensor_i2c_driver;

    sensor_i2c_client = i2c;

    /* attach i2c client device */
    ret = i2c_attach_client(i2c);
    if (ret < 0) {
        sensor_dbg("%s: failed to attach codec at addr %x\n", __func__, addr);
        goto err;
    }
	//HwGPIOF->GPEN	|= Hw17;
	//HwGPIOF->GPDAT	|= Hw17;
//	HwGPIOD->GPEN	&= ~Hw19;
	//HwGPIOD->GPDAT	|= Hw19;

	//synaptics_io_init();
	SENSOR_READ_DAT(0x19); //soft reset
	SENSOR_READ_DAT(0x19); //back into operation from soft reset
	
	old_ctrl = SENSOR_READ_DAT(0x00);
	sensor_dbg("%s: identification : %x\n", __func__, old_ctrl);
	old_ctrl = SENSOR_READ_DAT(0x01);
	sensor_dbg("%s: ASIC revition ID : %x\n", __func__, old_ctrl);

	old_ctrl = SENSOR_READ_DAT(0x02);
	new_ctrl = (0x80|0x00|0x00|0x00|0x02|0x00);	// range, level, exit, , mode, int_s
	SENSOR_SEND_CMD(0x02, new_ctrl);

    return ret;
err:
    kfree(i2c);
    return ret;
}

#endif // defined(CONFIG_ARCH_TCC93XX)

static int sensor_i2c_detach(struct i2c_client *client)
{
		sensor_dbg("%s\n", __func__);
    i2c_detach_client(client);
    kfree(sensor_i2c_client);
    return 0;
}

static int sensor_i2c_attach(struct i2c_adapter *adap)
{
		sensor_dbg("%s\n", __func__);
    return i2c_probe(adap, &addr_data, sensor_i2c_probe);
}


#if defined(CONFIG_ARCH_TCC93XX)
/* tsc2003 i2c codec control layer */
static struct i2c_driver sensor_i2c_driver = {
	.driver = {
		.name	= "tcc-accel-sensor",
	},
	.probe		= sensor_i2c_probe,
	.remove		= sensor_i2c_remove,
	.id_table	= sensor_i2c_id,
};
#else // defined(CONFIG_ARCH_TCC93XX)

/* synaptics i2c codec control layer */
static struct i2c_driver sensor_i2c_driver = {
    .driver = {
        .name   = "tcc-accel-sensor",
        .owner  = THIS_MODULE,
    },
    .id             = I2C_DRIVERID_ACCEL_SENSOR,
    .attach_adapter = sensor_i2c_attach,
    .detach_client  = sensor_i2c_detach,
    .command        = NULL,
};
#endif // defined(CONFIG_ARCH_TCC93XX)
#endif

static void sensor_openi2c(void)
{
#ifdef CONFIG_I2C
	int ret = 0;

		sensor_dbg("%s\n", __func__);
	ret = i2c_add_driver(&sensor_i2c_driver);
	if (ret != 0)
		sensor_dbg("%s: can't add i2c driver\n", __func__);
#endif
}

static int tcc_sensor_get_accel(void)
{
	int x, y, z;
	
	x = (int)SENSOR_READ_DAT(0x06);
	y = (int)SENSOR_READ_DAT(0x07);
	z = (int)SENSOR_READ_DAT(0x08);

	if (x >= 128)
		x = - (256 - x);

	if (y >= 128)
		y = - (256 - y);

	if (z >= 128)
		z = - (256 - z);

	tcc_sensor_accel.x = x/5;
	tcc_sensor_accel.y = y/5;
	tcc_sensor_accel.z = z/5;

	//sensor_dbg("%s: %d, %d, %d\n", __func__, tcc_sensor_accel.x, tcc_sensor_accel.y, tcc_sensor_accel.z);

	return 0;
}

static void sensor_fetch_thread(struct work_struct *work)
{
	sensor_dbg("%s: sensor_duration =%d \n", __func__, sensor_duration);
	tcc_sensor_get_accel();
	sensor_timer_registertimer( sensor_timer, sensor_duration );
}

static ssize_t tcc_sensor_write(struct file *file, const char __user *user, size_t size, loff_t *o)
 {
	sensor_dbg("%s\n", __func__);
	return 0;
 }

static ssize_t tcc_sensor_read(struct file *file, char __user *user, size_t size, loff_t *o)
{
	sensor_dbg("%s\n", __func__);	
	sensor_dbg("%s: IOCTL_read_SENSORS_ACCELERATION  %d, %d, %d\n", __func__, tcc_sensor_accel.x, tcc_sensor_accel.y, tcc_sensor_accel.z);

//	tcc_sensor_get_accel();   //test  reverse

	if(copy_to_user(( tcc_sensor_accel_t*) user, (const void *)&tcc_sensor_accel, sizeof( tcc_sensor_accel_t))!=0)
	{
		sensor_dbg("tcc_gsensor_read error\n");
	}


	return 0;
}

static int tcc_sensor_ioctl(struct inode *inode, struct file *filp, 
							unsigned int cmd, void *arg)
{
//	if (!sensor_used_count)
//		return -1;

	sensor_dbg(KERN_INFO "%s  (0x%x)  \n", __FUNCTION__, cmd);

	switch (cmd) {
		case IOCTL_SENSOR_GET_DATA_ACCEL:

			if(copy_to_user((tcc_sensor_accel_t*)arg, (const void *)&tcc_sensor_accel, sizeof(tcc_sensor_accel_t))!=0)
			{
					sensor_dbg("copy_to error\n");
			}
			sensor_dbg("%s: IOCTL_SENSOR_GET_DATA_ACCEL %d, %d, %d\n", __func__, tcc_sensor_accel.x, tcc_sensor_accel.y, tcc_sensor_accel.z);
			break;

		case IOCTL_SENSOR_SET_DELAY_ACCEL:
			if(copy_from_user((void *)&sensor_duration, (unsigned int*) arg, sizeof(unsigned int))!=0)
			{
					sensor_dbg("copy_from error\n");
			}					
			sensor_dbg(KERN_INFO "%s:  IOCTL_SENSOR_SET_DELAY_ACCEL (0x%x) %d \n", __FUNCTION__, cmd, sensor_duration);
			//printk(KERN_INFO "%s:  IOCTL_SENSOR_SET_DELAY_ACCEL (0x%x) %d \n", __FUNCTION__, cmd, sensor_duration);
			break;

		case IOCTL_SENSOR_GET_STATE_ACCEL:
			sensor_state_flag = 1;
			if(copy_to_user((unsigned int*) arg, (const void *)&sensor_state_flag, sizeof(unsigned int))!=0)
			{
					sensor_dbg("copy_to error\n");
			}			
			
			sensor_dbg(KERN_INFO "%s: IOCTL_SENSOR_GET_STATE_ACCEL  (0x%x) %d \n", __FUNCTION__, cmd, sensor_state_flag);	
			break;
			
		case IOCTL_SENSOR_SET_STATE_ACCEL:
//			arg = sensor_state_flag;

			if(copy_from_user((void *)&sensor_state_flag, (unsigned int*) arg, sizeof(unsigned int))!=0)
			{
					sensor_dbg("copy_from error\n");
			}			
			sensor_dbg(KERN_INFO "%s: IOCTL_SENSOR_SET_STATE_ACCEL  (0x%x) %d \n", __FUNCTION__, cmd, sensor_state_flag);			
			break;
			
		default:
			sensor_dbg("sensor: unrecognized ioctl (0x%x)\n", cmd); 
			return -EINVAL;
			break;
	}
	return 0;
}

static int tcc_sensor_release(struct inode *inode, struct file *filp)
{
	sensor_dbg(KERN_INFO "%s (%d)\n", __FUNCTION__, sensor_used_count);
	sensor_used_count--;
	if (sensor_used_count < 0) {
		sensor_dbg("sensor: release error (over)\n"); 
		sensor_used_count = 0;
	}

	if (sensor_used_count <= 0) {
		flush_scheduled_work();
		del_timer_sync( sensor_timer );
		//del_timer( sensor_timer);
		kfree( sensor_timer);
		i2c_del_driver(&sensor_i2c_driver);
	}
	return 0;
}

static int tcc_sensor_open(struct inode *inode, struct file *filp)
{
	unsigned char old_ctrl, new_ctrl;
	int num = MINOR(inode->i_rdev);
	
	sensor_dbg(KERN_INFO "%s -> minor : %d, (cnt:%d)\n", __FUNCTION__, num, sensor_used_count+1);
	if (sensor_used_count == 0) {
		sensor_openi2c();

		//synaptics_io_init();
		old_ctrl = SENSOR_READ_DAT(0x00);
		if (old_ctrl == 0xFF) {
			i2c_del_driver(&sensor_i2c_driver);

			sensor_dbg("%s: No such device or address\n", __func__);
			return -ENXIO;
		}
		sensor_dbg("%s: identification : %x\n", __func__, old_ctrl);

		old_ctrl = SENSOR_READ_DAT(0x01);
		sensor_dbg("%s: ASIC revition ID : %x\n", __func__, old_ctrl);

		old_ctrl = SENSOR_READ_DAT(0x02);
		new_ctrl = (0x80|0x00|0x00|0x00|0x02|0x00);	// range, level, exit, , mode, int_s
		SENSOR_SEND_CMD(0x02, new_ctrl);
		sensor_timer= kmalloc( sizeof( struct timer_list ), GFP_KERNEL );
		if (sensor_timer == NULL) return -ENOMEM;
		memset(sensor_timer, 0, sizeof(struct timer_list));
		sensor_dbg("%s: sensor_duration = %d\n", __func__, sensor_duration);	
		sensor_timer_registertimer( sensor_timer, sensor_duration );
	}
	
	sensor_used_count++;	
	return 0;
}

struct file_operations tcc_sensor_fops =
{
	.owner		= THIS_MODULE,
	.open		= tcc_sensor_open,
	.release	= tcc_sensor_release,
	.ioctl		= tcc_sensor_ioctl,
	.read      = tcc_sensor_read,
	.write	= tcc_sensor_write,
};

static struct class *sensor_class;

int __init tcc_sensor_init(void)
{
	int res;

	sensor_dbg(KERN_INFO "tcc_sensor_init \n", __FUNCTION__);
	res = register_chrdev(SENSOR_DEV_MAJOR, SENSOR_DEV_NAME, &tcc_sensor_fops);
	if (res < 0)
		return res;

	sensor_class = class_create(THIS_MODULE, SENSOR_DEV_NAME);
	device_create(sensor_class,NULL,MKDEV(SENSOR_DEV_MAJOR,SENSOR_DEV_MINOR),NULL,SENSOR_DEV_NAME);

	INIT_WORK(&sensor_work_q, sensor_fetch_thread);
	//init_waitqueue_head(&(tsc2003_ts->wait_q));
	//mutex_init(&(tcc_sensor->mutex));

	sensor_dbg(KERN_INFO "%s\n", __FUNCTION__);
	return 0;
}

void __exit tcc_sensor_exit(void)
{
    sensor_dbg(KERN_INFO "%s\n", __FUNCTION__);
	unregister_chrdev(SENSOR_DEV_MAJOR, SENSOR_DEV_NAME);

}

module_init(tcc_sensor_init);
module_exit(tcc_sensor_exit);

MODULE_AUTHOR("Telechips Inc. c2-g2-2 linux@telechips.com");
MODULE_DESCRIPTION("TCCxxx accel-gsensor driver");
MODULE_LICENSE("GPL");

