
#include <linux/init.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/moduleparam.h>
#include <linux/fs.h>
#include <linux/errno.h>
#include <asm/uaccess.h>
#include <linux/platform_device.h>
#include <linux/wakelock.h>
#include <linux/leds.h>
#include <linux/device.h>
#include <linux/err.h>
#include <linux/backlight.h>
#include <linux/kdev_t.h>
#include <linux/kobject.h>
#include <linux/sysfs.h>

#include "../char/tca_backlight.h"


extern void tca_bkl_setpowerval(unsigned int inValue);
extern unsigned int tca_bkl_getpowerval(void);

static void tcc_bl_set(struct led_classdev *led_cdev, enum led_brightness value)
{
//	printk("---> %s level:%d \n", __func__, value);
	tca_bkl_setpowerval(value);
}



static struct led_classdev tcc_bl_supplies[] = {
    {
        .name = "lcd-backlight",
		.brightness_set = tcc_bl_set,
    },
    {
        .name = "button-backlight",
        .brightness_set = tcc_bl_set,

    },
    {
        .name = "keyboard-backlight",
        .brightness_set = tcc_bl_set,

    },

};


static struct backlight_device *tcc_backlight_device;

static int read_brightness(struct backlight_device *bd)
{
	printk("->%s\n", __func__);
	int value;
	value = tca_bkl_getpowerval();
	return value;
}

static int update_bl_status(struct backlight_device *bd)
{
	int intensity = bd->props.brightness;

//	printk("->%s brightness:%d \n", __func__, intensity);

	tca_bkl_setpowerval(intensity);

	return 0;
}


static struct backlight_ops tcc_bl_ops = {
	.get_brightness = read_brightness,
	.update_status 	= update_bl_status,
};

static ssize_t tcc_bl_show_property(struct device *dev,
                                     struct device_attribute *attr,
                                     char *buf)
{
    int i = 0;
	printk("->%s\n", __func__);
	return 0;
}



#define TCC_BL_ATTR(_name)                                            		\
{                                                                            \
     .attr = { .name = #_name, .mode = S_IRUGO | S_IWUSR, .owner = THIS_MODULE },      \
     .show = tcc_bl_show_property,                                      	\
     .store = NULL,                                                          \
}

static struct device_attribute tcc_bl_attrs[] = {
     TCC_BL_ATTR(lcd-backlight),
     TCC_BL_ATTR(button-backlight),
     TCC_BL_ATTR(keyboard-backlight),

};



static int tcc_bl_create_attrs(struct device * dev)
{
     int i, j, rc;
     
     for (i = 0; i < ARRAY_SIZE(tcc_bl_attrs); i++) {
         rc = device_create_file(dev, &tcc_bl_attrs[i]);
         if (rc)
         	goto tcc_attrs_failed;
     }
     
     goto succeed;
     
tcc_attrs_failed:
     while (i--)
     	device_remove_file(dev, &tcc_bl_attrs[i]);

succeed:        
     return rc;
}


static int tcc_bl_probe(struct platform_device *pdev)
{
	int ret, i, err;

	for (i = 0; i < ARRAY_SIZE(tcc_bl_supplies); i++) {
        err = led_classdev_register(&pdev->dev, &tcc_bl_supplies[i]);
        if (err)
        	printk(KERN_ERR "Failed to register power supply (%d)\n", err);
    }
	
	/* create tcc detail attributes */
    tcc_bl_create_attrs(tcc_bl_supplies[0].dev);

	return ret;
}

static int tcc_bl_suspend(struct platform_device *dev, pm_message_t state)
{
	printk("---> %s \n", __func__);
	return 0;
}


static int tcc_bl_resume(struct platform_device *dev)
{

//	printk("---> %s \n", __func__);
	return 0;
}


static struct platform_driver tcc_bl_driver = {
    .driver.name	= "tcc-backlight",
	.driver.owner	= THIS_MODULE,
	.probe			= tcc_bl_probe,
	.suspend		= tcc_bl_suspend,
	.resume			= tcc_bl_resume,
};

static int __init tcc92x_bl_init(void)
{
	int res;
	
	printk("---> %s \n", __func__);
	res = platform_driver_register(&tcc_bl_driver);

	return 0;
}

static void __exit tcc92x_bl_exit(void)
{
	printk("---> %s \n", __func__);

	platform_driver_unregister(&tcc_bl_driver);
}


module_init(tcc92x_bl_init);
module_exit(tcc92x_bl_exit);

MODULE_AUTHOR("Telechips Inc. SYS3-4 linux@telechips.com");
MODULE_DESCRIPTION("TCC92x back-light driver");
MODULE_LICENSE("GPL");
	
