/************************************************************************
*    Telechips Multi Media Player
*    ------------------------------------------------
*
*    FUNCTION    : CAMERA INTERFACE API
*    MODEL        : DMP
*    CPU NAME    : TCCXXX
*    SOURCE        : cam.c
*
*    START DATE    : 2006. 4. 17.
*    MODIFY DATE :
*    DEVISION    : DEPT. SYSTEM 3-2 TEAM
*                : TELECHIPS, INC.
************************************************************************/
#include <linux/delay.h> 
#include <mach/hardware.h>
#include <linux/clk.h>


#if defined(CONFIG_ARCH_TCC92X) || defined(CONFIG_ARCH_TCC93XX)
#include <mach/bsp.h>
#elif defined(CONFIG_ARCH_TCC79X)
#include <mach/tcc79x.h>
#include <asm/plat-tcc/tcc_ckc.h>
#endif

#include "cam.h"
#include "cam_reg.h"
#include "sensor_if.h"


static struct clk *cifmc_clk;
#if defined(CONFIG_USE_ISP)
static struct clk *isps_clk;
static struct clk *ispj_clk;
#else
static struct clk *cifsc_clk;
#endif

/**********************************************************
*
*    Function of    
*
*    
*
*    Input    : 
*    Output    : 
*    Return    : 
*
*    Description    : 
**********************************************************/
void CIF_Clock_Get(void)
{
	cifmc_clk = clk_get(NULL, "cifmc");	
	BUG_ON(cifmc_clk == NULL);
	
	#if defined(CONFIG_USE_ISP)
	isps_clk = clk_get(NULL, "isps");
	BUG_ON(isps_clk == NULL);

	ispj_clk = clk_get(NULL, "ispj");
	BUG_ON(ispj_clk == NULL);
	#else
	cifsc_clk = clk_get(NULL, "cifsc");	
	BUG_ON(cifsc_clk == NULL);
	#endif
}

void CIF_Clock_Put(void)
{
	clk_put(cifmc_clk);
	#if defined(CONFIG_USE_ISP)
	clk_put(isps_clk);
	clk_put(ispj_clk);
	#else
	clk_put(cifsc_clk);
	#endif
}

/**********************************************************
*
*    Function of    
*
*    
*
*    Input    : 
*    Output    : 
*    Return    : 
*
*    Description    : 
**********************************************************/
void CIF_Open(void)
{
#if defined(CONFIG_ARCH_TCC93XX)
	volatile PCAMBUSCFG pCamBusCfg = (PCAMBUSCFG)tcc_p2v(HwCAMBUSCFG_BASE);
#endif

#if !defined(CONFIG_VIDEO_ATV_SENSOR_TVP5150)
	//PLL2 : 400Mhz
	tca_ckc_setperi(PERI_CIFMC, DISABLE, CKC_CAMERA_MCLK, CKC_CAMERA_MCLK_SRC);
#endif
#if defined(CONFIG_USE_ISP)
	//tca_ckc_setperi(PERI_ISPS, DISABLE, 1920000, DIRECTPLL1);
	//tca_ckc_setperi(PERI_ISPJ, DISABLE, 1920000*2, DIRECTPLL1);
	tca_ckc_setperi(PERI_ISPS, DISABLE, 1620000, DIRECTPLL2);
	tca_ckc_setperi(PERI_ISPJ, DISABLE, 1620000*2, DIRECTPLL2);
	printk("ISP CLK :: MCLK = %d, ISPS = %d, ISPJ = %d. \n", CKC_CAMERA_MCLK/10000, 1620000/10000, 1620000*2/10000);
#else
	tca_ckc_setperi(PERI_CIFSC, DISABLE, CKC_CAMERA_SCLK, CKC_CAMERA_SCLK_SRC);
	printk("CLK :: MCLK = %d, SCLK = %d. \n", CKC_CAMERA_MCLK/10000, CKC_CAMERA_SCLK/10000);
#endif

	clk_enable(cifmc_clk);
#if defined(CONFIG_USE_ISP)
	clk_enable(isps_clk);
	clk_enable(ispj_clk);
#else
	clk_enable(cifsc_clk);
#endif

#if defined(CONFIG_ARCH_TCC79X)
	BITSET(HwSWRESET, HwSWRESET_CIC_ON);
	BITCLR(HwSWRESET, HwSWRESET_CIC_ON);
#elif defined(CONFIG_ARCH_TCC92X)
	BITSET(HwDDI_CONFIG->SWRESET, HwDDIC_SWRESET_CIF);   // Reset
	BITCLR(HwDDI_CONFIG->SWRESET, HwDDIC_SWRESET_CIF);  // Normal
#elif defined(CONFIG_ARCH_TCC93XX)

	BITCLR(pCamBusCfg->CAMBusClk0Sel, HwCAMBUS_CLKO_SEL);
#if defined(CONFIG_USE_ISP)
	BITSET(pCamBusCfg->SoftResetRegister, HwCAMBUS_SWRESET_ISP);   // Reset
	BITCLR(pCamBusCfg->SoftResetRegister, HwCAMBUS_SWRESET_ISP);  // Normal
#else
	BITSET(pCamBusCfg->SoftResetRegister, HwCAMBUS_SWRESET_CIF);   // Reset
	BITCLR(pCamBusCfg->SoftResetRegister, HwCAMBUS_SWRESET_CIF);  // Normal
#endif
#endif
}

/**********************************************************
*
*    Function of    
*
*    
*
*    Input    : 
*    Output    : 
*    Return    : 
*
*    Description    : 
**********************************************************/
void CIF_Close(void)
{
#if defined(CONFIG_ARCH_TCC93XX)
	volatile PCAMBUSCFG pCamBusCfg = (PCAMBUSCFG)tcc_p2v(HwCAMBUSCFG_BASE);

	clk_disable(cifmc_clk);
	#if defined(CONFIG_USE_ISP)
	clk_disable(isps_clk);
	clk_disable(ispj_clk);
	#else
	clk_disable(cifsc_clk);
	#endif

	BITSET(pCamBusCfg->CAMBusClk0Sel, HwCAMBUS_CLKO_SEL);
#else
	clk_disable(cifmc_clk);
	clk_disable(cifsc_clk);
#endif
}

/**********************************************************
*
*    Function of    
*
*    
*
*    Input    : 
*    Output    : 
*    Return    : 
*
*    Description    : 
**********************************************************/
void CIF_ONOFF(unsigned int uiOnOff)
{
#ifdef CONFIG_ARCH_TCC79X
	if(uiOnOff == ON)
	{
		BITCSET(HwICPCR1, HwICPCR1_ON, (uiOnOff << 31));
	}
	else if(uiOnOff == OFF)
	{
		BITCSET(HwICPCR1, HwICPCR1_ON, (uiOnOff << 31));
	}
#else  //CONFIG_ARCH_TCC92X
	volatile PCIF pCIF = (PCIF)tcc_p2v(HwCIF_BASE);

	if(uiOnOff == ON)
	{
		BITCSET(pCIF->ICPCR1, HwICPCR1_ON, (uiOnOff << 31));
	}
	else if(uiOnOff == OFF)
	{
		BITCSET(pCIF->ICPCR1, HwICPCR1_ON, (uiOnOff << 31));
	}
#endif
}


/**********************************************************
*
*    Function of    
*
*
*    Input    : 
*    Output    : 
*    Return    : 
*
*    Description    : 
**********************************************************/
void CIF_WaitFrameSync(unsigned int exp_timer)
{
	unsigned int cnt=0;
	
#ifdef CONFIG_ARCH_TCC79X
	while((HwCIRQ & HwCIRQ_SOF) != HwCIRQ_SOF)
	{
		mdelay(1);
		cnt++;

		if(cnt>exp_timer)
			return;

	}
#else  //CONFIG_ARCH_TCC92X
	PCIF pCIF = (PCIF)tcc_p2v(HwCIF_BASE);

	while((pCIF->CIRQ & HwCIRQ_SOF) != HwCIRQ_SOF)
	{
		mdelay(1);
		cnt++;

		if(cnt>exp_timer)
			return;

	}
#endif
}

void CIF_OpStop(char wait_SOF, char sw_reset )
{
#if defined(CONFIG_ARCH_TCC79X)	
	if(wait_SOF)
	{
		BITSET(HwCIF->CIRQ, HwCIRQ_SOF);
		CIF_WaitFrameSync(400);
		BITSET(HwCIF->CIRQ, HwCIRQ_SOF);
	}
#elif defined(CONFIG_ARCH_TCC92X) || defined(CONFIG_ARCH_TCC93XX)
	volatile PCIF 		pCIF 		= (PCIF)tcc_p2v(HwCIF_BASE);
	#if defined(CONFIG_ARCH_TCC92X)
	volatile PDDICONFIG pDDICfg 	= (PDDICONFIG)tcc_p2v(HwDDI_CONFIG_BASE);
	#elif defined(CONFIG_ARCH_TCC93XX)
	volatile PCAMBUSCFG pCamBusCfg 	= (PCAMBUSCFG)tcc_p2v(HwCAMBUSCFG_BASE);
	#endif
	
	if(wait_SOF)
	{
		BITSET(pCIF->CIRQ, HwCIRQ_SOF);
		CIF_WaitFrameSync(400);
		BITSET(pCIF->CIRQ, HwCIRQ_SOF);
	}
#endif

	CIF_ONOFF(OFF);

	if(sw_reset)
	{
#if defined(CONFIG_ARCH_TCC79X)
		BITSET(HwSWRESET, HwSWRESET_CIC_ON);
		BITCLR(HwSWRESET, HwSWRESET_CIC_ON);
#elif defined(CONFIG_ARCH_TCC92X)
		BITSET(pDDICfg->SWRESET, HwDDIC_SWRESET_CIF);   // Reset
		BITCLR(pDDICfg->SWRESET, HwDDIC_SWRESET_CIF);  // Normal
		printk("cif reset out \n");
#elif defined(CONFIG_ARCH_TCC93XX)
		BITSET(pCamBusCfg->SoftResetRegister, HwCAMBUS_SWRESET_CIF);   // Reset
		BITCLR(pCamBusCfg->SoftResetRegister, HwCAMBUS_SWRESET_CIF);  // Normal
#endif
	}
}
/* end of file */
