/*====*====*====*====*====*====*====*====*====*====*====*====*====*====*====*

               CAMERA    API    M O D U L E

                        EDIT HISTORY FOR MODULE

when        who       what, where, why
--------    ---       -------------------------------------------------------
10/xx/08  Telechips   Created file.
*====*====*====*====*====*====*====*====*====*====*====*====*====*====*====*/

/*===========================================================================

                           INCLUDE FILES FOR MODULE

===========================================================================*/
#include <linux/delay.h>
#include <asm/system.h>
#include <mach/hardware.h>
#include <asm/io.h>
#include "sensor_if.h"
#include "cam.h"
#include "tcc_cam_i2c.h"

#if defined(CONFIG_ARCH_TCC92X) || defined(CONFIG_ARCH_TCC93XX)
#include <mach/bsp.h>
#elif defined(CONFIG_ARCH_TCC79X)
#include <mach/tcc79x.h>
#endif

#ifdef CONFIG_VIDEO_CAMERA_SENSOR_MT9P111

#if (0)
#if defined(CONFIG_MACH_TCC9200S)
#define CAM_PWR_ENABLE()		(HwGPIOD->GPEN |= Hw10);	(HwGPIOD->GPDAT |= Hw10);
#define CAM_PWR_DISABLE()		(HwGPIOD->GPEN |= Hw10);	(HwGPIOD->GPDAT &= ~Hw10);
#define CAM_PWRDN_ENABLE()		(HwGPIOF->GPEN |= Hw21);	(HwGPIOF->GPDAT |= Hw21);
#define CAM_PWRDN_DISABLE()		(HwGPIOF->GPEN |= Hw21);	(HwGPIOF->GPDAT &= ~Hw21);
#define CAM_RESET_HIGH()		(HwGPIOE->GPEN |= Hw2);		(HwGPIOE->GPDAT |= Hw2);
#define CAM_RESET_LOW()			(HwGPIOE->GPEN |= Hw2);		(HwGPIOE->GPDAT &= ~Hw2);
#elif defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900) 
#define CAM_PWR_ENABLE()
#define CAM_PWR_DISABLE()
#define CAM_PWRDN_ENABLE()		//(HwGPIOF->GPEN |= Hw21);	(HwGPIOF->GPDAT |= Hw21);
#define CAM_PWRDN_DISABLE()		//(HwGPIOF->GPEN |= Hw21);	(HwGPIOF->GPDAT &= ~Hw21);
#define CAM_RESET_HIGH()		(HwGPIOE->GPEN |= Hw26);	(HwGPIOE->GPDAT |= Hw26);
#define CAM_RESET_LOW()			(HwGPIOE->GPEN |= Hw26);	(HwGPIOE->GPDAT &= ~Hw26);
#else
#define CAM_PWR_ENABLE()
#define CAM_PWR_DISABLE()
#define CAM_PWRDN_ENABLE()		(HwGPFEN |= Hw1); 	(HwGPFDAT |= Hw1);
#define CAM_PWRDN_DISABLE()		(HwGPFEN |= Hw1); 	(HwGPFDAT &= ~Hw1);
#define CAM_RESET_HIGH()		(HwGPEEN |= Hw9); 	(HwGPEDAT |= Hw9);
#define CAM_RESET_LOW()			(HwGPEEN |= Hw9); 	(HwGPEDAT &= ~Hw9);
#endif
#endif

/* Array of image sizes supported by MV9317.  These must be ordered from 
 * smallest image size to largest.
 */
struct capture_size sensor_sizes[] = {
	{ 2560, 1920 },	/* QQXGA */
	{ 2048, 1536 },	/* QXGA */
	{ 1600, 1200 },	/* UXGA */
	{ 1280,  720 },	/* SXGA */
	{ 1024,  768 },	/* XGA */
	{  800,  600 },	/* SVGA */
	{  640,  480 },	/* VGA */
	{  320,  240 },	/* QVGA */
	{  176,  144 }	/* QCIF */
};


/* register initialization tables for sensor */
/* common sensor register initialization for all image sizes, pixel formats, 
 * and frame rates
 */
static struct sensor_reg sensor_initialize[] = {
//PART 0 BURST-DATA
	{0x0010, 0x0120, 0x0070, 0x2025, 0x1111}, 
	{0x001E, 0x0777, 0x1111},	  // PAD_SLEW_PAD_CONFIG
	{0x0022, 0x01E0, 0x1111},	  // VDD_DIS_COUNTER
	{0x002A, 0x7FFF, 0x0000, 0x0000, 0x1111},
	{0x0018, 0x4008, 0x1111},	  // STANDBY_CONTROL_AND_STATUS
	{0x098E, 0x8001, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [MON_MODE]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x0982, 0x0000, 0x1111},	  // ACCESS_CTL_STAT
	{0x098A, 0x0838, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0xC0F1, 0xC5E1, 0x246A, 0x1280, 0xC4E1, 0xD30E, 0x2069, 0x0000, 0x1111},
	{0x098A, 0x0848, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x1305, 0x0084, 0x1730, 0x7005, 0xD801, 0x8B41, 0xD900, 0x0D0A, 0x1111},
	{0x098A, 0x0858, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x0064, 0x8B63, 0xE809, 0xD509, 0xF004, 0x1504, 0x1400, 0x7840, 0x1111},
	{0x098A, 0x0868, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0xD007, 0x0DFB, 0x9004, 0xC4C1, 0x2029, 0x0300, 0x0191, 0x0684, 0x1111},
	{0x098A, 0x0878, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0xFF80, 0x0894, 0xFF80, 0x0838, 0xFF80, 0x0888, 0xFF80, 0x0894, 0x1111},
	{0x098A, 0x0888, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0xFF80, 0x0AA4, 0xFF80, 0x0A10, 0xFF80, 0x0B78, 0x0001, 0x0003, 0x1111},
	{0x098A, 0x0898, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x0000, 0x0000, 0xC0F1, 0x0DBA, 0x01C4, 0xD167, 0xD067, 0x1952, 0x1111},
	{0x098A, 0x08A8, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x0024, 0x208A, 0x0004, 0x199C, 0x0024, 0xD165, 0x8101, 0x2004, 0x1111},
	{0x098A, 0x08B8, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x8FBE, 0x0100, 0xF0F0, 0xF208, 0x8901, 0x080D, 0x005E, 0xD061, 0x1111},
	{0x098A, 0x08C8, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0xD901, 0xA820, 0xC0D1, 0x7EE0, 0xC0F1, 0x08BE, 0x0684, 0xD75C, 0x1111},
	{0x098A, 0x08D8, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x970E, 0x0813, 0x015E, 0xD15C, 0x814E, 0x9239, 0xB734, 0x925A, 0x1111},
	{0x098A, 0x08E8, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0xF005, 0xD25A, 0x922A, 0x924B, 0xB734, 0xD653, 0xD558, 0x2053, 0x1111},
	{0x098A, 0x08F8, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x8040, 0x1E40, 0x1064, 0x1E42, 0x10A4, 0xB755, 0xF220, 0x0A0E, 0x1111},
	{0x098A, 0x0908, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x01E4, 0xD801, 0xD800, 0xAF00, 0x1E3C, 0x1024, 0x8500, 0x8009, 0x1111},
	{0x098A, 0x0918, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x7860, 0xDF00, 0x8500, 0x8004, 0x7840, 0x8500, 0x8005, 0x7840, 0x1111},
	{0x098A, 0x0928, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x8500, 0x8006, 0x7840, 0xD045, 0x1E52, 0x1024, 0x208A, 0x0004, 0x1111},
	{0x098A, 0x0938, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x1E9C, 0x1024, 0x8500, 0x8003, 0x7840, 0xF021, 0x8500, 0x8003, 0x1111},
	{0x098A, 0x0948, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x7840, 0xD800, 0x163C, 0x1901, 0x2153, 0x00C2, 0x0A25, 0x0153, 0x1111},
	{0x098A, 0x0958, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x695C, 0xBAC3, 0x0A1D, 0x0153, 0x2942, 0x0202, 0xBAC3, 0xE284, 0x1111},
	{0x098A, 0x0968, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x29C2, 0x032C, 0x21D3, 0x00EC, 0x21CC, 0x812C, 0x20CA, 0x006C, 0x1111},
	{0x098A, 0x0978, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0xE86A, 0xDF00, 0x0996, 0x01E4, 0x70E9, 0xD032, 0x8800, 0xE809, 0x1111},
	{0x098A, 0x0988, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x167E, 0x1900, 0xE080, 0x27CA, 0x1061, 0x1E7E, 0x13E4, 0x8500, 0x1111},
	{0x098A, 0x0998, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x8007, 0x7840, 0x0059, 0x0684, 0xC0F1, 0x0A5A, 0x01C4, 0xD228, 0x1111},
	{0x098A, 0x09A8, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x920E, 0x2051, 0x8180, 0x9212, 0xF205, 0x204F, 0x01C1, 0xB232, 0x1111},
	{0x098A, 0x09B8, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0xF004, 0xB8A7, 0x7910, 0xB212, 0xD026, 0x1804, 0x0064, 0xF184, 0x1111},
	{0x098A, 0x09C8, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0xC0F1, 0xC5E1, 0xD21E, 0x8A01, 0x9234, 0x9275, 0x0815, 0x009E, 0x1111},
	{0x098A, 0x09D8, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x8A31, 0x8A12, 0x8A73, 0xB908, 0x7905, 0x8A14, 0xBB08, 0x7B05, 0x1111},
	{0x098A, 0x09E8, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x8A00, 0x92AE, 0xB8A2, 0xAA00, 0x0D13, 0x105E, 0xDD05, 0xBD09, 0x1111},
	{0x098A, 0x09F8, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x0D0B, 0x1043, 0xB882, 0xAA00, 0x71A9, 0xD017, 0x1808, 0x0064, 0x1111},
	{0x098A, 0x0A08, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x180A, 0x00E4, 0x07F9, 0x0644, 0xC0F1, 0x0F82, 0x0664, 0xDA28, 0x1111},
	{0x098A, 0x0A18, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0xD112, 0xD513, 0x76A9, 0x0D16, 0x0664, 0x70C9, 0xD011, 0xA507, 0x1111},
	{0x098A, 0x0A28, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0xD011, 0xA502, 0xD011, 0xA503, 0xD011, 0xA509, 0xD008, 0x07C9, 0x1111},
	{0x098A, 0x0A38, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x0664, 0xA0C0, 0xFF00, 0x3BC4, 0x0000, 0xFFFF, 0xFF80, 0x0290, 0x1111},
	{0x098A, 0x0A48, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0xFF80, 0x0BF8, 0xFF80, 0x0340, 0xFF80, 0x02D0, 0x8000, 0x0070, 0x1111},
	{0x098A, 0x0A58, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0xFF00, 0x3C3C, 0xFF00, 0x3C00, 0x0000, 0xE758, 0xFF80, 0x0BFC, 0x1111},
	{0x098A, 0x0A68, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0xFF80, 0x089C, 0xFF80, 0x08D0, 0xFF80, 0x09A0, 0xFF80, 0x09C8, 0x1111},
	{0x098A, 0x0A78, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0xC0F1, 0xC5E1, 0x099E, 0xFFE3, 0x7508, 0xD111, 0x79B6, 0x8910, 0x1111},
	{0x098A, 0x0A88, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x080D, 0x0091, 0xD20F, 0x9201, 0xB886, 0xB201, 0x890F, 0x080D, 0x1111},
	{0x098A, 0x0A98, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x0111, 0xD00D, 0xD905, 0xB021, 0x0765, 0x0644, 0xC0F1, 0x0EEA, 0x1111},
	{0x098A, 0x0AA8, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x0664, 0xDA34, 0xD709, 0xD60A, 0x75E9, 0x70A9, 0x0D2E, 0x0664, 0x1111},
	{0x098A, 0x0AB8, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x8620, 0xD008, 0xA701, 0x0739, 0x0664, 0xA6A0, 0xFF80, 0x01B0, 0x1111},
	{0x098A, 0x0AC8, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0xFF80, 0x05E8, 0xFF80, 0x0698, 0xFF80, 0x0C24, 0x8000, 0x0000, 0x1111},
	{0x098A, 0x0AD8, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0xFF80, 0x0A78, 0xC0F1, 0x0EAA, 0x0644, 0x0DAE, 0x0544, 0xD02D, 0x1111},
	{0x098A, 0x0AE8, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0xD52D, 0x88D2, 0x08C2, 0x00A4, 0x8D04, 0x7EDB, 0xD72B, 0x218A, 0x1111},
	{0x098A, 0x0AF8, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x0FCF, 0x2805, 0x03BE, 0x2941, 0x7140, 0x208C, 0x8FCF, 0x20CA, 0x1111},
	{0x098A, 0x0B08, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x0046, 0x1F2E, 0x1024, 0x08A2, 0x00A4, 0x9507, 0xD624, 0x16E3, 0x1111},
	{0x098A, 0x0B18, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x1082, 0x703A, 0x1660, 0x1104, 0x9507, 0x7148, 0x2186, 0x03FC, 0x1111},
	{0x098A, 0x0B28, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0xB944, 0xBAC3, 0x0876, 0x00A4, 0x165E, 0x1103, 0x6018, 0x7815, 0x1111},
	{0x098A, 0x0B38, 0x1111},	  // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x0F0E, 0x0664, 0x712A, 0x16E2, 0x1082, 0x2840, 0x0210, 0x1660, 0x1111},
	{0x098A, 0x0B48, 0x1111}, // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x1104, 0x9507, 0x7148, 0x2186, 0x03FC, 0xB944, 0xBAC3, 0x084E, 0x1111},
	{0x098A, 0x0B58, 0x1111}, // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x00A4, 0x165E, 0x1103, 0x6018, 0x7815, 0x0EE6, 0x0664, 0x712A, 0x1111},
	{0x098A, 0x0B68, 0x1111}, // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0xB80C, 0x2005, 0x0400, 0x1F32, 0x1024, 0x0675, 0x0644, 0x78E0, 0x1111},
	{0x098A, 0x0B78, 0x1111}, // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0xC0F1, 0x0E1A, 0x0664, 0xDA38, 0xD10A, 0xD50B, 0x76A9, 0x0BAE, 0x1111},
	{0x098A, 0x0B88, 0x1111}, // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x0664, 0x70C9, 0xD009, 0xA503, 0xD009, 0x066D, 0x0664, 0xA0C0, 0x1111},
	{0x098A, 0x0B98, 0x1111}, // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0xFF80, 0x05D4, 0xFF80, 0x07C8, 0xFF00, 0x3390, 0xFF80, 0x06E4, 0x1111},
	{0x098A, 0x0BA8, 0x1111}, // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x0000, 0xF170, 0xFF80, 0x0C58, 0xFF80, 0x0ADC, 0x8000, 0x014C, 0x1111},
	{0x098A, 0x0BB8, 0x1111}, // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0xEA0A, 0x242F, 0x7089, 0x78E0, 0x20A8, 0x0180, 0x1002, 0x0D02, 0x1111},
	{0x098A, 0x0BC8, 0x1111}, // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x1902, 0x0094, 0x7FE0, 0x7028, 0x7308, 0x1000, 0x0900, 0x7904, 0x1111},
	{0x098A, 0x0BD8, 0x1111}, // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x7947, 0x1B00, 0x0064, 0x7EE0, 0xEA0A, 0x242F, 0x7089, 0x78E0, 0x1111},
	{0x098A, 0x0BE8, 0x1111}, // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x20A8, 0x0180, 0x1102, 0x0502, 0x1802, 0x00B4, 0x7FE0, 0x7028, 0x1111},
	{0x098A, 0x0BF8, 0x1111}, // PHYSICAL_ADDRESS_ACCESS
	{0x0990, 0x0000, 0x0000, 0x1111},
	{0x098E, 0x0016, 0x1111}, // LOGICAL_ADDRESS_ACCESS [MON_ADDRESS_LO]
	{0x0990, 0x0838, 0x1111}, // MCU_VARIABLE_DATA0
	{0x098E, 0x0002, 0x1111}, // LOGICAL_ADDRESS_ACCESS [MON_CMD]
	
	{REG_TERM, 0x0050, 0x1111}, //delay 80ms

//PART 1
	{0x0990, 0x0001, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x30D4, 0xB080, 0x1111},	  // COLUMN_CORRECTION
	{0x316E, 0xC400, 0x1111},	  // DAC_ECL
	{0x305E, 0x10A0, 0x1111},	  // GLOBAL_GAIN
	{0x3E00, 0x0010, 0x1111},	  // SAMP_CONTROL
	{0x3E02, 0xED02, 0x1111},	  // SAMP_ADDR_EN
	{0x3E04, 0xC88C, 0x1111},	  // SAMP_RD1_SIG
	{0x3E06, 0xC88C, 0x1111},	  // SAMP_RD1_SIG_BOOST
	{0x3E08, 0x700A, 0x1111},	  // SAMP_RD1_RST
	{0x3E0A, 0x701E, 0x1111},	  // SAMP_RD1_RST_BOOST
	{0x3E0C, 0x00FF, 0x1111},	  // SAMP_RST1_EN
	{0x3E0E, 0x00FF, 0x1111},	  // SAMP_RST1_BOOST
	{0x3E10, 0x00FF, 0x1111},	  // SAMP_RST1_CLOOP_SH
	{0x3E12, 0x0000, 0x1111},	  // SAMP_RST_BOOST_SEQ
	{0x3E14, 0xC78C, 0x1111},	  // SAMP_SAMP1_SIG
	{0x3E16, 0x6E06, 0x1111},	  // SAMP_SAMP1_RST
	{0x3E18, 0xA58C, 0x1111},	  // SAMP_TX_EN
	{0x3E1A, 0xA58E, 0x1111},	  // SAMP_TX_BOOST
	{0x3E1C, 0xA58E, 0x1111},	  // SAMP_TX_CLOOP_SH
	{0x3E1E, 0xC0D0, 0x1111},	  // SAMP_TX_BOOST_SEQ
	{0x3E20, 0xEB00, 0x1111},	  // SAMP_VLN_EN
	{0x3E22, 0x00FF, 0x1111},	  // SAMP_VLN_HOLD
	{0x3E24, 0xEB02, 0x1111},	  // SAMP_VCL_EN
	{0x3E26, 0xEA02, 0x1111},	  // SAMP_COLCLAMP
	{0x3E28, 0xEB0A, 0x1111},	  // SAMP_SH_VCL
	{0x3E2A, 0xEC01, 0x1111},	  // SAMP_SH_VREF
	{0x3E2C, 0xEB01, 0x1111},	  // SAMP_SH_VBST
	{0x3E2E, 0x00FF, 0x1111},	  // SAMP_SPARE
	{0x3E30, 0x00F3, 0x1111},	  // SAMP_READOUT
	{0x3E32, 0x3DFA, 0x1111},	  // SAMP_RESET_DONE
	{0x3E34, 0x00FF, 0x1111},	  // SAMP_VLN_CLAMP
	{0x3E36, 0x00F3, 0x1111},	  // SAMP_ASC_INT
	{0x3E38, 0x0000, 0x1111},	  // SAMP_RS_CLOOP_SH_R
	{0x3E3A, 0xF802, 0x1111},	  // SAMP_RS_CLOOP_SH
	{0x3E3C, 0x0FFF, 0x1111},	  // SAMP_RS_BOOST_SEQ
	{0x3E3E, 0xEA10, 0x1111},	  // SAMP_TXLO_GND
	{0x3E40, 0xEB05, 0x1111},	  // SAMP_VLN_PER_COL
	{0x3E42, 0xE5C8, 0x1111},	  // SAMP_RD2_SIG
	{0x3E44, 0xE5C8, 0x1111},	  // SAMP_RD2_SIG_BOOST
	{0x3E46, 0x8C70, 0x1111},	  // SAMP_RD2_RST
	{0x3E48, 0x8C71, 0x1111},	  // SAMP_RD2_RST_BOOST
	{0x3E4A, 0x00FF, 0x1111},	  // SAMP_RST2_EN
	{0x3E4C, 0x00FF, 0x1111},	  // SAMP_RST2_BOOST
	{0x3E4E, 0x00FF, 0x1111},	  // SAMP_RST2_CLOOP_SH
	{0x3E50, 0xE38D, 0x1111},	  // SAMP_SAMP2_SIG
	{0x3E52, 0x8B0A, 0x1111},	  // SAMP_SAMP2_RST
	{0x3E58, 0xEB0A, 0x1111},	  // SAMP_PIX_CLAMP_EN
	{0x3E5C, 0x0A00, 0x1111},	  // SAMP_PIX_PULLUP_EN
	{0x3E5E, 0x00FF, 0x1111},	  // SAMP_PIX_PULLDOWN_EN_R
	{0x3E60, 0x00FF, 0x1111},	  // SAMP_PIX_PULLDOWN_EN_S
	{0x3E90, 0x3C01, 0x1111},	  // RST_ADDR_EN
	{0x3E92, 0x00FF, 0x1111},	  // RST_RST_EN
	{0x3E94, 0x00FF, 0x1111},	  // RST_RST_BOOST
	{0x3E96, 0x3C00, 0x1111},	  // RST_TX_EN
	{0x3E98, 0x3C00, 0x1111},	  // RST_TX_BOOST
	{0x3E9A, 0x3C00, 0x1111},	  // RST_TX_CLOOP_SH
	{0x3E9C, 0xC0E0, 0x1111},	  // RST_TX_BOOST_SEQ
	{0x3E9E, 0x00FF, 0x1111},	  // RST_RST_CLOOP_SH
	{0x3EA0, 0x0000, 0x1111},	  // RST_RST_BOOST_SEQ
	{0x3EA6, 0x3C00, 0x1111},	  // RST_PIX_PULLUP_EN
	{0x3ED8, 0x3057, 0x1111},	  // DAC_LD_12_13
	{0x3172, 0x0113, 0x1111},	  // ANALOG_CONTROL2
	{0x316C, 0xB44F, 0x1111},	  // DAC_TXLO
	{0x316E, 0xC6FF, 0x1111},	  // DAC_ECL
	{0x3ED2, 0xEA0F, 0x1111},	  // DAC_LD_6_7
	{0x3ED4, 0x0004, 0x1111},	  // DAC_LD_8_9
	{0x3EDC, 0x6020, 0x1111},	  // DAC_LD_16_17
	{0x3EE6, 0xA5C0, 0x1111},	  // DAC_LD_26_27
	{0x098E, 0x483A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_Y_ADDR_START]
	{0x0990, 0x000C, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x483C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_X_ADDR_START]
	{0x0990, 0x0018, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x483E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_Y_ADDR_END]
	{0x0990, 0x07B1, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4840, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_X_ADDR_END]
	{0x0990, 0x0A45, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4842, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_ROW_SPEED]
	{0x0990, 0x0001, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4844, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_SKIP_X_CORE]
	{0x0990, 0x0103, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4846, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_SKIP_Y_CORE]
	{0x0990, 0x0103, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4848, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_SKIP_X_PIPE]
	{0x0990, 0x0103, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x484A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_SKIP_Y_PIPE]
	{0x0990, 0x0103, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x484C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_POWER_MODE]
	{0x0990, 0x00F6, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x484E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_BIN_MODE]
	{0x0990, 0x0001, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xC850, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_ORIENTATION]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xC851, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_PIXEL_ORDER]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4852, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_FINE_CORRECTION]
	{0x0990, 0x019C, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4854, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_FINE_ITMIN]
	{0x0990, 0x0732, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4858, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_COARSE_ITMIN]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x485A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_COARSE_ITMAX_MARGIN]
	{0x0990, 0x0001, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x485C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_MIN_FRAME_LENGTH_LINES]
	{0x0990, 0x0423, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x485E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_MAX_FRAME_LENGTH_LINES]
	{0x0990, 0xFFFF, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4860, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_BASE_FRAME_LENGTH_LINES]
	{0x0990, 0x0423, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4862, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_MIN_LINE_LENGTH_PCLK]
	{0x0990, 0x0F06, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4864, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_MAX_LINE_LENGTH_PCLK]
	{0x0990, 0xFFFE, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4866, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_P4_5_6_DIVIDER]
	{0x0990, 0x7F7C, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4868, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_FRAME_LENGTH_LINES]
	{0x0990, 0x0423, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x486A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_LINE_LENGTH_PCK]
	{0x0990, 0x0F06, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x486C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_OUTPUT_SIZE_WIDTH]
	{0x0990, 0x0518, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x486E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_OUTPUT_SIZE_HEIGHT]
	{0x0990, 0x03D4, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4870, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_RX_FIFO_TRIGGER_MARK]
	{0x0990, 0x0014, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x48B8, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_OUTPUT_0_JPEG_CONTROL]
	{0x0990, 0x0004, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x48AE, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_OUTPUT_0_OUTPUT_FORMAT]
	{0x0990, 0x0001, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x48AA, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_OUTPUT_0_IMAGE_WIDTH]
	{0x0990, 0x0280, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x48AC, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_OUTPUT_0_IMAGE_HEIGHT]
	{0x0990, 0x01E0, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4872, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_Y_ADDR_START]
	{0x0990, 0x0010, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4874, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_X_ADDR_START]
	{0x0990, 0x001C, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4876, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_Y_ADDR_END]
	{0x0990, 0x07AF, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4878, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_X_ADDR_END]
	{0x0990, 0x0A43, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x487A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_ROW_SPEED]
	{0x0990, 0x0001, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x487C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_SKIP_X_CORE]
	{0x0990, 0x0101, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x487E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_SKIP_Y_CORE]
	{0x0990, 0x0101, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4880, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_SKIP_X_PIPE]
	{0x0990, 0x0101, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4882, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_SKIP_Y_PIPE]
	{0x0990, 0x0101, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4884, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_POWER_MODE]
	{0x0990, 0x00F2, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4886, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_BIN_MODE]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xC888, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_ORIENTATION]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xC889, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_PIXEL_ORDER]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x488A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_FINE_CORRECTION]
	{0x0990, 0x009C, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x488C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_FINE_ITMIN]
	{0x0990, 0x034A, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4890, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_COARSE_ITMIN]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4892, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_COARSE_ITMAX_MARGIN]
	{0x0990, 0x0001, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4894, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_MIN_FRAME_LENGTH_LINES]
	{0x0990, 0x082F, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4896, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_MAX_FRAME_LENGTH_LINES]
	{0x0990, 0xFFFF, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4898, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_BASE_FRAME_LENGTH_LINES]
	{0x0990, 0x082F, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x489A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_MIN_LINE_LENGTH_PCLK]
	{0x0990, 0x10CC, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x489C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_MAX_LINE_LENGTH_PCLK]
	{0x0990, 0xFFFE, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x489E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_P4_5_6_DIVIDER]
	{0x0990, 0x7F7C, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x48A0, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_FRAME_LENGTH_LINES]
	{0x0990, 0x082F, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x48A2, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_LINE_LENGTH_PCK]
	{0x0990, 0x10CC, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x48A4, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_OUTPUT_SIZE_WIDTH]
	{0x0990, 0x0A28, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x48A6, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_OUTPUT_SIZE_HEIGHT]
	{0x0990, 0x07A0, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x48A8, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_RX_FIFO_TRIGGER_MARK]
	{0x0990, 0x0014, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x48CE, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_OUTPUT_1_JPEG_CONTROL]
	{0x0990, 0x0015, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x48C4, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_OUTPUT_1_OUTPUT_FORMAT]
	{0x0990, 0x0001, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x48C0, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_OUTPUT_1_IMAGE_WIDTH]
	{0x0990, 0x0A20, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x48C2, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_OUTPUT_1_IMAGE_HEIGHT]
	{0x0990, 0x0798, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x301A, 0x10F4, 0x1111},	  // RESET_REGISTER
	{0x301E, 0x0000, 0x1111},	  // DATA_PEDESTAL
	{0x301A, 0x10FC, 0x1111},	  // RESET_REGISTER
	{0x098E, 0xDC33, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [SYS_FIRST_BLACK_LEVEL]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xDC35, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [SYS_UV_COLOR_BOOST]
	{0x0990, 0x0400, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x326E, 0x0004, 0x1111},	  // LOW_PASS_YUV_FILTER
	{0x098E, 0x4858, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_COARSE_ITMIN]
	{0x0990, 0x0002, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4890, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_B_COARSE_ITMIN]
	{0x0990, 0x0002, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xDC37, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [SYS_BRIGHT_COLORKILL]
	{0x0990, 0x6200, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x35A4, 0x0596, 0x1111},	  // BRIGHT_COLOR_KILL_CONTROLS
	{0x35A2, 0x0094, 0x1111},	  // DARK_COLOR_KILL_CONTROLS
	{0x098E, 0xDC36, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [SYS_DARK_COLOR_KILL]
	{0x0990, 0x2300, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x8404, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [SEQ_CMD]
	{0x0990, 0x0600, 0x1111},	  // MCU_VARIABLE_DATA0

	{REG_TERM, 0x0050, 0x1111}, //delay 80ms

//PART 2
	{0x098E, 0x3C14, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_FADE_TO_BLACK_START_POS]
	{0x0990, 0xFFFE, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3C16, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_FADE_TO_BLACK_END_POS]
	{0x0990, 0xFFFF, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x32BA, 0x0000, 0x1111},	  // RED_OFFSET_TO_CCM
	{0x32BC, 0x0000, 0x1111},	  // GREEN_OFFSET_TO_CCM
	{0x32BE, 0x0000, 0x1111},	  // BLUE_OFFSET_TO_CCM
	{0x3330, 0x0000, 0x1111},	  // OUTPUT_FORMAT_TEST
	{0x337C, 0x0004, 0x1111},	  // YUV_YCBCR_CONTROL
	{0x337E, 0x0000, 0x1111},	  // Y_RGB_OFFSET
	{0x0010, 0x0320, 0x1111},	  // PLL_DIVIDERS
	{0x0010, 0x0360, 0x1111},	  // PLL_DIVIDERS
	{0x32BA, 0x0000, 0x1111},	  // RED_OFFSET_TO_CCM
	{0x32BC, 0x0000, 0x1111},	  // GREEN_OFFSET_TO_CCM
	{0x32BE, 0x0000, 0x1111},	  // BLUE_OFFSET_TO_CCM
	{0x098E, 0xA00E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [FD_MAX_NUM_AUTOCOR_FUNC_VALUES_TO_CHECK]
	{0x0990, 0x3200, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2018, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [FD_EXPECTED50HZ_FLICKER_PERIOD_IN_CONTEXT_A]
	{0x0990, 0x0133, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x201A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [FD_EXPECTED50HZ_FLICKER_PERIOD_IN_CONTEXT_B]
	{0x0990, 0x0113, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x201C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [FD_EXPECTED60HZ_FLICKER_PERIOD_IN_CONTEXT_A]
	{0x0990, 0x00FF, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x201E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [FD_EXPECTED60HZ_FLICKER_PERIOD_IN_CONTEXT_B]
	{0x0990, 0x00E4, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2010, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [FD_MIN_EXPECTED50HZ_FLICKER_PERIOD]
	{0x0990, 0x0129, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2012, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [FD_MAX_EXPECTED50HZ_FLICKER_PERIOD]
	{0x0990, 0x013D, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2014, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [FD_MIN_EXPECTED60HZ_FLICKER_PERIOD]
	{0x0990, 0x00F5, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2016, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [FD_MAX_EXPECTED60HZ_FLICKER_PERIOD]
	{0x0990, 0x0109, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xA000, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [FD_STATUS]
	{0x0990, 0x1400, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x3380, 0x0504, 0x1111},	  // KERNEL_CONFIG
	{0x098E, 0xBCB2, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_CDC_DARK_CLUS_SLOPE]
	{0x0990, 0x2800, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBCB3, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_CDC_DARK_CLUS_SATUR]
	{0x0990, 0x5F00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2C46, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_LEFT_CCM_0]
	{0x0990, 0x0227, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2C48, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_LEFT_CCM_1]
	{0x0990, 0xFEBE, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2C4A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_LEFT_CCM_2]
	{0x0990, 0x0038, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2C4C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_LEFT_CCM_3]
	{0x0990, 0xFFCB, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2C4E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_LEFT_CCM_4]
	{0x0990, 0x014D, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2C50, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_LEFT_CCM_5]
	{0x0990, 0x0006, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2C52, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_LEFT_CCM_6]
	{0x0990, 0xFFC6, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2C54, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_LEFT_CCM_7]
	{0x0990, 0xFED3, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2C56, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_LEFT_CCM_8]
	{0x0990, 0x0284, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2C5C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_RIGHT_CCM_0]
	{0x0990, 0x024A, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2C5E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_RIGHT_CCM_1]
	{0x0990, 0xFF25, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2C60, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_RIGHT_CCM_2]
	{0x0990, 0xFFAE, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2C62, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_RIGHT_CCM_3]
	{0x0990, 0xFFCC, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2C64, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_RIGHT_CCM_4]
	{0x0990, 0x0150, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2C66, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_RIGHT_CCM_5]
	{0x0990, 0x0002, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2C68, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_RIGHT_CCM_6]
	{0x0990, 0xFFFB, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2C6A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_RIGHT_CCM_7]
	{0x0990, 0xFF69, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2C6C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_RIGHT_CCM_8]
	{0x0990, 0x01B9, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2C58, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_LEFT_CCM_R2BRATIO]
	{0x0990, 0x00CD, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2C6E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_RIGHT_CCM_R2BRATIO]
	{0x0990, 0x0050, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x3240, 0x0012, 0x1111},	  // AWB_XY_SCALE
	{0x3242, 0xF31A, 0x1111},	  // AWB_WEIGHT_R0
	{0x3244, 0xFE36, 0x1111},	  // AWB_WEIGHT_R1
	{0x3246, 0xDF28, 0x1111},	  // AWB_WEIGHT_R2
	{0x3248, 0xD724, 0x1111},	  // AWB_WEIGHT_R3
	{0x324A, 0xE3B5, 0x1111},	  // AWB_WEIGHT_R4
	{0x324C, 0xD587, 0x1111},	  // AWB_WEIGHT_R5
	{0x324E, 0xD3F1, 0x1111},	  // AWB_WEIGHT_R6
	{0x3250, 0x2C8F, 0x1111},	  // AWB_WEIGHT_R7
	{0x098E, 0x3842, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [STAT_AWB_GRAY_CHECKER_OFFSET_X]
	{0x0990, 0x003A, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3844, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [STAT_AWB_GRAY_CHECKER_OFFSET_Y]
	{0x0990, 0x0037, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xAC01, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_MODE]
	{0x0990, 0xAB00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xAC3C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_MIN_ACCEPTED_PRE_AWB_R2G_RATIO]
	{0x0990, 0x3900, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xAC3D, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_MAX_ACCEPTED_PRE_AWB_R2G_RATIO]
	{0x0990, 0x6E00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xAC3E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_MIN_ACCEPTED_PRE_AWB_B2G_RATIO]
	{0x0990, 0x2800, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xAC3F, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AWB_MAX_ACCEPTED_PRE_AWB_B2G_RATIO]
	{0x0990, 0x6B00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBCDE, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_START_SYS_THRESHOLD]
	{0x0990, 0x0400, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBCDF, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_STOP_SYS_THRESHOLD]
	{0x0990, 0x1800, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBCE0, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_START_SYS_GAIN]
	{0x0990, 0x0400, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBCE1, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_STOP_SYS_GAIN]
	{0x0990, 0x0400, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xB801, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [STAT_MODE]
	{0x0990, 0xE000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xB862, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [STAT_BMTRACKING_SPEED]
	{0x0990, 0x0400, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xB829, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [STAT_LL_BRIGHTNESS_METRIC_DIVISOR]
	{0x0990, 0x1100, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xB863, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [STAT_BM_MUL]
	{0x0990, 0x1700, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xB827, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [STAT_AE_EV_SHIFT]
	{0x0990, 0x0E00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xA409, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AE_RULE_BASE_TARGET]
	{0x0990, 0x3800, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3C52, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_START_BRIGHTNESS_METRIC]
	{0x0990, 0x00E0, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3C54, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_END_BRIGHTNESS_METRIC]
	{0x0990, 0x32B8, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3C58, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_START_GAIN_METRIC]
	{0x0990, 0x012C, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3C5A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_END_GAIN_METRIC]
	{0x0990, 0x106C, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3C5E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_START_APERTURE_GAIN_BM]
	{0x0990, 0x00FA, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3C60, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_END_APERTURE_GAIN_BM]
	{0x0990, 0x02B2, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3C66, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_START_APERTURE_GM]
	{0x0990, 0x00FA, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3C68, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_END_APERTURE_GM]
	{0x0990, 0x02B2, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3C86, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_START_FFNR_GM]
	{0x0990, 0x00C8, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3C88, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_END_FFNR_GM]
	{0x0990, 0x0658, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3CBC, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_SFFB_START_GAIN]
	{0x0990, 0x0040, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3CBE, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_SFFB_END_GAIN]
	{0x0990, 0x01FC, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3CCC, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_SFFB_START_MAX_GM]
	{0x0990, 0x00C8, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3CCE, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_SFFB_END_MAX_GM]
	{0x0990, 0x0633, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3C90, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_START_GRB_GM]
	{0x0990, 0x00C8, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3C92, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_END_GRB_GM]
	{0x0990, 0x0658, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3C0E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_CURVE_ADJ_START_POS]
	{0x0990, 0x0001, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3C10, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_CURVE_ADJ_MID_POS]
	{0x0990, 0x0002, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3C12, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_CURVE_ADJ_END_POS]
	{0x0990, 0x02BC, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3CAA, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_CDC_THR_ADJ_START_POS]
	{0x0990, 0x32B8, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3CAC, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_CDC_THR_ADJ_MID_POS]
	{0x0990, 0x09B1, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3CAE, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_CDC_THR_ADJ_END_POS]
	{0x0990, 0x00E0, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3CD8, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_PCR_START_BM]
	{0x0990, 0x00E0, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3CDA, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_PCR_END_BM]
	{0x0990, 0x05DC, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC94, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GB_START_THRESHOLD_0]
	{0x0990, 0x0600, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC95, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GB_START_THRESHOLD_1]
	{0x0990, 0x0500, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC96, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GB_START_THRESHOLD_2]
	{0x0990, 0x0600, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC97, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GB_START_THRESHOLD_3]
	{0x0990, 0x0500, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC9C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GB_END_THRESHOLD_0]
	{0x0990, 0x0900, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC9D, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GB_END_THRESHOLD_1]
	{0x0990, 0x0500, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC9E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GB_END_THRESHOLD_2]
	{0x0990, 0x0900, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC9F, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GB_END_THRESHOLD_3]
	{0x0990, 0x0500, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x30B6, 0x0001, 0x1111},	  // AUTOLR_CONTROL
	{0x33B0, 0x2A16, 0x1111},	  // FFNR_ALPHA_BETA
	{0x098E, 0xBC8A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_START_FF_MIX_THRESH_Y]
	{0x0990, 0x2C00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC8B, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_END_FF_MIX_THRESH_Y]
	{0x0990, 0xCF00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC8C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_START_FF_MIX_THRESH_YGAIN]
	{0x0990, 0x8B00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC8D, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_END_FF_MIX_THRESH_YGAIN]
	{0x0990, 0xD700, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC8E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_START_FF_MIX_THRESH_GAIN]
	{0x0990, 0x7F00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC8F, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_END_FF_MIX_THRESH_GAIN]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x33BA, 0x0086, 0x1111},	  // APEDGE_CONTROL
	{0x33BE, 0x0000, 0x1111},	  // UA_KNEE_L
	{0x33C2, 0x7300, 0x1111},	  // UA_WEIGHTS
	{0x098E, 0xBC62, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_START_APERTURE_KPGAIN]
	{0x0990, 0x1900, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC63, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_END_APERTURE_KPGAIN]
	{0x0990, 0x1F00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC64, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_START_APERTURE_KNGAIN]
	{0x0990, 0x1900, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC65, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_END_APERTURE_KNGAIN]
	{0x0990, 0x1F00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC6A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_START_APERTURE_INTEGER_GAIN]
	{0x0990, 0x0400, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC6B, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_END_APERTURE_INTEGER_GAIN]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC6C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_START_APERTURE_EXP_GAIN]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC6D, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_END_APERTURE_EXP_GAIN]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x3284, 0x0000, 0x1111},	  // APERTURE_KNEE_VALUES
	{0x3210, 0x49B0, 0x1111},	  // COLOR_PIPELINE_CONTROL
	{0x098E, 0xBCC0, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_SFFB_RAMP_START]
	{0x0990, 0x1F00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBCC1, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_SFFB_RAMP_STOP]
	{0x0990, 0x0300, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBCC2, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_SFFB_SLOPE_START]
	{0x0990, 0x1E00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBCC3, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_SFFB_SLOPE_STOP]
	{0x0990, 0x0100, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBCC4, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_SFFB_THSTART]
	{0x0990, 0x0A00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBCC5, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_SFFB_THSTOP]
	{0x0990, 0x1F00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3CBA, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_SFFB_CONFIG]
	{0x0990, 0x0009, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3CD0, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_SFFB_SOBEL_FLAT_START]
	{0x0990, 0x000A, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3CD2, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_SFFB_SOBEL_FLAT_STOP]
	{0x0990, 0x00FE, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3CD4, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_SFFB_SOBEL_SHARP_START]
	{0x0990, 0x001E, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3CD6, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_SFFB_SOBEL_SHARP_STOP]
	{0x0990, 0x00FF, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBCC6, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_SFFB_SHARPENING_START]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBCC7, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_SFFB_SHARPENING_STOP]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBCC8, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_SFFB_FLATNESS_START]
	{0x0990, 0x2000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBCC9, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_SFFB_FLATNESS_STOP]
	{0x0990, 0x4000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBCCA, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_SFFB_TRANSITION_START]
	{0x0990, 0x0400, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBCCB, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_SFFB_TRANSITION_STOP]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC18, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_CONTRAST_CURVE_0]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC19, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_CONTRAST_CURVE_1]
	{0x0990, 0x1100, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC1A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_CONTRAST_CURVE_2]
	{0x0990, 0x2300, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC1B, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_CONTRAST_CURVE_3]
	{0x0990, 0x3F00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC1C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_CONTRAST_CURVE_4]
	{0x0990, 0x6700, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC1D, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_CONTRAST_CURVE_5]
	{0x0990, 0x8500, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC1E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_CONTRAST_CURVE_6]
	{0x0990, 0x9B00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC1F, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_CONTRAST_CURVE_7]
	{0x0990, 0xAD00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC20, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_CONTRAST_CURVE_8]
	{0x0990, 0xBB00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC21, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_CONTRAST_CURVE_9]
	{0x0990, 0xC700, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC22, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_CONTRAST_CURVE_10]
	{0x0990, 0xD100, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC23, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_CONTRAST_CURVE_11]
	{0x0990, 0xDA00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC24, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_CONTRAST_CURVE_12]
	{0x0990, 0xE100, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC25, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_CONTRAST_CURVE_13]
	{0x0990, 0xE800, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC26, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_CONTRAST_CURVE_14]
	{0x0990, 0xEE00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC27, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_CONTRAST_CURVE_15]
	{0x0990, 0xF300, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC28, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_CONTRAST_CURVE_16]
	{0x0990, 0xF700, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC29, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_CONTRAST_CURVE_17]
	{0x0990, 0xFB00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC2A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_CONTRAST_CURVE_18]
	{0x0990, 0xFF00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC2B, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NEUTRAL_CURVE_0]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC2C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NEUTRAL_CURVE_1]
	{0x0990, 0x1100, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC2D, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NEUTRAL_CURVE_2]
	{0x0990, 0x2300, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC2E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NEUTRAL_CURVE_3]
	{0x0990, 0x3F00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC2F, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NEUTRAL_CURVE_4]
	{0x0990, 0x6700, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC30, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NEUTRAL_CURVE_5]
	{0x0990, 0x8500, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC31, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NEUTRAL_CURVE_6]
	{0x0990, 0x9B00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC32, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NEUTRAL_CURVE_7]
	{0x0990, 0xAD00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC33, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NEUTRAL_CURVE_8]
	{0x0990, 0xBB00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC34, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NEUTRAL_CURVE_9]
	{0x0990, 0xC700, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC35, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NEUTRAL_CURVE_10]
	{0x0990, 0xD100, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC36, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NEUTRAL_CURVE_11]
	{0x0990, 0xDA00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC37, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NEUTRAL_CURVE_12]
	{0x0990, 0xE100, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC38, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NEUTRAL_CURVE_13]
	{0x0990, 0xE800, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC39, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NEUTRAL_CURVE_14]
	{0x0990, 0xEE00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC3A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NEUTRAL_CURVE_15]
	{0x0990, 0xF300, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC3B, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NEUTRAL_CURVE_16]
	{0x0990, 0xF700, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC3C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NEUTRAL_CURVE_17]
	{0x0990, 0xFB00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC3D, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NEUTRAL_CURVE_18]
	{0x0990, 0xFF00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC3E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NR_CURVE_0]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC3F, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NR_CURVE_1]
	{0x0990, 0x1800, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC40, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NR_CURVE_2]
	{0x0990, 0x2500, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC41, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NR_CURVE_3]
	{0x0990, 0x3A00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC42, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NR_CURVE_4]
	{0x0990, 0x5900, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC43, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NR_CURVE_5]
	{0x0990, 0x7000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC44, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NR_CURVE_6]
	{0x0990, 0x8100, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC45, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NR_CURVE_7]
	{0x0990, 0x9000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC46, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NR_CURVE_8]
	{0x0990, 0x9E00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC47, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NR_CURVE_9]
	{0x0990, 0xAB00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC48, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NR_CURVE_10]
	{0x0990, 0xB600, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC49, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NR_CURVE_11]
	{0x0990, 0xC100, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC4A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NR_CURVE_12]
	{0x0990, 0xCB00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC4B, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NR_CURVE_13]
	{0x0990, 0xD500, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC4C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NR_CURVE_14]
	{0x0990, 0xDE00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC4D, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NR_CURVE_15]
	{0x0990, 0xE700, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC4E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NR_CURVE_16]
	{0x0990, 0xEF00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC4F, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NR_CURVE_17]
	{0x0990, 0xF700, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xBC50, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_NR_CURVE_18]
	{0x0990, 0xFF00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x8404, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [SEQ_CMD]
	{0x0990, 0x0600, 0x1111},	  // MCU_VARIABLE_DATA0

	{REG_TERM, 0x0050, 0x1111}, //delay 80ms

//PART 3
	{0x098E, 0x3C14, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_FADE_TO_BLACK_START_POS]
	{0x0990, 0xFFFE, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x3C16, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [LL_GAMMA_FADE_TO_BLACK_END_POS]
	{0x0990, 0xFFFF, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x31E0, 0x0003, 0x1111},	  // PIX_DEF_ID
	{0x098E, 0x281C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AE_TRACK_MIN_AGAIN]
	{0x0990, 0x0040, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2820, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AE_TRACK_MAX_AGAIN]
	{0x0990, 0x01FC, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2822, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AE_TRACK_MIN_DGAIN]
	{0x0990, 0x0080, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2824, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AE_TRACK_MAX_DGAIN]
	{0x0990, 0x0100, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x8404, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [SEQ_CMD]
	{0x0990, 0x0600, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x0018, 0x2008, 0x1111},	  // STANDBY_CONTROL_AND_STATUS
	{0x098E, 0x483A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_Y_ADDR_START]
	{0x0990, 0x0106, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x483C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_X_ADDR_START]
	{0x0990, 0x0018, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x483E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_Y_ADDR_END]
	{0x0990, 0x06B7, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4840, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_X_ADDR_END]
	{0x0990, 0x0A45, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4842, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_ROW_SPEED]
	{0x0990, 0x0001, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4844, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_SKIP_X_CORE]
	{0x0990, 0x0103, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4846, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_SKIP_Y_CORE]
	{0x0990, 0x0103, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4848, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_SKIP_X_PIPE]
	{0x0990, 0x0103, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x484A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_SKIP_Y_PIPE]
	{0x0990, 0x0103, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x484C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_POWER_MODE]
	{0x0990, 0x00F6, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x484E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_BIN_MODE]
	{0x0990, 0x0001, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xC850, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_ORIENTATION]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xC851, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_PIXEL_ORDER]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4852, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_FINE_CORRECTION]
	{0x0990, 0x019C, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4854, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_FINE_ITMIN]
	{0x0990, 0x0732, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4858, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_COARSE_ITMIN]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x485A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_COARSE_ITMAX_MARGIN]
	{0x0990, 0x0001, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x485C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_MIN_FRAME_LENGTH_LINES]
	{0x0990, 0x03E8, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x485E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_MAX_FRAME_LENGTH_LINES]
	{0x0990, 0xFFFF, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4860, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_BASE_FRAME_LENGTH_LINES]
	{0x0990, 0x03E8, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4862, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_MIN_LINE_LENGTH_PCLK]
	{0x0990, 0x0F06, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4864, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_MAX_LINE_LENGTH_PCLK]
	{0x0990, 0xFFFE, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4866, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_P4_5_6_DIVIDER]
	{0x0990, 0x7F7C, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4868, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_FRAME_LENGTH_LINES]
	{0x0990, 0x03E8, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x486A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_LINE_LENGTH_PCK]
	{0x0990, 0x0F06, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x486C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_OUTPUT_SIZE_WIDTH]
	{0x0990, 0x0518, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x486E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_OUTPUT_SIZE_HEIGHT]
	{0x0990, 0x02D8, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4870, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_RX_FIFO_TRIGGER_MARK]
	{0x0990, 0x0014, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x48B8, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_OUTPUT_0_JPEG_CONTROL]
	{0x0990, 0x0004, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x48AE, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_OUTPUT_0_OUTPUT_FORMAT]
	{0x0990, 0x0001, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x48AA, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_OUTPUT_0_IMAGE_WIDTH]
	{0x0990, 0x0500, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x48AC, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_OUTPUT_0_IMAGE_HEIGHT]
	{0x0990, 0x02D0, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xA80A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AE_TRACK_NUM_FRAMES_TO_SKIP]
	{0x0990, 0x0200, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xA807, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AE_TRACK_JUMP_DIVISOR]
	{0x0990, 0x0800, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xA409, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AE_RULE_BASE_TARGET]
	{0x0990, 0x4000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2010, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [FD_MIN_EXPECTED50HZ_FLICKER_PERIOD]
	{0x0990, 0x0122, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2012, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [FD_MAX_EXPECTED50HZ_FLICKER_PERIOD]
	{0x0990, 0x0118, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2014, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [FD_MIN_EXPECTED60HZ_FLICKER_PERIOD]
	{0x0990, 0x00D2, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2016, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [FD_MAX_EXPECTED60HZ_FLICKER_PERIOD]
	{0x0990, 0x0118, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2018, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [FD_EXPECTED50HZ_FLICKER_PERIOD_IN_CONTEXT_A]
	{0x0990, 0x0133, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x201C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [FD_EXPECTED60HZ_FLICKER_PERIOD_IN_CONTEXT_A]
	{0x0990, 0x00FF, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2818, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AE_TRACK_TARGET_INT_TIME_ROWS]
	{0x0990, 0x03E8, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x281A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AE_TRACK_MAX_INT_TIME_ROWS]
	{0x0990, 0x03E8, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2824, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AE_TRACK_MAX_DGAIN]
	{0x0990, 0x0180, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xB829, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [STAT_LL_BRIGHTNESS_METRIC_DIVISOR]
	{0x0990, 0x0A00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x8404, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [SEQ_CMD]
	{0x0990, 0x0600, 0x1111},	  // MCU_VARIABLE_DATA0

	{REG_TERM, 0x0050, 0x1111}, //delay 80ms

//PART 4
	{0x098E, 0x483A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_Y_ADDR_START]
	{0x0990, 0x0106, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x483C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_X_ADDR_START]
	{0x0990, 0x0018, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x483E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_Y_ADDR_END]
	{0x0990, 0x06B7, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4840, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_X_ADDR_END]
	{0x0990, 0x0A45, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4842, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_ROW_SPEED]
	{0x0990, 0x0001, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4844, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_SKIP_X_CORE]
	{0x0990, 0x0103, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4846, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_SKIP_Y_CORE]
	{0x0990, 0x0103, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4848, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_SKIP_X_PIPE]
	{0x0990, 0x0103, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x484A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_SKIP_Y_PIPE]
	{0x0990, 0x0103, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x484C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_POWER_MODE]
	{0x0990, 0x00F6, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x484E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_BIN_MODE]
	{0x0990, 0x0001, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xC850, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_ORIENTATION]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xC851, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_PIXEL_ORDER]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4852, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_FINE_CORRECTION]
	{0x0990, 0x019C, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4854, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_FINE_ITMIN]
	{0x0990, 0x0732, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4858, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_COARSE_ITMIN]
	{0x0990, 0x0000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x485A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_COARSE_ITMAX_MARGIN]
	{0x0990, 0x0001, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x485C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_MIN_FRAME_LENGTH_LINES]
	{0x0990, 0x03E8, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x485E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_MAX_FRAME_LENGTH_LINES]
	{0x0990, 0xFFFF, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4860, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_BASE_FRAME_LENGTH_LINES]
	{0x0990, 0x03E8, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4862, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_MIN_LINE_LENGTH_PCLK]
	{0x0990, 0x0F06, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4864, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_MAX_LINE_LENGTH_PCLK]
	{0x0990, 0xFFFE, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4866, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_P4_5_6_DIVIDER]
	{0x0990, 0x7F7C, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4868, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_FRAME_LENGTH_LINES]
	{0x0990, 0x03E8, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x486A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_LINE_LENGTH_PCK]
	{0x0990, 0x0F06, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x486C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_OUTPUT_SIZE_WIDTH]
	{0x0990, 0x0518, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x486E, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_OUTPUT_SIZE_HEIGHT]
	{0x0990, 0x02D8, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x4870, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_CORE_A_RX_FIFO_TRIGGER_MARK]
	{0x0990, 0x0014, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x48B8, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_OUTPUT_0_JPEG_CONTROL]
	{0x0990, 0x0004, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x48AE, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_OUTPUT_0_OUTPUT_FORMAT]
	{0x0990, 0x0001, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x48AA, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_OUTPUT_0_IMAGE_WIDTH]
	{0x0990, 0x0500, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x48AC, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [CAM_OUTPUT_0_IMAGE_HEIGHT]
	{0x0990, 0x02D0, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xA80A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AE_TRACK_NUM_FRAMES_TO_SKIP]
	{0x0990, 0x0200, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xA807, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AE_TRACK_JUMP_DIVISOR]
	{0x0990, 0x0800, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xA409, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AE_RULE_BASE_TARGET]
	{0x0990, 0x4000, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2010, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [FD_MIN_EXPECTED50HZ_FLICKER_PERIOD]
	{0x0990, 0x0122, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2012, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [FD_MAX_EXPECTED50HZ_FLICKER_PERIOD]
	{0x0990, 0x0118, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2014, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [FD_MIN_EXPECTED60HZ_FLICKER_PERIOD]
	{0x0990, 0x00D2, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2016, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [FD_MAX_EXPECTED60HZ_FLICKER_PERIOD]
	{0x0990, 0x0118, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2018, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [FD_EXPECTED50HZ_FLICKER_PERIOD_IN_CONTEXT_A]
	{0x0990, 0x0133, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x201C, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [FD_EXPECTED60HZ_FLICKER_PERIOD_IN_CONTEXT_A]
	{0x0990, 0x00FF, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2818, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AE_TRACK_TARGET_INT_TIME_ROWS]
	{0x0990, 0x03E8, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x281A, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AE_TRACK_MAX_INT_TIME_ROWS]
	{0x0990, 0x03E8, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x2824, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [AE_TRACK_MAX_DGAIN]
	{0x0990, 0x0180, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0xB829, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [STAT_LL_BRIGHTNESS_METRIC_DIVISOR]
	{0x0990, 0x0A00, 0x1111},	  // MCU_VARIABLE_DATA0
	{0x098E, 0x8404, 0x1111},	  // LOGICAL_ADDRESS_ACCESS [SEQ_CMD]
	{0x0990, 0x0600, 0x1111},	  // MCU_VARIABLE_DATA0

	{REG_TERM, 0x0050, 0x1111}, //delay 80ms

	//[FTB_OFF]
	{0x098E, 0x3C16, 0x1111},  	// LOGICAL_ADDRESS_ACCESS [LL_GAMMA_FADE_TO_BLACK_END_POS]
	{0x0990, 0xFFFF, 0x1111},  	// MCU_VARIABLE_DATA0
	{0x098E, 0xBC18, 0x1111},  	// LOGICAL_ADDRESS_ACCESS [LL_GAMMA_CONTRAST_CURVE_0]
	{0x0990, 0x0000, 0x1111},  	// MCU_VARIABLE_DATA0

	//[Thermal setting]
	{0x3ED0, 0x2409, 0x1111},
	{0x3EDE, 0x0A49, 0x1111},
	{0x3EE0, 0x4949, 0x1111},
	{0x3EE2, 0x4950, 0x1111},
	{0x0018, 0x2008, 0x1111}, 	// STANDBY_CONTROL_AND_STATUS

	//To support HD mode	
	{0x0010, 0x0248, 0x1111}, 	// PLL_DIVIDERS
	{0x0012, 0x0070, 0x1111}, 	// PLL_P_DIVIDERS
	{0x002A, 0x7F7C, 0x1111}, 	// PLL_P4_P5_P6_DIVIDERS

	//[AWB & AE OFF Mode]
	{0x098E, 0x840E, 0x1111},
	{0x840E, 0x0000, 0x1111},
	{0x8416, 0x0000, 0x1111},
	{0x841E, 0x0000, 0x1111},
	{0x842E, 0x0000, 0x1111},
	{0x8436, 0x0000, 0x1111},
	{0x8410, 0x0000, 0x1111},
	{0x8418, 0x0000, 0x1111},
	{0x8420, 0x0000, 0x1111},
	{0x8430, 0x0000, 0x1111},
	{0x8438, 0x0000, 0x1111},
	{0x8404, 0x0006, 0x1111},

	{REG_TERM, 0x0050, 0x1111}, //delay 80ms
	
    {REG_TERM, VAL_TERM}
};


static struct sensor_reg sensor_set_preview[] = {
	{ 0x098E,0x843C, 0x1111},//LOGICAL_ADDRESS_ACCESs[SEQ_STATE_CFG_5_MAX_FRAME_CNT]
	{ 0x0990,0x0100, 0x1111}, //MCU_VARIABLE_DATA0
	{ 0x098E,0x8404, 0x1111}, //LOGICAL_ADDRESS_ACCESS[SEQ_CMD]
	{ 0x0990,0x0100, 0x1111}, //MCU_VARIABLE_DATA0
	{ 0x0016, 0x0647, 0x1111},  // CLOCKS_CONTROL

	{REG_TERM, 0x0080, 0x1111}, //delay 120ms

	{0xC83A, 0x0106, 0x1111},
	{0xC83E, 0x06B7, 0x1111},
	{0xC858, 0x0000, 0x1111},
	{0xC85C, 0x03E8, 0x1111},
	{0xC860, 0x03E8, 0x1111},
	{0xC866, 0x7F7C, 0x1111},
	{0xC868, 0x03E8, 0x1111},
	{0xC86E, 0x02D8, 0x1111},
	{0xC8AA, 0x0500, 0x1111},
	{0xC8AC, 0x02D0, 0x1111},
	{0xA80A, 0x0002, 0x1111},
	{0xA807, 0x0008, 0x1111},
	{0xA409, 0x0040, 0x1111},
	{0xA010, 0x0122, 0x1111},
	{0xA012, 0x0118, 0x1111},
	{0xA014, 0x00D2, 0x1111},
	{0xA016, 0x0118, 0x1111},
	{0xA018, 0x0133, 0x1111},
	{0xA01C, 0x00FF, 0x1111},
	{0xA81A, 0x03E8, 0x1111},
	{0xA824, 0x0180, 0x1111},
	{0xAC09, 0x0035, 0x1111},
	{0xB829, 0x000A, 0x1111},
	{0x8404, 0x0006, 0x1111},

	{REG_TERM, 0x0080, 0x1111}, //delay 120ms

    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_set_capture[] = {
	{ 0x3CA2, 0x0007, 0x1111},   // TXC_PARAMETERS
	{ 0x098E, 0x5820, 0x1111},   // LOGICAL_ADDRESS_ACCESS [JPEG_JPSS_CTRL_VAR]
	{ 0x0990, 0x4510, 0x1111},   // MCU_VARIABLE_DATA0
	{ 0x098E, 0x8404, 0x1111},   // LOGICAL_ADDRESS_ACCESS [SEQ_CMD]
	{ 0x0990, 0x0600, 0x1111},   // MCU_VARIABLE_DATA0

	{REG_TERM, 0x0080, 0x1111}, //delay 120ms

	{ 0x098E,0x843C, 0x1111}, //LOGICAL_ADDRESS_ACCESS[SEQ_STATE_CFG_5_MAX_FRAME_CNT]
	{ 0x0990,0xFF00, 0x1111}, //MCU_VARIABLE_DATA0
	{ 0x098E,0x8404, 0x1111}, //LOGICAL_ADDRESS_ACCESS[SEQ_CMD]
	{ 0x0990,0x0200, 0x1111}, //MCU_VARIABLE_DATA0

	{REG_TERM, 0x0080, 0x1111}, //delay 120ms
	
    {REG_TERM, VAL_TERM}
};

 struct sensor_reg* sensor_reg_common[3] =
{
	sensor_initialize,
	sensor_set_preview,
	sensor_set_capture
};

static struct sensor_reg sensor_brightness_0[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_brightness_1[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_brightness_2[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_brightness_3[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_brightness_4[] = {
    {REG_TERM, VAL_TERM}
};

struct sensor_reg* sensor_reg_brightness[5] =
{
	sensor_brightness_0,
	sensor_brightness_1,
	sensor_brightness_2,
	sensor_brightness_3,
	sensor_brightness_4
};


static struct sensor_reg sensor_awb_auto[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_awb_daylight[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_awb_incandescent[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_awb_fluorescent[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_awb_cloudy[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_awb_sunset[] = {
    {REG_TERM, VAL_TERM}
};

struct sensor_reg* sensor_reg_awb[6] =
{
	sensor_awb_auto,
	sensor_awb_daylight,
	sensor_awb_incandescent,
	sensor_awb_fluorescent,
	sensor_awb_cloudy,
	sensor_awb_sunset
	
};


static struct sensor_reg sensor_iso_auto[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_iso_100[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_iso_200[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_iso_400[] = {
	{REG_TERM, VAL_TERM}
};

struct sensor_reg* sensor_reg_iso[4] =
{
	sensor_iso_auto,
	sensor_iso_100,
	sensor_iso_200,
	sensor_iso_400
};


static struct sensor_reg sensor_effect_normal[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_effect_gray[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_effect_negative[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_effect_sepia[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_effect_sharpness[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_effect_sketch[] = {
    {REG_TERM, VAL_TERM}
};

struct sensor_reg* sensor_reg_effect[6] =
{
	sensor_effect_normal,
	sensor_effect_gray,
	sensor_effect_negative,
	sensor_effect_sepia,
	sensor_effect_sharpness,
	sensor_effect_sketch,
};


static struct sensor_reg sensor_reg_flipnone[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_reg_hflip[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_reg_vflip[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_reg_hvflip[] = {
    {REG_TERM, VAL_TERM}
};

struct sensor_reg* sensor_reg_flip[4] =
{
	sensor_reg_flipnone,
	sensor_reg_hflip,
	sensor_reg_vflip,
	sensor_reg_hvflip,
};


static struct sensor_reg sensor_secne_auto[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_secne_night[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_secne_landscape[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_secne_portrait[] = {
	{REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_secne_sport[] = {
	{REG_TERM, VAL_TERM}
};

struct sensor_reg* sensor_reg_scene[5] =
{
	sensor_secne_auto,
	sensor_secne_night,
	sensor_secne_landscape,
	sensor_secne_portrait,
	sensor_secne_sport
};

static struct sensor_reg sensor_me_mtrix[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_me_center_weighted[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_me_spot[] = {
    {REG_TERM, VAL_TERM}
};

struct sensor_reg* sensor_reg_metering_exposure[3] =
{
	sensor_me_mtrix,
	sensor_me_center_weighted,
	sensor_me_spot,
};

static struct sensor_reg sensor_af_single[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_af_manual[] = {
    {REG_TERM, VAL_TERM}
};

struct sensor_reg* sensor_reg_af[2] =
{
	sensor_af_single,
	sensor_af_manual,
};

static int write_regs(const struct sensor_reg reglist[])
{
	int cnt;
	int err;
	int err_cnt = 0;	
	unsigned char data[132];
	unsigned char bytes;
	const struct sensor_reg *next = reglist;
	
	while (!((next->reg == REG_TERM) && (next->val[0] == VAL_TERM)))
	{
	
		if(next->reg == REG_TERM && next->val[1] == VAL_END)
		{
			mdelay(next->val[0]);
			printk("Sensor init Delay[%d]!!!! \n", next->val[0]);
			next++;
		}
		else
		{
			bytes = 0;
			data[bytes]= next->reg>>8;		bytes++;		
			data[bytes]= (u8)next->reg&0xff;	bytes++;

			for( cnt = 0; cnt<25 ; cnt++)
			{
				if(next->val[cnt] == VAL_END)
					break;
				
				data[bytes + (cnt*2)] = next->val[cnt] >> 8;
				data[bytes + (cnt*2) + 1] = (u8)next->val[cnt] & 0xff;
			}
			bytes += cnt*2;

			
			err = DDI_I2C_Write(data, 2, bytes-2);

	        if (err)
	        {
				err_cnt++;
				if(err_cnt >= 3)
				{
					printk("ERROR: Sensor I2C !!!! \n"); 
					return err;
				}
	        }
	        else
	        {
	        	err_cnt = 0;
	            next++;
	        }
		}
	}

	return 0;
}

void sensor_delay(int ms)
{
#if 1 //ZzaU
	unsigned int msec;

	msec = ms / 10; //10msec unit

	if(!msec)
		msleep(1);
	else
		msleep(msec);
#else
	int totCnt, iCnt, temp = 0;

	totCnt = ms*20000;
	for(iCnt = 0; iCnt < totCnt; iCnt++)
	{
		temp++;
	}
#endif
}

static int sensor_open(void)
{
#if (1)
	sensor_power_disable();
	sensor_delay(50);

	sensor_reset_low();
	sensor_delay(50);

	sensor_power_enable();
	sensor_delay(100);
	
	CIF_Open();
	sensor_delay(50);

	sensor_reset_high();
	sensor_delay(100);	
#else
	sensor_power_disable();
	sensor_delay(20);

	sensor_power_enable();
	sensor_delay(20);

	sensor_powerdown_disable();
	sensor_delay(20);

	sensor_reset_low();
	sensor_delay(40);

	CIF_Open();
	sensor_delay(80); // 40
	
	//start standby!!
	sensor_powerdown_enable();
	sensor_delay(40);

	CIF_Close();
	sensor_delay(40);

	sensor_powerdown_disable();
	sensor_delay(40);
	//end standby!!

	CIF_Open();
	sensor_delay(80);
	
	sensor_reset_high();
	sensor_delay(30);
#endif
	return write_regs(sensor_reg_common[0]);
}

static int sensor_close(void)
{
	CIF_ONOFF(OFF);

	sensor_reset_low();
	sensor_power_disable();
	sensor_powerdown_disable();

	CIF_Close();
	msleep(5);

	return 0;
}

static int sensor_preview(void)
{
	return write_regs(sensor_reg_common[1]);
}

static int sensor_capture(void)
{
	return write_regs(sensor_reg_common[2]);
}

static int sensor_capturecfg(int width, int height)
{
	return 0;
}

static int sensor_zoom(int val)
{
	return 0;
}

static int sensor_autofocus(int val)
{
	return write_regs(sensor_reg_af[val]);
}

static int sensor_effect(int val)
{
	return write_regs(sensor_reg_effect[val]);
}

static int sensor_flip(int val)
{
	return write_regs(sensor_reg_flip[val]);
}

static int sensor_iso(int val)
{
	return write_regs(sensor_reg_iso[val]);
}

static int sensor_me(int val)
{
	return write_regs(sensor_reg_metering_exposure[val]);
}

static int sensor_wb(int val)
{
	return write_regs(sensor_reg_awb[val]);
}

static int sensor_bright(int val)
{
	return write_regs(sensor_reg_brightness[val]);
}

static int sensor_scene(int val)
{
	return write_regs(sensor_reg_scene[val]);
}

static int sensor_check_esd(int val)
{
	return 0;
}

static int sensor_check_luma(int val)
{
	return 0;
}

/**********************************************************
*    Function  
**********************************************************/
void sensor_init_fnc(SENSOR_FUNC_TYPE *sensor_func)
{
	sensor_func->Open 			= sensor_open;
	sensor_func->Close 			= sensor_close;

	sensor_func->Set_Preview 	= sensor_preview;
	sensor_func->Set_Capture 	= sensor_capture;
	sensor_func->Set_CaptureCfg = sensor_capturecfg;

	sensor_func->Set_Zoom		= sensor_zoom;
	sensor_func->Set_AF 		= sensor_autofocus;
	sensor_func->Set_Effect 	= sensor_effect;
	sensor_func->Set_Flip 		= sensor_flip;
	sensor_func->Set_ISO 		= sensor_iso;
	sensor_func->Set_ME 		= sensor_me;
	sensor_func->Set_WB 		= sensor_wb;
	sensor_func->Set_Bright 	= sensor_bright;
	sensor_func->Set_Scene 		= sensor_scene;

	sensor_func->Check_ESD 		= sensor_check_esd;
	sensor_func->Check_Luma 	= sensor_check_luma;
}

#endif

