
/*
 * drivers/media/video/tcccam/sensor_if.c
 *
 * COMMON Sensor driver for  camera sensor interface
 *
 *
 *
 * This file is licensed under the terms of the GNU General Public License 
 * version 2. This program is licensed "as is" without any warranty of any 
 * kind, whether express or implied.
 */

#include <linux/errno.h>
#include <linux/i2c.h>
#include <linux/videodev.h>
#include <linux/videodev2.h>
//#include <media/video-buf.h>
#include <linux/delay.h>
#include <asm/mach-types.h>
#include <linux/regulator/consumer.h>

#include <asm/io.h>

#if defined(CONFIG_ARCH_TCC92X) || defined(CONFIG_ARCH_TCC93XX)
#include <mach/bsp.h>
#include <linux/gpio.h>
#include <mach/gpio.h>
#elif defined(CONFIG_ARCH_TCC79X)
#include <mach/tcc79x.h>
#endif
#include "tcc_cam_i2c.h"  
#include <mach/tcc_cam_ioctrl.h>
#include "sensor_if.h"
#include "tdd_cif.h"
#include "cam.h"

#if defined(CONFIG_REGULATOR) && (defined(CONFIG_MACH_TCC9200S_SPACEY) || defined(CONFIG_MACH_TCC9200S_M340EVB))
#if defined(CONFIG_MACH_TCC9200S_M340EVB)
#define VDD_AF_NAME "vdd_cam_af28"
#define VDD_IO_NAME "vdd_cam_io27"
#define VDD_CORE_NAME "vdd_cam_core12"
#else
#define VDD_AF_NAME "vdd_isp_ram18"
#define VDD_IO_NAME "vdd_isp_io28"
#define VDD_CORE_NAME "vdd_isp_core12"
#endif

struct regulator *vdd_cam_core;
struct regulator *vdd_cam_af;
struct regulator *vdd_cam_io;
#endif


static int enabled = 0;

/* list of image formats supported by sensor sensor */
const static struct v4l2_fmtdesc sensor_formats[] = {
	{
		/* Note:  V4L2 defines RGB565 as:
		 *
		 *	Byte 0			  Byte 1
		 *	g2 g1 g0 r4 r3 r2 r1 r0	  b4 b3 b2 b1 b0 g5 g4 g3
		 *
		 * We interpret RGB565 as:
		 *
		 *	Byte 0			  Byte 1
		 *	g2 g1 g0 b4 b3 b2 b1 b0	  r4 r3 r2 r1 r0 g5 g4 g3
		 */
		.description	= "RGB565, le",
		.pixelformat	= V4L2_PIX_FMT_RGB565,
	},{
		/* Note:  V4L2 defines RGB565X as:
		 *
		 *	Byte 0			  Byte 1
		 *	b4 b3 b2 b1 b0 g5 g4 g3	  g2 g1 g0 r4 r3 r2 r1 r0
		 *
		 * We interpret RGB565X as:
		 *
		 *	Byte 0			  Byte 1
		 *	r4 r3 r2 r1 r0 g5 g4 g3	  g2 g1 g0 b4 b3 b2 b1 b0
		 */
		.description	= "RGB565, be",
		.pixelformat	= V4L2_PIX_FMT_RGB565X,
	},
	{
		.description	= "YUYV (YUV 4:2:2), packed",
		.pixelformat	= V4L2_PIX_FMT_YUYV,
	},{
		.description	= "UYVY, packed",
		.pixelformat	= V4L2_PIX_FMT_UYVY,
	},
	{
		/* Note:  V4L2 defines RGB555 as:
		 *
		 *	Byte 0			  Byte 1
		 *	g2 g1 g0 r4 r3 r2 r1 r0	  x  b4 b3 b2 b1 b0 g4 g3
		 *
		 * We interpret RGB555 as:
		 *
		 *	Byte 0			  Byte 1
		 *	g2 g1 g0 b4 b3 b2 b1 b0	  x  r4 r3 r2 r1 r0 g4 g3
		 */
		.description	= "RGB555, le",
		.pixelformat	= V4L2_PIX_FMT_RGB555,
	},{
		/* Note:  V4L2 defines RGB555X as:
		 *
		 *	Byte 0			  Byte 1
		 *	x  b4 b3 b2 b1 b0 g4 g3	  g2 g1 g0 r4 r3 r2 r1 r0
		 *
		 * We interpret RGB555X as:
		 *
		 *	Byte 0			  Byte 1
		 *	x  r4 r3 r2 r1 r0 g4 g3	  g2 g1 g0 b4 b3 b2 b1 b0
		 */
		.description	= "RGB555, be",
		.pixelformat	= V4L2_PIX_FMT_RGB555X,
	}
};

#define NUM_CAPTURE_FORMATS ARRAY_SIZE(sensor_formats)

#if defined(SENSOR_2M)
enum image_size { UXGA, SXGA, XGA, SVGA, VGA, QVGA, QCIF};
#define NUM_IMAGE_SIZES 7
#elif defined(SENSOR_3M)
enum image_size { QXGA, UXGA, SXGA, XGA, SVGA, VGA, QVGA, QCIF};
#define NUM_IMAGE_SIZES 8
#elif defined(SENSOR_5M)
enum image_size { QQXGA, QXGA, UXGA, SXGA, XGA, SVGA, VGA, QVGA, QCIF};
#define NUM_IMAGE_SIZES 9
#elif defined(SENSOR_VGA)
enum image_size {VGA, sCIF, QVGA, QCIF};
#define NUM_IMAGE_SIZES 4
#elif defined(SENSOR_TVP5150) || defined(SENSOR_RDA5888)
enum image_size { XGA, VGA};
#define NUM_IMAGE_SIZES 2
#endif

enum pixel_format { YUV, RGB565, RGB555 };
#define NUM_PIXEL_FORMATS 3

SENSOR_FUNC_TYPE sensor_func;

//need to revised  MCC
#ifdef CONFIG_ARCH_OMAP24XX
#define NUM_OVERLAY_FORMATS 4
#else
#define NUM_OVERLAY_FORMATS 2
#endif


#define DEF_BRIGHTNESS      	3
#define DEF_ISO      			0
#define DEF_AWB           		0
#define DEF_EFFECT				0
#define DEF_ZOOM				0
#define DEF_FLIP				0
#define DEF_SCENE				0
#define DEF_METERING_EXPOSURE	0
#define DEF_FLASH				0

/*  Video controls  */
static unsigned int need_new_set = 0;
static struct vcontrol {
        struct v4l2_queryctrl qc;
        int current_value;
		int	need_set;
} control[] = {
        { { V4L2_CID_BRIGHTNESS, V4L2_CTRL_TYPE_INTEGER, "Brightness", 0, 4, 1,
            DEF_BRIGHTNESS },
          0, 0},
          
        { { V4L2_CID_AUTO_WHITE_BALANCE, V4L2_CTRL_TYPE_INTEGER, "Auto White Balance", 0,5,1,
            DEF_AWB },
          0, 0},

        { { V4L2_CID_ISO, V4L2_CTRL_TYPE_INTEGER, "Iso", 0, 3, 1,
            DEF_ISO },
          0, 0},
          
        { { V4L2_CID_EFFECT, V4L2_CTRL_TYPE_INTEGER, "Special Effect", 0, 6, 1,
            DEF_EFFECT },
          0, 0},
          
        { { V4L2_CID_ZOOM, V4L2_CTRL_TYPE_INTEGER, "Zoom", 0, 25, 1,
            DEF_ZOOM },
          0, 0},    
                    
        { { V4L2_CID_FLIP, V4L2_CTRL_TYPE_INTEGER, "Flip", 0, 3, 1,
            DEF_FLIP },
          0, 0},
          
        { { V4L2_CID_SCENE, V4L2_CTRL_TYPE_INTEGER, "Scene", 0, 5, 1,
            DEF_SCENE},
          0, 0},  
          
        { { V4L2_CID_METERING_EXPOSURE, V4L2_CTRL_TYPE_INTEGER, "Metering Exposure", 0, 3, 1,
            DEF_METERING_EXPOSURE},
          0, 0},
          
        { { V4L2_CID_FLASH, V4L2_CTRL_TYPE_INTEGER, "Flash", 0, 4, 1,
            DEF_FLASH},
          0, 0},	
};


unsigned int sensor_control[] = 
{
	V4L2_CID_BRIGHTNESS,
	V4L2_CID_AUTO_WHITE_BALANCE,
	V4L2_CID_ISO,
	V4L2_CID_EFFECT,
	V4L2_CID_ZOOM,
	V4L2_CID_FLIP,
	V4L2_CID_SCENE,
	V4L2_CID_METERING_EXPOSURE,
	//V4L2_CID_FLASH
};

#ifndef USE_SENSOR_EFFECT_IF
extern int tccxxx_cif_set_effect (u8 nCameraEffect);
#endif
#ifndef USE_SENSOR_ZOOM_IF
extern int tccxxx_cif_set_zoom(unsigned char arg);
#endif

int sensor_if_change_mode(unsigned char capture_mode)
{
    int ret=0;

	if(capture_mode)
		ret = sensor_func.Set_Capture();
	else
		ret = sensor_func.Set_Preview();
	
    return ret;
}

int sensor_if_adjust_autofocus(void)
{
	return sensor_func.Set_AF(0);
}

/* Returns the index of the requested ID from the control structure array */
static int find_vctrl(int id)
{
	int i;
	int ctrl_cnt = ARRAY_SIZE(control);

	if (id < V4L2_CID_BASE)
		return -EDOM;

	for (i = ctrl_cnt - 1; i >= 0; i--)
	{
		if (control[i].qc.id == id)
			break;
	}

	if (i < 0)
		i = -EINVAL;
	
	return i;
}

/* initialize Sensor after module was reset. */
int sensor_init_module(enum image_size isize, enum pixel_format pfmt,unsigned long xclk)
{
	return 0;
}


/* Find the best match for a requested image capture size.  The best match 
 * is chosen as the nearest match that has the same number or fewer pixels 
 * as the requested size, or the smallest image size if the requested size 
 * has fewer pixels than the smallest image.
 */
static enum image_size sensor_find_size(unsigned int width, unsigned int height)
{
	enum image_size isize;
	unsigned long pixels = width*height;

	for (isize = 0; isize < (NUM_IMAGE_SIZES-1); isize++) {
		if (sensor_sizes[isize + 1].height*
			sensor_sizes[isize + 1].width > pixels)
		{
			return isize;
		}
	}
#if defined(CONFIG_VIDEO_ATV_SENSOR_TVP5150) || defined(CONFIG_VIDEO_ATV_SENSOR_RDA5888) || defined(CONFIG_VIDEO_CAMERA_SENSOR_OV7690) 
	return VGA;
#else
	return UXGA;
#endif
}

/* following are sensor interface functions implemented by 
 * sensor driver.
 */
int sensor_if_query_control(struct v4l2_queryctrl *qc)
{
	int i;

	i = find_vctrl (qc->id);
	if (i == -EINVAL) 
	{
		qc->flags = V4L2_CTRL_FLAG_DISABLED;
		return 0;
	}
	
	if (i < 0)
		return -EINVAL;

	*qc = control[i].qc;
	
	return 0;
}

int sensor_if_get_control(struct v4l2_control *vc)
{
	int i;
	struct vcontrol * lvc;
	
	if((i = find_vctrl(vc->id)) < 0)
		return -EINVAL;

	lvc = &control[i];	
	vc->value = lvc->current_value;
	
	return 0;
}

int sensor_if_set_control(struct v4l2_control *vc, unsigned char init)
{	
	struct vcontrol *lvc;
	int val = vc->value;
	int i;
	
	if((i = find_vctrl(vc->id)) < 0)
		return -EINVAL;

	lvc = &control[i];

	if(lvc->qc.maximum < val)
		return val;

	if(lvc->current_value != val || init)
	{
		lvc->current_value = val;
		lvc->need_set = 1;
		need_new_set = 1;
	}
	
	return 0;
}

int sensor_if_check_control(void)
{
	int i, err;
	struct vcontrol *lvc;
	int ctrl_cnt = ARRAY_SIZE(control);

	if(!need_new_set)
		return 0;
	
	for (i = ctrl_cnt - 1; i >= 0; i--)
	{
		lvc = &control[i];
		if(lvc->need_set)
		{
			switch(lvc->qc.id)
			{
				case V4L2_CID_BRIGHTNESS:
					err = sensor_func.Set_Bright(lvc->current_value);
				break;
			
				case V4L2_CID_AUTO_WHITE_BALANCE:
					err = sensor_func.Set_WB(lvc->current_value);
				break;
			
				case V4L2_CID_ISO:
					err = sensor_func.Set_ISO(lvc->current_value);
				break;
			
				case V4L2_CID_EFFECT:
		#ifdef USE_SENSOR_EFFECT_IF
					err = sensor_func.Set_Effect(lvc->current_value);
		#else
					err = tccxxx_cif_set_effect(lvc->current_value);
		#endif			
				break;
					
				case V4L2_CID_ZOOM:
		#ifdef USE_SENSOR_ZOOM_IF
					err = sensor_func.Set_Zoom(lvc->current_value);
		#else
					err = tccxxx_cif_set_zoom(lvc->current_value);
		#endif
					break;
					
				case V4L2_CID_FLIP:
					err = sensor_func.Set_Flip(lvc->current_value);
				break;
			
				case V4L2_CID_SCENE:
					err = sensor_func.Set_Scene(lvc->current_value);
				break;
					
				case V4L2_CID_METERING_EXPOSURE:
					err = sensor_func.Set_ME(lvc->current_value);
					break;
					
				case V4L2_CID_FLASH:
					//err = ;
					break;	
					
				default:
					break;	
				
			}

			lvc->need_set = 0;
		}
	}

	return 0;
}


/* In case of ESD-detection, to set current value in sensor after module was resetted. */
int sensor_if_set_current_control(void)
{
	struct v4l2_control vc;
	int i;
	int ctrl_cnt = ARRAY_SIZE(sensor_control);

	printk("Setting Sensor-ctrl!! %d /n", ctrl_cnt);
	
	for (i = ctrl_cnt - 1; i >= 0; i--)
	{
		vc.id = sensor_control[i];
		sensor_if_get_control(&vc);
		sensor_if_set_control(&vc, 1);
	}

	return 0;
}

/* Implement the VIDIOC_ENUM_FMT ioctl for the CAPTURE buffer type.
 */
int sensor_if_enum_pixformat(struct v4l2_fmtdesc *fmt)
{
	int index = fmt->index;
	enum v4l2_buf_type type = fmt->type;

	memset(fmt, 0, sizeof(*fmt));
	fmt->index = index;
	fmt->type = type;

	switch (fmt->type) {
		case V4L2_BUF_TYPE_VIDEO_CAPTURE:
		if (index >= NUM_CAPTURE_FORMATS)
			return -EINVAL;
		break;

		case V4L2_BUF_TYPE_VIDEO_OVERLAY:
		if (index >= NUM_OVERLAY_FORMATS)
			return -EINVAL;
		break;

		default:
			return -EINVAL;
	}

	fmt->flags = sensor_formats[index].flags;
	strlcpy(fmt->description, sensor_formats[index].description, sizeof(fmt->description));
	fmt->pixelformat = sensor_formats[index].pixelformat;

	return 0;
}

/* Implement the VIDIOC_TRY_FMT ioctl for the CAPTURE buffer type.  This 
 * ioctl is used to negotiate the image capture size and pixel format 
 * without actually making it take effect.
 */
static int sensor_try_format(struct v4l2_pix_format *pix)
{
	enum image_size isize;
	int ifmt;

	isize = sensor_find_size(pix->width, pix->height);
	pix->width  = sensor_sizes[isize].width;
	pix->height = sensor_sizes[isize].height;
	
	for (ifmt = 0; ifmt < NUM_CAPTURE_FORMATS; ifmt++) 
	{
		if (pix->pixelformat == sensor_formats[ifmt].pixelformat)
			break;
	}
	
	if (ifmt == NUM_CAPTURE_FORMATS)
		ifmt = 0;
	
	pix->pixelformat = sensor_formats[ifmt].pixelformat;
	pix->field = V4L2_FIELD_NONE;
	pix->bytesperline = pix->width*2;
	pix->sizeimage = pix->bytesperline*pix->height;
	pix->priv = 0;
	
	switch (pix->pixelformat) 
	{
		case V4L2_PIX_FMT_YUYV:
		case V4L2_PIX_FMT_UYVY:
		default:
			pix->colorspace = V4L2_COLORSPACE_JPEG;
			break;
			
		case V4L2_PIX_FMT_RGB565:
		case V4L2_PIX_FMT_RGB565X:
		case V4L2_PIX_FMT_RGB555:
		case V4L2_PIX_FMT_RGB555X:
			pix->colorspace = V4L2_COLORSPACE_SRGB;
			break;
	}
	
	return 0;
}



/* Given a capture format in pix, the frame period in timeperframe, and
 * the xclk frequency, set the capture format of the sensor.
 * The actual frame period will be returned in timeperframe.
 */
int sensor_if_configure(struct v4l2_pix_format *pix, unsigned long xclk)
{
  	enum pixel_format pfmt = YUV;

	switch (pix->pixelformat) {
		case V4L2_PIX_FMT_RGB565:
		case V4L2_PIX_FMT_RGB565X:
			pfmt = RGB565;
			break;
		case V4L2_PIX_FMT_RGB555:
		case V4L2_PIX_FMT_RGB555X:
			pfmt = RGB555;
			break;
		case V4L2_PIX_FMT_YUYV:
		case V4L2_PIX_FMT_UYVY:
		default:
			pfmt = YUV;
 	}

	return sensor_init_module(sensor_find_size(pix->width, pix->height), pfmt, xclk);
}


/* Initialize the sensor.
 * This routine allocates and initializes the data structure for the sensor, 
 * powers up the sensor, registers the I2C driver, and sets a default image 
 * capture format in pix.  The capture format is not actually programmed 
 * into the sensor by this routine.
 * This function must return a non-NULL value to indicate that 
 * initialization is successful.
 */
int sensor_if_init(struct v4l2_pix_format *pix)
{
 	int ret;

#if defined(CONFIG_REGULATOR) && (defined(CONFIG_MACH_TCC9200S_SPACEY)|| defined(CONFIG_MACH_TCC9200S_M340EVB))
	vdd_cam_core = regulator_get(NULL, VDD_CORE_NAME);
	if (vdd_cam_core == NULL) {
		pr_warning("vdd_cam_core: failed to get ISP 1.2V regulator \n");
		return -1;
	} 

	vdd_cam_af = regulator_get(NULL, VDD_AF_NAME);
	if (vdd_cam_af == NULL) {
		pr_warning("vdd_cam_af: failed to get ISP 1.8V regulator \n");
		return -1;
	} 

	vdd_cam_io = regulator_get(NULL, VDD_IO_NAME);
	if (vdd_cam_io == NULL) {
		pr_warning("vdd_cam_io: failed to get ISP 2.8V regulator \n");
		return -1;
	} 
#elif defined(CONFIG_MACH_TCC9300)
	#ifdef CONFIG_VIDEO_ATV_SENSOR_RDA5888
		gpio_direction_output(TCC_GPEXT1(14), 1); // CAM2_ON, sign-up to external gpio.
		gpio_direction_output(TCC_GPEXT1(12), 1); // sign-up to external gpio.
	#else
		gpio_direction_output(TCC_GPEXT1(13), 1); // sign-up to external gpio.
	#endif
#elif defined(CONFIG_MACH_TCC9300ST) || defined(CONFIG_MACH_TCC9300CM)
	// Todo. RonaldLEE_0929
#endif

	sensor_init_fnc(&sensor_func);
	TDD_CIF_Initialize();

	if(!enabled)
	{
#if !defined(CONFIG_VIDEO_CAMERA_SENSOR_AIT848_ISP)
		if((ret = DDI_I2C_Initialize()) < 0)
		{
			return -1;
		}
		enabled = 1;
#endif
		
	}
	
	/* common register initialization */
	if(sensor_func.Open() < 0)
	{
		return -1;
	}
	
	/* Make the default capture format QVGA YUV422 */
	pix->width = sensor_sizes[VGA].width;
	pix->height = sensor_sizes[VGA].height;
	pix->pixelformat = V4L2_PIX_FMT_YUYV;
	sensor_try_format(pix);

	need_new_set = 0;
	
	return 0;
}

int sensor_if_deinit(void)
{
#if defined(CONFIG_REGULATOR) && (defined(CONFIG_MACH_TCC9200S_SPACEY)|| defined(CONFIG_MACH_TCC9200S_M340EVB))
	if (vdd_cam_core)
		regulator_put(vdd_cam_core);

	if (vdd_cam_af)
		regulator_put(vdd_cam_af);

	if (vdd_cam_io)
		regulator_put(vdd_cam_io);
#endif

	return 0;
}

void sensor_pwr_control(int name, int enable)
{
#if defined(CONFIG_REGULATOR) && (defined(CONFIG_MACH_TCC9200S_SPACEY) || defined(CONFIG_MACH_TCC9200S_M340EVB))
	if(enable)
	{
		switch(name)
		{
			case CAM_CORE:
				if (vdd_cam_core)
					regulator_enable(vdd_cam_core);
				break;

			case CAM_ETC:
				if (vdd_cam_af)
					regulator_enable(vdd_cam_af);
				break;

			case CAM_IO:
				if (vdd_cam_io)
					regulator_enable(vdd_cam_io);
				break;

			default:
				break;
		}
	}
	else
	{
		switch(name)
		{
			case CAM_CORE:
				if (vdd_cam_core)
					regulator_disable(vdd_cam_core);
				break;

			case CAM_ETC:
				if (vdd_cam_af)
					regulator_disable(vdd_cam_af);
				break;

			case CAM_IO:
				if (vdd_cam_io)
					regulator_disable(vdd_cam_io);
				break;

			default:
				break;
		}
		
	}	
#endif
}

#if defined(CONFIG_VIDEO_CAMERA_SENSOR_AIT848_ISP)
void sensor_if_set_enable(void)
{
	DDI_I2C_Initialize();
	enabled = 1;
}

int sensor_if_get_enable(void)
{
	return enabled;
}
#endif

int sensor_if_restart(void)
{
	sensor_func.Open();

	return 0;
}

int sensor_if_capture_config(int width, int height)
{
	return sensor_func.Set_CaptureCfg(width, height);
}

int sensor_if_isESD(void)
{
	return sensor_func.Check_ESD(0);
}
/* Prepare for the driver to exit.
 * Balances sensor_if_init().
 * This function must de-initialize the sensor and its associated data 
 * structures.
 */
int sensor_if_cleanup(void)
{
	if(enabled)
	{
		sensor_func.Close();
		TDD_CIF_Termination();
		DDI_I2C_Terminate();
		
		enabled = 0;
	}

    return 0;
}

void sensor_power_enable(void)
{
	#if defined(CONFIG_MACH_TCC9300)
		#ifdef CONFIG_VIDEO_ATV_SENSOR_RDA5888
			gpio_direction_output(TCC_GPEXT1(14), 1); // CAM2_ON, sign-up to external gpio.
			gpio_direction_output(TCC_GPEXT1(12), 1); // P-CAM_PWR_ON, sign-up to external gpio.
			printk("[RDATV] Power Enabled!! \n");
		#else
			gpio_direction_output(TCC_GPEXT1(13), 1); // CAM1_ON, sign-up to external gpio.
		#endif
	#elif defined(CONFIG_MACH_TCC9200S)
	volatile PGPION pGPIO_D = (PGPION)tcc_p2v(HwGPIOD_BASE);
	pGPIO_D->GPEN |= Hw10;
	pGPIO_D->GPDAT |= Hw10;
	#elif defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900) || defined(CONFIG_MACH_TCC9300CM)
	// Todo : 
	#elif defined(CONFIG_MACH_TCC9300ST)
	// Todo : RonaldLEE_0929
	#else
	// Todo : 
	#endif
}

void sensor_power_disable(void)
{
	#if defined(CONFIG_MACH_TCC9300)
		#ifdef CONFIG_VIDEO_ATV_SENSOR_RDA5888
			gpio_direction_output(TCC_GPEXT1(12), 0); // P-CAM_PWR_ON, sign-up to external gpio.
			gpio_direction_output(TCC_GPEXT1(14), 0); // CAM2_ON, sign-up to external gpio.
			printk("[RDATV] Power Disabled!! \n");
		#else
			gpio_direction_output(TCC_GPEXT1(13), 0); // CAM1_ON, sign-up to external gpio.
		#endif
	#elif defined(CONFIG_MACH_TCC9200S)
	volatile PGPION pGPIO_D = (PGPION)tcc_p2v(HwGPIOD_BASE);
	pGPIO_D->GPEN |= Hw10;
	pGPIO_D->GPDAT &= ~Hw10;
	#elif defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900) || defined(CONFIG_MACH_TCC9300CM)
	// Todo : 
	#elif defined(CONFIG_MACH_TCC9300ST)
	// Todo : RonaldLEE_0929
	#else
	// Todo : 
	#endif
}

void sensor_powerdown_enable(void)
{
	#if defined(CONFIG_MACH_TCC9300)
	volatile PGPION pGPIO_E = (PGPION)tcc_p2v(HwGPIOE_BASE);
	pGPIO_E->GPEN |= Hw24;
	pGPIO_E->GPDAT |= Hw24;
	#elif defined(CONFIG_MACH_TCC9200S)
	volatile PGPION pGPIO_F = (PGPION)tcc_p2v(HwGPIOF_BASE);
	pGPIO_F->GPEN |= Hw21;
	pGPIO_F->GPDAT |= Hw21;
	#elif defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900) || defined(CONFIG_MACH_TCC9300CM)
	// Todo : 
	#elif defined(CONFIG_MACH_TCC9300ST)
	// Todo : RonaldLEE_0929
	#else
	HwGPFEN |= Hw1;
	HwGPFDAT |= Hw1;
	#endif
}

void sensor_powerdown_disable(void)
{
	#if defined(CONFIG_MACH_TCC9300)
	volatile PGPION pGPIO_E = (PGPION)tcc_p2v(HwGPIOE_BASE);
	pGPIO_E->GPEN |= Hw24;
	pGPIO_E->GPDAT &= ~Hw24;
	#elif defined(CONFIG_MACH_TCC9200S)
	volatile PGPION pGPIO_F = (PGPION)tcc_p2v(HwGPIOF_BASE);
	pGPIO_F->GPEN |= Hw21;
	pGPIO_F->GPDAT &= ~Hw21;
	#elif defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900) || defined(CONFIG_MACH_TCC9300CM) 
	// Todo : 
	#elif defined(CONFIG_MACH_TCC9300ST)
	// Todo : RonaldLEE_0929
	#else
	HwGPFEN |= Hw1;
	HwGPFDAT &= ~Hw1;
	#endif
}

void sensor_reset_high(void)
{
	volatile PGPION pGPIO_E = (PGPION)tcc_p2v(HwGPIOE_BASE);
	#if defined(CONFIG_MACH_TCC9300)
	pGPIO_E->GPEN |= Hw25;
	pGPIO_E->GPDAT |= Hw25;
	#elif defined(CONFIG_MACH_TCC9200S)
	pGPIO_E->GPEN |= Hw2;
	pGPIO_E->GPDAT |= Hw2;
	#elif defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900) || defined(CONFIG_MACH_TCC9300CM)
	pGPIO_E->GPEN |= Hw26;
	pGPIO_E->GPDAT |= Hw26;
	#elif defined(CONFIG_MACH_TCC9300ST)
	// Todo : RonaldLEE_0929
	#else
	HwGPEEN |= Hw9;
	HwGPEDAT |= Hw9;
	#endif
}

void sensor_reset_low(void)
{
	volatile PGPION pGPIO_E = (PGPION)tcc_p2v(HwGPIOE_BASE);
	#if defined(CONFIG_MACH_TCC9300)
	pGPIO_E->GPEN |= Hw25;
	pGPIO_E->GPDAT &= ~Hw25;
	#elif defined(CONFIG_MACH_TCC9200S)
	pGPIO_E->GPEN |= Hw2;
	pGPIO_E->GPDAT &= ~Hw2;
	#elif defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900) || defined(CONFIG_MACH_TCC9300CM)
	pGPIO_E->GPEN |= Hw26;
	pGPIO_E->GPDAT &= ~Hw26;
	#elif defined(CONFIG_MACH_TCC9300ST)
	// Todo : RonaldLEE_0929
	#else
	HwGPEEN |= Hw9;
	HwGPEDAT &= ~Hw9;
	#endif
}

/* ----------------------------------------------------------------------- */
MODULE_DESCRIPTION("Camera SENSOR  driver");
MODULE_AUTHOR("Telechips");
MODULE_LICENSE("GPL");

