
 /****************************************************************************
 *   FileName    : tca_cs8900_hwset.c
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/
#ifdef __KERNEL__
#include "tca_cs8900_hwset.h"
#include <linux/delay.h>
#define CS_SLEEP(x)	mdelay(x)
#define COUNT		5
#else
#include "tca_cs8900_hwset.h"
#include <windows.h>
#define CS_SLEEP(x)	Sleep(x)
#define COUNT		100
#endif

void tca_cs8900_init(void)
{
#if defined(_TCC79x_)
	/* CS0 */
	HwCSCFG0 = 0x468B0000 | (2<<11) | (30<<3) | 2 | HwCSCFG_OD;
	HwPORTCFG4 &= ~(1<<0); // set CS0
	HwGPCEN |= (Hw24);

	/* ETH_RESET */
/*
	HwPORTCFG2 &= ~(Hw16);	// GPIO_F16
	HwGPFEN   |= (Hw16);	// GPIO_F16 is Output
	HwGPFCLR   = (Hw16);	// Reset Low
	CS_SLEEP(COUNT);			// Wait 2msec
	HwGPFSET   = (Hw16);
*/
#elif defined(_TCC83x_)
	/* CS2 */
	HwCSCFG2 = (0x668A0000 | (2<<11) | (30<<3) | 2 | HwCSCFG_OD);	
	/* GPIO_XA */
	HwGSEL_XA |= (Hw31 | Hw25 | Hw24 | 0xE);
	/* ETH_RESET */
	HwNGSEL_A |= Hw9;		// GPIO_A[9] be used for ETH_RESET
	HwGIOCON_A |= Hw9; 		// GPIO_A[9] Output mode
#endif
}

void tca_cs8900_irq(void)
{
#if defined(_TCC79x_)
    //IRQ initializing

    HwPORTCFG3 &=~(1<<16); //GPIO_F3
    HwGPFEN |= (1<<3);
    HwEINTSEL0 = (HwEINTSEL0 & (~0x1f)) | HwEINTSEL0_EINT0(SEL_HPXD3);

    HwSEL |= (1<<IRQ_EI0); //EINT_n be used
    HwIRQMSK |= (1<<IRQ_EI0);

    //set EINT_n as level sensitive; HIGH active
    HwPOL &= ~(1<<IRQ_EI0);
    HwMODE |= (1<<IRQ_EI0);

    HwGPFDAT |= (1<<16);
    CS_SLEEP(COUNT);
    HwGPFDAT &= ~(1<<16);
    CS_SLEEP(COUNT);

#elif defined(_TCC83x_)
	/* ETH_IRQ */
	BITSET(HwGSEL_B, Hw3);		//GPIO_B[5] EINT_1
	BITCLR(HwGIOCON_B, Hw5);	//GPIO_B[5] InputMode
	BITSET(HwGPIOB_PE, Hw5);	//GPIO_B[5] Enable pull down/up
	BITCLR(HwGPIOB_PS, Hw5);	//GPIO_B[5] Pull down

	/* ETH init */
	BITSET(HwSEL_B, Hw1);
	BITCLR(HwMODE_B, Hw1);
	BITSET(HwCLR_B, Hw1);

	BITSET(HwICFG, Hw7|Hw6|Hw5);

	//Reset CS8900 hardware
	BITSET(HwGDATA_A, Hw9);
	CS_SLEEP(COUNT);
	BITCLR(HwGDATA_A, Hw9);
	CS_SLEEP(COUNT);
#endif
}

void tca_cs8900_clear_irq(void)
{
#ifdef _TCC79x_
	HwCLR = Hw1;
#elif _TCC83x_
	HwCLR_B = Hw1;
#endif
}

/* end of tca_cs8900_hwset.c */
