/*
 * drivers/regulator/act8810-regulator.c
 *
 * ACT8810 PMIC driver
 *
 * Copyright (C) 2009 Telechips, Inc.
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <linux/kernel.h>
#include <linux/err.h>
#include <linux/platform_device.h>
#include <linux/mfd/act8810.h>
#include <linux/regulator/driver.h>
#include <linux/regulator/machine.h>
#include <linux/regulator/act8810.h>

struct act8810_voltage {
	int uV;
	int val;
};

static int current_voltages[ACT8810_NUM_REGULATORS];

static int find_voltage_from_value(struct act8810_voltage *voltages, int val)
{
	int i;

	for (i = 0; i < 32; i++) {
		if (voltages[i].val == val)
			return voltages[i].uV;
	}
	return 0;
}

static int find_value_from_voltage(struct act8810_voltage *voltages, int uV)
{
	int i;

	for (i = 0; i < 32; i++) {
		if (voltages[i].uV == uV)
			return voltages[i].val;
	}
	return 0;
}

static struct act8810_voltage out_voltages[] = {
	{  800000, ACT8810_OUT_0_800V },
	{  850000, ACT8810_OUT_0_850V },
	{  900000, ACT8810_OUT_0_900V },
	{  950000, ACT8810_OUT_0_950V },
	{ 1000000, ACT8810_OUT_1_000V },
	{ 1050000, ACT8810_OUT_1_050V },
	{ 1100000, ACT8810_OUT_1_100V },
	{ 1150000, ACT8810_OUT_1_150V },
	{ 1200000, ACT8810_OUT_1_200V },
	{ 1250000, ACT8810_OUT_1_250V },
	{ 1300000, ACT8810_OUT_1_300V },
	{ 1350000, ACT8810_OUT_1_350V },
	{ 1400000, ACT8810_OUT_1_400V },
	{ 1500000, ACT8810_OUT_1_500V },
	{ 1600000, ACT8810_OUT_1_600V },
	{ 1700000, ACT8810_OUT_1_700V },
	{ 1750000, ACT8810_OUT_1_750V },
	{ 1800000, ACT8810_OUT_1_800V },
	{ 1850000, ACT8810_OUT_1_850V },
	{ 1900000, ACT8810_OUT_1_900V },
	{ 2000000, ACT8810_OUT_2_000V },
	{ 2100000, ACT8810_OUT_2_100V },
	{ 2200000, ACT8810_OUT_2_200V },
};

static int act8810_ldo_is_enabled(struct regulator_dev *dev)
{
	struct act8810 *act8810 = rdev_get_drvdata(dev);
	int id = rdev_get_id(dev);
	int enable = 1;
	int reg;

	reg = 0; //act8810_reg_read(act8810, LP8725_REG_LDO_CONTROL);
	if (reg < 0)
		return -EINVAL;

	switch (id) {
	case ACT8810_OUT1:
	case ACT8810_OUT2:
	case ACT8810_OUT3:
		break;
		
	default:
		return -EINVAL;
	}
	return enable;
}

static int act8810_ldo_enable(struct regulator_dev *dev)
{
	struct act8810 *act8810 = rdev_get_drvdata(dev);
	int id = rdev_get_id(dev);
	int mask;
	int ret;

	switch (id) {
	case ACT8810_OUT1:
	case ACT8810_OUT2:
	case ACT8810_OUT3:
		break;

	default:
		return -EINVAL;
	}
	ret = 0; //act8810_reg_set_bit_mask(act8810, LP8725_REG_LDO_CONTROL, mask, 0xff);
	return ret;
}

static int act8810_ldo_disable(struct regulator_dev *dev)
{
	struct act8810 *act8810 = rdev_get_drvdata(dev);
	int id = rdev_get_id(dev);
	int mask;
	int ret;

	switch (id) {
	case ACT8810_OUT1:
	case ACT8810_OUT2:
	case ACT8810_OUT3:
		break;

	default:
		return -EINVAL;
	}
	ret = 0;//act8810_reg_set_bit_mask(act8810, LP8725_REG_LDO_CONTROL, mask, 0);
	return ret;
}


static int act8810_get_voltage(struct regulator_dev *dev)
{
	struct act8810 *act8810 = rdev_get_drvdata(dev);
	int id = rdev_get_id(dev);
	int reg;
	int volt;

	switch (id) {
	case ACT8810_OUT1:
		reg = 0;//act8810_reg_read(act8810, ACT8810_REG_BUCK1_V1);
		if (reg < 0)
			return -EINVAL;
		break;

	case ACT8810_OUT2:
	case ACT8810_OUT3:
		break;

	default:
		return -EINVAL;
	}
	reg &= 0x1F;
	volt = find_voltage_from_value(out_voltages, reg);

	return volt;
}

static int act8810_set_voltage(struct regulator_dev *dev,
				   int min_uV, int max_uV)
{
	struct act8810 *act8810 = rdev_get_drvdata(dev);
	int id = rdev_get_id(dev);
	int reg;
	int ret;

	if (current_voltages[id] == min_uV)
		return 0;

	//pr_debug("act8810-regulator: set '%s' to %dmV\n", act8810->regulators[id].name, min_uV / 1000);
	printk("act8810-regulator: set '%s' to %dmV\n", act8810->regulators[id].name, min_uV / 1000);

	reg = find_value_from_voltage(out_voltages, min_uV);
	
	printk("act8810-regulator: id:%d  reg:%x\n", id, reg);

	switch (id) {
	case ACT8810_OUT1:
	case ACT8810_OUT2:
		ret = act8810_reg_set_bit_mask(act8810, ACT8810_REG_OUT1_STBY, 0x1F, reg);
		ret = act8810_reg_set_bit_mask(act8810, ACT8810_REG_OUT1_VSET, 0x3F, reg);
		break;

	case ACT8810_OUT3:
		ret = 0;
		break;

	default:
		return -EINVAL;
	}
	current_voltages[id] = min_uV;

	return ret;
}

static int act8810_set_suspend_enable(struct regulator_dev *dev)
{
	return 0;
}

static int act8810_set_suspend_disable(struct regulator_dev *dev)
{
	return 0;
}

static int act8810_set_suspend_voltage(struct regulator_dev *dev, int uV)
{
	return act8810_set_voltage(dev, uV, uV);
}

static struct regulator_ops act8810_ops = {
	.is_enabled = act8810_ldo_is_enabled,
	.enable = act8810_ldo_enable,
	.disable = act8810_ldo_disable,
	.get_voltage = act8810_get_voltage,
	.set_voltage = act8810_set_voltage,
	.set_suspend_enable = act8810_set_suspend_enable,
	.set_suspend_disable = act8810_set_suspend_disable,
	.set_suspend_voltage = act8810_set_suspend_voltage,
};

static struct regulator_consumer_supply vdd_usb33_consumers[] = {
	{
		.supply = "vdd_usb33",
	}
};

static struct regulator_consumer_supply vdd_usb12_consumers[] = {
	{
		.supply = "vdd_usb12",
	}
};

static struct regulator_consumer_supply vdd_core_consumers[] = {
	{
		.supply = "vdd_core",
	},
};

static struct regulator_init_data regulator_initdata[ACT8810_NUM_REGULATORS] = {
	[ACT8810_OUT2] = {
		.constraints = {
			.name = "vdd_usb33",
			.min_uV = 1200000,
			.max_uV = 3300000,
			.valid_modes_mask = REGULATOR_MODE_NORMAL,
			.valid_ops_mask = REGULATOR_CHANGE_VOLTAGE,
			.boot_on = 1,
		},
		.num_consumer_supplies = ARRAY_SIZE(vdd_usb33_consumers),
		.consumer_supplies = vdd_usb33_consumers,
	},
	[ACT8810_OUT3] = {
		.constraints = {
			.name = "vdd_usb12",
			.min_uV = 800000,
			.max_uV = 3300000,
			.valid_modes_mask = REGULATOR_MODE_NORMAL,
			.valid_ops_mask = REGULATOR_CHANGE_VOLTAGE,
			.boot_on = 1,
		},
		.num_consumer_supplies = ARRAY_SIZE(vdd_usb12_consumers),
		.consumer_supplies = vdd_usb12_consumers,
	},

	[ACT8810_OUT1] = {
		.constraints = {
			.name = "vdd_core",
			.min_uV = 1000000,
			.max_uV = 1500000,
			.valid_modes_mask = REGULATOR_MODE_NORMAL,
			.valid_ops_mask = REGULATOR_CHANGE_VOLTAGE,
			.state_mem = {
				.uV = 1050000,
				.mode = REGULATOR_MODE_NORMAL,
				.enabled = 1,
			},
			.always_on = 1,
			.boot_on = 1,
		},
		.num_consumer_supplies = ARRAY_SIZE(vdd_core_consumers),
		.consumer_supplies = vdd_core_consumers,
	},
};

static struct regulator_desc regulators[ACT8810_NUM_REGULATORS] = {
	{
		.name = "vdd_usb33",
		.id = ACT8810_OUT2,
		.ops = &act8810_ops,
		.type = REGULATOR_VOLTAGE,
		.owner = THIS_MODULE,
	},
	{
		.name = "vdd_usb12",
		.id = ACT8810_OUT3,
		.ops = &act8810_ops,
		.type = REGULATOR_VOLTAGE,
		.owner = THIS_MODULE,
	},
	{
		.name = "vdd_core",
		.id = ACT8810_OUT1,
		.ops = &act8810_ops,
		.type = REGULATOR_VOLTAGE,
		.owner = THIS_MODULE,
	},
};

static int __devinit act8810_regulator_probe(struct platform_device *pdev)
{
	struct regulator_dev *rdev;

	rdev = regulator_register(&regulators[pdev->id], &pdev->dev, platform_get_drvdata(pdev));
	if (IS_ERR(rdev))
		return PTR_ERR(rdev);
	return 0;
}

static int __devexit act8810_regulator_remove(struct platform_device *pdev)
{
	struct regulator_dev *rdev = platform_get_drvdata(pdev);

	regulator_unregister(rdev);

	return 0;
}

int act8810_register_regulator(struct device *dev, int reg)
{
	struct act8810 *act8810 = dev_get_drvdata(dev);
	struct regulator_init_data *initdata;

	if (act8810->regulators[reg].name)
		return -EBUSY;

	initdata = &regulator_initdata[reg];
	initdata->driver_data = act8810;

	act8810->regulators[reg].name = "act8810-regulator";
	act8810->regulators[reg].id = reg;
	act8810->regulators[reg].dev.parent = dev;
	dev_set_drvdata(&act8810->regulators[reg].dev, act8810);
	act8810->regulators[reg].dev.platform_data = initdata;

	return platform_device_register(&act8810->regulators[reg]);
}
EXPORT_SYMBOL(act8810_register_regulator);

static struct platform_driver act8810_regulator_driver = {
	.driver = {
		.name = "act8810-regulator",
	},
	.probe = act8810_regulator_probe,
	.remove = act8810_regulator_remove,
};

static int __init act8810_regulator_init(void)
{
	return platform_driver_register(&act8810_regulator_driver);
}
module_init(act8810_regulator_init);

static void __exit act8810_regulator_exit(void)
{
	platform_driver_unregister(&act8810_regulator_driver);
}
module_exit(act8810_regulator_exit);

MODULE_AUTHOR("Telechips, Inc.");
MODULE_DESCRIPTION("ACT8810 regulator driver");
MODULE_LICENSE("GPL");
