
/****************************************************************************
 *   FileName    : tca_tccserial.c
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/

/*****************************************************************************
*
* Include
*
******************************************************************************/

#ifdef __KERNEL__
	#if defined(_TCC79x_)
		#include <mach/tcc79x.h> 
	#elif defined(_TCC83x_)
		#include <mach/tcc83x_virt.h>
	#endif
#include <asm/plat-tcc/tcc_ckc.h>

#else

	#if defined(_TCC79x_)
		#include "TCC79x_Virtual.h"
	#elif defined(_TCC83x_)
		#include "TCC83x_Virtual.h"
	#endif

#include "tca_tccserial.h"
#include "tca_ckcstr.h"
#include "tcc_IoctlCode.h"
#include "tca_peri_ckc.h"

#endif
/*****************************************************************************
*
* Defines
*
******************************************************************************/



/*****************************************************************************
* Function Name : tca_serial_intrinit()
******************************************************************************
* Desription    : serial Interrupt Init
* Parameter     : void
* Return        : 
******************************************************************************/
void	tca_serial_intrinit(void)
{

	#if defined(_TCC79x_)
		BITCLR(HwIRQSEL, Hw5|Hw4); //UART Intr select
		HwSEL	 |= HwINT_SEL2;
		HwIRQMSK |= HwINT_SEL2;
		HwMODE	 |= HwINT_SEL2;
		//HwPOL	 |= HwINT_SEL2;
    #elif defined(CONFIG_ARCH_TCC83X)
#ifdef __KERNEL
        BITCLR(HwIEN, Hw7|Hw29);
        BITCLR(HwIEN_B, Hw8|Hw9);
        BITSET(HwSEL, Hw7|Hw29);
        BITSET(HwSEL_B, Hw8|Hw9);
        BITSET(HwMODE, Hw7|Hw29);
        BITSET(HwMODE_B, Hw8|Hw9);
        //BITSET(HwPOL, (1 << port->irq));
#endif
    #endif
}

/*****************************************************************************
* Function Name : tca_serial_intrdone()
******************************************************************************
* Desription    : serial Interrupt Init
* Parameter     : void
* Return        : 
******************************************************************************/
void	tca_serial_intrdone(void)
{
	#if defined(_TCC79x_)
		HwINTMSK |= HwINT_SEL2;
		//HwIEN |= HwINT_SEL2;
    #elif defined(CONFIG_ARCH_TCC83X)
#ifdef __KERNEL__
        BITSET(HwIEN, Hw7|Hw29);
        BITSET(HwIEN_B, Hw8|Hw9);
#endif
    #endif
}

/*****************************************************************************
* Function Name : tca_serial_portinit(int nPort)
******************************************************************************
* Desription    : tca_serial_portinit
* Parameter     : int nPort
* Return        : 
******************************************************************************/
void	tca_serial_portinit(int nPort)
{
		if(nPort == 0)
		{
		//Set Uart Peri CKC
            tca_ckc_com_setperibus(RB_UART0, 1);
			tca_ckc_com_setperi(PERI_UART0,MHz,52,DIRECTPLL0);

			#if defined(_TCC79x_)
				BITSET(HwBCLKCTR, Hw11); 
				BITCLR(HwPORTCFG9, Hw16-Hw12);//UTX0 GPIOE[0]
				BITCLR(HwPORTCFG9, Hw12-Hw8);//URX0 GPIOE[1]
#ifdef __KERNEL__
				BITCLR(HwPORTCFG9, Hw8-Hw4);//UCTS0 GPIOE[2]
				BITCLR(HwPORTCFG9, Hw4-Hw0);//URTS0 GPIOE[3]
#endif
			#elif defined(_TCC83x_)
				BITSET(HwBCLKCTR, Hw5); 
				BITCSET(HwGSEL_B, Hw11|Hw10, Hw11);//UTX0 GPIO_B8 -GPIO_B11
			#endif

		}
		else if(nPort == 1)
		{
		//Set Uart Peri CKC
            tca_ckc_com_setperibus(RB_UART1, 1);
			tca_ckc_com_setperi(PERI_UART1,MHz,52,DIRECTPLL0);
			
		//Set Uart Peri BUS, GPIO
			#if defined(_TCC79x_)
				BITSET(HwBCLKCTR, Hw12); 
				BITCLR(HwPORTCFG10, Hw32-Hw28);//UTX1
				BITCLR(HwPORTCFG10, Hw28-Hw24);//URX1
#ifdef __KERNEL__
				BITCLR(HwPORTCFG10, Hw24-Hw20);//UCTS1 GPIOE[6]
				BITCLR(HwPORTCFG10, Hw20-Hw16);//URTS1 GPIOE[7]
#endif
			#elif defined(_TCC83x_)
				BITSET(HwBCLKCTR, Hw23); 
				BITCSET(HwGSEL_B, Hw13|Hw12, Hw13);//UTX1 GPIO_B12 - GPIO_B15
			#endif
		}

		else if(nPort == 2)
		{
		//Set Uart Peri CKC
            tca_ckc_com_setperibus(RB_UART2, 1);
			tca_ckc_com_setperi(PERI_UART2,MHz,52,DIRECTPLL0);
			
		//Set Uart Peri BUS, GPIO
			#if defined(_TCC79x_)
				BITSET(HwBCLKCTR, Hw15);

				BITCLR(HwPORTCFG10, Hw16-Hw12);//UTX2
				BITCLR(HwPORTCFG10, Hw12-Hw8); //URX2
#ifdef __KERNEL__
#else
				BITCSET(HwUART2_IRCFG, (Hw4-Hw0), (Hw3|Hw2|Hw1|Hw0));	// (Hw3)RXP=1,(Hw2)TXP=1,(Hw1)RXE=1,(Hw0)TXE=1 
#endif
      
			#elif defined(_TCC83x_)
				BITSET(HwBCLKCTR, Hw6); 
				BITCSET(HwGSEL_F, Hw11|Hw10, Hw10);//GPIOF0 -F3
			#endif
		}
		else if(nPort == 3)
		{
		//Set Uart Peri CKC
            tca_ckc_com_setperibus(RB_UART3, 1);
			tca_ckc_com_setperi(PERI_UART3,MHz,52,DIRECTPLL0);
		
		//Set Uart Peri BUS, GPIO
			#if defined(_TCC79x_)
				BITSET(HwBCLKCTR1, Hw0); 
				BITCLR(HwPORTCFG10, Hw8-Hw4);//UTX3
				BITCLR(HwPORTCFG10, Hw4-Hw0);//URX3
			#elif defined(_TCC83x_)
				BITSET(HwBCLKCTR, Hw8); 
				BITCSET(HwGSEL_F, Hw13|Hw12, Hw13);//UTX3 GPIO_F12 - GPIO_F15
			#endif
		}
		else if(nPort == 4)
		{
		//Set Uart Peri BUS, GPIO
#ifdef __KERNEL__

#else
			#if defined(_TCC79x_)
				BITCSET(HwPORTCFG9, Hw8-Hw4, Hw5);//UTX4
				BITCSET(HwPORTCFG9, Hw4-Hw0, Hw1); //URX4
			#elif defined(_TCC83x_)

			#endif
#endif 
		}

}

//TODO: TCC83x DMA
//#ifdef _TCC79x_
#if defined(_TCC79x_) || defined(_TCC80x_)
/*****************************************************************************
* Function Name : tca_serial_dmaclrinterrupt()
******************************************************************************
* Desription    : dma clear interrupt status
* Parameter     : dma number
* Return        : success(SUCCESS) 
******************************************************************************/
unsigned int tca_serial_dmaclrinterrupt(unsigned nDmanum)
{
//RETAILMSG(1, (TEXT("-tca_serial_dmaclrinterrupt--nDmanum:%d\r\n"),nDmanum));
	switch(	nDmanum )
	{
	case 0:
		HwCHCTRL0 |= Hw3; 
		break;
	case 1:
		HwCHCTRL1 |= Hw3;
		break;
	case 2:
		HwCHCTRL2 |= Hw3; 
		break;
	case 3:
		HwCHCTRL3 |= Hw3; 
		break;
	case 4:
		HwCHCTRL4 |= Hw3;
		break;
	case 5:
		HwCHCTRL5 |= Hw3;
		break;
	case 6:
		HwCHCTRL6 |= Hw3;
		break;
	case 7:
		HwCHCTRL7 |= Hw3;
		break;
	case 8:
		HwCHCTRL8 |= Hw3;
		break;
#if defined(_TCC80x_)
	case 9:
		HwCHCTRL9 |= Hw3;
		break;
	case 10:
		HwCHCTRL10 |= Hw3;
		break;
	case 11:
		HwCHCTRL11 |= Hw3;
		break;
#endif
	}
	return 0;
}

/*****************************************************************************
* Function Name : tca_dma_ctrl(int m_DmaNumber)
******************************************************************************
* Desription    : tca_dma_ctrl
* Parameter     : int m_DmaNumber
* Return        : 
******************************************************************************/
int     tca_dma_ctrl(int m_DmaNumber)
{
//RETAILMSG(1, (TEXT("-tca_dma_ctrl--m_DmaNumber:%d\r\n"),m_DmaNumber));
	switch(m_DmaNumber)
	{
	case 0:
		return HwCHCTRL0;
		break;
	case 1:
		return HwCHCTRL1;
		break;
	case 2:
		return HwCHCTRL2;
		break;
	case 3:
		return HwCHCTRL3;
		break;
	case 4:
		return HwCHCTRL4;
		break;
	case 5:
		return HwCHCTRL5;
		break;
	case 6:
		return HwCHCTRL6;
		break;
	case 7:
		return HwCHCTRL7;
		break;
	case 8:
		return HwCHCTRL8;
		break;
#if defined(_TCC80x_)
	case 9:
		return HwCHCTRL9;
		break;
	case 10:
		return HwCHCTRL10;
		break;
	case 11:
		return HwCHCTRL11;
		break;
#endif
	default:
		return 0;
	}
	
}

/*****************************************************************************
* Function Name : tca_uart_channelselectint channel, int mode)
******************************************************************************
* Desription    : tca_serial_portinit
* Parameter     : int channel, int mode
* Return        : 
******************************************************************************/
int
tca_uart_channelselect(int channel, int mode)
{
//RETAILMSG(1, (TEXT("-tca_uart_channelselect--channel:%d, mode:%d\r\n"),channel,mode));
	switch(channel)
	{
	case 0:
		return (mode?HwEXTREQ_UART0_RX:HwEXTREQ_UART0_TX);
		break;
	case 1:
		return (mode?HwEXTREQ_UART1_RX:HwEXTREQ_UART1_TX);
		break;
	case 2:
		return (mode?HwEXTREQ_UART2_RX:HwEXTREQ_UART2_TX);
		break;
	case 3:
		return (mode?HwEXTREQ_UART3_RX:HwEXTREQ_UART3_TX);
		break;
#if defined(_TCC80x_)
	case 4:
		return (mode?HwEXTREQ_UART4_RX:HwEXTREQ_UART4_TX);
		break;
#endif
	default :
		return (mode?HwEXTREQ_UART3_RX:HwEXTREQ_UART3_TX);
		break;
	}
}

/*****************************************************************************
* Function Name : tca_dma_setconfig(
	iunsigned uCH,
	void *pSRT, 
	unsigned uSPARAM,
	void *pDST, 
	unsigned uDPARAM,
	unsigned	uCHCTRL,
	unsigned uSize,
	unsigned channel,
	unsigned mode
)
******************************************************************************
* Desription    : tca_dma_setconfig
* Parameter     : 
* Return        : 
******************************************************************************/
void tca_dma_setconfig(
    unsigned uCH,
	void *pSRT, 
	unsigned uSPARAM,
	void *pDST, 
	unsigned uDPARAM,
	unsigned	uCHCTRL,
	unsigned uSize,
	unsigned channel,
	unsigned mode
)
{

//RETAILMSG(1, (TEXT("-tca_dma_setconfig--uCH:%d,uCHCTRL:%x,uSize:%d,channel:%d,mode:%d\r\n"),uCH,uCHCTRL,uSize,channel,mode ));

    switch( uCH )
    {
		    case 2:
        	if (uSize)
        	{
        		// Set Source Address & Source Parameter (mask + increment)
				HwST_SADR2	= (int)pSRT;
        		HwSPARAM2 = uSPARAM;

        		// Set Dest Address & Dest Parameter (mask + increment)
        		HwST_DADR2	= (int)pDST;
        		HwDPARAM2 = uDPARAM;
				HwRPTCTRL2 &= ~Hw31;
				HwEXTREQ2 =  tca_uart_channelselect(channel,mode);
				// now one HOP is 1*8/8 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
				HwHCOUNT2 = uSize; //((uSize)>>0x01)>>0x03;
				
        	}

        	HwCHCTRL2  = uCHCTRL;
			HwCHCTRL2 |= HwCHCTRL_EN_EN;
        	break;
  
         case 3:
        	if (uSize)
        	{
        		// Set Source Address & Source Parameter (mask + increment)
				HwST_SADR3	= (int)pSRT;
        		HwSPARAM3 = uSPARAM;

        		// Set Dest Address & Dest Parameter (mask + increment)
        		HwST_DADR3	= (int)pDST;
        		HwDPARAM3 = uDPARAM;
				HwRPTCTRL3 &= ~Hw31;
				HwEXTREQ3 =  tca_uart_channelselect(channel,mode);
				// now one HOP is 1*8/8 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
				HwHCOUNT3 = uSize; //((uSize)>>0x01)>>0x03;
				
        	}

        	HwCHCTRL3  = uCHCTRL;
			HwCHCTRL3 |= HwCHCTRL_EN_EN;
        	break;
		 case 4:
        	if (uSize)
        	{
        		// Set Source Address & Source Parameter (mask + increment)
				HwST_SADR4	= (int)pSRT;
        		HwSPARAM4 = uSPARAM;

        		// Set Dest Address & Dest Parameter (mask + increment)
        		HwST_DADR4	= (int)pDST;
        		HwDPARAM4 = uDPARAM;
				HwRPTCTRL4 &= ~Hw31;
				HwEXTREQ4 = tca_uart_channelselect(channel,mode);
				// now one HOP is 1*8/8 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
				HwHCOUNT4 = uSize; // ((uSize)>>0x01)>>0x03;
				
        	}

        	HwCHCTRL4  = uCHCTRL;
			HwCHCTRL4 |= HwCHCTRL_EN_EN;
        	break;
		case 5:
        	if (uSize)
        	{
        		// Set Source Address & Source Parameter (mask + increment)
				HwST_SADR5	= (int)pSRT;
        		HwSPARAM5 = uSPARAM;

        		// Set Dest Address & Dest Parameter (mask + increment)
        		HwST_DADR5	= (int)pDST;
        		HwDPARAM5 = uDPARAM;
				HwRPTCTRL5 &= ~Hw31;
				HwEXTREQ5 = tca_uart_channelselect(channel,mode);
				// now one HOP is 1*8/8 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
				HwHCOUNT5 = uSize; // ((uSize)>>0x01)>>0x03;
				
        	}

        	HwCHCTRL5  = uCHCTRL;
			HwCHCTRL5 |= HwCHCTRL_EN_EN;
        	break;
		 case 6:
        	if (uSize)
        	{
        		// Set Source Address & Source Parameter (mask + increment)
				HwST_SADR6	= (int)pSRT;
        		HwSPARAM6 = uSPARAM;

        		// Set Dest Address & Dest Parameter (mask + increment)
        		HwST_DADR6	= (int)pDST;
        		HwDPARAM6 = uDPARAM;
				HwRPTCTRL6 &= ~Hw31;

				HwEXTREQ6 =  tca_uart_channelselect(channel,mode);
				// now one HOP is 1*8/8 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
				HwHCOUNT6 = uSize; // ((uSize)>>0x01)>>0x03;
				
        	}

        	HwCHCTRL6  = uCHCTRL;
			HwCHCTRL6 |= HwCHCTRL_EN_EN;
        	break;
		 case 7:
        	if (uSize)
        	{
        		// Set Source Address & Source Parameter (mask + increment)
				HwST_SADR7	= (int)pSRT;
        		HwSPARAM7 = uSPARAM;
        		// Set Dest Address & Dest Parameter (mask + increment)
        		HwST_DADR7	= (int)pDST;
        		HwDPARAM7 = uDPARAM;
				HwRPTCTRL7 &= ~Hw31;

				HwEXTREQ7 =  tca_uart_channelselect(channel,mode);
				// now one HOP is 1*8/8 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
#if defined(_TCC80x_) 
				HwHCOUNT7 = (uSize/2); // ((uSize)>>0x01)>>0x03;
#else
				HwHCOUNT7 = (uSize); // ((uSize)>>0x01)>>0x03;
#endif

				
        	}

        	HwCHCTRL7  = uCHCTRL;
			HwCHCTRL7 |= HwCHCTRL_EN_EN;
        	break;
		 case 8:
        	if (uSize)
        	{
        		// Set Source Address & Source Parameter (mask + increment)
				HwST_SADR8	= (int)pSRT;
        		HwSPARAM8 = uSPARAM;

        		// Set Dest Address & Dest Parameter (mask + increment)
        		HwST_DADR8	= (int)pDST;
        		HwDPARAM8 = uDPARAM;
				HwRPTCTRL8 &= ~Hw31;
				HwEXTREQ8 =  tca_uart_channelselect(channel,mode);
				// now one HOP is 1*8/8 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
				HwHCOUNT8 = uSize; // ((uSize)>>0x01)>>0x03;
				
        	}

        	HwCHCTRL8  = uCHCTRL;
			HwCHCTRL8 |= HwCHCTRL_EN_EN;
        	break;
#if defined(_TCC80x_)
		 case 9:
        	if (uSize)
        	{
        		// Set Source Address & Source Parameter (mask + increment)
				HwST_SADR9	= (int)pSRT;
        		HwSPARAM9 = uSPARAM;

        		// Set Dest Address & Dest Parameter (mask + increment)
        		HwST_DADR9	= (int)pDST;
        		HwDPARAM9 = uDPARAM;
				HwRPTCTRL9 &= ~Hw31;
				HwEXTREQ9 =  tca_uart_channelselect(channel,mode);
				// now one HOP is 1*8/8 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
				HwHCOUNT9 = uSize; // ((uSize)>>0x01)>>0x03;

      }

        	HwCHCTRL9  = uCHCTRL;
			HwCHCTRL9 |= HwCHCTRL_EN_EN;
        	break;
		 case 10:
        	if (uSize)
        	{
        		// Set Source Address & Source Parameter (mask + increment)
				HwST_SADR10	= (int)pSRT;
        		HwSPARAM10 = uSPARAM;

        		// Set Dest Address & Dest Parameter (mask + increment)
        		HwST_DADR10	= (int)pDST;
        		HwDPARAM10 = uDPARAM;
				HwRPTCTRL10 &= ~Hw31;
				HwEXTREQ10 =  tca_uart_channelselect(channel,mode);
				// now one HOP is 1*8/8 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
				HwHCOUNT10 = uSize; // ((uSize)>>0x01)>>0x03;
				
        	}

        	HwCHCTRL10  = uCHCTRL;
			HwCHCTRL10 |= HwCHCTRL_EN_EN;
        	break;
		 case 11:
        	if (uSize)
        	{
        		// Set Source Address & Source Parameter (mask + increment)
				HwST_SADR11	= (int)pSRT;
        		HwSPARAM11 = uSPARAM;

        		// Set Dest Address & Dest Parameter (mask + increment)
        		HwST_DADR11	= (int)pDST;
        		HwDPARAM11 = uDPARAM;
				HwRPTCTRL11 &= ~Hw31;
				HwEXTREQ11 =  tca_uart_channelselect(channel,mode);
				// now one HOP is 1*8/8 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
				HwHCOUNT11 = uSize; // ((uSize)>>0x01)>>0x03;
				
        	}

        	HwCHCTRL11  = uCHCTRL;
			HwCHCTRL11 |= HwCHCTRL_EN_EN;
        	break;

#endif
      }

}
#endif	//_TCC79x_
