
/****************************************************************************
 *   FileName    : tca_tccserial.c
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/

/*****************************************************************************
*
* Include
*
******************************************************************************/

#include <bsp.h>
#include "tcc_serial.h"
#include "tca_serial.h"


/*****************************************************************************
*
* Defines
*
******************************************************************************/


/*****************************************************************************
* Function Name : tca_serial_portinit(int nCh, int nPort, unsigned long* pvGpioAddr, unsigned long* pvPortMuxAddr)
******************************************************************************
* Desription    : tca_serial_portinit
* Parameter     : int nCh, int nPort, unsigned long* pvGpioAddr, unsigned long* pvPortMuxAddr
* Return        : None
******************************************************************************/
void tca_serial_portinit(int nCh, int nPort, unsigned long* pvGpioAddr, unsigned long* pvPortMuxAddr)
{

	PGPIO pGpio = (PGPIO)pvGpioAddr;
	PUARTPORTMUX pPortmux = (PUARTPORTMUX)pvPortMuxAddr;

	// ch5 cannot use DMA (spec.)
	// each value of CHSEL shuld be different
	//                          chnum   ,  portnum
#if defined(CONFIG_MACH_TCC9300)   // Original source for TCC9300
    BITCSET(pPortmux->CHSEL, 0x7 <<(4*0), 0 << (4*0));  //portmuxinit
    BITCSET(pPortmux->CHSEL, 0x7 <<(4*1), 1 << (4*1));  //portmuxinit
    BITCSET(pPortmux->CHSEL, 0x7 <<(4*2), 2 << (4*2));  //portmuxinit
    BITCSET(pPortmux->CHSEL, 0x7 <<(4*3), 3 << (4*3));  //portmuxinit
    BITCSET(pPortmux->CHSEL, 0x7 <<(4*4), 4 << (4*4));  //portmuxinit
    BITCSET(pPortmux->CHSEL, 0x7 <<(4*5), 5 << (4*5));  //portmuxinit
#else
	BITCSET(pPortmux->CHSEL, 0x7 <<(4*0), 0 << (4*0));  //portmuxinit
	BITCSET(pPortmux->CHSEL, 0x7 <<(4*1), 1 << (4*1));  //portmuxinit
	BITCSET(pPortmux->CHSEL, 0x7 <<(4*2), 4 << (4*2));  //portmuxinit
	BITCSET(pPortmux->CHSEL, 0x7 <<(4*3), 5 << (4*3));  //portmuxinit
	BITCSET(pPortmux->CHSEL, 0x7 <<(4*4), 2 << (4*4));  //portmuxinit
	BITCSET(pPortmux->CHSEL, 0x7 <<(4*5), 3 << (4*5));  //portmuxinit
#endif

	switch(nPort){//do port set
	case 0: 
		//TXD: GPIO_G25
		//RXD: GPIO_G24
		BITCSET(pGpio->GPGFN3, Hw4-Hw0, Hw1);
		BITCSET(pGpio->GPGFN3, Hw8-Hw4, Hw5);
		break;
	case 1:
		#if defined(CONFIG_MACH_TCC9300) || defined(CONFIG_MACH_TCC9300CM)
			//TXD: GPIO_E4
			//RXD: GPIO_E5
			BITCSET(pGpio->GPEFN0, Hw24-Hw20, Hw20);
			BITCSET(pGpio->GPEFN0, Hw20-Hw16, Hw16);
		#elif defined(CONFIG_MACH_TCC9300ST)
			//TXD: GPIO_G29
			//RXD: GPIO_G28
            BITCSET(pGpio->GPGFN3, Hw24-Hw20, Hw21);
            BITCSET(pGpio->GPGFN3, Hw20-Hw16, Hw17);
		#endif
		break;
	case 2:
		#if defined(CONFIG_MACH_TCC9300) || defined(CONFIG_MACH_TCC9300CM)
			//TXD: GPIO_E8
			//RXD: GPIO_E9
			BITCSET(pGpio->GPEFN1, Hw4-Hw0, Hw0);
			BITCSET(pGpio->GPEFN1, Hw8-Hw4, Hw4);
		#elif defined(CONFIG_MACH_TCC9300ST)
			//TXD: GPIO_G7
			//RXD: GPIO_G6
            BITCSET(pGpio->GPGFN0, Hw32-Hw28, Hw29);
            BITCSET(pGpio->GPGFN0, Hw28-Hw24, Hw25);
		#endif
		break;
	case 3:
		//TXD: GPIO_E10
		//RXD: GPIO_E11
		BITCSET(pGpio->GPEFN1, Hw16-Hw12, Hw12);
		BITCSET(pGpio->GPEFN1, Hw12-Hw8, Hw8);		
		break;
	case 4:
		//TXD: GPIO_D13
		//RXD: GPIO_D14
		BITCSET(pGpio->GPDFN1, Hw28-Hw24, Hw24);
		BITCSET(pGpio->GPDFN1, Hw24-Hw20, Hw20);
		break;
	case 5:
		//TXD: GPIO_D17
		//RXD: GPIO_D18
		BITCSET(pGpio->GPDFN2, Hw8-Hw4, Hw4);
		BITCSET(pGpio->GPDFN2, Hw12-Hw8, Hw8);
		break;
	}
	
}

void tca_serial_gpio_setting(int nCh, int nPort, unsigned long* pvGpioAddr, unsigned long* pvPortMuxAddr)
{

	PGPIO pGpio = (PGPIO)pvGpioAddr;
	PUARTPORTMUX pPortmux = (PUARTPORTMUX)pvPortMuxAddr;
	
	switch(nPort){//do port set
	case 0: 
		//TXD: GPIO_E0
		//RXD: GPIO_E1
		BITCSET(pGpio->GPEFN0, Hw4-Hw0, Hw0);
		BITCSET(pGpio->GPEFN0, Hw8-Hw4, Hw4);
		break;
	case 1:
		#if defined(CONFIG_MACH_TCC9300) || defined(CONFIG_MACH_TCC9300CM)
			//TXD: GPIO_E4
			//RXD: GPIO_E5
			BITCSET(pGpio->GPEFN0, Hw24-Hw20, Hw20);
			BITCSET(pGpio->GPEFN0, Hw20-Hw16, Hw16);
		#elif defined(CONFIG_MACH_TCC9300ST)
			//TXD: GPIO_G29
			//RXD: GPIO_G28
            BITCSET(pGpio->GPGFN3, Hw24-Hw20, Hw21);
            BITCSET(pGpio->GPGFN3, Hw20-Hw16, Hw17);
		#endif
		break;
	case 2:
		#if defined(CONFIG_MACH_TCC9300) || defined(CONFIG_MACH_TCC9300CM)
			//TXD: GPIO_E8
			//RXD: GPIO_E9
			BITCSET(pGpio->GPEFN1, Hw4-Hw0, Hw0);
			BITCSET(pGpio->GPEFN1, Hw8-Hw4, Hw4);
		#elif defined(CONFIG_MACH_TCC9300ST)
			//TXD: GPIO_G7
			//RXD: GPIO_G6
            BITCSET(pGpio->GPGFN0, Hw32-Hw28, Hw29);
            BITCSET(pGpio->GPGFN0, Hw28-Hw24, Hw25);
		#endif
		break;
	case 3:
		//TXD: GPIO_E10
		//RXD: GPIO_E11
		BITCSET(pGpio->GPEFN1, Hw16-Hw12, Hw12);
		BITCSET(pGpio->GPEFN1, Hw12-Hw8, Hw8);		
		break;
	case 4:
		//TXD: GPIO_D13
		//RXD: GPIO_D14
		BITCSET(pGpio->GPDFN1, Hw28-Hw24, Hw24);
		BITCSET(pGpio->GPDFN1, Hw24-Hw20, Hw20);
		break;
	case 5:
		//TXD: GPIO_D17
		//RXD: GPIO_D18
		BITCSET(pGpio->GPDFN2, Hw8-Hw4, Hw4);
		BITCSET(pGpio->GPDFN2, Hw12-Hw8, Hw8);
		break;
	}
}

void tca_serial_gpio_default(int nCh, int nPort, unsigned long* pvGpioAddr, unsigned long* pvPortMuxAddr)
{

	PGPIO pGpio = (PGPIO)pvGpioAddr;
	PUARTPORTMUX pPortmux = (PUARTPORTMUX)pvPortMuxAddr;
	
	switch(nPort){//do port set
	case 0: 
		//TXD: GPIO_G25
		//RXD: GPIO_G24
		BITCLR(pGpio->GPGFN3, Hw4-Hw0);
		BITCLR(pGpio->GPGFN3, Hw8-Hw4);
		BITCLR(pGpio->GPGEN, Hw24|Hw25);		
		break;
	case 1:
		#if defined(CONFIG_MACH_TCC9300) || defined(CONFIG_MACH_TCC9300CM)
			//TXD: GPIO_E4
			//RXD: GPIO_E5
			BITCLR(pGpio->GPEFN0, Hw24-Hw20);
			BITCLR(pGpio->GPEFN0, Hw20-Hw16);
			BITCLR(pGpio->GPEEN, Hw4|Hw5);
		#elif defined(CONFIG_MACH_TCC9300ST)
			//TXD: GPIO_G29
			//RXD: GPIO_G28
            BITCLR(pGpio->GPGFN3, Hw24-Hw20);
            BITCLR(pGpio->GPGFN3, Hw20-Hw16);
            BITCLR(pGpio->GPGEN, Hw28|Hw29);
		#endif
		break;
	case 2:
		#if defined(CONFIG_MACH_TCC9300) || defined(CONFIG_MACH_TCC9300CM)
			//TXD: GPIO_E8
			//RXD: GPIO_E9
			BITCLR(pGpio->GPEFN1, Hw4-Hw0);
			BITCLR(pGpio->GPEFN1, Hw8-Hw4);
			BITCLR(pGpio->GPEEN, Hw8|Hw9);
		#elif defined(CONFIG_MACH_TCC9300ST)
			//TXD: GPIO_G7
			//RXD: GPIO_G6
			BITCLR(pGpio->GPGFN0, Hw32-Hw28);
            BITCLR(pGpio->GPGFN0, Hw28-Hw24);
            BITCLR(pGpio->GPGEN, Hw6|Hw7);
		#endif
		break;
	case 3:
		//TXD: GPIO_E10
		//RXD: GPIO_E11
		BITCLR(pGpio->GPEFN1, Hw16-Hw12);
		BITCLR(pGpio->GPEFN1, Hw12-Hw8);		
		BITCLR(pGpio->GPEEN, Hw10|Hw11);
		break;
	case 4:
		//TXD: GPIO_D13
		//RXD: GPIO_D14
		BITCLR(pGpio->GPDFN1, Hw28-Hw24);
		BITCLR(pGpio->GPDFN1, Hw24-Hw20);
		BITCLR(pGpio->GPDEN, Hw13|Hw14);
		break;
	case 5:
		//TXD: GPIO_D17
		//RXD: GPIO_D18
		BITCLR(pGpio->GPDFN2, Hw8-Hw4);
		BITCLR(pGpio->GPDFN2, Hw12-Hw8);
		BITCLR(pGpio->GPDEN, Hw17|Hw18);
		break;
	}

		
}

/*****************************************************************************
* Function Name : tca_serial_dmaclrinterrupt(unsigned nDmanum, unsigned long* pVirtualDmaAddr)
******************************************************************************
* Desription    : tca_serial_dmaclrinterrupt
* Parameter     : unsigned nDmanum, unsigned long* pVirtualDmaAddr
* Return        : success(SUCCESS) 
******************************************************************************/
unsigned int tca_serial_dmaclrinterrupt(unsigned nDmanum, unsigned long* pVirtualDmaAddr)
{
	PGDMACTRL pDMA = (PGDMACTRL)pVirtualDmaAddr;
	switch(	nDmanum ){
	case 0:
		pDMA->CHCTRL0 |= Hw3; 
		break;
	case 1:
		pDMA->CHCTRL1 |= Hw3; 
		break;
	case 2:
		pDMA->CHCTRL2 |= Hw3; 
		break;
	default:
		return 0;
	}
	return 1;
}

/*****************************************************************************
* Function Name : tca_dma_dmacurrentaddress(int m_DmaNumber, unsigned long* pVirtualDmaAddr)
******************************************************************************
* Desription    : tca_dma_dmacurrentaddress
* Parameter     : int m_DmaNumber, unsigned long* pVirtualDmaAddr
* Return        : 
******************************************************************************/
int tca_dma_dmacurrentaddress(int m_DmaNumber, unsigned long* pVirtualDmaAddr)
{
	PGDMACTRL pDMA = (PGDMACTRL)pVirtualDmaAddr;
	switch(m_DmaNumber)
	{
	case 0:
		return  (pDMA->C_DADR0);
	case 1:
		return  (pDMA->C_DADR1);
	case 2:
		return  (pDMA->C_DADR2);
	default:
		return 0;
	}
}

/*****************************************************************************
* Function Name : tca_dma_dmadeststartaddress(int m_DmaNumber, unsigned long* pVirtualDmaAddr)
******************************************************************************
* Desription    : tca_dma_dmadeststartaddress
* Parameter     : int m_DmaNumber, unsigned long* pVirtualDmaAddr
* Return        : 
******************************************************************************/
int tca_dma_dmadeststartaddress(int m_DmaNumber, unsigned long* pVirtualDmaAddr)
{
	PGDMACTRL pDMA = (PGDMACTRL)pVirtualDmaAddr;
	switch(m_DmaNumber){
	case 0:
		return  (pDMA->ST_DADR0);
	case 1:
		return  (pDMA->ST_DADR1);
	case 2:
		return  (pDMA->ST_DADR2);
	default:
		return 0;
	}
	return 0;
}

/*****************************************************************************
* Function Name : tca_dma_clrien(int m_DmaNumber, unsigned long* pVirtualDmaAddr)
******************************************************************************
* Desription    : tca_dma_clrien
* Parameter     : int m_DmaNumber, unsigned long* pVirtualDmaAddr
* Return        : 
******************************************************************************/
int tca_dma_clrien(int m_DmaNumber, unsigned long* pVirtualDmaAddr)
{
	PGDMACTRL pDMA = (PGDMACTRL)pVirtualDmaAddr;
	switch(m_DmaNumber){
	case 0:
		 pDMA->CHCTRL0 &= ~Hw2; 
		break;
	case 1:
		 pDMA->CHCTRL1 &= ~Hw2; 
		break;
	case 2:
		 pDMA->CHCTRL2 &= ~Hw2; 
		break;
	default:
		return 0;
	}
	return 0;
}

/*****************************************************************************
* Function Name : tca_dma_setien(int m_DmaNumber, unsigned long* pVirtualDmaAddr)
******************************************************************************
* Desription    : tca_dma_setien
* Parameter     : int m_DmaNumber, unsigned long* pVirtualDmaAddr
* Return        : 
******************************************************************************/
int tca_dma_setien(int m_DmaNumber, unsigned long* pVirtualDmaAddr)
{
	PGDMACTRL pDMA = (PGDMACTRL)pVirtualDmaAddr;
	switch(m_DmaNumber){
	case 0:
		 pDMA->CHCTRL0 |= Hw2; 
		break;
	case 1:
		 pDMA->CHCTRL1 |= Hw2; 
		break;
	case 2:
		 pDMA->CHCTRL2 |= Hw2; 
		break;
	default:
		return 0;
	}
	return 0;
}

/*****************************************************************************
* Function Name : tca_dma_ctrl(int m_DmaNumber, unsigned long* pVirtualDmaAddr)
******************************************************************************
* Desription    : tca_dma_ctrl
* Parameter     : int m_DmaNumber, unsigned long* pVirtualDmaAddr
* Return        : 
******************************************************************************/
int tca_dma_ctrl(int m_DmaNumber, unsigned long* pVirtualDmaAddr)
{
	PGDMACTRL pDMA = (PGDMACTRL)pVirtualDmaAddr;

	switch(m_DmaNumber){
	case 0:
		return  (pDMA->CHCTRL0);
	case 1:
		return  (pDMA->CHCTRL1);
	case 2:
		return  (pDMA->CHCTRL2);
	default:
		return 0;
	}
}

/*****************************************************************************
* Function Name : tca_dma_setconfig(
	iunsigned uCH,
	void *pSRT, 
	unsigned uSPARAM,
	void *pDST, 
	unsigned uDPARAM,
	unsigned	uCHCTRL,
	unsigned uSize,
	unsigned channel,
	unsigned mode,
	unsigned long* pVirtualDmaAddr
)
******************************************************************************
* Desription    : tca_dma_setconfig
* Parameter     : 
* Return        : 
******************************************************************************/
void tca_dma_setconfig(
    unsigned uCH,
	void* pSRT, 
	unsigned uSPARAM,
	void* pDST, 
	unsigned uDPARAM,
	unsigned	uCHCTRL,
	unsigned uSize,
	unsigned channel,
	unsigned mode,
	unsigned long* pVirtualDmaAddr
)
{
	PGDMACTRL pDMA = (PGDMACTRL)pVirtualDmaAddr;


	switch( uCH )   {
	case 0:
		if (uSize){
        	// Set Source Address & Source Parameter (mask + increment)
			 pDMA->ST_SADR0 = (int)pSRT;
			 pDMA->SPARAM0 = uSPARAM;
        	
			 // Set Dest Address & Dest Parameter (mask + increment)
			 pDMA->ST_DADR0 = (int)pDST;
			 pDMA->DPARAM0 = uDPARAM;
			 pDMA->RPTCTRL0 &= ~Hw31;
			 

			 pDMA->EXTREQ0 = tca_uart_channelselect(channel, mode);
			 pDMA->HCOUNT0 = uSize;
		}

		 pDMA->CHCTRL0 = uCHCTRL;
		 pDMA->CHCTRL0 |= Hw0;
		break;
	case 1:
		if (uSize){
		// Set Source Address & Source Parameter (mask + increment)
			 pDMA->ST_SADR1 = (int)pSRT;
			 pDMA->SPARAM1 = uSPARAM;

        	// Set Dest Address & Dest Parameter (mask + increment)
			 pDMA->ST_DADR1 = (int)pDST;
			 pDMA->DPARAM1 = uDPARAM;
			 pDMA->RPTCTRL1 &= ~Hw31;
			 
			 pDMA->EXTREQ1 = tca_uart_channelselect(channel, mode);
			 pDMA->HCOUNT1 = uSize;
        	}

		 pDMA->CHCTRL1 = uCHCTRL;
		 pDMA->CHCTRL1 |= Hw0;
        break;
	case 2:
		if (uSize){
		// Set Source Address & Source Parameter (mask + increment)
			 pDMA->ST_SADR2 = (int)pSRT;
			 pDMA->SPARAM2 = uSPARAM;

        	// Set Dest Address & Dest Parameter (mask + increment)
			 pDMA->ST_DADR2 = (int)pDST;
			 pDMA->DPARAM2 = uDPARAM;
			 pDMA->RPTCTRL2 &= ~Hw31;
			 
			 pDMA->EXTREQ2 = tca_uart_channelselect(channel, mode);
			 pDMA->HCOUNT2 = uSize;
        	}

		 pDMA->CHCTRL2 = uCHCTRL;
		 pDMA->CHCTRL2 |= Hw0;
        break;
	}
}

/*****************************************************************************
* Function Name : tca_uart_channelselectint channel, int mode)
******************************************************************************
* Desription    : tca_serial_portinit
* Parameter     : int channel, int mode
* Return        : 
******************************************************************************/
int tca_uart_channelselect(int channel, int mode)
{
	
	switch(channel){
	case 0:
		return (mode?HwEXTREQ_UART0_RX:HwEXTREQ_UART0_TX);
		break;
	case 1:
		return (mode?HwEXTREQ_UART1_RX:HwEXTREQ_UART1_TX);
		break;
	case 2:
		return (mode?HwEXTREQ_UART2_RX:HwEXTREQ_UART2_TX);
		break;
	case 3:
		return (mode?HwEXTREQ_UART3_RX:HwEXTREQ_UART3_TX);
	default :
		break;
	
	}
	return 0;
}
