/*
 * linux/drivers/spi/tcc9200_tsif.c
 *
 * Author:  <linux@telechips.com>
 * Created: 1st April, 2009
 * Description: Driver for Telechips SPI (GPSB) Controllers
 *              SPI slave mode for DXB TSIF (Transport Stream Interface) modules
 *
 * Copyright (c) Telechips, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/spinlock.h>
#include <linux/workqueue.h>
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <linux/errno.h>
#include <linux/err.h>
#include <linux/clk.h>
#include <linux/dma-mapping.h>
#include <linux/poll.h>
#include <linux/spi/spi.h>
#include <linux/compile.h>

#include <asm/uaccess.h>
#include <asm/io.h>
#include <asm/dma.h>

#include <mach/bsp.h>
#include <mach/gpio.h>
#include <linux/spi/tcc_tsif.h>
#include "tcc/tca_spi_hwset.h"
#include "tsdemux/TSDEMUX_ref_timer.h"
#include "tsdemux/TSDEMUX_sys.h"


static unsigned int guiPCRPID;
static struct clk *gpsb_clk;

//#define SPI_DMA_SIZE 0x100000
#define SPI_DMA_SIZE 0x200000

struct tca_spi_pri_handle {
    wait_queue_head_t wait_q;
    struct mutex mutex;
    int open_cnt;
	u32 gpio_port;
};

static tca_spi_handle_t tsif_handle;
static struct tca_spi_pri_handle tsif_pri;


static ssize_t show_port(struct device *dev, struct device_attribute *attr, char *buf)
{
#if 0
	struct platform_device *pdev = to_platform_device(dev);
	tca_spi_handle_t *ptsif_tca = platform_get_drvdata(pdev);
	struct tca_spi_pri_handle *ptsif_pri = ptsif_tca->private_data;
	return sprintf(buf, "0x%x\n", ptsif_pri->gpio_port);
#else
	return sprintf(buf, "0x%x\n", tsif_pri.gpio_port);
#endif
}
static ssize_t store_port(struct device *dev, struct device_attribute *attr, const char *buf, size_t count)
{
#if 0
	struct platform_device *pdev = to_platform_device(dev);
	tca_spi_handle_t *ptsif_tca = platform_get_drvdata(pdev);
	struct tca_spi_pri_handle *ptsif_pri = ptsif_tca->private_data;
	u32 port;

	port = simple_strtoul(buf, (char **)NULL, 16);
	/* valid port: 0xC, 0xD, 0xE */
	if (port < 0xC || port > 0xE) {
		printk("%s%d: invalid port! (use 0xc/d/e)\n", pdev->name, pdev->id);
		return -EINVAL;
}

	ptsif_pri->gpio_port = port;
	return count;
#else
	u32 port;

	port = simple_strtoul(buf, (char **)NULL, 16);
	/* valid port: 0xC, 0xD, 0xE */
	if (port < 0xC || port > 0xE) {
		printk("tcc-tsif: invalid port! (use 0xc/d/e)\n");
		return -EINVAL;
	}

	tsif_pri.gpio_port = port;
	return count;
#endif
}
static DEVICE_ATTR(tcc_port, S_IRUSR|S_IWUSR, show_port, store_port);

static int __init tsif_drv_probe(struct platform_device *pdev)
{
	struct resource *port = NULL;

	port = platform_get_resource(pdev, IORESOURCE_IO, 0);
	if (!port) {
		return -ENXIO;
		}
	tsif_pri.gpio_port = port->start;

	platform_set_drvdata(pdev, &tsif_handle);

	/* TODO: device_remove_file(&pdev->dev, &dev_attr_tcc_port); */
	device_create_file(&pdev->dev, &dev_attr_tcc_port);

	printk("%s%d: init\n", pdev->name, TSIF_SPI_NUM);
	return 0;
		}

#ifdef CONFIG_PM
/*
 * NOTE: before suspend, you must close spi.
 */
static int tsif_drv_suspend(struct platform_device *pdev, pm_message_t state)
{
	return 0;
	}
static int tsif_drv_resume(struct platform_device *pdev)
{
	return 0;
}
#else
#define tsif_drv_suspend NULL
#define tsif_drv_resume  NULL
#endif

static struct platform_driver tsif_platform_driver = {
	.driver = {
		.name	= "tcc-tsif",
		.owner	= THIS_MODULE,
	},
	.suspend = tsif_drv_suspend,
	.resume  = tsif_drv_resume,
};


static int tcc_dxb_power(unsigned long flag)
{
	#if defined(CONFIG_MACH_TCC9300ST)
 	if (flag == 1) {
                /* GPIO_EXPAND DXB_ON Power-on */
                //gpio_direction_output(TCC_GPEXT1(8), 1);
                //gpio_set_value(TCC_GPEXT1(8), 1);
        } else if( flag == 2 ) {
                /* GPIO_EXPAND DEEPPWDN Power-on */
                //gpio_direction_output(TCC_GPEXT2(8), 1);
                //gpio_set_value(TCC_GPEXT2(8), 1);
        } else if (flag == 0) {
                /* GPIO_EXPAND DXB_ON Power-off */
                //gpio_direction_output(TCC_GPEXT1(8), 0);
                //gpio_set_value(TCC_GPEXT1(8), 0);

                /* GPIO_EXPAND DEEPPWDN Power-off */
                //gpio_direction_output(TCC_GPEXT2(8), 0);
                //gpio_set_value(TCC_GPEXT2(8), 0);
        } else {
                return -EINVAL;
        }
    #elif defined(CONFIG_MACH_TCC9300CM)
    if (flag == 1) {
		/* GPIO_EXPAND DXB_ON Power-on */
        	gpio_direction_output(TCC_GPEXT2(12), 1);
	        gpio_set_value(TCC_GPEXT2(12), 1);
	        gpio_direction_output(TCC_GPEXT1(15), 1);
	        gpio_set_value(TCC_GPEXT1(15), 1);
	} else if (flag == 0) {
		/* GPIO_EXPAND DXB_ON Power-off */
	        gpio_direction_output(TCC_GPEXT2(12), 0);
	        gpio_set_value(TCC_GPEXT2(12), 0);
	        gpio_direction_output(TCC_GPEXT1(15), 0);
	        gpio_set_value(TCC_GPEXT1(15), 0);
	} else {
		return -EINVAL;
	}
	#else
	if (flag == 1) {
		/* GPIO_EXPAND DXB_ON Power-on */
        	gpio_direction_output(TCC_GPEXT1(8), 1);
	        gpio_set_value(TCC_GPEXT1(8), 1);
	} else if( flag == 2 ) {
		/* GPIO_EXPAND DEEPPWDN Power-on */
	        gpio_direction_output(TCC_GPEXT2(8), 1);
        	gpio_set_value(TCC_GPEXT2(8), 1);
	} else if( flag == 3 ) {
		/* GPIO_EXPAND_DXB_PWRDWN_N_IN Power-on */
			gpio_direction_output(TCC_GPEXT2(9), 1);
			gpio_set_value(TCC_GPEXT2(9), 1);
	} else if (flag == 0) {
		/* GPIO_EXPAND DXB_ON Power-off */
	        gpio_direction_output(TCC_GPEXT1(8), 0);
	        gpio_set_value(TCC_GPEXT1(8), 0);

		/* GPIO_EXPAND DEEPPWDN Power-off */
	        gpio_direction_output(TCC_GPEXT2(8), 0);
	        gpio_set_value(TCC_GPEXT2(8), 0);

		/* GPIO_EXPAND PWRDWN_N_IN Power-off */
			gpio_direction_output(TCC_GPEXT2(9), 0);
			gpio_set_value(TCC_GPEXT2(9), 0);
	} else {
		return -EINVAL;
	}
	#endif

	return 0;
}

static void tea_free_dma_linux(struct tea_dma_buf *tdma)
{
    if (tdma) {
        if (tdma->v_addr != 0) {
            dma_free_writecombine(0, tdma->buf_size, tdma->v_addr, tdma->dma_addr);
        }
        memset(tdma, 0, sizeof(struct tea_dma_buf));
    }
}

static int tea_alloc_dma_linux(struct tea_dma_buf *tdma, unsigned int size)
{
    int ret = -1;
    if (tdma) {
        tea_free_dma_linux(tdma);
        tdma->buf_size = size;
        tdma->v_addr = dma_alloc_writecombine(0, tdma->buf_size, &tdma->dma_addr, GFP_KERNEL);
        printk("tcc_tsif: alloc DMA buffer @0x%X(Phy=0x%X), size:%d\n",
               (unsigned int)tdma->v_addr,
               (unsigned int)tdma->dma_addr,
               tdma->buf_size);
        ret = tdma->v_addr ? 0 : 1;
    }
    return ret;
}

static irqreturn_t tcc_tsif_dma_handler(int irq, void *dev_id)
{
    struct tca_spi_handle *tspi = (struct tca_spi_handle *)dev_id;
    struct tca_spi_pri_handle *tpri = (struct tca_spi_pri_handle *)tspi->private_data;
    unsigned long dma_done_reg = 0;

    dma_done_reg = tspi->regs->DMAICR;
    if (dma_done_reg & (Hw28 | Hw29)) {
        BITSET(tspi->regs->DMAICR, Hw29 | Hw28);
        if (tpri->open_cnt > 0) {
            tspi->cur_q_pos = (int)(tspi->regs->DMASTR >> 17);
			//Check PCR & Make STC
			if(guiPCRPID < 0x1FFF )
			{	
				int start_pos, search_pos, search_size;
				start_pos = tspi->cur_q_pos - tspi->dma_intr_packet_cnt;
				if(start_pos > 0 )
				{
					search_pos = start_pos;
					search_size = tspi->dma_intr_packet_cnt;
				}	
				else
				{
					search_pos = 0;
					search_size = tspi->cur_q_pos;
				}	
				TSDEMUX_MakeSTC((unsigned char *)tsif_handle.rx_dma.v_addr + search_pos*TSIF_PACKET_SIZE, search_size*TSIF_PACKET_SIZE, guiPCRPID);				
			}			
            wake_up(&(tpri->wait_q));
        }
    }
    return IRQ_HANDLED;
}

static int tsif_get_readable_cnt(tca_spi_handle_t *H)
{
    if (H) {
        int dma_pos = H->cur_q_pos;
        int q_pos = H->q_pos;
        int readable_cnt = 0;

        if (dma_pos > q_pos) {
            readable_cnt = dma_pos - q_pos;
        } else if (dma_pos < q_pos) {
            readable_cnt = H->dma_total_packet_cnt - q_pos;
            readable_cnt += dma_pos;
        } 
        return readable_cnt;
    }
    return 0;
}

static ssize_t tcc_tsif_read(struct file *filp, char *buf, size_t len, loff_t *ppos)
{
    int readable_cnt = 0, copy_cnt = 0;
    int copy_byte = 0;

    readable_cnt = tsif_get_readable_cnt(&tsif_handle);
    if (readable_cnt > 0) {
        copy_byte = readable_cnt * TSIF_PACKET_SIZE;
        if (copy_byte > len) {
            copy_byte = len;
        }

        copy_byte -= copy_byte % TSIF_PACKET_SIZE;
        copy_cnt = copy_byte / TSIF_PACKET_SIZE;
        copy_cnt -= copy_cnt % tsif_handle.dma_intr_packet_cnt;
        copy_byte = copy_cnt * TSIF_PACKET_SIZE;

        if (copy_cnt >= tsif_handle.dma_intr_packet_cnt) {
            int offset = tsif_handle.q_pos * TSIF_PACKET_SIZE;
            if (copy_cnt > tsif_handle.dma_total_packet_cnt - tsif_handle.q_pos) {
                int first_copy_byte = (tsif_handle.dma_total_packet_cnt - tsif_handle.q_pos) * TSIF_PACKET_SIZE;
                int first_copy_cnt = first_copy_byte / TSIF_PACKET_SIZE;
                int second_copy_byte = (copy_cnt - first_copy_cnt) * TSIF_PACKET_SIZE;

                if (copy_to_user(buf, tsif_handle.rx_dma.v_addr + offset, first_copy_byte)) {
                    return -EFAULT;
                }
                if (copy_to_user(buf + first_copy_byte, tsif_handle.rx_dma.v_addr, second_copy_byte)) {
                    return -EFAULT;
                }

                tsif_handle.q_pos = copy_cnt - first_copy_cnt;
            } else {
                if (copy_to_user(buf, tsif_handle.rx_dma.v_addr + offset, copy_byte)) {
                    return -EFAULT;
                }

                tsif_handle.q_pos += copy_cnt;
                if (tsif_handle.q_pos >= tsif_handle.dma_total_packet_cnt) {
                    tsif_handle.q_pos = 0;
                }
            }
            return copy_byte;
        }
    }
    return 0;
}

static ssize_t tcc_tsif_write(struct file *filp, const char *buf, size_t len, loff_t *ppos)
{
    return 0;
}

static unsigned int tcc_tsif_poll(struct file *filp, struct poll_table_struct *wait)
{
    if (tsif_get_readable_cnt(&tsif_handle) >= tsif_handle.dma_intr_packet_cnt) {
		return  (POLLIN | POLLRDNORM);
    }

    poll_wait(filp, &(tsif_pri.wait_q), wait);
    if (tsif_get_readable_cnt(&tsif_handle) >= tsif_handle.dma_intr_packet_cnt) {
		return  (POLLIN | POLLRDNORM);
    }
    return 0;
}

static int tcc_tsif_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg)
{
    int ret = 0;
    switch (cmd) {
    case IOCTL_TSIF_DMA_START:
        {
            struct tcc_tsif_param param;
            if (copy_from_user(&param, (void *)arg, sizeof(struct tcc_tsif_param))) {
                printk("cannot copy from user tcc_tsif_param in IOCTL_TSIF_DMA_START !!! \n");
                return -EFAULT;
            }

            if (((TSIF_PACKET_SIZE * param.ts_total_packet_cnt) > tsif_handle.dma_total_size)
                || (param.ts_total_packet_cnt <= 0)) {
                printk("so big ts_total_packet_cnt !!! \n");
                return -EFAULT;
            }

            tsif_handle.dma_stop(&tsif_handle);

            tca_spi_setCPHA(tsif_handle.regs, param.mode & SPI_CPHA);
            tca_spi_setCPOL(tsif_handle.regs, param.mode & SPI_CPOL);
            tca_spi_setCS_HIGH(tsif_handle.regs, param.mode & SPI_CS_HIGH);
            tca_spi_setLSB_FIRST(tsif_handle.regs, param.mode & SPI_LSB_FIRST);

			tsif_handle.dma_mode = param.dma_mode;
			if (tsif_handle.dma_mode == 0) {
				tsif_handle.set_mpegts_pidmode(&tsif_handle, 0);
			}

            tsif_handle.dma_total_packet_cnt = param.ts_total_packet_cnt;
            tsif_handle.dma_intr_packet_cnt = param.ts_intr_packet_cnt;

            tsif_handle.clear_fifo_packet(&tsif_handle);
            tsif_handle.q_pos = tsif_handle.cur_q_pos = 0;

            tsif_handle.set_packet_cnt(&tsif_handle, MPEG_PACKET_SIZE);
            printk("interrupt packet count [%u]\n", tsif_handle.dma_intr_packet_cnt);
            tsif_handle.dma_start(&tsif_handle);
        }
        break;
		
    case IOCTL_TSIF_DMA_STOP:
            tsif_handle.dma_stop(&tsif_handle);
        break;
		
    case IOCTL_TSIF_GET_MAX_DMA_SIZE:
        {
            struct tcc_tsif_param param;

            param.ts_total_packet_cnt = tsif_handle.dma_total_size / TSIF_PACKET_SIZE;
            param.ts_intr_packet_cnt = 1;

            if (copy_to_user((void *)arg, (void *)&param, sizeof(struct tcc_tsif_param))) {
                printk("cannot copy to user tcc_tsif_param in IOCTL_TSIF_GET_MAX_DMA_SIZE !!! \n");
                return -EFAULT;
            }
        }
        break;
		
    case IOCTL_TSIF_SET_PID:
        {
            struct tcc_tsif_pid_param param;
            if (copy_from_user(&param, (void *)arg, sizeof(struct tcc_tsif_pid_param))) {
                printk("cannot copy from user tcc_tsif_pid_param in IOCTL_TSIF_SET_PID !!! \n");
                return -EFAULT;
            }

            if (param.valid_data_cnt <= PID_MATCH_TABLE_MAX_CNT) {
				volatile unsigned long* PIDT;

                int i = 0;
                for (i = 0; i < PID_MATCH_TABLE_MAX_CNT; i++) {
					PIDT = (volatile unsigned long *)tcc_p2v(HwGPSB_PIDT(i));
					*PIDT = 0;

                    //HwGPSB_PIDT(i) = 0;
                }
                if (param.valid_data_cnt > 0) {
                    for (i = 0; i < param.valid_data_cnt; i++) {
						PIDT = (volatile unsigned long *)tcc_p2v(HwGPSB_PIDT(i));
                        *PIDT = (param.pid_data)[i] & 0x1FFFFFFF;
                        BITSET(*PIDT, (TSIF_SPI_NUM == 0) ? HwGPSB_PIDT_CH0 : HwGPSB_PIDT_CH1);
					    printk("PIDT 0x%08X : 0x%08X\n", PIDT, *PIDT);

                        //HwGPSB_PIDT(i) = (param.pid_data)[i] & 0x1FFFFFFF;
                        //BITSET(HwGPSB_PIDT(i), (TSIF_SPI_NUM == 0) ? HwGPSB_PIDT_CH0 : HwGPSB_PIDT_CH1);
                    }
                    tsif_handle.set_mpegts_pidmode(&tsif_handle, 1);
                } else {
                    tsif_handle.set_mpegts_pidmode(&tsif_handle, 0);
                }
            } else {
                printk("tsif: PID TABLE is so big !!! (0x%X)\n", cmd);
                ret = -EINVAL;
            }
        }
        break;
		
	case IOCTL_TSIF_DXB_POWER:
		{
			unsigned long flag;
			if (copy_from_user((void *)&flag, (const void *)arg, sizeof(flag))) {
				return -EFAULT;
			}
			if (tcc_dxb_power(flag)) {
				return -EINVAL;
			}
		}
		break;
	case IOCTL_TSIF_SET_PCRPID:		
		if (copy_from_user((void *)&guiPCRPID, (const void *)arg, sizeof(int))) {
			return -EFAULT;
		}		
		printk("Set PCR PID[0x%X]\n", guiPCRPID);
		if( guiPCRPID < 0x1FFF)
			TSDEMUX_Open();
		break;
	case IOCTL_TSIF_GET_STC:	
		{
			unsigned int uiSTC;
			uiSTC = TSDEMUX_GetSTC();
			//printk("STC %d\n", uiSTC);
			if (copy_to_user((void *)arg, (void *)&uiSTC, sizeof(int))) {
                printk("cannot copy to user tcc_tsif_param in IOCTL_TSIF_GET_PCR !!! \n");
                return -EFAULT;
            }
		}
		break;		
    default:
        printk("tsif: unrecognized ioctl (0x%X)\n", cmd);
        ret = -EINVAL;
        break;
    }
    return ret;
}

static int tcc_tsif_init(int tsif_num)
{
	int ret = 0;
    volatile struct tca_spi_regs *reg_addr = NULL;
	int port;

	//printk("%s: spi%d open\n", __func__, tsif_num);
    reg_addr = (volatile struct tca_spi_regs *)((TSIF_SPI_NUM == 0)
												? &(HwGPSBCH0_BASE) : &(HwGPSBCH1_BASE));

    memset(&tsif_handle, 0, sizeof(tca_spi_handle_t));
    //memset(&tsif_pri, 0, sizeof(struct tca_spi_pri_handle));

	/* GPIO_EXPAND DXB Power-on */
	//tcc_dxb_power(1);

	// reset
	tca_ckc_set_iobus_swreset(tsif_num == 0 ? RB_GPSBCONTROLLER0 : RB_GPSBCONTROLLER1, OFF);
	tca_ckc_set_iobus_swreset(tsif_num == 0 ? RB_GPSBCONTROLLER0 : RB_GPSBCONTROLLER1, ON);

    tca_ckc_setiobus(tsif_num == 0 ? RB_GPSBCONTROLLER0 : RB_GPSBCONTROLLER1, ENABLE);

	/* GPIO port */
	port = tsif_pri.gpio_port;

	if (tca_spi_init(&tsif_handle,
                     reg_addr,
                     (TSIF_SPI_NUM == 0) ? INT_GPSB0_DMA : INT_GPSB1_DMA,
                     tea_alloc_dma_linux,
                     tea_free_dma_linux,
                     SPI_DMA_SIZE,
                     TSIF_SPI_NUM,
                     1,
					 port)) {
        printk("%s: tca_spi_init error !!!!!\n", __func__);
		ret = -EBUSY;
		goto err_spi;
    }

	tsif_handle.private_data = (void *)&tsif_pri;
    tsif_handle.clear_fifo_packet(&tsif_handle);
    tsif_handle.dma_stop(&tsif_handle);

    //init_waitqueue_head(&(tsif_pri.wait_q));
    //mutex_init(&(tsif_pri.mutex));

    tsif_handle.dma_total_packet_cnt = tsif_handle.dma_total_size / TSIF_PACKET_SIZE;
    tsif_handle.dma_intr_packet_cnt = 1;

    tsif_handle.hw_init(&tsif_handle);
	
    ret = request_irq(tsif_handle.irq, tcc_tsif_dma_handler, IRQF_SHARED, TSIF_DEV_NAME, &tsif_handle);
	if (ret) { 
		goto err_irq; 
	}

    tsif_handle.set_packet_cnt(&tsif_handle, MPEG_PACKET_SIZE);

	return 0;

err_irq:
	free_irq(tsif_handle.irq, &tsif_handle);
	
err_spi:
	tca_spi_clean(&tsif_handle);
	tca_ckc_setiobus(tsif_num == 0 ? RB_GPSBCONTROLLER0 : RB_GPSBCONTROLLER1, DISABLE);
	/* GPIO_EXPAND DXB Power-off */
	//tcc_dxb_power(0);
	
	return ret;
}

static void tcc_tsif_deinit(int tsif_num)
{
	free_irq(tsif_handle.irq, &tsif_handle);
    tca_spi_clean(&tsif_handle);
    tca_ckc_setiobus(tsif_num == 0 ? RB_GPSBCONTROLLER0 : RB_GPSBCONTROLLER1, DISABLE);
	/* GPIO_EXPAND DXB Power-off */
	//tcc_dxb_power(0);
}

static int tcc_tsif_open(struct inode *inode, struct file *filp)
{
    int ret = 0;
 
    clk_enable(gpsb_clk);

    mutex_lock(&(tsif_pri.mutex));
	guiPCRPID = 0xFFFF;
	//TSDEMUX_Open();
	
    if (tsif_pri.open_cnt == 0) {
        tsif_pri.open_cnt++;
/* jhlim
		ret = tcc_tsif_init(TSIF_SPI_NUM);
		if (ret == 0) {
			tsif_handle.set_mpegts_pidmode(&tsif_handle, 0);
		}
*/
    } else {
        ret = -EBUSY;
    }
    mutex_unlock(&(tsif_pri.mutex));

// jhlim
    tsif_handle.set_mpegts_pidmode(&tsif_handle, 0);	
	
    return ret;
}

static int tcc_tsif_release(struct inode *inode, struct file *filp)
{
    mutex_lock(&(tsif_pri.mutex));
    tsif_handle.dma_stop(&tsif_handle);
	TSDEMUX_Close();
    if (tsif_pri.open_cnt > 0) {
        tsif_pri.open_cnt--;
/* jhlim
		tcc_tsif_deinit(TSIF_SPI_NUM);
*/
    }	
    mutex_unlock(&(tsif_pri.mutex));

    clk_disable(gpsb_clk);

    return 0;
}

struct file_operations tcc_tsif_fops =
{
    .owner      = THIS_MODULE,
    .read       = tcc_tsif_read,
    .write      = tcc_tsif_write,
    .ioctl      = tcc_tsif_ioctl,
    .open       = tcc_tsif_open,
    .release    = tcc_tsif_release,
    .poll       = tcc_tsif_poll,
};

static struct class *tsif_class; //jhlim

static int __init tsif_init(void)
{
    int ret;

    printk("tcc_tsif: spi%d init (svn%s)\n", TSIF_SPI_NUM, UTS_VERSION);

    if( 0 == TSIF_SPI_NUM )
        gpsb_clk = clk_get(NULL, "gpsb0");
    else
        gpsb_clk = clk_get(NULL, "gpsb1");
    if(gpsb_clk == NULL)
    {
        printk("gpsb clock error on tsif: cannot get clock\n");
        return -EINVAL;
    }

    memset(&tsif_pri, 0, sizeof(struct tca_spi_pri_handle));
    ret = register_chrdev(TSIF_DEV_MAJOR, TSIF_DEV_NAME, &tcc_tsif_fops);
    if (ret < 0) {
        printk("[%s:%d] register_chrdev error !!!!!\n", __func__, __LINE__); 
        return ret;
    }

    init_waitqueue_head(&(tsif_pri.wait_q));
    mutex_init(&(tsif_pri.mutex));

    ret = platform_driver_probe(&tsif_platform_driver, tsif_drv_probe);

// jhlim
    tcc_tsif_init(TSIF_SPI_NUM);

    tsif_class = class_create(THIS_MODULE, TSIF_DEV_NAME);
    device_create(tsif_class, NULL, MKDEV(TSIF_DEV_MAJOR, 1), NULL, TSIF_DEV_NAME);
// jhlim
	guiPCRPID = 0xFFFF;
    return ret;
}

static void __exit tsif_exit(void)
{
// jhlim
    tcc_tsif_deinit(TSIF_SPI_NUM);

    unregister_chrdev(TSIF_DEV_MAJOR, TSIF_DEV_NAME);
    platform_driver_unregister(&tsif_platform_driver);

    clk_put(gpsb_clk);
}
module_init(tsif_init);
module_exit(tsif_exit);

MODULE_AUTHOR("Telechips Inc. linux@telechips.com");
MODULE_DESCRIPTION("Telechips GPSB Slave (TSIF) driver");
MODULE_LICENSE("GPL");
