/*
 *
 */
#ifndef __TCC_USB_PHY_H__
#define __TCC_USB_PHY_H__

#include "tcc_usb_def.h"
#include "dwc_otg_cil.h"
#include "dwc_otg_core_if.h"

typedef enum {
	USBPHY_MODE_RESET,
	USBPHY_MODE_HOST,
	USBPHY_MODE_DEVICE,
	USBPHY_MODE_ON,
	USBPHY_MODE_OFF,
	USBPHY_MODE_DEVICE_DETACH
} USBPHY_MODE_T;

/*
 * PWR_GP1's status value for USB OHCI & OTG
 *
 *  b 0 0 0 0  0 0 0 0
 *        a b      c d
 *  - a : USB_OTG exist (Hw5)
 *  - b : USB_OHCI exist (Hw4)
 *  - c : USB_OTG status (Hw1)
 *  - d : USB_OHCI status (Hw0)
 */
extern unsigned int PWR_GP1_STATUS;
#define PWR_GP1_OTG_EXIST	Hw5
#define PWR_GP1_OHCI_EXIST	Hw4
#define PWR_GP1_OTG_STATUS	Hw1
#define PWR_GP1_OHCI_STATUS	Hw0

extern const unsigned char * USBPHY_GetVersion(void);
extern void USBPHY_EnableClock(void);
extern USBPHY_MODE_T USBPHY_SetMode(dwc_otg_core_if_t *_core_if, USBPHY_MODE_T mode);
void USBPHY_DEVICE_VBUS(unsigned int valid);
#if defined(CONFIG_MACH_TCC8900) || defined(CONFIG_MACH_TCC9200S) || defined(CONFIG_MACH_TCC9201)
extern void USBPHY_SetID(unsigned int ID);
#endif

#if defined(CONFIG_MACH_TCC9300) || defined(CONFIG_MACH_TCC9300CM) || defined(CONFIG_MACH_TCC9300ST)
extern void USBPHY_PWR_Control(int port_index, int on);
#endif

extern void TCC_DVBUS_Control(dwc_otg_core_if_t *_core_if, int on);
extern void OTGCORE_Init(dwc_otg_core_if_t *_core_if);

#endif /*__TCC_USB_PHY_H__*/
