/*
 * linux/drivers/usb/gadget/tcc_udc.h
 *
 * Author:  <linux@telechips.com>
 * Created: 10th Jun, 2008 
 * Description: Intel TCC83x on-chip full speed USB device controller
 *
 * Copyright (C) 2008 Telechips Inc.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __LINUX_USB_GADGET_TCC83X_H
#define __LINUX_USB_GADGET_TCC83X_H

#include <linux/types.h>
//#include <linux/module.h>
//#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/io.h>
#include <linux/device.h>
#include <linux/list.h>
#include <linux/platform_device.h>
#include <linux/usb/ch9.h>
#include <linux/usb/gadget.h>

#define ALIGN_32	__aligned(8)

#define N_ENDPOINT	4

// EP0 Buffer Register
#if defined(CONFIG_ARCH_TCC79X)
#define	HwEP0BUF_PHY	*(volatile unsigned long *)0xF0010060
#elif defined(CONFIG_ARCH_TCC83X)
#define	HwEP0BUF_PHY	*(volatile unsigned long *)0x80008060
#endif

//#define DEBUG

#ifdef DEBUG
#define DBG_ERR		0x0001	/* report all error returns */
#define DBG_HW		0x0002	/* debug hardware initialization */
#define DBG_GADGET	0x0004	/* calls to/from gadget driver */
#define DBG_INT		0x0008	/* interrupts */
#define DBG_BUS		0x0010	/* report changes in bus state */
#define DBG_QUEUE	0x0020  /* debug request queue processing */
#define DBG_FIFO	0x0040  /* debug FIFO contents */
#define DBG_DMA		0x0080  /* debug DMA handling */
#define DBG_REQ		0x0100	/* print out queued request length */
//#define DBG_ALL		0xffff
//#define DBG_NONE	0x0000

enum {
	DBG_ALL,
	DBG_DEBUG,
	DBG_INFO,
	DBG_ERROR,
	DBG_NONE
};

#define DEBUG_LEVEL	(DBG_ALL)
#define DBG(level, fmt, ...)					\
	if ((level) >= DEBUG_LEVEL) {			\
		if( level == DBG_DEBUG ) \
			printk("\n" KERN_DEBUG "tcc-udc:[DEBUG] " fmt, ## __VA_ARGS__); \
		else if( level == DBG_INFO ) \
			printk("\n" KERN_DEBUG "tcc-udc:[INFO] " fmt, ## __VA_ARGS__); \
		else if( level == DBG_ERROR ) \
			printk("\n" KERN_DEBUG "tcc-udc:[ERROR] " fmt, ## __VA_ARGS__); \
		else \
			printk("\n" KERN_DEBUG "tcc-udc:[ETC] " fmt, ## __VA_ARGS__); \
	}
#else
#define DBG(level, fmt...)
#endif

struct usbt_ep_t {
	//int							state;
	//void __iomem				*ep_regs;
	//void __iomem				*dma_regs;
	//void __iomem				*fifo;
	struct usb_ep				ep;
	struct usbt_udc_t			*udc;
	
	struct list_head			queue;
	const struct usb_endpoint_descriptor	*desc;

	u16							fifo_size;
	//u8						nr_banks;
	u8							index;
	//unsigned int				can_dma:1;
	//unsigned int				can_isoc:1;
	//unsigned int				is_isoc:1;
	unsigned int				is_in:1;
	unsigned int				is_working:1;

//#ifdef CONFIG_USB_GADGET_DEBUG_FS
//	u32					last_dma_status;
//	struct dentry				*debugfs_dir;
//	struct dentry				*debugfs_queue;
//	struct dentry				*debugfs_dma_status;
//	struct dentry				*debugfs_state;
//#endif
};

struct usbt_udc_t {
	/* Protect hw registers from concurrent modifications */
	spinlock_t lock;

	//void __iomem *regs;
	//void __iomem *fifo;

	struct usb_gadget gadget;
	struct usb_gadget_driver *driver;
	struct platform_device *pdev;
	int irq;

	int dma_irq;
	struct usbt_ep_t *dmaisr_ep;
	u32 dmaisr_length;
	//int vbus_pin;
	//struct clk *pclk;
	//struct clk *hclk;

	u16 devstatus;
	//
	//u16 test_mode;
	int vbus_prev;

	struct task_struct	*thread_task;

	void __iomem *dtcm;		// dtcm iomapped address
	u32 dtcm_pa;			// dtcm physical address

//#ifdef CONFIG_USB_GADGET_DEBUG_FS
//	struct dentry *debugfs_root;
//	struct dentry *debugfs_regs;
//#endif
};

struct usbt_request_t {
	struct usb_request			req;
	struct list_head			queue;

	//u32					ctrl;
	//
	//unsigned int				submitted:1;
	//unsigned int				last_transaction:1;
	//unsigned int				using_dma:1;
	//unsigned int				mapped:1;
};

static inline struct usbt_ep_t *to_usbt_ep(struct usb_ep *ep)
{
	return container_of(ep, struct usbt_ep_t, ep);
}

static inline struct usbt_request_t *to_usbt_req(struct usb_request *req)
{
	return container_of(req, struct usbt_request_t, req);
}

static inline u16 *usbt_get_epbuf(u8 index)
{
	return (u16*)( (u32)&HwEP0BUF + ((u32)index<<2) );
}

static inline u16 *usbt_get_epbuf_physical(u8 index)
{
	return (u16*)( (u32)&HwEP0BUF_PHY + ((u32)index<<2) );
}



#endif /* __TCC_UDC_H__ */
