
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <linux/clk.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/div64.h>
#include <asm/mach/map.h>
#ifdef CONFIG_PM
#include <linux/pm.h>
#endif

#include <mach/bsp.h>
#include <mach/tca_ckc.h>

#ifdef CONFIG_FB_TCC92XX_QVGA_HTD240000D
#include "TCC_LCD_Interface.h"
#include "TCC_LCD_DriverCtrl.h"

#include "tca_lcdc.h"

#define HTN240000D_CMD_WRITE(addr) 	DEV_LCDSI_DATA(LCDSI_CS00, addr)

#define HTN240000D_DATA_WRITE(data)	DEV_LCDSI_DATA(LCDSI_CS01, data)


void HTN240000D_CMDDATA_WRITE(addr, data)
{
	DEV_LCDSI_DATA(LCDSI_CS00, addr);
	DEV_LCDSI_DATA(LCDSI_CS01, data);

}

typedef struct 
{
    unsigned short Add;
    unsigned short Data;
} LCD_CONTROL_DATA;

const LCD_CONTROL_DATA LCD_CSTN_ROT0L[] = {
	0x0003, 	0x1080,
};

const LCD_CONTROL_DATA LCD_CSTN_ROT90L[] = {
	0x0003, 	0x1098,
};
const LCD_CONTROL_DATA LCD_CSTN_ROT180L[] = {
	0x0003, 	0x10b0,
};

const LCD_CONTROL_DATA LCD_CSTN_ROT270L[] = {
	0x0003,		0x10a8,
};

const LCD_CONTROL_DATA LCD_CSTN_I_ROT0L[] = {
	0x0003,		0x10a0,
};

const LCD_CONTROL_DATA LCD_CSTN_I_ROT90L[] = {
	0x0003,		0x10b8,
};
const LCD_CONTROL_DATA LCD_CSTN_I_ROT180L[] = {
	0x0003,		0x1090,
};

const LCD_CONTROL_DATA LCD_CSTN_I_ROT270L[] = {
	0x0003,		0x1088,
};

void HTN240000D_initialize(void)
{

	// Oscillation Start
	HTN240000D_CMDDATA_WRITE(0x00A4, 0x0001);
	LCD_OS_DELAY(20);

// Display Sequence Control
	HTN240000D_CMDDATA_WRITE(0x0007, 0x0021);
	LCD_OS_DELAY(2);

	HTN240000D_CMDDATA_WRITE(0x0017, 0x0001);
	LCD_OS_DELAY(2);

// Power Control(1)
	HTN240000D_CMDDATA_WRITE(0x0010, 0x17B0);
	HTN240000D_CMDDATA_WRITE(0x0011, 0x0017);
	HTN240000D_CMDDATA_WRITE(0x0012, 0x019E);
	HTN240000D_CMDDATA_WRITE(0x0013, 0x1700);
	HTN240000D_CMDDATA_WRITE(0x0014, 0x8000);
	LCD_OS_DELAY(2);

// Power Control(2)
	HTN240000D_CMDDATA_WRITE(0x0012, 0x01BE);
	LCD_OS_DELAY(24);

// Driver Output Control
	HTN240000D_CMDDATA_WRITE(0x0001, 0x0500);

// LCD Driving Wave Control
	HTN240000D_CMDDATA_WRITE(0x0002, 0x0400);

// Entry Mode
//	HTN240000D_CMDDATA_WRITE(0x0003, 0x10A8);
	HTN240000D_CMDDATA_WRITE(LCD_CSTN_ROT0L[0].Add, LCD_CSTN_ROT0L[0].Data);	

	HTN240000D_CMDDATA_WRITE(0x0004, 0x0000);

// Display Control(4)
	HTN240000D_CMDDATA_WRITE(0x0008, 0x0808);

// ISC Control
	HTN240000D_CMDDATA_WRITE(0x0009, 0x0000);

// Display Control(4)
	HTN240000D_CMDDATA_WRITE(0x000A, 0x0008);
	HTN240000D_CMDDATA_WRITE(0x000C, 0x0000);
	HTN240000D_CMDDATA_WRITE(0x000D, 0x0000);

// Gamma Setting
	HTN240000D_CMDDATA_WRITE(0x0030, 0x0703);
	HTN240000D_CMDDATA_WRITE(0x0031, 0x0001);
	HTN240000D_CMDDATA_WRITE(0x0032, 0x0104);
	HTN240000D_CMDDATA_WRITE(0x0033, 0x0102);
	HTN240000D_CMDDATA_WRITE(0x0034, 0x0300);
	HTN240000D_CMDDATA_WRITE(0x0035, 0x0103);
	HTN240000D_CMDDATA_WRITE(0x0036, 0x001F);
	HTN240000D_CMDDATA_WRITE(0x0037, 0x0703);
	HTN240000D_CMDDATA_WRITE(0x0038, 0x0001);
	HTN240000D_CMDDATA_WRITE(0x0039, 0x0104);
	HTN240000D_CMDDATA_WRITE(0x003A, 0x0102);
	HTN240000D_CMDDATA_WRITE(0x003B, 0x0300);
	HTN240000D_CMDDATA_WRITE(0x003C, 0x0103);
	HTN240000D_CMDDATA_WRITE(0x003D, 0x001F);

// Horizontal RAM address Start
	HTN240000D_CMDDATA_WRITE(0x0050, 	0x0000);

// Horizontal RAM address End
	HTN240000D_CMDDATA_WRITE(0x0051, 	0x00EF);

// Vertical RAM address Start
	HTN240000D_CMDDATA_WRITE(0x0052, 	0x0000);

// Vertical RAM address End
	HTN240000D_CMDDATA_WRITE(0x0053, 	0x013F);

	HTN240000D_CMDDATA_WRITE(0x0060, 	0x2700);
	HTN240000D_CMDDATA_WRITE(0x0061, 	0x0000);
	HTN240000D_CMDDATA_WRITE(0x006A, 	0x0000);

// Frame Cycle Control 1
	HTN240000D_CMDDATA_WRITE(0x0090, 	0x0011);
	HTN240000D_CMDDATA_WRITE(0x0092, 	0x0000);
	HTN240000D_CMDDATA_WRITE(0x0093, 	0x0002);
	
	HTN240000D_CMDDATA_WRITE(0x0020, 	0x0000);
	HTN240000D_CMDDATA_WRITE(0x0021, 	0x0000);

// Display ON
	HTN240000D_CMDDATA_WRITE(0x0007, 	0x0021);
	LCD_OS_DELAY(4);
	HTN240000D_CMDDATA_WRITE(0x0010, 	0x18B0);
	HTN240000D_CMDDATA_WRITE(0x0011, 	0x0017);
	HTN240000D_CMDDATA_WRITE(0x0007, 	0x0061);
	LCD_OS_DELAY(4);
	HTN240000D_CMDDATA_WRITE(0x0007, 	0x0173);
	LCD_OS_DELAY(4);

	HTN240000D_CMDDATA_WRITE(0x0020, 	0x0000);
	HTN240000D_CMDDATA_WRITE(0x0021, 	0x0000);
	HTN240000D_CMD_WRITE(0x0022);

}



 
 // Unit 100hz
#define LCDC_CLK		420000
#define LCDSI_CLK		460000
 
 
  // LCDSI Output Signals Timing Configuration
#define LCDSI_TIME_STP	2	// setup time	(nsec)
#define LCDSI_TIME_PW	8	// plus width	(nsec)
#define LCDSI_TIME_HLD	2	// hold time	(nsec)
 
#define LCD_BUS_WIDTH	16	
#define LCD_DIVIDER		2
 
 
#define LPW		0	// line pulse width
#define LPC		240 // line pulse count (active horizontal pixel - 1)
#define LSWC	9	// line start wait clock (the number of dummy pixel clock - 1)
#define LEWC	6	// line end wait clock (the number of dummy pixel clock - 1)
#define VDB		0	// Back porch Vsync delay
#define VDF		0 	// front porch of Vsync delay
 
#define FPW1	0	// TFT/TV : Frame pulse width is the pulse width of frmae clock
#define FLC1	320	// frmae line count is the number of lines in each frmae on the screen
#define FSWC1	4	// frmae start wait cycle is the number of lines to insert at the end each frame
#define FEWC1	2	// frame start wait cycle is the number of lines to insert at the begining each frame
#define FPW2	0	// TFT/TV : Frame pulse width is the pulse width of frmae clock
#define FLC2	320	// frmae line count is the number of lines in each frmae on the screen
#define FSWC2	4	// frmae start wait cycle is the number of lines to insert at the end each frame
#define FEWC2 	2	// frame start wait cycle is the number of lines to insert at the begining each frame
 
 
 //LCD Driver time ns unit 
#define SETUP_T			10// setup time
#define PLUS_W			40// plus width
#define HOLD_T			10// hold time
 
#define SYNC_INVET (ID_INVERT_EN|IV_INVERT_EN|IH_INVERT_EN|IP_INVERT_EN)
	
void DEV_LCD_MODULE_SPEC(LCD_MODULE_SPEC_TYPE* spec)
{
	spec->lcdc_clk = LCDC_CLK;
	spec->lcdc_clk_div = LCD_DIVIDER;
	spec->lcd_bus_width = LCD_BUS_WIDTH;
	
	spec->lcd_width = 240;
	spec->lcd_height = 320;
	spec->lcd_LPW = LPW;		// line pulse width
	spec->lcd_LPC = LPC;		// line pulse count (active horizontal pixel - 1)
	spec->lcd_LSWC = LSWC;		// line start wait clock (the number of dummy pixel clock - 1)
	spec->lcd_LEWC = LEWC;		// line end wait clock (the number of dummy pixel clock - 1)
	spec->lcd_VDB = VDB;		// Back porch Vsync delay
	spec->lcd_VDF = VDF;		// front porch of Vsync delay

	spec->lcd_FPW1 = FPW1;		// TFT/TV : Frame pulse width is the pulse width of frmae clock
	spec->lcd_FLC1 = FLC1;		// frmae line count is the number of lines in each frmae on the screen
	spec->lcd_FSWC1 = FSWC1;		// frmae start wait cycle is the number of lines to insert at the end each frame
	spec->lcd_FEWC1 = FEWC1;		// frame start wait cycle is the number of lines to insert at the begining each frame

	spec->lcd_FPW2 = FPW2;		// TFT/TV : Frame pulse width is the pulse width of frmae clock
	spec->lcd_FLC2 = FLC2;		// frmae line count is the number of lines in each frmae on the screen
	spec->lcd_FSWC2 = FSWC2;		// frmae start wait cycle is the number of lines to insert at the end each frame
	spec->lcd_FEWC2 = FEWC2;		// frame start wait cycle is the number of lines to insert at the begining each frame
	spec->SyncSignal_Invert = SYNC_INVET;
}


#define LCD_LC	TCC_LCDC1

 void DEV_LCD_Module (LCD_MODULE_CTRL uiControl)
{
	unsigned int	uiMode;


//	DEV_LCDSI_Time(LCDSI_CLK, LCDSI_TIME_STP, LCDSI_TIME_PW, LCDSI_TIME_HLD);

	uiMode = DEV_LCDSI_CTRL(LCD_LC, SET_LCDSI_ALL, IF80, CPU_MODE, FMT565_16BIT, LCDSI_RS_HIGH, LCDSI_CS0, 1, 0);
	DEV_LCDSI_BW_Set(32);

	switch(uiControl)
	{
		case LCD_M_INITIALIZE:
			HTN240000D_initialize();
			break;
		case LCD_M_ON:
			HTN240000D_initialize();
			break;
		case LCD_M_OFF:

			break;
		default:
			break;
	}

	DEV_LCDSI_BW_Set(16);

	uiMode = DEV_LCDSI_CTRL(LCD_LC, SET_LCDSI_INPUTMODE, IF80, LCDC_MODE, FMT565_16BIT_SWAP, LCDSI_RS_HIGH, LCDSI_CS0, 1, 0);

	}
#endif//
