#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <linux/clk.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/div64.h>
#include <asm/mach/map.h>
#ifdef CONFIG_PM
#include <linux/pm.h>
#endif

#include <mach/bsp.h>
#include <mach/tca_ckc.h>


#ifdef CONFIG_FB_TCC92XX_WVGA_LMS480KF
#include <mach/TCC_LCD_DriverCtrl.h>
#include <mach/TCC_LCD_Interface.h>

PGPION pGPIO_A;
PGPION pGPIO_B;
PGPION pGPIO_C;
PGPION pGPIO_D;
PGPION pGPIO_E;



#if 0
#define dprintk(msg...)	 { printk( "LMS480KF: " msg); }
#else
#define dprintk(msg...)	 
#endif

// Unit 100hz
#define LCDC_CLK		(230000)
#define LCD_DIVIDER		2

#define LCD_BUS_WIDTH	24

#define LPW		2	// line pulse width
#define LPC		800	// line pulse count (active horizontal pixel - 1)
#define LSWC	12	// line start wait clock (the number of dummy pixel clock - 1)
#define LEWC	7	// line end wait clock (the number of dummy pixel clock - 1)
#define VDB		0	// Back porch Vsync delay
#define VDF		0 	// front porch of Vsync delay

#define FPW1	0	// TFT/TV : Frame pulse width is the pulse width of frmae clock
#define FLC1	480	// frmae line count is the number of lines in each frmae on the screen
#define FSWC1	6	// frmae start wait cycle is the number of lines to insert at the end each frame
#define FEWC1	4	// frame start wait cycle is the number of lines to insert at the begining each frame
#define FPW2	0	// TFT/TV : Frame pulse width is the pulse width of frmae clock
#define FLC2	480	// frmae line count is the number of lines in each frmae on the screen
#define FSWC2	6	// frmae start wait cycle is the number of lines to insert at the end each frame
#define FEWC2 	4	// frame start wait cycle is the number of lines to insert at the begining each frame

#define SYNC_INVET (IV_INVERT_EN |IH_INVERT_EN)
	


#ifdef CONFIG_MACH_TCC9200S
//E3 : LCD_BL_EN, C29: LCD_RESET	C30:LCD_DISP  C31:LCD_PWRCTL
#define LCD_PROT_INT() 		BITCSET(pGPIO_C->GPEN, Hw29 | Hw30| Hw31, Hw29 | Hw30| Hw31)//;// 	\
							//BITCSET(pGPIO_C->GPDAT, Hw29 | Hw30| Hw31, 0)

#define LCD_POWER_ON()		BITCSET(pGPIO_C->GPDAT, Hw31, Hw31)
#define LCD_POWER_OFF()		BITCSET(pGPIO_C->GPDAT, Hw31, 0)
#define LCD_DISPLAYON()		BITCSET(pGPIO_C->GPDAT, Hw30, Hw30)
#define LCD_DISPLAYOFF()	BITCSET(pGPIO_C->GPDAT, Hw30, 0)

#define LCD_RESET_H()		BITCSET(pGPIO_C->GPDAT, Hw29, Hw29)
#define LCD_RESET_L()		BITCSET(pGPIO_C->GPDAT, Hw29, 0)

#elif defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900) 
#include <mach/tcc_pca953x.h>

//  A7 : LCD_bl_en ,  D22 : LCD_RST,   C28:LCD_DISP ,  extension: LCD_PWRCTL
#define LCD_PROT_INT() 		BITCSET(pGPIO_A->GPEN, Hw7, Hw7); 	\
							BITCSET(pGPIO_D->GPEN, Hw22, Hw22); \
							BITCSET(pGPIO_C->GPEN, Hw28 , Hw28); \
					 		BITCSET(pGPIO_A->GPDAT, Hw7, 0);  \
							BITCSET(pGPIO_D->GPDAT, Hw22, 0); \
							BITCSET(pGPIO_C->GPDAT, Hw28 , 0); 

#define LCD_POWER_ON()		tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, LCD, OUTPUT, ON, SET_DIRECTION|SET_VALUE)
#define LCD_POWER_OFF()		tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, LCD, OUTPUT, OFF, SET_DIRECTION|SET_VALUE)
#define LCD_DISPLAYON()		BITCSET(pGPIO_C->GPDAT, Hw28 ,Hw28)
#define LCD_DISPLAYOFF()	BITCSET(pGPIO_C->GPDAT, Hw28, 0)
#define LCD_RESET_H()		BITCSET(pGPIO_D->GPDAT, Hw22 ,Hw22)
#define LCD_RESET_L()		BITCSET(pGPIO_D->GPDAT, Hw22, 0)
#elif defined(CONFIG_MACH_TCC9300CM)
#include <linux/gpio.h>
#include <mach/gpio.h>

//  A7 : LCD_bl_en ,  D22 : LCD_RST,   C28:LCD_DISP ,  extension: LCD_PWRCTL
#define LCD_PROT_INT() 		BITCSET(pGPIO_A->GPEN, Hw7, Hw7); 	\
							BITCSET(pGPIO_D->GPEN, Hw22, Hw22); \
							BITCSET(pGPIO_C->GPEN, Hw28 , Hw28); \
					 		BITCSET(pGPIO_A->GPDAT, Hw7, 0);  \
							BITCSET(pGPIO_D->GPDAT, Hw22, 0); \
							BITCSET(pGPIO_C->GPDAT, Hw28 , 0); 

#define LCD_POWER_ON()		gpio_set_value(TCC_GPEXT3(1), 1)
#define LCD_POWER_OFF()		gpio_set_value(TCC_GPEXT3(1), 0)
#define LCD_DISPLAYON()		BITCSET(pGPIO_C->GPDAT, Hw28 ,Hw28)
#define LCD_DISPLAYOFF()	BITCSET(pGPIO_C->GPDAT, Hw28, 0)
#define LCD_RESET_H()		BITCSET(pGPIO_D->GPDAT, Hw22 ,Hw22)
#define LCD_RESET_L()		BITCSET(pGPIO_D->GPDAT, Hw22, 0)
#elif defined(CONFIG_MACH_TCC9300)
#include <linux/gpio.h>
#include <mach/gpio.h>
#define EXP_LCD_RESET		TCC_GPEXT3(12)
#define EXP_LCD_POWER		TCC_GPEXT1(0)

//E3 : LCD_BL_EN, C29: LCD_RESET	C30:LCD_DISP  C31:LCD_PWRCTL
#define LCD_PROT_INT() 		BITCSET(pGPIO_E->GPEN, Hw1, Hw1);	\
							gpio_direction_output(EXP_LCD_POWER, 1); \
							gpio_direction_output(EXP_LCD_RESET, 1)
							
#define LCD_POWER_ON()		gpio_set_value(EXP_LCD_POWER, 1)
#define LCD_POWER_OFF()		gpio_set_value(EXP_LCD_POWER, 0)
#define LCD_RESET_H()		gpio_set_value(EXP_LCD_RESET, 1)
#define LCD_RESET_L()		gpio_set_value(EXP_LCD_RESET, 0)

#define LCD_DISPLAYON()		BITCSET(pGPIO_E->GPDAT, Hw1, Hw1)
#define LCD_DISPLAYOFF()	BITCSET(pGPIO_E->GPDAT, Hw1, 0)
#elif defined(CONFIG_MACH_TCC9300ST)
#include <linux/gpio.h>
#include <mach/gpio.h>

#define EXP_LCD_RESET           //TCC_GPEXT1(2)
#define EXP_LCD_POWER           //TCC_GPEXT3(12)

//E3 : LCD_BL_EN, C29: LCD_RESET        D7:LCD_DISP  C31:LCD_PWRCTL
#define LCD_PROT_INT()          BITCSET(pGPIO_D->GPEN, Hw7, Hw7);


#define LCD_POWER_ON()          //gpio_set_value(EXP_LCD_POWER, 1)
#define LCD_POWER_OFF()         //gpio_set_value(EXP_LCD_POWER, 0)
#define LCD_RESET_H()           //gpio_set_value(EXP_LCD_RESET, 1)
#define LCD_RESET_L()           //gpio_set_value(EXP_LCD_RESET, 0)

#define LCD_DISPLAYON()         BITCSET(pGPIO_D->GPDAT, Hw7, Hw7)
#define LCD_DISPLAYOFF()        BITCSET(pGPIO_D->GPDAT, Hw7, 0)
#endif//

void LM480KF_backlight_ctrl(unsigned int level)
{
	dprintk("%s level : %d ~", __func__, level);
	#ifdef CONFIG_MACH_TCC9200S
	BITCSET(HwGPIOE->GPFN0, HwPORTCFG_GPFN3_MASK, HwPORTCFG_GPFN3(0)); 
	BITCSET(HwGPIOE->GPEN, Hw3, Hw3);

	if(level)	{
		BITCSET(HwGPIOE->GPDAT, Hw3, Hw3);
	}
	else	{
		BITCSET(HwGPIOE->GPDAT, Hw3, 0); // BL OFF	
	}
	#elif defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900) || defined(CONFIG_MACH_TCC9300CM)
	BITCSET(pGPIO_A->GPEN, Hw7, Hw7);
	if(level)	{
		BITCSET(pGPIO_A->GPDAT, Hw7, Hw7);
	}
	else	{
		BITCSET(pGPIO_A->GPDAT, Hw7, 0); // BL OFF	
	}
	#elif defined(CONFIG_MACH_TCC9300)
	BITCSET(pGPIO_A->GPEN, Hw12, Hw12);

	if(level)
		BITCSET(pGPIO_A->GPDAT, Hw12, Hw12);	
	else
		BITCSET(pGPIO_A->GPDAT, Hw12, 0);	

	#elif defined(CONFIG_MACH_TCC9300ST)

	#endif
}

int LMS480KF_Module_initialize(char LCDC_num)
{
	LCD_POWER_ON();
	LCD_Driver_mdelay(10);

	LCD_RESET_H();

	LCD_Driver_mdelay(16);

	LCDC_IO_Set(LCDC_num, LCD_BUS_WIDTH);
	
	LCD_DISPLAYON();

	return 1;
}


int LMS480KF_Module_Off(void)
{
	printk("%s \n", __func__);

	LCD_DISPLAYOFF();
	
	LCD_Driver_mdelay(10);

	LCD_PROT_INT();
	
	LCD_RESET_L();
 	LCD_POWER_OFF();

	LCDC_IO_Disable(0, LCD_BUS_WIDTH);
	return 1;
}

int LMS480KF_Module_On(char LCDC_num)
{
	printk("%s LCDC_num:%d \n", __func__, LCDC_num);
	LCD_POWER_ON();

	LCD_Driver_mdelay(10);

	LCD_RESET_H();

	LCD_Driver_mdelay(1);

	LCDC_IO_Set(LCDC_num, LCD_BUS_WIDTH);
	LCD_Driver_mdelay(10);
	
	LCD_DISPLAYON();	
	return 1;
}

void LMS480KF_Module_Spec(LCD_MODULE_SPEC_TYPE* spec)
{
	spec->lcdc_clk = LCDC_CLK;
	spec->lcdc_clk_div = LCD_DIVIDER;
	spec->lcd_bus_width = LCD_BUS_WIDTH;
	spec->lcd_width = 800;
	spec->lcd_height = 480;
	spec->lcd_LPW = LPW;		// line pulse width
	spec->lcd_LPC = LPC;		// line pulse count (active horizontal pixel - 1)
	spec->lcd_LSWC = LSWC;		// line start wait clock (the number of dummy pixel clock - 1)
	spec->lcd_LEWC = LEWC;		// line end wait clock (the number of dummy pixel clock - 1)
	spec->lcd_VDB = VDB;		// Back porch Vsync delay
	spec->lcd_VDF = VDF;		// front porch of Vsync delay

	spec->lcd_FPW1 = FPW1;		// TFT/TV : Frame pulse width is the pulse width of frmae clock
	spec->lcd_FLC1 = FLC1;		// frmae line count is the number of lines in each frmae on the screen
	spec->lcd_FSWC1 = FSWC1;		// frmae start wait cycle is the number of lines to insert at the end each frame
	spec->lcd_FEWC1 = FEWC1;		// frame start wait cycle is the number of lines to insert at the begining each frame

	spec->lcd_FPW2 = FPW2;		// TFT/TV : Frame pulse width is the pulse width of frmae clock
	spec->lcd_FLC2 = FLC2;		// frmae line count is the number of lines in each frmae on the screen
	spec->lcd_FSWC2 = FSWC2;		// frmae start wait cycle is the number of lines to insert at the end each frame
	spec->lcd_FEWC2 = FEWC2;		// frame start wait cycle is the number of lines to insert at the begining each frame
	spec->SyncSignal_Invert = SYNC_INVET;
}


int LMS480KF_Module_probe(void)
{
	    pGPIO_A = (volatile PGPION)tcc_p2v(HwGPIOA_BASE);
		pGPIO_B = (volatile PGPION)tcc_p2v(HwGPIOB_BASE);
	    pGPIO_C = (volatile PGPION)tcc_p2v(HwGPIOC_BASE);
		pGPIO_D = (volatile PGPION)tcc_p2v(HwGPIOD_BASE);
	    pGPIO_E = (volatile PGPION)tcc_p2v(HwGPIOE_BASE);


	#if defined(CONFIG_MACH_TCC9300)
		gpio_direction_output(EXP_LCD_RESET, 1); //LCD Power
		gpio_direction_output(EXP_LCD_POWER, 1); //LCD Reset
	#elif defined(CONFIG_MACH_TCC9300CM)
		gpio_direction_output(TCC_GPEXT3(1), 1); //LCD Reset
	#elif defined(CONFIG_MACH_TCC9300ST )
	    //gpio_direction_output(EXP_LCD_RESET, 1); //LCD Power RonaldLEE_0929
	    //gpio_direction_output(EXP_LCD_POWER, 1); //LCD Reset RonaldLEE_0929

	#endif//
	dprintk("%s \n", __func__);
	return 0;
}

void LMS480KF_Module_func(LCD_MODULE_FUNC_TYPE *func)
{
	func->Lcd_Module_probe = LMS480KF_Module_probe;
	func->Lcd_Initailize = LMS480KF_Module_initialize;
	func->Lcd_on = LMS480KF_Module_On;
	func->Lcd_off = LMS480KF_Module_Off;
	func->Backligth_ctrl = LM480KF_backlight_ctrl;
	func->Lcd_GetModuleSpec = LMS480KF_Module_Spec;
}
EXPORT_SYMBOL(LMS480KF_Module_func);

#endif//CONFIG_FB_TCC92XX_WVGA_LMS480KF
