/*
 * drivers/video/Tcc_overlay1.c
 *
 * Copyright (C) 2004 Telechips, Inc. 
 *
 * Video-for-Linux (Version 2) graphic capture driver
 *
 * 
 * This package is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. 
 * 
 * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR 
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED 
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE. 
 *
 */
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <linux/clk.h>
#include <linux/videodev.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/div64.h>
#include <asm/mach/map.h>
#include <linux/poll.h>

#if defined(_TCC79x_)
#include <mach/tcc79x.h>
#elif defined(_TCC83x_)
#include <mach/tcc83x_virt.h>
#elif defined(CONFIG_ARCH_TCC92X)|| defined(CONFIG_ARCH_TCC93XX)
#include <mach/bsp.h>
#endif

#include <mach/tcc_overlay_ioctl.h>

#if 0
static int debug	   = 1;
#else
static int debug	   = 0;
#endif

#define dprintk(msg...)	if (debug) { printk( "tcc_overlay1: " msg); }


#define DEVICE_NAME			"overlay1"
#define MAJOR_ID			203
#define MINOR_ID			1

//#define OVERLAY_CNT_DEFAULT 2

static struct clk *overlay1_lcdc_clk;

static overlay_config_t overlay1_cfg;
static struct v4l2_format overlay1_fmt;


/* LCD Overlay Setting*/
#define IMG_INTL	       	0
#define IMG_AEN           	0
#define IMG_CEN           	0
#define IMG_IEN           	1
#define IMG_AOPT          	2
#define IMG_ASEL          	0
#define IMG_PD            	0
#define IMG_Y2RMD         	1
#define IMG_Y2R          	1
#define IMG_BR            	0

#define IMG_RGB565_FMT           	10	//RGB888
#define IMG_RGB888_FMT           	12	//RGB888
#define IMG_RGBYUV420_FMT         24	//RGB888
#define IMG_RGBYUV422SP_FMT         25	//RGB888
#define IMG_RGBYUV422SQ_FMT         26	//RGB888
#define IMG_RGBYUV420I_FMT         28	//YUV420_INTERLEAVED

#define IMG_POSITION_Y    	0
#define IMG_POSITION_X    	0
#define IMG_HEIGHT	       	LCD_HEIGHT
#define IMG_WIDTH         		LCD_WIDTH
#define IMG_AVAL0         		95
#define IMG_AVAL1         		95
static PLCDC		pLCDC1;
static unsigned char start_en = 0;
static unsigned char wait_restart = 0;
static unsigned char pos_reset = 0;

unsigned char tcc_overlay1_use = 0;

//#define VC_OVERLAY_PROFILE // CAM_28pin, GPIO_D24
#ifdef VC_OVERLAY_PROFILE
static unsigned char toggle_int = 0;
#endif
extern void tccxxx_overlay_common_enable(void);
extern void tccxxx_overlay_common_disable(int channel);

unsigned char tccxxx_overlay1_use(void)
{
	return tcc_overlay1_use;
}
EXPORT_SYMBOL(tccxxx_overlay1_use);

static int tccxxx_overlay1_mmap(struct file *file, struct vm_area_struct *vma)
{
	vma->vm_page_prot = pgprot_noncached(vma->vm_page_prot);
	if(remap_pfn_range(vma,vma->vm_start, vma->vm_pgoff , vma->vm_end - vma->vm_start, vma->vm_page_prot))
	{
		return -EAGAIN;
	}

	vma->vm_ops	= NULL;
	vma->vm_flags 	|= VM_IO;
	vma->vm_flags 	|= VM_RESERVED;
	
	return 0;
}


DECLARE_WAIT_QUEUE_HEAD(overlay1_wait);

static unsigned int tccxxx_overlay1_poll(struct file *file, struct poll_table_struct *wait)
{
	dprintk(" tccxxx_overlay1_poll wait[%d][%d]!!!\n", (unsigned)wait, (unsigned)&overlay1_wait);
	poll_wait(file, &(overlay1_wait), wait);	
	dprintk(" tccxxx_overlay1_poll finish[%d][%d]!!!\n", (unsigned)wait, (unsigned)&overlay1_wait);	
	return POLLIN;
}

static int tccxxx_overlay1_get_pos(overlay_config_t * arg )
{
	overlay_config_t *pos;
	
	pos = (overlay_config_t *)arg;
	if(copy_from_user(pos, (overlay_config_t *)arg, sizeof(*pos)))
		return -EFAULT;

	pos->sx 		= overlay1_cfg.sx;
	pos->sy 		= overlay1_cfg.sy;
	pos->width	= overlay1_cfg.width;
	pos->height	= overlay1_cfg.height;
	dprintk(" Overlay -> Get Position :: (%d,%d) | (%d,%d) \n", overlay1_cfg.sx, overlay1_cfg.sy, overlay1_cfg.width, overlay1_cfg.height);
	
	if(copy_to_user((overlay_config_t *)arg, pos, sizeof(*pos)))
		return -EFAULT;

	return 0;
}

static int tccxxx_overlay1_get_screenInfo(overlay_config_t * arg )
{
	unsigned int screen_width, screen_height;

	overlay_config_t *screen_info;
	
	screen_info = (overlay_config_t *)arg;
	if(copy_from_user(screen_info, (overlay_config_t *)arg, sizeof(*screen_info)))
		return -EFAULT;

        screen_width      = SCREEN_WIDTH;
        screen_height     = SCREEN_HEIGHT;

#if defined(CONFIG_TCC_HDMI_UI_SIZE_1280_720) && defined(COMFIG_ARCH_TCC92X)
        if(tcc_display_data.resolution == 1)
        {
                screen_width      = 720;
                screen_height     = 576;
        }
        else if(tcc_display_data.resolution == 2)
        {
                screen_width      = 800;
                screen_height     = 480;
        }
#endif	

	screen_info->sx 		= 0;
	screen_info->sy 		= 0;
	screen_info->width	= screen_width;
	screen_info->height	= screen_height;
	
	dprintk(" Overlay -> Get ScreenInfo :: (%d,%d) | (%d,%d) \n", screen_info->sx, screen_info->sy, screen_info->width, screen_info->height);
	
	if(copy_to_user((overlay_config_t *)arg, screen_info, sizeof(*screen_info)))
		return -EFAULT;

	return 0;
}

void tccxxx_overlay1_fmt_set(unsigned int fmt)
{
	dprintk(" Overlay -> S_FMT :: format = 0x%x(RGB565-0x%x, YUV420-0x%x, YUV420inter-0x%x) \n", fmt, V4L2_PIX_FMT_RGB565,V4L2_PIX_FMT_YVU420, V4L2_PIX_FMT_NV12);

	if(fmt == V4L2_PIX_FMT_RGB565)
	{
		BITCSET (pLCDC1->LI1C, 0x1f<< 0,	(IMG_RGB565_FMT) <<  0); // format
		BITCSET (pLCDC1->LI1O, 0x0000FFFF,	overlay1_cfg.width * 2); // format
		BITCSET (pLCDC1->LI1C, 0x1<<  8,	(0)  <<  8); // y2r converter enable
	}
	else if(fmt == V4L2_PIX_FMT_NV12)
	{
		BITCSET (pLCDC1->LI1C, 0x1f<< 0,	(IMG_RGBYUV420I_FMT) <<  0); // format
		BITCSET (pLCDC1->LI1O, 0xFFFFFFFF,	((overlay1_cfg.width)<<16) | (overlay1_cfg.width)); // format 			
		BITCSET (pLCDC1->LI1C, 0x1<<  8,	(IMG_Y2R)  <<  8); // y2r converter enable
	}
	else if(fmt == V4L2_PIX_FMT_YUV422P)
	{
		BITCSET (pLCDC1->LI1C, 0x1f<< 0,	(IMG_RGBYUV422SQ_FMT) <<  0); // format
		BITCSET (pLCDC1->LI1O, 0x0000FFFF,	overlay1_cfg.width * 2); // format
		BITCSET (pLCDC1->LI1C, 0x1<<  8,	(IMG_Y2R)  <<  8); // y2r converter enable
	}
	else
	{
		BITCSET (pLCDC1->LI1C, 0x1f<< 0,	(IMG_RGBYUV420_FMT) <<	0); // format
		BITCSET (pLCDC1->LI1O, 0xFFFFFFFF,	((overlay1_cfg.width/2)<<16) | (overlay1_cfg.width)); // format				
		BITCSET (pLCDC1->LI1C, 0x1<<  8,	(IMG_Y2R)  <<  8); // y2r converter enable
	}
}

int tccxxx_overlay1_q_buffer(unsigned int* arg )
{
	unsigned int *curY_phyAddr, curU_phyAddr, curV_phyAddr;

	curY_phyAddr = (unsigned int *)arg;
	if(copy_from_user(curY_phyAddr, (unsigned int *)arg, sizeof(*curY_phyAddr)))
		return -EFAULT;

	//dprintk(" Overlay -> Q_Buffer :: buffer = 0x%x \n", *curY_phyAddr);

#ifdef VC_OVERLAY_PROFILE
	if(toggle_int)
	{
		(HwGPIOD->GPEN |= Hw24);	(HwGPIOD->GPDAT |= Hw24);
		toggle_int = 0;
	}
	else
	{
		(HwGPIOD->GPEN |= Hw24);	(HwGPIOD->GPDAT &= ~Hw24);
		toggle_int = 1;
	}
#endif

	//in case position reset in streamming.
	if(pos_reset)
	{
		pos_reset = 0;
		// position
		BITCSET (pLCDC1->LI1P, 0xffff<< 16, (overlay1_cfg.sy)  << 16); // position y
		BITCSET (pLCDC1->LI1P, 0xffff<<  0, (overlay1_cfg.sx)  <<  0); // position x
		
		// size
		BITCSET (pLCDC1->LI1S, 0xffff<< 16, (overlay1_cfg.height) << 16); // height
		BITCSET (pLCDC1->LI1S, 0xffff<<  0, (overlay1_cfg.width) <<	0); // width
		
		tccxxx_overlay1_fmt_set(overlay1_fmt.fmt.pix.pixelformat);		
	}

	// image address
	curU_phyAddr = GET_ADDR_YUV42X_spU(*curY_phyAddr, overlay1_cfg.width, overlay1_cfg.height); 
	curV_phyAddr = GET_ADDR_YUV420_spV(curU_phyAddr, overlay1_cfg.width, overlay1_cfg.height);
	
	BITCSET (pLCDC1->LI1BA0, 0xFFFFFFFF,  *curY_phyAddr); // address0
	BITCSET (pLCDC1->LI1BA1, 0xFFFFFFFF,  curU_phyAddr); // address1	
	BITCSET (pLCDC1->LI1BA2, 0xFFFFFFFF,  curV_phyAddr); // address2

	if(!start_en)
	{
		tccxxx_overlay_common_enable();
		tccxxx_overlay1_fmt_set(overlay1_fmt.fmt.pix.pixelformat);
		BITCLR (pLCDC1->LI1C, HwLIC_INTL);				 // not interlace format

		BITCSET (pLCDC1->LI1C, 0x1<<28,     (1)  << 28); // Enable Image
		BITSET(pLCDC1->LCTRL, Hw0);	
		start_en = 1;
	}
	else if(wait_restart)
	{
		BITCSET (pLCDC1->LI1C, 0x1<<28,     (1)  << 28); // Enable Image
	}
	
	//ZzaU:: For Camcorder...
	if(!(pLCDC1->LI1C & Hw28))
	{
		BITCSET (pLCDC1->LI1C, 0x1<<28, 	(1)  << 28); // Enable Image
	}

#ifdef CONFIG_ARCH_TCC93XX
	BITCSET (pLCDC1->LI1C, HwLIC_UPD, HwLIC_UPD); //Image update
#endif//	CONFIG_ARCH_TCC93XX
	return 0;
}


static int tccxxx_overlay1_disable(void)
{
	BITSCLR (pLCDC1->LI1C, 0x1<<28, 	(1)  << 28); // Disable Image

#ifdef CONFIG_ARCH_TCC93XX
	BITCSET (pLCDC1->LI1C, HwLIC_UPD, HwLIC_UPD); //Image update
#endif//	CONFIG_ARCH_TCC93XX

	wait_restart = 1;

	return 0;
}


static int tccxxx_overlay1_set_pos(overlay_config_t * arg )
{
	unsigned int screen_width, screen_height;

	overlay_config_t *pos;
	
	pos = (overlay_config_t *)arg;
	if(copy_from_user(pos, (overlay_config_t *)arg, sizeof(*pos)))
		return -EFAULT;

	if(!start_en)
	{
		BITSCLR (pLCDC1->LI1C, 0x1<<28, 	(1)  << 28); // Disable Image
		wait_restart = 1;
	}

	overlay1_cfg.sx		=	pos->sx; 	
	overlay1_cfg.sy		=	pos->sy; 	
	overlay1_cfg.width	=	pos->width;

	if(overlay1_fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_RGB565 || overlay1_fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_YUV422P)
		overlay1_cfg.width	=	((overlay1_cfg.width+3) >> 2)<<2;
	else
		overlay1_cfg.width	=	((overlay1_cfg.width+15) >> 4)<<4;
	
	overlay1_cfg.height	=	pos->height;

        screen_width      = SCREEN_WIDTH;
        screen_height     = SCREEN_HEIGHT;
#if defined(CONFIG_TCC_HDMI_UI_SIZE_1280_720) && defined(COMFIG_ARCH_TCC92X)
        if(tcc_display_data.resolution == 1)
        {
                screen_width      = 720;
                screen_height     = 576;
        }
        else if(tcc_display_data.resolution == 2)
        {
                screen_width      = 800;
                screen_height     = 480;
        }
#endif

	if(overlay1_cfg.width !=	pos->width)
	{
		if(overlay1_cfg.width > screen_width)
			overlay1_cfg.width = screen_width;
		
		overlay1_cfg.sx = (screen_width - overlay1_cfg.width)/2;
	}

	if(overlay1_cfg.sy + overlay1_cfg.height > screen_height)
		overlay1_cfg.height = screen_height - overlay1_cfg.sy;
	
	dprintk(" Overlay -> Set Position :: (%d,%d) | (%d,%d) \n", overlay1_cfg.sx, overlay1_cfg.sy, overlay1_cfg.width, overlay1_cfg.height);

	//in case position reset in streamming.
	if(start_en)
	{
		pos_reset = 1;
		return 0;
	}

	// position
	BITCSET (pLCDC1->LI1P, 0xffff<< 16, (overlay1_cfg.sy)  << 16); // position y
	BITCSET (pLCDC1->LI1P, 0xffff<<  0, (overlay1_cfg.sx)  <<  0); // position x

	// size
	BITCSET (pLCDC1->LI1S, 0xffff<< 16, (overlay1_cfg.height) << 16); // height
	BITCSET (pLCDC1->LI1S, 0xffff<<  0, (overlay1_cfg.width) <<  0); // width

	tccxxx_overlay1_fmt_set(overlay1_fmt.fmt.pix.pixelformat);

	return 0;
}

static int tccxxx_overlay1_set_configure(overlay_config_t* arg)
{
	unsigned int screen_width, screen_height;

	overlay_config_t *config;

	config = (overlay_config_t*)arg;
	if(copy_from_user(config, (unsigned int *)arg, sizeof(*config)))
		return -EFAULT;

	overlay1_fmt.fmt.pix.width 	 	= config->width;
	overlay1_fmt.fmt.pix.height 		= config->height ;
	overlay1_fmt.fmt.pix.pixelformat	= config->format;
	dprintk(" Overlay -> S_FMT :: size(%d,%d), format = 0x%x(RGB565-0x%x, YUV420-0x%x, YUV420inter-0x%x) \n", overlay1_fmt.fmt.pix.width, overlay1_fmt.fmt.pix.height, overlay1_fmt.fmt.pix.pixelformat, V4L2_PIX_FMT_RGB565,V4L2_PIX_FMT_YVU420, V4L2_PIX_FMT_NV12);

	overlay1_cfg.sx		=	config->sx;	
	overlay1_cfg.sy		=	config->sy;		
	overlay1_cfg.width	=	overlay1_fmt.fmt.pix.width;	
	if(overlay1_fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_RGB565 || overlay1_fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_YUV422P)
		overlay1_cfg.width	=	((overlay1_cfg.width+3) >> 2)<<2;
	else
		overlay1_cfg.width	=	((overlay1_cfg.width+15) >> 4)<<4;
	overlay1_cfg.height	=	overlay1_fmt.fmt.pix.height;

        screen_width      = SCREEN_WIDTH;
        screen_height     = SCREEN_HEIGHT;
#if defined(CONFIG_TCC_HDMI_UI_SIZE_1280_720) && defined(COMFIG_ARCH_TCC92X)
        if(tcc_display_data.resolution == 1)
        {
                screen_width      = 720;
                screen_height     = 576;
        }
        else if(tcc_display_data.resolution == 2)
        {
                screen_width      = 800;
                screen_height     = 480;
        }
#endif
	if(overlay1_cfg.sx + overlay1_cfg.width > screen_width)
	{
		if(overlay1_cfg.width > screen_width)
			overlay1_cfg.width = screen_width;
		
		overlay1_cfg.sx = (screen_width - overlay1_cfg.width)/2;
	}

	if(overlay1_cfg.sy + overlay1_cfg.height > screen_height)
		overlay1_cfg.height = screen_height - overlay1_cfg.sy;

	dprintk(" Overlay -> S_FMT :: Real => size(%d,%d ~ %d,%d) \n", overlay1_cfg.sx, overlay1_cfg.sy, overlay1_cfg.width, overlay1_cfg.height);

	// position
	BITCSET (pLCDC1->LI1P, 0xffff<< 16, (overlay1_cfg.sy)  << 16); // position y
	BITCSET (pLCDC1->LI1P, 0xffff<<  0, (overlay1_cfg.sx)  <<  0); // position x

	// size
	BITCSET (pLCDC1->LI1S, 0xffff<< 16, (overlay1_cfg.height) << 16); // height
	BITCSET (pLCDC1->LI1S, 0xffff<<  0, (overlay1_cfg.width) <<  0); // width


	tccxxx_overlay1_fmt_set(overlay1_fmt.fmt.pix.pixelformat);

	BITCLR(pLCDC1->LI1C, HwLIC_SRC);

	return 0;
}

static int overlay1_forbid;
static int tccxxx_overlay1_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
{
	int intArg;

	if(overlay1_forbid && (cmd != OVERLAY_FORBID))
		return 0;

	switch(cmd)
	{
		case OVERLAY_FORBID:
			if(copy_from_user(&intArg, (int *)arg, sizeof(int)))
				return -EFAULT;
			overlay1_forbid = intArg;
			return 0;

		case OVERLAY_GET_POSITION:
			return tccxxx_overlay1_get_pos((overlay_config_t*)arg);

		case OVERLAY_GET_SCREEN_INFO:
			return tccxxx_overlay1_get_screenInfo((overlay_config_t*)arg);
			
		case OVERLAY_SET_POSITION:
			return tccxxx_overlay1_set_pos((overlay_config_t*)arg);

		case OVERLAY_QUEUE_BUFFER:
			return tccxxx_overlay1_q_buffer((unsigned int*)arg);

		case OVERLAY_SET_CONFIGURE:
			return tccxxx_overlay1_set_configure((overlay_config_t*)arg);

		case OVERLAY_SET_DISABLE:
			return tccxxx_overlay1_disable();


		default:
			dprintk(" Unsupported IOCTL!!!\n");      
			break;			
		}


		return 0;
	}

static int tccxxx_overlay1_release(struct inode *inode, struct file *file)
{
	start_en = 0;
	wait_restart = 0;
	tcc_overlay1_use --;
	dprintk(" ===========> tccxxx_overlay1_release num:%d \n", tcc_overlay1_use);

	tccxxx_overlay_common_disable(1);

	clk_disable(overlay1_lcdc_clk);

	return 0;
}

static int tccxxx_overlay1_open(struct inode *inode, struct file *file)
{
	dprintk(" ===========> tccxxx_overlay1_open num:%d \n", tcc_overlay1_use);

	tcc_overlay1_use ++;
	clk_enable(overlay1_lcdc_clk);

	pLCDC1  = (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);


	return 0;	
}

static struct file_operations tcc_overlay1_fops = 
{
	.owner			= THIS_MODULE,
	.poll 			= tccxxx_overlay1_poll,
	.ioctl			= tccxxx_overlay1_ioctl,
	.mmap			= tccxxx_overlay1_mmap,
	.open			= tccxxx_overlay1_open,
	.release		= tccxxx_overlay1_release,
};

static void __exit
tccxxx_overlay1_cleanup(void)
{
	unregister_chrdev(MAJOR_ID, DEVICE_NAME);
/*
	dma_free_writecombine(0,(PAGE_ALIGN((OVERLAY_BUFFER_SIZE*overlay_buffer_cnt)+PAGE_SIZE)),overlay_virtual_buffer, overlay_physical_buffer);
	overlay_virtual_buffer = NULL;
*/

	dprintk(" ===========> tccxxx_overlay1_cleanup \n");
	
	return;
}

static char banner[] __initdata = KERN_INFO "TCC Overlay driver initializing\n";
static struct class *overlay1_class;

static int __init 
tccxxx_overlay1_init(void)
{
	printk(banner);

/*
	overlay_buffer_cnt = OVERLAY_CNT_DEFAULT;
	overlay_virtual_buffer  = dma_alloc_writecombine(0, (PAGE_ALIGN((OVERLAY_BUFFER_SIZE*overlay_buffer_cnt)+PAGE_SIZE)), &overlay_physical_buffer, GFP_KERNEL);
	if (overlay_virtual_buffer == NULL)
	{
		printk("ERROR: overlay buffer allocation Failed!!");
		return -EFAULT;
	}

	printk("INFO: %d 'th allocation.. virt = 0x%x, phy = 0x%x !!", overlay_buffer_cnt, overlay_virtual_buffer, overlay_physical_buffer);	
*/
	register_chrdev(MAJOR_ID, DEVICE_NAME, &tcc_overlay1_fops);

	overlay1_class = class_create(THIS_MODULE, DEVICE_NAME);
	device_create(overlay1_class,NULL,MKDEV(MAJOR_ID,MINOR_ID),NULL,DEVICE_NAME);

	overlay1_lcdc_clk = clk_get(0, "lcdc1");
	BUG_ON(overlay1_lcdc_clk == NULL);

	return 0;
}


MODULE_AUTHOR("Telechips.");
MODULE_DESCRIPTION("TCC Video for Linux overlay1 driver");
MODULE_LICENSE("GPL");


module_init(tccxxx_overlay1_init);
module_exit(tccxxx_overlay1_cleanup);

