/*
 * include/linux/regulator/lp8725.h
 *
 * Copyright (C) 2009 Telechips, Inc.
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef __LINUX_REGULATOR_LP8725_H
#define __LINUX_REGULATOR_LP8725_H

#define LP8725_NUM_REGULATORS	9

#define LP8725_LDO1	0
#define LP8725_LDO2	1
#define LP8725_LDO3	2
#define LP8725_LDO4	3
#define LP8725_LDO5	4
#define LP8725_LILO1	5
#define LP8725_LILO2	6
#define LP8725_DCDC1	7
#define LP8725_DCDC2	8

#define LP8725_REG_GENERAL	0x0
#define LP8725_REG_LDO1		0x1
#define LP8725_REG_LDO2		0x2
#define LP8725_REG_LDO3		0x3
#define LP8725_REG_LDO4		0x4
#define LP8725_REG_LDO5		0x5
#define LP8725_REG_LILO1	0x6
#define LP8725_REG_LILO2	0x7
#define LP8725_REG_BUCK1_V1	0x8
#define LP8725_REG_BUCK1_V2	0x9
#define LP8725_REG_BUCK2_V1	0xa
#define LP8725_REG_BUCK2_V2	0xb
#define LP8725_REG_BUCK_CONTROL	0xc
#define LP8725_REG_LDO_CONTROL	0xd
#define LP8725_REG_PULLDOWN	0xe
#define LP8725_REG_STATUS	0xf

#define LP8725_LDO1_EN		(1 << 0)
#define LP8725_LDO2_EN		(1 << 1)
#define LP8725_LDO3_EN		(1 << 2)
#define LP8725_LDO4_EN		(1 << 3)
#define LP8725_LDO5_EN		(1 << 4)
#define LP8725_LILO1_EN		(1 << 5)
#define LP8725_LILO2_EN		(1 << 6)

#define LP8725_LDO_1_20V	0
#define LP8725_LDO_1_25V	1
#define LP8725_LDO_1_30V	2
#define LP8725_LDO_1_35V	3
#define LP8725_LDO_1_40V	4
#define LP8725_LDO_1_45V	5
#define LP8725_LDO_1_50V	6
#define LP8725_LDO_1_55V	7
#define LP8725_LDO_1_60V	8
#define LP8725_LDO_1_65V	9
#define LP8725_LDO_1_70V	10
#define LP8725_LDO_1_75V	11
#define LP8725_LDO_1_80V	12
#define LP8725_LDO_1_85V	13
#define LP8725_LDO_1_90V	14
#define LP8725_LDO_2_00V	15
#define LP8725_LDO_2_10V	16
#define LP8725_LDO_2_20V	17
#define LP8725_LDO_2_30V	18
#define LP8725_LDO_2_40V	19
#define LP8725_LDO_2_50V	20
#define LP8725_LDO_2_60V	21
#define LP8725_LDO_2_65V	22
#define LP8725_LDO_2_70V	23
#define LP8725_LDO_2_75V	24
#define LP8725_LDO_2_80V	25
#define LP8725_LDO_2_85V	26
#define LP8725_LDO_2_90V	27
#define LP8725_LDO_2_95V	28
#define LP8725_LDO_3_00V	29
#define LP8725_LDO_3_10V	30
#define LP8725_LDO_3_30V	31

#define LP8725_LILO_0_80V	0
#define LP8725_LILO_3_30V	31

#define LP8725_DCDC_0_80V	0
#define LP8725_DCDC_0_85V	1
#define LP8725_DCDC_0_90V	2
#define LP8725_DCDC_0_95V	3
#define LP8725_DCDC_1_00V	4
#define LP8725_DCDC_1_05V	5
#define LP8725_DCDC_1_10V	6
#define LP8725_DCDC_1_15V	7
#define LP8725_DCDC_1_20V	8
#define LP8725_DCDC_1_25V	9
#define LP8725_DCDC_1_30V	10
#define LP8725_DCDC_1_35V	11
#define LP8725_DCDC_1_40V	12
#define LP8725_DCDC_1_50V	13
#define LP8725_DCDC_1_60V	14
#define LP8725_DCDC_1_70V	15
#define LP8725_DCDC_1_75V	16
#define LP8725_DCDC_1_80V	17
#define LP8725_DCDC_1_85V	18
#define LP8725_DCDC_1_90V	19
#define LP8725_DCDC_2_00V	20
#define LP8725_DCDC_2_10V	21
#define LP8725_DCDC_2_20V	22
#define LP8725_DCDC_2_30V	23
#define LP8725_DCDC_2_40V	24
#define LP8725_DCDC_2_50V	25
#define LP8725_DCDC_2_60V	26
#define LP8725_DCDC_2_70V	27
#define LP8725_DCDC_2_80V	28
#define LP8725_DCDC_2_85V	29
#define LP8725_DCDC_2_90V	30
#define LP8725_DCDC_3_00V	31

int lp8725_register_regulator(struct device *dev, int reg);

#endif
