//
// Copyright (c) Telechips Corporation.  All rights reserved.
//
//    


#include <asm/arch/tcc79x.h>
#include "tcc7901_i2s.h"
#include "tcc7901_i2c.h"

static void InitOutputDMA(void);
static void InitInputDMA(void);

int I2S_Init(void)
{
	HwDAVC = 0;
	HwDAMR = 0
		//| Hw31
		//| HwDAMR_TE_EN
		//| HwDAMR_RE_EN
		//| HwDAMR_MD_MSB			// 0: IIS, 1:MSB
		| HwDAMR_SM_INT
		| HwDAMR_BM_INT
		| HwDAMR_FM_INT
		//| HwDAMR_CC_EN
		//| HwDAMR_BP_NEGA
		//| HwDAMR_CM_EN
		//| HwDAMR_MM_EN
		//| HwDAMR_LB_EN
		//| HwDAMR_BD_DIV4
		| HwDAMR_FD_DIV64
		| HwDAMR_TXS_LSB16
		| HwDAMR_RXS_LSB16
		;
	
	//CKC DAI clock enable
	BITSET(HwPCK_DAI, HwPCK_EN_EN);
	//TX enable
	HwDAMR |= HwDAMR_TE_EN;
	//DAI enable
	HwDAMR |= HwDAMR_EN_ON;
	
	//enable DMA Interrupt
	HwMODE &= ~HwINT_DMA;
	HwMODEA &= ~HwINT_DMA;
	HwSEL |= HwINT_DMA;
	HwIRQMSK |= HwINT_DMA;

	SetI2SClockRate(IS2LRCLK_44100);

	//----- 4. For power management purposes, shut the clocks off! -----
	StopI2SOutputClock();

	InitOutputDMA();
	InitInputDMA();
    	
	return(1);
}

static void IO_DMA_SetDMA(
	unsigned uCh,
	unsigned uSPARAM,
	void *pDST, unsigned uDPARAM,
	unsigned uCHCTRL
)
{
	// audio output --- DMA channel 0
	// audio input  --- DMA channel 1
	switch (uCh)
	{
		case 0:
			// Set Source Parameter (mask + increment)
			HwSPARAM0 = uSPARAM;
			// Set Dest Address & Dest Parameter (mask + increment)
			HwST_DADR0 = (int)pDST;
			HwDPARAM0 = uDPARAM;
		
			HwCHCTRL0 = uCHCTRL;
		break;
		case 1:
			// Set Source Parameter (mask + increment)
			HwSPARAM1 = uSPARAM;
			// Set Dest Address & Dest Parameter (mask + increment)
			HwST_DADR1 = (int)pDST;
			HwDPARAM1 = uDPARAM;
		
			HwCHCTRL1 = uCHCTRL;
		break;
	}
}

// Set the start address & length of DMA source buffer
void SetDMASourceBuffer(unsigned int sAddr, unsigned int b_length)
{
	unsigned long DMABuffer = 0;

	HwCHCTRL0 &= ~HwCHCTRL_EN_EN;
    
	DMABuffer = 0xFFFFFF00 / (((b_length) >> 4) << 8);
	DMABuffer = DMABuffer * (((b_length) >>4)<<8);
	
	//DMABuffer = 0xFFFFF8000;
	//printk("SetDMABuffer(%08X,%08X, %08X)\n", sAddr, b_length, DMABuffer);
	HwSPARAM0 = DMABuffer | 2;
	
	// generate interrupt when half of buffer was played
	HwHCOUNT0 = ((b_length/2)>>0x04);
	
	HwST_SADR0 = sAddr;
    
	HwRPTCTRL0 &= ~Hw31;
	
	HwCHCTRL0 |= HwCHCTRL_EN_EN;
}
// Set the start address & length of input DMA dest buffer
void SetInputDMADestBuffer(unsigned int sAddr, unsigned int b_length)
{
	unsigned long DMABuffer = 0;

	//printk("SetInputDMADestBuffer(%p,%d)\n", sAddr, b_length);
	HwCHCTRL1 &= ~HwCHCTRL_EN_EN;
    
	DMABuffer = 0xFFFFFF00 / (((b_length) >> 4) << 8);
	DMABuffer = DMABuffer * (((b_length) >>4)<<8);
	
	//DMABuffer = 0xFFFFF8000;
	HwDPARAM1 = DMABuffer | 2;
	
	// generate interrupt when half of buffer was filled
	HwHCOUNT1 = ((b_length/2)>>0x04);
	
	HwST_DADR1 = sAddr;
    
	HwRPTCTRL1 &= ~Hw31;
	
	HwCHCTRL1 |= HwCHCTRL_EN_EN;
}

static void InitOutputDMA(void)
{
	unsigned long DMABuffer = 0;

	DMABuffer = 0xFFFFFF00 / (((AUDIO_DMA_OUT_BUF_SIZE) >> 4) << 8);
	DMABuffer = DMABuffer * (((AUDIO_DMA_OUT_BUF_SIZE) >>4)<<8);
	
	//printk("HwSPARAM0:%08X\n", DMABuffer);
	HwEXTREQ0 = Hw23;
	IO_DMA_SetDMA(0, (2|DMABuffer), (void*)&HwDADO_L0, (4|0xFFFFFE00), 
				//HwCHCTRL_DTM_EN	|
				HwCHCTRL_CONT_C		| 
				HwCHCTRL_SYNC_EN	|
				HwCHCTRL_TYPE_SE	| 
				HwCHCTRL_BSIZE_8	|
				HwCHCTRL_WSIZE_16	| 
				HwCHCTRL_FLAG		|
				HwCHCTRL_IEN_EN     |
				HwCHCTRL_REP_EN
				);	

    // now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
	HwHCOUNT0 = ((AUDIO_DMA_OUT_PAGE_SIZE)>>0x04);

	HwRPTCTRL0 &= ~Hw31;
	
	//HwCHCONFIG0 &= ~HwCHCONFIG_SWP0_EN;

	HwCHCTRL0 |= HwCHCTRL_EN_EN;
}

static void InitInputDMA(void)
{
	unsigned long DMABuffer = 0;

	DMABuffer = 0xFFFFFF00 / (((AUDIO_DMA_IN_BUF_SIZE) >> 4) << 8);
	DMABuffer = DMABuffer * (((AUDIO_DMA_IN_BUF_SIZE) >>4)<<8);
	
	HwEXTREQ1 = Hw24;
	// Set Source Address
	HwST_SADR1 = (int)&HwDADI_L0; 
	// Set Source Parameter (mask + increment)
	HwSPARAM1 = 0xFFFFFE00 | 4;
	// Set Dest Address & Dest Parameter (mask + increment)
	//HwST_DADR1 = (int)pDST;
	HwDPARAM1 = DMABuffer|2;
		
	HwCHCTRL1 = 
		//HwCHCTRL_DTM_EN	|
		HwCHCTRL_CONT_C		| 
		HwCHCTRL_SYNC_EN	|
		HwCHCTRL_TYPE_SE	| 
		HwCHCTRL_BSIZE_8	|
		HwCHCTRL_WSIZE_16	| 
		HwCHCTRL_FLAG		|
		HwCHCTRL_IEN_EN     |
		HwCHCTRL_REP_EN;
    
	// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
	HwHCOUNT1 = ((AUDIO_DMA_IN_PAGE_SIZE)>>0x04);

	HwRPTCTRL1 &= ~Hw31;

	//HwCHCONFIG0 &= ~HwCHCONFIG_SWP1_EN;

	HwCHCTRL1 |= HwCHCTRL_EN_EN;
}

int I2S_Deinit(void)
{
    //----- 1. Stop the I2S clocks -----
    StopI2SOutputClock();

	return 1;
}


#if 0
int I2S_WriteData(unsigned long *lpBuff, unsigned long dwLen)
{
	static unsigned long waitCount	= 0;
	static unsigned long i			= 0;

	//----- 1. Check the parameters -----
	if(!lpBuff || (dwLen == 0))
	{
        goto WRITE_ERROR;
	}

//	StartI2SClock();


	StopI2SOutputClock();
	return 1;

WRITE_ERROR:
	StopI2SOutputClock();
	return 0;
}
#endif

//------------------------------------ Helper Routines ------------------------------------

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Function:		StartI2SClock()

Description:	Enables the I2S clock that drives the audio codec chip.

Returns:		N/A
-------------------------------------------------------------------*/
void StartI2SOutputClock(void)
{
	//TX enable
	HwDAMR |= HwDAMR_TE_EN;
	HwCHCTRL0 |= HwCHCTRL_EN_EN;

	return;
}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Function:		StopI2SClock()

Description:	Disables the I2S clock that drives the audio codec chip.

Returns:		N/A
-------------------------------------------------------------------*/
void StopI2SOutputClock(void)
{
	//TX disable
	HwDAMR &= ~HwDAMR_TE_EN;
	HwCHCTRL0 &= ~HwCHCTRL_EN_EN;

	// reset HOP counter
	//HwHCOUNT0 = 0;
	//printk("StopI2SOutputClock()\n");
		
	return;
}

void StartI2SInputClock(void)
{
	//RX enable
	HwDAMR |= HwDAMR_RE_EN;

	return;
}

void EnableDAI(void)
{
	HwDAMR |= HwDAMR_EN_ON;
}

void DisableDAI(void)
{
	HwDAMR &= ~HwDAMR_EN_ON;
}

void StopI2SInputClock(void)
{
	//RX disable
	HwDAMR &= ~HwDAMR_RE_EN;

	return;
}

void SetI2SClockRate(unsigned long ClockRate)
{
	
	//HwPCK_DAI = 0x1000f151;
/*

	unsigned int dwPclk;
	unsigned int dwM, dwS, dwP, dwDiv;
	unsigned long uTmpL;

	HwPCK_DAI = 0;
	dwPclk = HwPLL0CFG;
	dwS = (dwPclk >> 16)&0x03;
	dwM = (dwPclk >> 8) &0xff;
	dwP = (dwPclk) & 0x3f;
	
	dwPclk = ((2*12000000) * (dwM+8)/(dwP+2)) >> dwS;
	
	uTmpL = (unsigned long)ClockRate * 256 * 0x10000;
	dwDiv = uTmpL / dwPclk;
	HwPCK_DAI = HwPCK_DAI_EN_EN + HwPCK_DAI_SEL_PLL0 + 0x10000 - dwDiv;
*/	
    return;
}




