/*
 * linux/sound/soc/codecs/tcc250_dac.c
 *
 * Author:  <linux@telechips.com>
 * Created:     June 10, 2008
 * Description: 
 *
 * Copyright (C) 2008-2009 Telechips 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see the file COPYING, or write
 * to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
#if defined(CONFIG_ARCH_TCC83X)
#include <asm/arch/tcc83x_virt.h>
#elif defined(CONFIG_ARCH_TCC79X)
#include <asm/arch/tcc79x.h>
#endif

#include <sound/driver.h>
#include <sound/core.h>
#include <sound/pcm.h>
#include <sound/pcm_params.h>
#include <sound/soc.h>
#include <sound/soc-dapm.h>
#include <linux/i2c.h>

#include "tcc250_dac.h"



/* extern HANDLE ghI2C; */
void _Delay(unsigned int count)
{
    volatile unsigned int i;
    for (i=0;i<count;i++);
}

void TCC250_SEND_CMD(void *handle, unsigned char hb, unsigned char lb)
{
    unsigned char data[2];
    struct snd_soc_codec *codec = (struct snd_soc_codec *)handle;

    data[0] = hb;
    data[1] = lb;
    codec->hw_write(codec->control_data, data, 2);
}

unsigned char TCC250_READ_CMD(void *handle, unsigned char hb)
{
    unsigned char data[2];
    struct snd_soc_codec *codec = (struct snd_soc_codec *)handle;

    data[0] = hb;
    data[1] = 0;
    codec->hw_write(codec->control_data, data, 1);
    codec->hw_read(codec->control_data, &(data[1]), 1);

    return data[1];
}

#if defined(CONFIG_ARCH_TCC83X)
void TCC250_I2S_Init(void)
{
    HwDAMR = (0
              | HwDAMR_BPS_EN
              | HwDAMR_LPS_EN
              //| HwDAMR_NMD_EN
              //| HwDAMR_RXE_EN
              | HwDAMR_TXS_LSB16
              | HwDAMR_RXS_LSB16
              //| HwDAMR_EN_ON
              //| HwDAMR_TE_EN
              //| HwDAMR_RE_EN
              //| HwDAMR_MD_MSB			// 0: IIS, 1:MSB
#ifdef MCLK_TO_TCC250_INCLUDE
              | HwDAMR_SM_INT
#endif

#ifdef FRAME_TO_TCC250_INCLUDE
              | HwDAMR_BM_INT
              | HwDAMR_FM_INT
#endif
              //| HwDAMR_CC_EN
              //| HwDAMR_BD_DIV4
              | HwDAMR_FD_DIV64
              //| HwDAMR_BP_NEGA
              //| HwDAMR_CM_EN
              //| HwDAMR_MM_EN
              //| HwDAMR_LB_EN
              );
}
#elif defined(CONFIG_ARCH_TCC79X)
void TCC250_I2S_Init(void)
{
    HwDAMR = (0        
              //| HwDAMR_BPS_EN
              //| HwDAMR_LPS_EN
              //| HwDAMR_NMD_EN
              //| HwDAMR_RXE_EN
              | HwDAMR_TXS_LSB16
              | HwDAMR_RXS_LSB16
              //| HwDAMR_EN_ON
              //| HwDAMR_TE_EN
              //| HwDAMR_RE_EN
              //| HwDAMR_MD_MSB			// 0: IIS, 1:MSB
#ifdef MCLK_TO_TCC250_INCLUDE
              | HwDAMR_SM_INT
#endif
#ifdef FRAME_TO_TCC250_INCLUDE
              | HwDAMR_BM_INT
              | HwDAMR_FM_INT
#endif
              //| HwDAMR_CC_EN
              //| HwDAMR_BD_DIV4
              | HwDAMR_FD_DIV64
              //| HwDAMR_BP_NEGA
              //| HwDAMR_CM_EN
              //| HwDAMR_MM_EN
              //| HwDAMR_LB_EN
              );
}
#endif


/*****************************************************************************
 * Function Name : TCC250_SetToLDOANA()
 ******************************************************************************
 * Desription  	: Set to LDO_ANA Voltage for Audio Codec
 * Parameter   	: mV   - Seeting Voltage [mV], Max 3600mV, Min 3300mV
 *                 DVS  - Digital Voltage Scaling Enable
 * Return      	: None
 * Note          : 
 ******************************************************************************/
void TCC250_SetToLDOANA(void *handle, unsigned short mV)
{
    if(mV < 2700) mV = 2700;
    else if(mV > 3300) mV = 3300;

#ifdef _TCC250_DVS_INCLUDE
    TCC250_SEND_CMD(handle, TCC250_ANA_CTRL_REG,   ((mV / 50) - 54));	// DVS Mode
#if 0
    Sleep(2);
#endif
    TCC250_SEND_CMD(handle, TCC250_ANA_CTRL_REG,   (0x10 + ((mV / 50) - 54)));	// DVS Mode
#else
    TCC250_SEND_CMD(handle, TCC250_ANA_CTRL_REG,   ((mV / 50) - 54));	// Normal Mode
#if 0
    Sleep(2);
#endif
    TCC250_SEND_CMD(handle, TCC250_ANA_CTRL_REG,   (0x20 + ((mV / 50) - 54)));	// Normal Mode
#endif
}

/*****************************************************************************
 * Function Name : TCC250_Initial()
 ******************************************************************************
 * Desription   	 	: The function for H/W Codec Initialize function
 * Parameter   	: None
 * Return        	: None
 ******************************************************************************/
void TCC250_CODEC_Init(void *handle)	
{
    TCC250_I2S_Init();
    TCC250_ModePlay(handle);
    TCC250_SetToLDOANA(handle, 2800);
}
#if 0
/*****************************************************************************
 * Function Name : TCC250_OutPutMute()
 ******************************************************************************
 * Desription  	: The function for H/W Codec Mute Control function
 * Parameter   	: ucOnOff - 1(Mute On), 0(Mute Off)
 * Return      	: None
 ******************************************************************************/
void TCC250_OutPutMute(unsigned char ucOnOff)
{
    if(ucOnOff)	// Mute
    {
        TCC250_SEND_CMD(TCC250_LRMUTE_REG, 		0x63);
    }
    else	// Mute Off			
    {
        TCC250_SEND_CMD(TCC250_LRMUTE_REG, 		0x00);
    }
}
#endif

/*****************************************************************************
 * Function Name : TCC250_InPutMute()
 ******************************************************************************
 * Desription  	: The function for H/W Codec Mute Control function
 * Parameter   	: ucOnOff - 1(Mute On), 0(Mute Off)
 * Return      	: None
 ******************************************************************************/
void TCC250_InPutMute(void *handle, unsigned char ucOnOff)
{
    if(ucOnOff)	// Mute
    {
        TCC250_SEND_CMD(handle, TCC250_LRMUTE_REG, 0x1C);
    }
    else	// Mute Off			
    {
        TCC250_SEND_CMD(handle, TCC250_LRMUTE_REG, 0x00);
    }
}
/*****************************************************************************
 * Function Name : TCC250_SoftMute()
 ******************************************************************************
 * Desription  	: The function for H/W Codec Mute Control function
 * Parameter   	: ucOnOff - 1(Mute On), 0(Mute Off)
 * Return      	: None
 ******************************************************************************/
void TCC250_SoftMute(void *handle, unsigned char ucOnOff)
{
    if(ucOnOff)	// Mute
    {
        TCC250_SEND_CMD(handle, TCC250_LRMUTE_REG, 		0x7f);
    }
    else	// Mute Off			
    {
        TCC250_SEND_CMD(handle, TCC250_LRMUTE_REG, 		0x00);
    }
}

/*****************************************************************************
 * Function Name : TCC250_ModeLineIn()
 ******************************************************************************
 * Desription   	 	: The function for H/W Codec Line-IN Control function
 * Parameter   		: None
 * Return        	: None
 ******************************************************************************/
// LineIn Jack -> Line2
void TCC250_ModeLineIn(void *handle)
{
    //	TCC250_SendCmd(TCC250_PUP_OFF_INPUT_REG,	0x5f);	// ADC off 

#ifdef	ANA_LDO_INCLUDE
    TCC250_SEND_CMD(handle, TCC250_AUTOSTART1_REG, 0x4E); // Path 14
    TCC250_SEND_CMD(handle, TCC250_AUTOSTART2_REG, 0x07); // 
#else
    TCC250_SEND_CMD(handle, TCC250_AUTOSTART2_REG, 0x87); // 
    TCC250_SEND_CMD(handle, TCC250_AUTOSTART1_REG, 0x0E); // Path 14 :LineIn Bypass --> HP + LineIN --> Digital Out
#endif
    TCC250_SEND_CMD(handle, TCC250_MIC_CTRL_REG, 0x00);	// Microphone Control : Bias Off, Det off
    TCC250_SEND_CMD(handle, TCC250_ISSCTRL_REG, TCC250_ISSCTRL_VALUE);	// MCLK By External OSC
    TCC250_SEND_CMD(handle, TCC250_PUP_OFF_OUTPUT_REG, 0x0C);	// DAC off , HP on


#ifdef	TCC250_CODEC_I2S
    TCC250_SEND_CMD(handle, TCC250_FRAME_CTRL_REG, 0x09); // I2S
#else
    TCC250_SEND_CMD(handle, TCC250_FRAME_CTRL_REG, 0x49); // Left Justfied (MSB First)
#endif
    TCC250_SEND_CMD(handle, TCC250_PLBCK_REG, 0x00);	// MonoADC -  Input samples sent to both Left and Right Audio Digital Interface

    TCC250_SEND_CMD(handle, TCC250_LRMUTE_REG, 0x64);
}

/*****************************************************************************
 * Function Name : TCC250_ModeLineIn()
 ******************************************************************************
 * Desription   	 	: The function for H/W Codec Line-IN Control function
 * Parameter   		: None
 * Return        	: None
 ******************************************************************************/
// LineIn Jack -> Line2
void TCC250_ModeLineInOut(void *handle)
{
    //	TCC250_SendCmd(TCC250_PUP_OFF_INPUT_REG,	0x5f);	// ADC off 

#ifdef	ANA_LDO_INCLUDE
    TCC250_SEND_CMD(handle, TCC250_AUTOSTART1_REG, 0x58); // Path 24 :LineIN --> Digital Out, Digital in -> Headset Out
    TCC250_SEND_CMD(handle, TCC250_AUTOSTART2_REG, 0x07); // 
#else
    TCC250_SEND_CMD(handle, TCC250_AUTOSTART1_REG, 0x08); //
    TCC250_SEND_CMD(handle, TCC250_AUTOSTART2_REG, 0x87); // 
#endif
    TCC250_SEND_CMD(handle, TCC250_MIC_CTRL_REG, 0x00);	// Microphone Control : Bias Off, Det off
    TCC250_SEND_CMD(handle, TCC250_ISSCTRL_REG,	 TCC250_ISSCTRL_VALUE);	// MCLK By External OSC
    TCC250_SEND_CMD(handle, TCC250_PUP_OFF_OUTPUT_REG, 0x0F);	// DAC on , HP on


#ifdef	TCC250_CODEC_I2S
    TCC250_SEND_CMD(handle, TCC250_FRAME_CTRL_REG, 0x09); // I2S
#else
    TCC250_SEND_CMD(handle, TCC250_FRAME_CTRL_REG, 0x49); // Left Justfied (MSB First)
#endif
    TCC250_SEND_CMD(handle, TCC250_PLBCK_REG, 0x00); // MonoADC -  Input samples sent to both Left and Right Audio Digital Interface

    TCC250_SEND_CMD(handle, TCC250_LRMUTE_REG, 0x04); //DAC Mute Off / MICLMUTE On
}


/*****************************************************************************
 * Function Name : TCC250_ModeAuxIn()
 ******************************************************************************
 * Desription   	 	: The function for H/W Codec FM-IN Control function
 * Parameter   	: None
 * Return        	: None
 ******************************************************************************/
// FM output -> AuxIn
void TCC250_ModeAuxIn(void *handle)
{
    //	TCC250_SendCmd(TCC250_PUP_OFF_INPUT_REG,	0x1f);	// MIC Mute, Left/Right Input Channel On

#ifdef	ANA_LDO_INCLUDE
    TCC250_SEND_CMD(handle, TCC250_AUTOSTART1_REG, 0x50); // Path 16
    TCC250_SEND_CMD(handle, TCC250_AUTOSTART2_REG, 0x07); // 
#else
    TCC250_SEND_CMD(handle, TCC250_AUTOSTART2_REG, 0x87); // 
    TCC250_SEND_CMD(handle, TCC250_AUTOSTART1_REG, 0x10); // Path 16
#endif
    TCC250_SEND_CMD(handle, TCC250_MIC_CTRL_REG, 0x00);	// Microphone Control : Bias Off, Det off
    TCC250_SEND_CMD(handle, TCC250_ISSCTRL_REG,	 TCC250_ISSCTRL_VALUE);	// MCLK By External OSC
    TCC250_SEND_CMD(handle, TCC250_PUP_OFF_OUTPUT_REG, 0x0C);	// DAC off , HP on

#ifdef	TCC250_CODEC_I2S
    TCC250_SEND_CMD(handle, TCC250_FRAME_CTRL_REG, 0x09); // I2S
#else
    TCC250_SEND_CMD(handle, TCC250_FRAME_CTRL_REG, 0x49); // Left Justfied (MSB First)
#endif
    TCC250_SEND_CMD(handle, TCC250_PLBCK_REG, 0x00); // MonoADC -  Input samples sent to both Left and Right Audio Digital Interface

    TCC250_SEND_CMD(handle, TCC250_LRMUTE_REG, 0x64);


}

/*****************************************************************************
 * Function Name : TCC250_ModeMicIn()
 ******************************************************************************
 * Desription   	 	: The function for H/W Codec Mic-IN Control function
 * Parameter   	: None
 * Return        	: None
 ******************************************************************************/
// MicIn -> Line1
void TCC250_ModeMicIn(void *handle)
{ 
    //	TCC250_SendCmd(TCC250_PUP_OFF_INPUT_REG,	0xA4);	// ADC off 

#ifdef	ANA_LDO_INCLUDE
    TCC250_SEND_CMD(handle, TCC250_AUTOSTART1_REG, 0x4C);	// Path 12
    TCC250_SEND_CMD(handle, TCC250_AUTOSTART2_REG, 0x07);	//
#else
    TCC250_SEND_CMD(handle, TCC250_AUTOSTART2_REG, 0x87);	//
    TCC250_SEND_CMD(handle, TCC250_AUTOSTART1_REG, 0x0C);	// Path 12
#endif
    TCC250_SEND_CMD(handle, TCC250_MIC_CTRL_REG, 0x07);	// Microphone Control : Bias On, Det on, Level 170mV

    TCC250_SEND_CMD(handle, TCC250_ISSCTRL_REG, TCC250_ISSCTRL_VALUE);	// MCLK By External OSC
    TCC250_SEND_CMD(handle, TCC250_PUP_OFF_OUTPUT_REG, 0x0C);	// DAC off , HP on

#ifdef	TCC250_CODEC_I2S
    TCC250_SEND_CMD(handle, TCC250_FRAME_CTRL_REG, 0x09);	// I2S
#else
    TCC250_SEND_CMD(handle, TCC250_FRAME_CTRL_REG, 0x49);	// Left Justfied (MSB First)
#endif
    TCC250_SEND_CMD(handle, TCC250_PLBCK_REG, 0x02);	// MonoADC -  Input samples sent to both Left and Right Audio Digital Interface

    TCC250_SEND_CMD(handle, TCC250_MICVOL_REG, 0x20);	// +24dB MAX +34DB

    TCC250_SEND_CMD(handle, TCC250_LRMUTE_REG, 0x78);
}

void TCC250_ModePlay(void *handle)
{

#ifdef	ANA_LDO_INCLUDE
    TCC250_SEND_CMD(handle, TCC250_AUTOSTART1_REG, 0x41);	// Enable ENACLDO and Path#1
    TCC250_SEND_CMD(handle, TCC250_AUTOSTART2_REG, 0x07);
#else
    TCC250_SEND_CMD(handle, TCC250_AUTOSTART2_REG, 0x87);
    TCC250_SEND_CMD(handle, TCC250_AUTOSTART1_REG, 0x01);	// Disable ENACLDO and Path#1
#endif
    TCC250_SEND_CMD(handle, TCC250_MIC_CTRL_REG, 0x00);	// Microphone Control : Bias Off, Det off
    TCC250_SEND_CMD(handle, TCC250_PLBCK_REG, 0x00);	//Enable High Pass Filter

#ifdef	TCC250_CODEC_I2S
    TCC250_SEND_CMD(handle, TCC250_FRAME_CTRL_REG, 0x09);	// I2S
#else
    TCC250_SEND_CMD(handle, TCC250_FRAME_CTRL_REG, 0x49);	// Left Justfied (MSB First)
#endif


    TCC250_SEND_CMD(handle, TCC250_ISSCTRL_REG,			TCC250_ISSCTRL_VALUE);	// MCLK By External OSC
    TCC250_SEND_CMD(handle, TCC250_PUP_OFF_OUTPUT_REG,		0x0F);	// DAC on HP on

    TCC250_SEND_CMD(handle, TCC250_LRMUTE_REG, 			0x1C);

}
void TCC250_SetAudioInVolume(void *handle, unsigned char ucVolume)
{

    const unsigned char TCC250_HP_IN_VOLUME[41]= {
        0x5D,	// Mute <= 0xB2
        0xD4,	// Vol 01 : -27dB
        0xD6,	// Vol 02 : -26dB
        0xD8,	// Vol 03 : -25dB
        0xDA,	// Vol 04 : -24dB
        0xDC,	// Vol 05 : -23dB
        0xDE,	// Vol 06 : -22dB
        0xE0,	// Vol 07 : -21dB
        0xE2,	// Vol 08 : -20dB
        0xE4,	// Vol 09 : -19dB
        0xE6,	// Vol 10 : -18dB
        0xE8,	// Vol 11 : -17dB
        0xEA,	// Vol 12 : -16dB
        0xEC,	// Vol 13 : -15dB
        0xEE,	// Vol 14 : -14dB
        0xF0,	// Vol 15 : -13dB
        0xF2,	// Vol 16 : -12dB
        0xF4,	// Vol 17 : -11dB
        0xF6,	// Vol 18 : -10dB
        0xF7,	// Vol 19 : -9dB
        0xF8,	// Vol 20 : -8dB
        0xF9,	// Vol 21 : -7dB
        0xFA,	// Vol 22 : -6dB
        0xFB,	// Vol 23 : -5dB
        0xFC,	// Vol 24 : -4dB
        0xFD,	// Vol 25 : -3dB
        0xFE,	// Vol 26 : -2dB
        0xFF,	// Vol 27 : -1dB
        0x00,	// Vol 28 : 0 dB
        0x01,	// Vol 29 : + 1dB
        0x02,	// Vol 30 : + 2dB
        0x03,	// Vol 31 : + 3dB
        0x04,	// Vol 32 : + 4dB
        0x05,	// Vol 33 : + 5dB
        0x06,	// Vol 34 : + 6dB
        0x07,	// Vol 35 : + 7dB
        0x08,	// Vol 36 : + 8dB
        0x09,	// Vol 37 : + 9dB
        0x0a,	// Vol 38 : + 10dB
        0x0b,	// Vol 39 : + 11dB
        0x0c,	// Vol 40 : + 12dB
    };

    TCC250_SEND_CMD(handle, TCC250_LINVOL_REG, 0x80+TCC250_HP_IN_VOLUME[ucVolume]); // +12db MAX +12DB
    TCC250_SEND_CMD(handle, TCC250_RINVOL_REG, 0x80+TCC250_HP_IN_VOLUME[ucVolume]);// +12db MAX +12DB

}

int TCC250_GetAudioInVolume(void *handle)
{
	unsigned int retVal = 0;
	int i = 0;

	const unsigned char TCC250_HP_IN_VOLUME[41]= {
        0x5D,	// Mute <= 0xB2
        0xD4,	// Vol 01 : -27dB
        0xD6,	// Vol 02 : -26dB
        0xD8,	// Vol 03 : -25dB
        0xDA,	// Vol 04 : -24dB
        0xDC,	// Vol 05 : -23dB
        0xDE,	// Vol 06 : -22dB
        0xE0,	// Vol 07 : -21dB
        0xE2,	// Vol 08 : -20dB
        0xE4,	// Vol 09 : -19dB
        0xE6,	// Vol 10 : -18dB
        0xE8,	// Vol 11 : -17dB
        0xEA,	// Vol 12 : -16dB
        0xEC,	// Vol 13 : -15dB
        0xEE,	// Vol 14 : -14dB
        0xF0,	// Vol 15 : -13dB
        0xF2,	// Vol 16 : -12dB
        0xF4,	// Vol 17 : -11dB
        0xF6,	// Vol 18 : -10dB
        0xF7,	// Vol 19 : -9dB
        0xF8,	// Vol 20 : -8dB
        0xF9,	// Vol 21 : -7dB
        0xFA,	// Vol 22 : -6dB
        0xFB,	// Vol 23 : -5dB
        0xFC,	// Vol 24 : -4dB
        0xFD,	// Vol 25 : -3dB
        0xFE,	// Vol 26 : -2dB
        0xFF,	// Vol 27 : -1dB
        0x00,	// Vol 28 : 0 dB
        0x01,	// Vol 29 : + 1dB
        0x02,	// Vol 30 : + 2dB
        0x03,	// Vol 31 : + 3dB
        0x04,	// Vol 32 : + 4dB
        0x05,	// Vol 33 : + 5dB
        0x06,	// Vol 34 : + 6dB
        0x07,	// Vol 35 : + 7dB
        0x08,	// Vol 36 : + 8dB
        0x09,	// Vol 37 : + 9dB
        0x0a,	// Vol 38 : + 10dB
        0x0b,	// Vol 39 : + 11dB
        0x0c,	// Vol 40 : + 12dB
    };


	retVal = TCC250_READ_CMD(handle, TCC250_LINVOL_REG) - 0x80;

	for(i = 0; i < 41; i++)
	{

		if(TCC250_HP_IN_VOLUME[i] == retVal)
			return i;
	}

	return  0;

}

/*****************************************************************************
 * Function Name : TCC250_SetAudioOutVolume()
 ******************************************************************************
 * Desription   	: The function for H/W Codec Output Volume Control function
 * Parameter   	: ucVolume (0 ~ 40)
 * Return        : None
 ******************************************************************************/
void TCC250_SetAudioOutVolume(void *handle, unsigned char ucVolume)
{
    const unsigned char TCC250_HP_OUT_VOLUME[41]= {
        0xB2,	// Mute <= 0xB2
        0xB8,	// Vol 01 : -72dB
        0xBB,	// Vol 02 : -69dB
        0xBE,	// Vol 03 : -66dB
        0xC1,	// Vol 04 : -63dB
        0xC4,	// Vol 05 : -60dB
        0xC6,	// Vol 06 : -58dB
        0xC8,	// Vol 07 : -56dB
        0xCA,	// Vol 08 : -54dB
        0xCC,	// Vol 09 : -52dB
        0xCE,	// Vol 10 : -50dB
        0xD0,	// Vol 11 : -48dB
        0xD2,	// Vol 12 : -46dB
        0xD4,	// Vol 13 : -44dB
        0xD6,	// Vol 14 : -42dB
        0xD8,	// Vol 15 : -40dB
        0xDA,	// Vol 16 : -38dB
        0xDC,	// Vol 17 : -36dB
        0xDE,	// Vol 18 : -34dB
        0xE0,	// Vol 19 : -32dB
        0xE2,	// Vol 20 : -30dB
        0xE4,	// Vol 21 : -28dB
        0xE6,	// Vol 22 : -26dB
        0xE8,	// Vol 23 : -24dB
        0xEA,	// Vol 24 : -22dB
        0xEC,	// Vol 25 : -20dB
        0xEE,	// Vol 26 : -18dB
        0xF0,	// Vol 27 : -16dB
        0xF2,	// Vol 28 : -14dB
        0xF4,	// Vol 29 : -12dB
        0xF6,	// Vol 30 : -10dB
        0xF7,	// Vol 31 : - 9dB
        0xF8,	// Vol 32 : - 8dB
        0xF9,	// Vol 33 : - 7dB
        0xFA,	// Vol 34 : - 6dB
        0xFB,	// Vol 35 : - 5dB
        0xFC,	// Vol 36 : - 4dB
        0xFD,	// Vol 37 : - 3dB
        0xFE,	// Vol 38 : - 2dB
        0xFF,	// Vol 39 : - 1dB
        0x00,	// Vol 40 : + 0dB
    };
    if(ucVolume > 40) ucVolume = 40;	// Value Limit is 40

    TCC250_SEND_CMD(handle, TCC250_HSOUTL_REG, TCC250_HP_OUT_VOLUME[ucVolume]);
    TCC250_SEND_CMD(handle, TCC250_HSOUTR_REG, TCC250_HP_OUT_VOLUME[ucVolume]);

}

int TCC250_GetAudioOutVolume(void *handle)
{
	unsigned int retVal = 0, i = 0;
    const unsigned char TCC250_HP_OUT_VOLUME[41]= {
        0xB2,	// Mute <= 0xB2
        0xB8,	// Vol 01 : -72dB
        0xBB,	// Vol 02 : -69dB
        0xBE,	// Vol 03 : -66dB
        0xC1,	// Vol 04 : -63dB
        0xC4,	// Vol 05 : -60dB
        0xC6,	// Vol 06 : -58dB
        0xC8,	// Vol 07 : -56dB
        0xCA,	// Vol 08 : -54dB
        0xCC,	// Vol 09 : -52dB
        0xCE,	// Vol 10 : -50dB
        0xD0,	// Vol 11 : -48dB
        0xD2,	// Vol 12 : -46dB
        0xD4,	// Vol 13 : -44dB
        0xD6,	// Vol 14 : -42dB
        0xD8,	// Vol 15 : -40dB
        0xDA,	// Vol 16 : -38dB
        0xDC,	// Vol 17 : -36dB
        0xDE,	// Vol 18 : -34dB
        0xE0,	// Vol 19 : -32dB
        0xE2,	// Vol 20 : -30dB
        0xE4,	// Vol 21 : -28dB
        0xE6,	// Vol 22 : -26dB
        0xE8,	// Vol 23 : -24dB
        0xEA,	// Vol 24 : -22dB
        0xEC,	// Vol 25 : -20dB
        0xEE,	// Vol 26 : -18dB
        0xF0,	// Vol 27 : -16dB
        0xF2,	// Vol 28 : -14dB
        0xF4,	// Vol 29 : -12dB
        0xF6,	// Vol 30 : -10dB
        0xF7,	// Vol 31 : - 9dB
        0xF8,	// Vol 32 : - 8dB
        0xF9,	// Vol 33 : - 7dB
        0xFA,	// Vol 34 : - 6dB
        0xFB,	// Vol 35 : - 5dB
        0xFC,	// Vol 36 : - 4dB
        0xFD,	// Vol 37 : - 3dB
        0xFE,	// Vol 38 : - 2dB
        0xFF,	// Vol 39 : - 1dB
        0x00,	// Vol 40 : + 0dB
    };

	retVal = TCC250_READ_CMD(handle, TCC250_HSOUTL_REG);
    printk("TCC250_HSOUTL_REG value [%u] \n", retVal);


	for(i = 0; i < 41; i++)
	{
		if(TCC250_HP_OUT_VOLUME[i] == retVal)
			return i;
	}
	return  0;
}

