/*
 * linux/sound/soc/tcc/tcc-i2s.c
 *
 * Based on:    linux/sound/soc/pxa/pxa2xx-i2s.h
 * Author: Liam Girdwood<liam.girdwood@wolfsonmicro.com or linux@wolfsonmicro.com>
 * Rewritten by:    <linux@telechips.com>
 * Created:     12th Aug 2005   Initial version.
 * Modified:    Nov 25, 2008
 * Description: ALSA PCM interface for the Intel PXA2xx chip
 *
 * Copyright 2005 Wolfson Microelectronics PLC.
 * Copyright (C) 2008-2009 Telechips 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see the file COPYING, or write
 * to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/device.h>
#include <linux/delay.h>
#include <linux/clk.h>
#include <sound/core.h>
#include <sound/pcm.h>
#include <sound/initval.h>
#include <sound/soc.h>
#ifdef CONFIG_HAS_EARLYSUSPEND
#include <linux/earlysuspend.h>
#endif

#include <mach/hardware.h>

#if defined(CONFIG_ARCH_TCC92X)
#include <mach/irqs.h>
#include <mach/bsp.h>
#include <mach/tca_ckc.h>
#elif defined(CONFIG_ARCH_TCC93XX)
#include <mach/irqs.h>
#include <mach/bsp.h>
#include <mach/tca_ckc.h>
#endif

#include "tcc-pcm.h"
#include "tcc-i2s.h"

#ifdef CONFIG_HAS_EARLYSUSPEND
    static struct early_suspend early_suspend;
#endif	

#include "tcc/tca_tcchwcontrol.h"



#undef alsa_dbg(f, a...)
#if 0
#define alsa_dbg(f, a...)  printk("== alsa-debug == " f, ##a)
#else
#define alsa_dbg(f, a...)  
#endif



#if defined(CONFIG_ARCH_TCC92X)
static struct tcc_pcm_dma_params tcc_i2s_pcm_stereo_out = {
	.name			 = "I2S PCM Stereo out",
    .dma_addr           = 0,
	.channel			 =0,
	//		 .dma_size

};

static struct tcc_pcm_dma_params tcc_i2s_pcm_stereo_in = {
	.name				   = "I2S PCM Stereo in",
    .dma_addr           = 0,
	.channel			   =1,
	//		  .dma_size 
};

#if !defined(CONFIG_SND_TCC_DAI2SPDIF)
static struct tcc_pcm_dma_params tcc_i2s_pcm_spdif_out = {
    .name		     = "I2S PCM S/PDIF out",
    .dma_addr           = 0,
    .channel             =0,
    //       .dma_size

};
#endif

#elif defined(CONFIG_ARCH_TCC93XX)
static struct tcc_pcm_dma_params tcc_i2s_pcm_stereo_out = {
	.name               = "I2S PCM Stereo out",
    .dma_addr           = 0,
	.channel            =0,
	//		 .dma_size

};

static struct tcc_pcm_dma_params tcc_i2s_pcm_stereo_in = {
	.name               = "I2S PCM Stereo in",
    .dma_addr           = 0,
	.channel            =1,
	//		  .dma_size 
};

#if !defined(CONFIG_SND_TCC_DAI2SPDIF)
static struct tcc_pcm_dma_params tcc_i2s_pcm_spdif_out = {
    .name		     = "I2S PCM S/PDIF out",
    .dma_addr           = 0,
    .channel             =0,
    //       .dma_size

};
#endif

#endif


#if defined(CONFIG_ARCH_TCC92X)
int alsa_I2S_Init(void)
{
	volatile PADMADAI pADMA_DAI = (volatile PADMADAI)tcc_p2v(HwADMA_DAIBASE);
	volatile PPIC pPIC = (volatile PPIC)tcc_p2v(HwPIC_BASE);

	tca_i2s_dai_init(pADMA_DAI);

	tca_ckc_setiobus(RB_SPDIFTXCONTROLLER, 1);
	tca_ckc_setiobus(RB_DAICDIFCONTROLLER, 1);
	tca_ckc_setiobus(RB_ADMACONTROLLER, 1);

#if 1 /* sgbaek */
	tca_ckc_set_iobus_swreset(RB_ADMACONTROLLER, 0);
	tca_ckc_set_iobus_swreset(RB_ADMACONTROLLER, 1);
#else
	tca_ckc_setswreset(RB_ADMACONTROLLER, 0);
	tca_ckc_setswreset(RB_ADMACONTROLLER, 1);
#endif

	//Config DMA Interrupt
	BITSET(pPIC->SEL1 , 1 << (INT_ADMA - 32));
	BITSET(pPIC->MODE1, 1 << (INT_ADMA - 32));
	BITCLR(pPIC->POL1 , 1 << (INT_ADMA - 32));

	tca_i2s_stop((void *)pADMA_DAI, 0);
	tca_i2s_stop((void *)pADMA_DAI, 1);

	return 1;
}

static int alsa_spdif_init(void)
{
    clk_enable(clk_get(NULL, "spdif_clk"));

    return 0;
}

#elif defined(CONFIG_ARCH_TCC93XX)
int alsa_I2S_Init(void)
{
	volatile PADMADAI pADMA_DAI = (volatile PADMADAI)tcc_p2v(HwDAI0_BASE);
	volatile PPIC pPIC = (volatile PPIC)tcc_p2v(HwPIC_BASE);
    volatile ADMASPDIFTX *p_adma_spdif_tx_base = (volatile ADMASPDIFTX *)tcc_p2v(HwSPDIF0TX_BASE);

    tca_i2s_dai_init(pADMA_DAI);
    
    clk_enable(clk_get(NULL, "dai0"));
    clk_enable(clk_get(NULL, "adma0"));

	//Config DMA Interrupt
	BITSET(pPIC->SEL1 , 1 << (INT_ADMA0 - 32));
	BITSET(pPIC->MODE1, 1 << (INT_ADMA0 - 32));
	BITCLR(pPIC->POL1 , 1 << (INT_ADMA0 - 32));

	tca_i2s_stop((void *)pADMA_DAI, 0);         // sgbaek, check?
	tca_i2s_stop((void *)pADMA_DAI, 1);

    alsa_dbg("== alsa-debug == %s() \n", __func__);
	return 1;
}

static int alsa_spdif_init(void)
{
    clk_enable(clk_get(NULL, "spdif0"));
    return 0;
}

#endif


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Function:		StartI2SClock()

Description:	Enables the I2S clock that drives the audio codec chip.

Returns:		N/A
-------------------------------------------------------------------*/
static int tcc_i2s_startup(struct snd_pcm_substream *substream, struct snd_soc_dai *dai)
{
    struct snd_soc_pcm_runtime *rtd = substream->private_data;
    struct snd_soc_dai *cpu_dai = rtd->dai->cpu_dai;

    if (!cpu_dai->active) {
//        alsa_I2S_Init();      // It doesn't need. sgbaek
    }

    return 0;
}

static int tcc_i2s_set_dai_fmt(struct snd_soc_dai *cpu_dai, unsigned int fmt)
{

    switch (fmt & SND_SOC_DAIFMT_MASTER_MASK) {
    case SND_SOC_DAIFMT_CBS_CFS:
        break;
    case SND_SOC_DAIFMT_CBM_CFS:
        break;
    default:
        break;
    }


    /* interface format */
    switch (fmt & SND_SOC_DAIFMT_FORMAT_MASK) {
    case SND_SOC_DAIFMT_I2S:
        break;
    case SND_SOC_DAIFMT_LEFT_J:
        break;
    }
    return 0;
}

static int tcc_i2s_set_dai_sysclk(struct snd_soc_dai *cpu_dai,
                                      int clk_id, unsigned int freq, int dir)
{
    if (clk_id != TCC_I2S_SYSCLK)
        return -ENODEV;  

    return 0;
}
static int tcc_i2s_hw_params(struct snd_pcm_substream *substream,
                                 struct snd_pcm_hw_params *params, struct snd_soc_dai *dai)
{
    struct snd_soc_pcm_runtime *rtd = substream->private_data;
    struct snd_soc_dai *cpu_dai = rtd->dai->cpu_dai;

    if (substream->pcm->device == __I2S_DEV_NUM__) {
        cpu_dai->dma_data = (substream->stream == SNDRV_PCM_STREAM_PLAYBACK) ?
            &tcc_i2s_pcm_stereo_out : &tcc_i2s_pcm_stereo_in;
#if defined(CONFIG_SND_TCC_DAI2SPDIF)
        tea_i2s_setclock(params_rate(params),
                         (substream->stream == SNDRV_PCM_STREAM_PLAYBACK) ? 0 : 1);
#else
        tea_i2s_setclock(params_rate(params));
#endif
    }

    return 0;
}

#if !defined(CONFIG_SND_TCC_DAI2SPDIF)
static int tcc_spdif_hw_params(struct snd_pcm_substream *substream,
                                 struct snd_pcm_hw_params *params, struct snd_soc_dai *dai)
{
    struct snd_soc_pcm_runtime *rtd = substream->private_data;
    struct snd_soc_dai *cpu_dai = rtd->dai->cpu_dai;

    if (substream->pcm->device == __SPDIF_DEV_NUM__) {
        cpu_dai->dma_data = &tcc_i2s_pcm_spdif_out;
        tea_spdif_setclock(params_rate(params));
    }
    return 0;
}
#endif

static int tcc_i2s_trigger(struct snd_pcm_substream *substream, int cmd, struct snd_soc_dai *dai)
{
    int ret = 0;
#if defined(CONFIG_ARCH_TCC92X)
//    volatile PADMADAI pDAI = (volatile PADMADAI)tcc_p2v(HwADMA_DAIBASE);
#elif defined(CONFIG_ARCH_TCC93XX)
//    volatile PADMADAI pDAI = (volatile PADMADAI)tcc_p2v(HwDAI0_BASE);
#endif

    switch (cmd) {
    case SNDRV_PCM_TRIGGER_START:
    case SNDRV_PCM_TRIGGER_RESUME:
    case SNDRV_PCM_TRIGGER_PAUSE_RELEASE:
        //tca_i2s_start(substream->stream == SNDRV_PCM_STREAM_PLAYBACK ? 0 : 1);        		
#if defined(CONFIG_ARCH_TCC92X)
//		BITSET(pDAI->DAMR, Hw15);
#elif defined(CONFIG_ARCH_TCC93XX)
//		BITSET(pDAI->DAMR, Hw15);
#else
        BITSET(HwDAMR, HwDAMR_EN_ON);
#endif		
        break;

    case SNDRV_PCM_TRIGGER_STOP:
    case SNDRV_PCM_TRIGGER_SUSPEND:
    case SNDRV_PCM_TRIGGER_PAUSE_PUSH:
        //tca_i2s_stop(substream->stream == SNDRV_PCM_STREAM_PLAYBACK ? 0 : 1);
        if (!(tcc_alsa_info.flag)) {
#if defined(CONFIG_ARCH_TCC92X)			
//            BITCLR(pDAI->DAMR, Hw15);
#elif defined(CONFIG_ARCH_TCC93XX)
//            BITCLR(pDAI->DAMR, Hw15);
#else
			HwDAMR &= ~HwDAMR_EN_ON;
#endif
        }
        break;

    default:
        ret = -EINVAL;
    }
    return ret;
}

static void tcc_i2s_shutdown(struct snd_pcm_substream *substream, struct snd_soc_dai *dai)
{
    if (substream->stream == SNDRV_PCM_STREAM_PLAYBACK) {        
    } else {        
    }
}

unsigned int i2s_clk_freq=0; 

#ifdef CONFIG_PM
static int tcc_i2s_suspend(struct snd_soc_dai *dai)
{
    return 0;
}
static int tcc_i2s_resume(struct snd_soc_dai *dai)
{
    return 0;
}
#else

#define tcc_i2s_suspend	NULL
#define tcc_i2s_resume	NULL

#endif

#define TCC_I2S_RATES (SNDRV_PCM_RATE_8000 | SNDRV_PCM_RATE_11025 |\
                           SNDRV_PCM_RATE_16000 | SNDRV_PCM_RATE_22050 |\
                           SNDRV_PCM_RATE_32000 |SNDRV_PCM_RATE_44100 |\
                           SNDRV_PCM_RATE_48000 )


#ifdef CONFIG_HAS_EARLYSUSPEND
static void tcc92xx_i2s_early_suspend(struct early_suspend *h);
static void tcc92xx_i2s_late_resume(struct early_suspend *h);
#endif


                           
int tcc_i2s_probe(struct platform_device *pdev, struct snd_soc_dai *dai)
{

#if defined(CONFIG_ARCH_TCC92X)
	((void)(dai));
#endif

    alsa_dbg("== alsa-debug == %s() \n", __func__);
    alsa_I2S_Init();

#if defined(CONFIG_SND_TCC_DAI2SPDIF)
    tea_i2s_setclock(44100, 1);
    alsa_spdif_init();
    tea_spdif_setclock(44100);
#else
    tea_i2s_setclock(44100);
#endif

#ifdef CONFIG_HAS_EARLYSUSPEND
	early_suspend.suspend = tcc92xx_i2s_early_suspend;
	early_suspend.resume  = tcc92xx_i2s_late_resume;
	early_suspend.level   = EARLY_SUSPEND_LEVEL_BLANK_SCREEN;
	//register_early_suspend(&early_suspend);
#endif    
    
    return 0;
}

#if !defined(CONFIG_SND_TCC_DAI2SPDIF)
int tcc_spdif_probe(struct platform_device *pdev, struct snd_soc_dai *dai)
{
    ((void)(dai));

    alsa_dbg("== alsa-debug == %s() \n", __func__);
    alsa_spdif_init();
    tea_spdif_setclock(44100);
    return 0;
}
#endif

struct snd_soc_dai tcc_i2s_dai[] = {
    [__I2S_DEV_NUM__] = {
        .name = "tcc-i2s",
        .id = 0,
        .suspend = tcc_i2s_suspend,
        .resume = tcc_i2s_resume,
        .playback = {
            .channels_min = 2,
            .channels_max = 2,
            .rates = TCC_I2S_RATES,
            .formats = SNDRV_PCM_FMTBIT_S16_LE | SNDRV_PCM_FMTBIT_IEC958_SUBFRAME_LE,},  //should be change? phys:32 width:16
        .capture = {
            .channels_min = 2,
            .channels_max = 2,
            .rates = TCC_I2S_RATES,
            .formats = SNDRV_PCM_FMTBIT_S16_LE,}, //should be change? phys:32 width:16
        .ops = {
            .startup = tcc_i2s_startup,
            .shutdown = tcc_i2s_shutdown,
            .trigger = tcc_i2s_trigger,
            .hw_params = tcc_i2s_hw_params,
            .set_fmt = tcc_i2s_set_dai_fmt,
            .set_sysclk = tcc_i2s_set_dai_sysclk,
        },
        .probe = tcc_i2s_probe,
    },
#if !defined(CONFIG_SND_TCC_DAI2SPDIF)
    [__SPDIF_DEV_NUM__] = {
        .name = "tcc-spdif",
        .id = 1,
        .suspend = tcc_i2s_suspend,
        .resume = tcc_i2s_resume,
        .playback = {
            .channels_min = 2,
            .channels_max = 2,
            .rates = TCC_I2S_RATES,
            .formats = SNDRV_PCM_FMTBIT_S16_LE | SNDRV_PCM_FMTBIT_IEC958_SUBFRAME_LE,},  //should be change? phys:32 width:16
        .ops = {
            .startup = tcc_i2s_startup,
            .shutdown = tcc_i2s_shutdown,
            .trigger = tcc_i2s_trigger,
            .hw_params = tcc_spdif_hw_params,
            .set_fmt = tcc_i2s_set_dai_fmt,
            .set_sysclk = tcc_i2s_set_dai_sysclk,
        },
        .probe = tcc_spdif_probe,
    }
#endif
};


static int __init tcc_i2s_init(void)
{
	return snd_soc_register_dais(tcc_i2s_dai, ARRAY_SIZE(tcc_i2s_dai));
}
module_init(tcc_i2s_init);

static void __exit tcc_i2s_exit(void)
{
	snd_soc_unregister_dais(tcc_i2s_dai, ARRAY_SIZE(tcc_i2s_dai));
}
module_exit(tcc_i2s_exit);


#ifdef CONFIG_HAS_EARLYSUSPEND
static void tcc92xx_i2s_early_suspend(struct early_suspend *h)
{
	alsa_dbg(" %s\n", __func__);
	tcc_i2s_dai[0].suspend;
#if !defined(CONFIG_SND_TCC_DAI2SPDIF)
	tcc_i2s_dai[1].suspend;
#endif
}

static void tcc92xx_i2s_late_resume(struct early_suspend *h)
{
	alsa_dbg(" %s\n", __func__);
	tcc_i2s_dai[0].resume;
#if !defined(CONFIG_SND_TCC_DAI2SPDIF)
	tcc_i2s_dai[1].resume;
#endif
}

#endif


EXPORT_SYMBOL_GPL(tcc_i2s_dai);

/* Module information */
MODULE_AUTHOR("linux <linux@telechips.com>");
MODULE_DESCRIPTION("Telechips TCC I2S SoC Interface");
MODULE_LICENSE("GPL");

