/*
 * linux/sound/soc/tcc/tcc-pcm.c  
 *
 * Based on:    linux/sound/arm/pxa2xx-pcm.c
 * Author:  <linux@telechips.com>
 * Created: Nov 30, 2004
 * Modified:    Nov 25, 2008
 * Description: ALSA PCM interface for the Intel PXA2xx chip
 *
 * Copyright:	MontaVista Software, Inc.
 * Copyright (C) 2008-2009 Telechips 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see the file COPYING, or write
 * to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/slab.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>

#include <linux/delay.h>
#include <sound/core.h>
#include <sound/pcm.h>
#include <sound/pcm_params.h>
#include <sound/soc.h>

#include <asm/dma.h>
#include <mach/hardware.h>
#if defined(CONFIG_ARCH_TCC83X)
#include <mach/tcc83x_virt.h>
#elif defined(CONFIG_ARCH_TCC79X)
#include <mach/tcc79x.h>
#endif

#include "tcc-i2s.h"
#include "tcc-pcm.h"

#include <asm/plat-tcc/tcc_ckc.h>
#include "./tca_tcchwcontrol.h"

#if defined(CONFIG_ARCH_TCC83X)
#define TCC_DMA_TYPE IRQ_ADMA
#elif defined(CONFIG_ARCH_TCC79X)
#define TCC_DMA_TYPE IRQ_DMA
#endif
#define alsa_dbg(f, a...)

#ifdef CONFIG_SND_TCC_LOOPBACK
#include "tcc-loopback.h"
static tcc_alsa_lb_handle_t *gs_p_alb_handle = NULL;
#endif

tcc_interrupt_info_x tcc_alsa_info;
EXPORT_SYMBOL(tcc_alsa_info);

static const struct snd_pcm_hardware tcc_pcm_hardware_play = {
    .info = (SNDRV_PCM_INFO_MMAP
             | SNDRV_PCM_INFO_MMAP_VALID
             | SNDRV_PCM_INFO_INTERLEAVED
             | SNDRV_PCM_INFO_BLOCK_TRANSFER
             | SNDRV_PCM_INFO_PAUSE
             | SNDRV_PCM_INFO_RESUME),

    .formats = (SNDRV_PCM_FMTBIT_S16_LE | SNDRV_PCM_FMTBIT_S24_LE),
    .rates      = SNDRV_PCM_RATE_8000_48000,
    .rate_min   = 8000,
    .rate_max   = 48000, 
    .channels_min = 2,
    .channels_max = 2,                                

#if 0
    .period_bytes_min = 16,
    .period_bytes_max = __play_buf_size,
    .periods_min = __play_buf_cnt,
    .periods_max = __play_buf_cnt,
    .buffer_bytes_max = __play_buf_cnt * __play_buf_size,
    .fifo_size = 16,  //ignored
#else
    .period_bytes_min = 128,
    .period_bytes_max = __play_buf_cnt * __play_buf_size,
    .periods_min = 1,
    .periods_max = __play_buf_cnt,
    .buffer_bytes_max = __play_buf_cnt * __play_buf_size,
    .fifo_size = 16,  //ignored
#endif
};

static const struct snd_pcm_hardware tcc_pcm_hardware_capture = {
    .info = (SNDRV_PCM_INFO_MMAP
             | SNDRV_PCM_INFO_MMAP_VALID
             | SNDRV_PCM_INFO_INTERLEAVED
             | SNDRV_PCM_INFO_BLOCK_TRANSFER
             | SNDRV_PCM_INFO_PAUSE
             | SNDRV_PCM_INFO_RESUME),

    .formats = SNDRV_PCM_FMTBIT_S16_LE | SNDRV_PCM_FMTBIT_S24_LE,
    .rates      = SNDRV_PCM_RATE_8000_48000,
    .rate_min   = 8000,
    .rate_max   = 48000, 
    .channels_min = 2,
    .channels_max = 2,                                

#if 0
    .period_bytes_min = 16,
    .period_bytes_max = __cap_buf_size,
    .periods_min = __cap_buf_cnt,
    .periods_max = __cap_buf_cnt,
    .buffer_bytes_max = __cap_buf_cnt * __cap_buf_size,
    .fifo_size = 16, //ignored
#else
    .period_bytes_min = 128,
    .period_bytes_max = __cap_buf_cnt * __cap_buf_size,
    .periods_min = 1,
    .periods_max = __cap_buf_cnt,
    .buffer_bytes_max = __cap_buf_cnt * __cap_buf_size,
    .fifo_size = 16, //ignored
#endif
};

/*** tea module ***/
static unsigned int io_ckc_get_dai_clock(unsigned int freq)
{
    switch (freq) {
    default:
    case 44100: return ((441 * 256) / 10);
    case 22000: return (((22050 * 256) / 100) / 10);
    case 11000: return (((11025 * 256) / 100) / 10);
    case 22050:
    case 11025:
    case 48000:
    case 24000:
    case 12000:
    case  8000:
    case 32000:
    case 16000: return (((freq * 256) / 100) / 10);
    }
    return 0;
}

static int alsa_get_intr_num(struct snd_pcm_substream *substream)
{
    if (substream) {
#if defined(CONFIG_ARCH_TCC83X)
        return IRQ_ADMA;
#elif defined(CONFIG_ARCH_TCC79X)
        return (substream->stream == SNDRV_PCM_STREAM_PLAYBACK) ? INT_DMA0_CH0 : INT_DMA0_CH1;
#endif
    }
    return -1;
}
void tea_i2s_setclock(unsigned int ClockRate)
{
    unsigned int ckc = 0;
    ckc = io_ckc_get_dai_clock(ClockRate);
    tca_ckc_com_setperi(PERI_DAI, KHz, ckc, DIRECTPLL0);
}
EXPORT_SYMBOL(tea_i2s_setclock);


static void set_dma_outbuffer(unsigned int addr, unsigned int length, unsigned int period)
{
    unsigned long dma_buffer = 0;

    alsa_dbg("%s, addr[0x%08X], len[%u], period[%u]\n", __func__, addr, length, period);
#if defined(CONFIG_ARCH_TCC83X)
    HwChCtrl &= ~HwChCtrl_DTEN_EN;

    dma_buffer = 0xFFFFFF00 / (((tcc_pcm_hardware_play.buffer_bytes_max) >> 4) << 8);
    dma_buffer = dma_buffer * (((tcc_pcm_hardware_play.buffer_bytes_max) >> 4) << 8);

    HwTxDaParam = dma_buffer | 4;

    // generate interrupt when half of buffer was played
    HwTxDaTCnt = (period >> 0x04) - 1;

    alsa_dbg("[%s] HwTxDaParam [0x%X]\n", __func__, (int)(dma_buffer | 4));
    alsa_dbg("[%s] HwTxDaTCnt [%d]\n", __func__, ((period) >> 0x04) - 1);
    HwTxDaSar = addr;

    HwTransCtrl |= (HwTransCtrl_TCN_EN
                    | HwTransCtrl_DTRPT_EN
                    | HwTransCtrl_DTBSIZE_4
                    | HwTransCtrl_DTWSIZE_32
                    | HwTransCtrl_DTLCK);

    HwRptCtrl = 0;
    BITCLR(HwRptCtrl, HwRptCtrl_DRI_EN);
    if (length < tcc_pcm_hardware_play.buffer_bytes_max)
        BITSET(HwRptCtrl, (length / period) - 1);

    HwChCtrl |= (HwChCtrl_DTEN_EN
                 | HwChCtrl_DTDW_16
                 | HwChCtrl_DTIEN);
#elif defined(CONFIG_ARCH_TCC79X)
    Tx_CHCTRL &= ~HwCHCTRL0_EN_EN;

    Tx_ST_SADR  = addr;    
    Tx_ST_DADR = (int)&HwDADO_L0;

    Tx_DPARAM =  0xFFFFFE00 | 4; 
    dma_buffer = 0xFFFFFF00 / (((tcc_pcm_hardware_play.buffer_bytes_max) >> 4) << 8);
    dma_buffer = dma_buffer * (((tcc_pcm_hardware_play.buffer_bytes_max) >> 4) << 8);
    Tx_SPARAM = dma_buffer | 2;

    // generate interrupt when period was played
    Tx_HCOUNT = (period >> 0x04);

    alsa_dbg("[%s] HwTxDaParam [0x%X]\n", __func__, (int)(dma_buffer | 4));
    alsa_dbg("[%s] HwTxDaTCnt [%d]\n", __func__, ((period) >> 0x04));

    Tx_EXTREQ = HwEXTREQ0_DAIT;

    Tx_CHCTRL |= (HwCHCTRL0_CONT_C
                  | HwCHCTRL0_SYNC_EN
                  | HwCHCTRL0_LOCK_EN 
                  | HwCHCTRL0_TYPE_SE
                  // | HwCHCTRL0_TYPE_SL
                  | HwCHCTRL0_BSIZE_8
                  | HwCHCTRL0_WSIZE_16
                  | HwCHCTRL0_FLAG
                  | HwCHCTRL0_IEN_EN
                  | HwCHCTRL0_REP_EN); 

    Tx_RPTCTRL = 0;
    BITCLR(Tx_RPTCTRL, HwRPTCTRL0_DRI_LAST);    
    if (length < tcc_pcm_hardware_play.buffer_bytes_max)
        BITSET(Tx_RPTCTRL, (length / period) - 1);

    Tx_CHCTRL |= HwCHCTRL0_EN_EN;
#endif
}

static void set_dma_inbuffer(unsigned int addr, unsigned int length, unsigned int period)
{
    unsigned long dma_buffer = 0;

    alsa_dbg("%s, addr[0x%08X], len[%d]\n", __func__, addr, length);

#if defined(CONFIG_ARCH_TCC83X)
    HwChCtrl &= ~HwChCtrl_DREN_EN;

    dma_buffer = 0xFFFFFF00 / (((tcc_pcm_hardware_capture.buffer_bytes_max) >> 4) << 8);
    dma_buffer = dma_buffer * (((tcc_pcm_hardware_capture.buffer_bytes_max) >> 4) << 8);

    //dma_buffer = 0xFFFFF8000;
    HwRxDaParam = dma_buffer | 4;

    // generate interrupt when half of buffer was filled
    HwRxDaTCnt = (period >> 0x04) - 1;

    alsa_dbg("[%s] HwRxDaParam [0x%X]\n", __func__, (int)dma_buffer | 4);
    alsa_dbg("[%s] HwRxDaTCnt [%d]\n", __func__, ((period) >> 0x04) - 1);
    HwRxDaDar = addr;

    HwTransCtrl |= (HwTransCtrl_RCN_EN
                    | HwTransCtrl_DRRPT_EN
                    | HwTransCtrl_DRBSIZE_4
                    | HwTransCtrl_DRWSIZE_32
                    | HwTransCtrl_DRLCK);

    HwRptCtrl = 0;
    BITCLR(HwRptCtrl,HwRptCtrl_DRI_EN);

    if (length<tcc_pcm_hardware_capture.buffer_bytes_max)
        BITSET(HwRptCtrl, (length / period) - 1);
    HwChCtrl |= (HwChCtrl_DREN_EN
                 | HwChCtrl_DRDW_16
                 | HwChCtrl_DRIEN);
#elif defined(CONFIG_ARCH_TCC79X)
    Rx_CHCTRL &= ~HwCHCTRL0_EN_EN;    

    Rx_ST_DADR = addr;    
    Rx_ST_SADR= (int)&HwDADI_L0;

    //Set Source Address & Parameter(mask + increment)
    Rx_SPARAM = 0xFFFFFE00 | 4;
    dma_buffer = 0xFFFFFF00 / (((tcc_pcm_hardware_capture.buffer_bytes_max) >> 4) << 8);
    dma_buffer = dma_buffer * (((tcc_pcm_hardware_capture.buffer_bytes_max) >> 4) << 8);
    Rx_DPARAM = dma_buffer | 2;


    // generate interrupt when period was filled
    Rx_HCOUNT = (period >> 0x04);

    alsa_dbg("[%s] HwRxDaParam [0x%X]\n", __func__, (int)dma_buffer | 4);
    alsa_dbg("[%s] HwRxDaTCnt [%d]\n", __func__, ((period) >> 0x04));

    Rx_EXTREQ = HwEXTREQ0_DAIR;

    Rx_CHCTRL |= (HwCHCTRL0_CONT_C
                  | HwCHCTRL0_SYNC_EN
                  | HwCHCTRL0_LOCK_EN
                  | HwCHCTRL0_TYPE_SE
                  // | HwCHCTRL0_TYPE_SL
                  | HwCHCTRL0_BSIZE_8
                  | HwCHCTRL0_WSIZE_16
                  | HwCHCTRL0_FLAG
                  | HwCHCTRL0_IEN_EN
                  | HwCHCTRL0_REP_EN); 

    Rx_RPTCTRL = 0;
    BITCLR(Rx_RPTCTRL, HwRPTCTRL0_DRI_LAST);    

    if (length < tcc_pcm_hardware_capture.buffer_bytes_max)
        BITSET(Rx_RPTCTRL, (length / period) - 1);
    
    Rx_CHCTRL |= HwCHCTRL0_EN_EN;
#endif
}

static irqreturn_t tcc_dma_done_handler(int irq, void *dev_id)
{
    struct snd_pcm_runtime *runtime;
    struct tcc_runtime_data *prtd;
    dma_addr_t cur_ptr;
    int dmaInterruptSource = 0;

#if defined(CONFIG_ARCH_TCC83X)
    unsigned long reg_temp = 0;                 

    if (HwIntStatus & (Hw4 | Hw0)) {
        dmaInterruptSource |= DMA_CH_OUT;
        reg_temp |= Hw4 | Hw0;
    }
    if (HwIntStatus & (Hw6 | Hw2)) {
        dmaInterruptSource |= DMA_CH_IN;
        reg_temp |= Hw6 | Hw2;
    }

    if (reg_temp) { 
        HwIntStatus |= reg_temp;
    }

    if ((dmaInterruptSource & DMA_CH_OUT)
        && (tcc_alsa_info.flag & TCC_RUNNING_PLAY)) {
        snd_pcm_period_elapsed(tcc_alsa_info.play_ptr);

        runtime = tcc_alsa_info.play_ptr->runtime;
        prtd = (struct tcc_runtime_data *) runtime->private_data;

        cur_ptr = HwTxDaCsar; 
        if (prtd->dma_buffer_end <= cur_ptr) {           
            HwChCtrl |= HwChCtrl_DTEN_EN;
        }
    }
    if ((dmaInterruptSource & DMA_CH_IN)
        && (tcc_alsa_info.flag & TCC_RUNNING_CAPTURE)) {
        snd_pcm_period_elapsed(tcc_alsa_info.cap_ptr);

        runtime = tcc_alsa_info.cap_ptr->runtime;
        prtd = (struct tcc_runtime_data *)runtime->private_data;

        cur_ptr = HwRxDaCdar; 
        if (prtd->dma_buffer_end <= cur_ptr) {           
            HwChCtrl |= HwChCtrl_DREN_EN;
        }       
    }
#else
    if (Tx_CHCONFIG & HwCHCONFIG_IS0_1)  // output interrupt           
        dmaInterruptSource |= DMA_CH_OUT; 
    if (Rx_CHCONFIG & HwCHCONFIG_IS1_1)  // input interrupt
        dmaInterruptSource |= DMA_CH_IN;

    if ((dmaInterruptSource & DMA_CH_OUT)
        && (tcc_alsa_info.flag & TCC_RUNNING_PLAY)) {

        runtime = tcc_alsa_info.play_ptr->runtime;
        prtd = (struct tcc_runtime_data *) runtime->private_data;

        cur_ptr = Tx_C_SADR;

        snd_pcm_period_elapsed(tcc_alsa_info.play_ptr);


        if (prtd->dma_buffer_end <= cur_ptr) {           
            Tx_CHCTRL |= HwCHCTRL0_EN_EN | HwCHCTRL0_FLAG;
        } else {
            //Tx_CHCTRL |= HwCHCTRL0_FLAG | HwCHCTRL0_EN_EN;
            Tx_CHCTRL |= HwCHCTRL0_FLAG;
        }

    }

    if ((dmaInterruptSource & DMA_CH_IN)
        && (tcc_alsa_info.flag & TCC_RUNNING_CAPTURE)) {
        snd_pcm_period_elapsed(tcc_alsa_info.cap_ptr);

        runtime = tcc_alsa_info.cap_ptr->runtime;
        prtd = (struct tcc_runtime_data *)runtime->private_data;

        // Rx_CHCTRL |= HwCHCTRL0_FLAG;       

        cur_ptr = Rx_C_DADR; 		
        if (prtd->dma_buffer_end <= cur_ptr) {           
            // HwChCtrl |= HwChCtrl_DREN_EN;
            Rx_CHCTRL |= HwCHCTRL0_EN_EN | HwCHCTRL0_FLAG;
        } else {
            Rx_CHCTRL |= HwCHCTRL0_FLAG | HwCHCTRL0_EN_EN;
        }
    }
#endif

    return IRQ_HANDLED;
}


static int tcc_pcm_hw_params(struct snd_pcm_substream *substream, struct snd_pcm_hw_params *params)
{
    struct snd_pcm_runtime *runtime = substream->runtime;
    struct tcc_runtime_data *prtd = runtime->private_data;
    struct snd_soc_pcm_runtime *rtd = substream->private_data;
    struct tcc_pcm_dma_params *dma = rtd->dai->cpu_dai->dma_data;
    size_t totsize = params_buffer_bytes(params);
    //size_t period = params_period_bytes(params); 
    size_t period_bytes = params_period_bytes(params); 
    //size_t period_num = params_periods(params);
    int ret = 0;

    /* return if this is a bufferless transfer e.g.
     * codec <--> BT codec or GSM modem -- lg FIXME */
    if (!dma)
        return 0;      

    /* this may get called several times by oss emulation
     * with different params */
    if (prtd->params == NULL) {
        prtd->params = dma;

        mutex_lock(&(tcc_alsa_info.mutex));
#if defined(CONFIG_ARCH_TCC83X)
        if (!(tcc_alsa_info.flag & TCC_INTERRUPT_REQUESTED)) {
            ret = request_irq(alsa_get_intr_num(substream),
                              tcc_dma_done_handler,
                              IRQF_SHARED,
                              "alsa-audio",
                              &tcc_alsa_info);
            if (ret < 0) {
                mutex_unlock(&(tcc_alsa_info.mutex));
                return -EBUSY;
            }
            tcc_alsa_info.flag |= TCC_INTERRUPT_REQUESTED;
        }
#elif defined(CONFIG_ARCH_TCC79X)
        ret = request_irq(alsa_get_intr_num(substream),
                          tcc_dma_done_handler,
                          IRQF_SHARED,
                          "alsa-audio",
                          &tcc_alsa_info);
        if (ret < 0) {
            mutex_unlock(&(tcc_alsa_info.mutex));
            return -EBUSY;
        }
#endif
        mutex_unlock(&(tcc_alsa_info.mutex));

        prtd->dma_ch = (substream->stream == SNDRV_PCM_STREAM_PLAYBACK) ? DMA_CH_OUT : DMA_CH_IN;
    } else {
        /* should works more? */
    }        

    //DMA  Ʈ
    //DMA    ʱȭ 

    snd_pcm_set_runtime_buffer(substream, &substream->dma_buffer);        
    runtime->dma_bytes = totsize;

    prtd->dma_buffer_end = runtime->dma_addr + totsize;
    prtd->dma_buffer = runtime->dma_addr;
    alsa_dbg("\n totsize=0x%X period=0x%X  period num=%d\n", totsize, period_bytes, params_periods(params));

    if (substream->stream == SNDRV_PCM_STREAM_PLAYBACK) {
        set_dma_outbuffer(substream->dma_buffer.addr, totsize, period_bytes);
#ifdef CONFIG_SND_TCC_LOOPBACK
        gs_p_alb_handle->set_dma_info(gs_p_alb_handle,
                                      Tx_C_SADR,
                                      substream->dma_buffer.addr,
                                      totsize,
                                      substream->dma_buffer.area);
#endif
    } else {
        set_dma_inbuffer(substream->dma_buffer.addr, totsize, period_bytes);
    }
    return ret;
}

static int tcc_pcm_hw_free(struct snd_pcm_substream *substream)
{
    struct tcc_runtime_data *prtd = substream->runtime->private_data;

    if (prtd->dma_ch) {
        snd_pcm_set_runtime_buffer(substream, NULL);
        //߰ ʿ  ??
    }

    return 0;
}

static int tcc_pcm_prepare(struct snd_pcm_substream *substream)
{
    struct snd_pcm_runtime *runtime = substream->runtime;

    //DMA initialize

#if defined(CONFIG_ARCH_TCC83X)
    /* Disable Noise */
    memset((void *)(&HwDADO_L0), 0, 32);
#endif
    memset(runtime->dma_area, 0x00, runtime->dma_bytes);

    return 0;
}

static int tcc_pcm_trigger(struct snd_pcm_substream *substream, int cmd)
{
    //struct tcc_runtime_data *prtd = substream->runtime->private_data;
    //    audio_stream_t *s = &output_stream;        
    int ret = 0;

    switch (cmd) {
    case SNDRV_PCM_TRIGGER_START:
    case SNDRV_PCM_TRIGGER_RESUME:        
    case SNDRV_PCM_TRIGGER_PAUSE_RELEASE:
#if defined(CONFIG_ARCH_TCC83X)
        if (substream->stream == SNDRV_PCM_STREAM_PLAYBACK) {
            tca_i2s_start(0);
            BITSET(HwChCtrl, HwChCtrl_DTIEN | HwChCtrl_DTEN_EN);
        } else {
            tca_i2s_start(1);
            BITSET(HwChCtrl, HwChCtrl_DRIEN | HwChCtrl_DREN_EN);
        }
#elif defined(CONFIG_ARCH_TCC79X)
        if (substream->stream == SNDRV_PCM_STREAM_PLAYBACK) {
            tca_i2s_start(0);
            BITSET(Tx_CHCTRL, HwCHCTRL0_IEN_EN | HwCHCTRL0_EN_EN);
        } else {
            tca_i2s_start(1);
            BITSET(Rx_CHCTRL, HwCHCTRL0_IEN_EN | HwCHCTRL0_EN_EN);
        }
#endif
        break;

    case SNDRV_PCM_TRIGGER_STOP:
    case SNDRV_PCM_TRIGGER_SUSPEND:
    case SNDRV_PCM_TRIGGER_PAUSE_PUSH:
#if defined(CONFIG_ARCH_TCC83X)
        if (substream->stream == SNDRV_PCM_STREAM_PLAYBACK) {
            tca_i2s_stop(0);
            BITCLR(HwChCtrl, HwChCtrl_DTIEN);
        } else {
            tca_i2s_stop(1);
            BITCLR(HwChCtrl, HwChCtrl_DRIEN);
        }
        if (!(tcc_alsa_info.flag)) {
            HwChCtrl &= ~HwChCtrl_DTEN_EN;              
        }
#elif defined(CONFIG_ARCH_TCC79X)
        if (substream->stream == SNDRV_PCM_STREAM_PLAYBACK) {
            tca_i2s_stop(0);
            Tx_CHCTRL &= ~HwCHCTRL0_IEN_EN;     
            if (!(tcc_alsa_info.flag)) {
                Tx_CHCTRL &= ~HwCHCTRL0_EN_EN;              
            }
        } else {
            tca_i2s_stop(1);
            Rx_CHCTRL &= ~HwCHCTRL0_IEN_EN;	
            if (!(tcc_alsa_info.flag)) {
                Rx_CHCTRL &= ~HwCHCTRL0_EN_EN;                       				
            }
        }
#endif
        break;

    default:
        ret = -EINVAL;
    }

    return ret;
}


//buffer point update
//current position   range 0-(buffer_size-1)
static snd_pcm_uframes_t tcc_pcm_pointer(struct snd_pcm_substream *substream)
{
    struct snd_pcm_runtime *runtime = substream->runtime;
    //struct tcc_runtime_data *prtd = runtime->private_data;
    snd_pcm_uframes_t x;

#if defined(CONFIG_ARCH_TCC83X)
    dma_addr_t ptr = (substream->stream == SNDRV_PCM_STREAM_PLAYBACK) ? HwTxDaCsar : HwRxDaCdar;
#elif defined(CONFIG_ARCH_TCC79X)
    dma_addr_t ptr = (substream->stream == SNDRV_PCM_STREAM_PLAYBACK) ? Tx_C_SADR: Rx_C_DADR;	
#ifdef CONFIG_SND_TCC_LOOPBACK
    if (substream->stream == SNDRV_PCM_STREAM_PLAYBACK) {
        if (gs_p_alb_handle->open_cnt > 0) {
            struct tcc_alsa_lb_handle *h = gs_p_alb_handle;
            if (h->cur_dma_addr == 0) {
                h->cur_dma_addr = ptr;
            } else {
                unsigned int offset = h->cur_dma_addr - h->dma_addr;
                unsigned int copy_size = ptr - h->cur_dma_addr;

                if (h->cur_dma_addr > ptr) {
                    copy_size = h->dma_addr + h->dma_size - h->cur_dma_addr;

                    h->push(h, h->v_addr + offset, copy_size);
                    h->cur_dma_addr = h->dma_addr;
                    if (h->cur_dma_addr != ptr) {
                        h->push(h, h->v_addr + offset, copy_size);
                        h->cur_dma_addr = ptr;
                    }
                } else if (h->cur_dma_addr < ptr) {
                    h->push(h, h->v_addr + offset, copy_size);
                    h->cur_dma_addr = ptr;
                }
            }
            wake_up(&(h->wait_q));
        }
    }
#endif
#endif

    x = bytes_to_frames(runtime, ptr - runtime->dma_addr);

    if (x < runtime->buffer_size) {
        return x;
    }
    return 0;
}

static int tcc_pcm_open(struct snd_pcm_substream *substream)
{
    struct snd_pcm_runtime *runtime = substream->runtime;
    struct tcc_runtime_data *prtd;
    int ret;

    mutex_lock(&(tcc_alsa_info.mutex));
    if (substream->stream == SNDRV_PCM_STREAM_PLAYBACK) {  
        tcc_alsa_info.flag |= TCC_RUNNING_PLAY;
        tcc_alsa_info.play_ptr = substream;
        snd_soc_set_runtime_hwparams(substream, &tcc_pcm_hardware_play);
    } else {
        tcc_alsa_info.flag |= TCC_RUNNING_CAPTURE;
        tcc_alsa_info.cap_ptr = substream;
        snd_soc_set_runtime_hwparams(substream, &tcc_pcm_hardware_capture);
    }
    mutex_unlock(&(tcc_alsa_info.mutex));

    prtd = kzalloc(sizeof(struct tcc_runtime_data), GFP_KERNEL);

    if (prtd == NULL) {
        ret = -ENOMEM;
        goto out;
    }

    runtime->private_data = prtd;

    return 0;
    kfree(prtd);
out:
    return ret;
}

static int tcc_pcm_close(struct snd_pcm_substream *substream)
{
    struct snd_pcm_runtime *runtime = substream->runtime;
    struct tcc_runtime_data *prtd = runtime->private_data;

    if (substream->stream == SNDRV_PCM_STREAM_PLAYBACK) {
        tcc_alsa_info.flag &= ~TCC_RUNNING_PLAY;
        tca_i2s_stop(0);
    } else {
        tcc_alsa_info.flag &= ~TCC_RUNNING_CAPTURE;
        tca_i2s_stop(1);
    }
    // dma_free_writecombine(substream->pcm->card->dev, PAGE_SIZE,); 

    if (prtd) {
        kfree(prtd);
    }

    mutex_lock(&(tcc_alsa_info.mutex));
#if defined(CONFIG_ARCH_TCC83X)
    if (tcc_alsa_info.flag & TCC_INTERRUPT_REQUESTED) {
        if (!(tcc_alsa_info.flag & (TCC_RUNNING_PLAY | TCC_RUNNING_CAPTURE))) {
            free_irq(alsa_get_intr_num(substream), &tcc_alsa_info);
            tcc_alsa_info.flag &= ~TCC_INTERRUPT_REQUESTED;
        }
    }
#elif defined(CONFIG_ARCH_TCC79X)
    free_irq(alsa_get_intr_num(substream), &tcc_alsa_info);
#endif
    mutex_unlock(&(tcc_alsa_info.mutex));

    return 0;
}

static int tcc_pcm_mmap(struct snd_pcm_substream *substream, struct vm_area_struct *vma)
{
    struct snd_pcm_runtime *runtime = substream->runtime;
    return dma_mmap_writecombine(substream->pcm->card->dev, vma,
        runtime->dma_area,
        runtime->dma_addr,
        runtime->dma_bytes);
}

struct snd_pcm_ops tcc_pcm_ops = {
    .open  = tcc_pcm_open,
    .close  = tcc_pcm_close,
    .ioctl  = snd_pcm_lib_ioctl,
    .hw_params = tcc_pcm_hw_params,
    .hw_free = tcc_pcm_hw_free,
    .prepare = tcc_pcm_prepare,
    .trigger = tcc_pcm_trigger,
    .pointer = tcc_pcm_pointer,
    .mmap = tcc_pcm_mmap,
};

static int tcc_pcm_preallocate_dma_buffer(struct snd_pcm *pcm, int stream)
{
    struct snd_pcm_substream *substream = pcm->streams[stream].substream;
    struct snd_dma_buffer *buf = &substream->dma_buffer;
    size_t size = 0;

    if (substream->stream == SNDRV_PCM_STREAM_PLAYBACK) {
        size = tcc_pcm_hardware_play.buffer_bytes_max;
    } else {
        size = tcc_pcm_hardware_capture.buffer_bytes_max;
    }

    buf->dev.type = SNDRV_DMA_TYPE_DEV;
    buf->dev.dev = pcm->card->dev;
    buf->private_data = NULL;

    alsa_dbg("tcc_pcm_preallocate_dma_buffer size [%d]\n", size);
    buf->area = dma_alloc_writecombine(pcm->card->dev, size, &buf->addr, GFP_KERNEL);
    if (!buf->area) {
        return -ENOMEM;
    }
    buf->bytes = size;
    return 0;
}

#ifdef CONFIG_SND_TCC_LOOPBACK
static ssize_t tcc_alb_read(struct file *filp, char *buffer, size_t count, loff_t *ppos)
{
    return gs_p_alb_handle->pop_user(gs_p_alb_handle, buffer, count);
}

static unsigned int tcc_alb_poll(struct file *filp, struct poll_table_struct *wait)
{
    if (gs_p_alb_handle->get_pop_size(gs_p_alb_handle) > 0) {
		return  (POLLIN | POLLRDNORM);
    } else {
        poll_wait(filp, &(gs_p_alb_handle->wait_q), wait);
        if (gs_p_alb_handle->get_pop_size(gs_p_alb_handle) > 0) {
            return  (POLLIN | POLLRDNORM);
        }
    }
    return 0;
}

static int tcc_alb_open(struct inode *inode, struct file *filp)
{
    gs_p_alb_handle->lock(gs_p_alb_handle);
    if (gs_p_alb_handle->open_cnt == 0) {
        gs_p_alb_handle->cur_dma_addr = Tx_C_SADR;
        gs_p_alb_handle->bottom = gs_p_alb_handle->top;
        ALB_START(gs_p_alb_handle);
        gs_p_alb_handle->unlock(gs_p_alb_handle);
        return 0;
    }
    gs_p_alb_handle->unlock(gs_p_alb_handle);
    return -EBUSY;
}

static int tcc_alb_release(struct inode *inode, struct file *filp)
{
    gs_p_alb_handle->lock(gs_p_alb_handle);
    ALB_STOP(gs_p_alb_handle);
    gs_p_alb_handle->unlock(gs_p_alb_handle);
    return 0;
}

static struct file_operations tcc_alsa_lb_fops = {
	.owner = THIS_MODULE,
	.open = tcc_alb_open,
	.read = tcc_alb_read,	
	.release = tcc_alb_release,
	.poll = tcc_alb_poll,
};
#endif
static void tcc_pcm_free_dma_buffers(struct snd_pcm *pcm)
{
    struct snd_pcm_substream *substream;
    struct snd_dma_buffer *buf;
    int stream;

    for (stream = 0; stream < 2; stream++) {
        substream = pcm->streams[stream].substream;
        if (!substream) { continue; }

        buf = &substream->dma_buffer;
        if (!buf->area) { continue; }

        dma_free_writecombine(pcm->card->dev, buf->bytes, buf->area, buf->addr);
        buf->area = NULL;
    }
    mutex_init(&(tcc_alsa_info.mutex));
#ifdef CONFIG_SND_TCC_LOOPBACK
    gs_p_alb_handle = delete_alsa_lb(gs_p_alb_handle);
	unregister_chrdev(LOOPBACK_MAJOR_NUM, DEVICE_NAME);
#endif
}

static u64 tcc_pcm_dmamask = DMA_32BIT_MASK;

int tcc_pcm_new(struct snd_card *card, struct snd_soc_dai *dai, struct snd_pcm *pcm)
{
    int ret = 0;

    memset(&tcc_alsa_info, 0, sizeof(tcc_interrupt_info_x));
    mutex_init(&(tcc_alsa_info.mutex));

    if (!card->dev->dma_mask) {
        card->dev->dma_mask = &tcc_pcm_dmamask;
    }
    if (!card->dev->coherent_dma_mask) {
        card->dev->coherent_dma_mask = DMA_32BIT_MASK;
    }
    if (dai->playback.channels_min) {
        ret = tcc_pcm_preallocate_dma_buffer(pcm,SNDRV_PCM_STREAM_PLAYBACK);
        if (ret) { goto out; }
    }

    if (dai->capture.channels_min) {
        ret = tcc_pcm_preallocate_dma_buffer(pcm,SNDRV_PCM_STREAM_CAPTURE);
        if (ret) { goto out; }
    } 

#ifdef CONFIG_SND_TCC_LOOPBACK
    {
        int result;

        alsa_dbg("[alsa_loopback] init ~\n");

        gs_p_alb_handle = new_alsa_lb(1024 * 64 * 2);
        if (gs_p_alb_handle == NULL) {
            return -ENOMEM;
        }
        if (gs_p_alb_handle->init(gs_p_alb_handle)) {
            gs_p_alb_handle = delete_alsa_lb(gs_p_alb_handle);
            return -ENOMEM;
        }

        result = register_chrdev(LOOPBACK_MAJOR_NUM, DEVICE_NAME, &tcc_alsa_lb_fops);
        if (result < 0) {
            alsa_dbg("Cannot register alb driver !!!\n");
            return result;
        }
    }
#endif
out:
    return ret;
}

int tcc_pcm_suspend(struct platform_device *pdev, struct snd_soc_cpu_dai *cpu_dai)
{
    return 0;
}

int tcc_pcm_resume(struct platform_device *pdev, struct snd_soc_cpu_dai *cpu_dai)
{
    return 0;
}
struct snd_soc_platform tcc_soc_platform = {
    .name  = "tcc-audio",
    .pcm_ops  = &tcc_pcm_ops,
    .pcm_new = tcc_pcm_new,
    .pcm_free = tcc_pcm_free_dma_buffers,
    .suspend = tcc_pcm_suspend,
    .resume = tcc_pcm_resume,
};

EXPORT_SYMBOL_GPL(tcc_soc_platform);

MODULE_AUTHOR("Telechips");
MODULE_DESCRIPTION("Telechips TCC PCM DMA module");
MODULE_LICENSE("GPL");

