/*
 * linux/drivers/ata/sata_snps.c
 *
 * Author:  <linux@telechips.com>
 * Created: 1st April, 2009
 * Description: Driver for SATA Host Controller
 *
 * Copyright (c) 2009 Telechips, Inc.
 * Copyright (c) 2005 SYNOPSYS, INC.  ALL RIGHTS RESERVED
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/blkdev.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/device.h>
#include <scsi/scsi_host.h>
#include <scsi/scsi_cmnd.h>
#include <scsi/scsi.h>
#include <linux/libata.h>
#include <linux/platform_device.h>

//#include <mach/TCC89x_Physical.h>
//#include <mach/TCC89x_Structures.h>
#include <bsp.h>
#include <mach/hardware.h>
#include <mach/irqs.h>
#include <mach/tca_ckc.h>
#include <linux/workqueue.h>
#include <linux/time.h>
#include <linux/kthread.h>
#include <mach/tcc_pca953x.h>
#include <linux/tcc_pwm.h>
#include "sata_snps.h"
#include <linux/tcc_ll.h>

#define DRV_NAME	"tcc_sata"
#define DRV_VERSION	"0.1"

#if 0
#define jprintk(f, a...)    printk("---J--- [%s:%d] " f, __func__, __LINE__, ##a)
#define sata_printk(f, a...)    printk("---sata--- [%s:%d] " f, __func__, __LINE__, ##a)
#else
#define jprintk(f, a...)
#define sata_printk(f, a...)
#endif

//#if defined(CONFIG_TCC_R_AX)
#if defined(CONFIG_TCC_CHIP_REV_A)
#define __RX_TEST_DBTSR__ 8
#define __TX_TEST_DBTSR__ 8
//#define HSATA_DMA_DBTSR_VAL (((__RX_TEST_DBTSR__) << 16) | ((__TX_TEST_DBTSR__) << 0))
#define HSATA_DMA_DBTSR_VAL (((__RX_TEST_DBTSR__) << 16) | ((254 - 8) << 0))
#elif defined(CONFIG_TCC_R_XX)
#define __RX_TEST_DBTSR__ 1
#define __TX_TEST_DBTSR__ 1
#define HSATA_DMA_DBTSR_VAL (((__RX_TEST_DBTSR__) << 16) | ((__TX_TEST_DBTSR__) << 0))
#else #error "Undefined TCC8900 revision type !!!"
#endif

#define WAIT_TIME_FOR_DMA_DONE (1000 * 10)

/********************************************************************************************
extern functions
********************************************************************************************/
extern struct clk *clk_get(struct device *dev, const char *id);
extern int clk_enable(struct clk *clk);
extern void clk_disable(struct clk *clk);

/********************************************************************************************
static local functions
********************************************************************************************/
static void tcc_exec_command_by_tag(struct ata_port *ap, const struct ata_taskfile *tf, u8 tag);
static void tcc_sata_setup_port(struct ata_ioports *port, void __iomem *base);

/********************************************************************************************
static variables
********************************************************************************************/
static unsigned int g_SataConnectionState = 0;

static struct task_struct *start_sata_thread(int (*threadfn)(void *data),
					struct tcc_sata_dev *tcc_dev,
					const char namefmt[])
{
	struct task_struct *ret_task = NULL;
	if (tcc_dev)
	{
		tcc_dev->is_continue = 1;
		ret_task = kthread_run(threadfn, tcc_dev, namefmt);
		if (IS_ERR(ret_task))
		{
			tcc_dev->is_continue = 0;
			ret_task = NULL;
			printk("[%s:%d] cannot run kernel_thread !!!! \n", __func__, __LINE__);
		}
	}
	return ret_task;
}

static void stop_sata_thread(struct tcc_sata_dev *tcc_dev)
{
	if (tcc_dev)
	{
		tcc_dev->is_continue = 0;
		if (tcc_dev->fetch_task)
		{
			wake_up(&(tcc_dev->job_wait_q));
			kthread_stop(tcc_dev->fetch_task);
			tcc_dev->fetch_task = NULL;
		}
	}
}

static void tcc_start_dma(GDMANCTRL *reg)
{
	BITSET(reg->CHCTRL, Hw0 | Hw2);
}

static void tcc_stop_dma(GDMANCTRL *reg)
{
	BITCLR(reg->CHCTRL, Hw0 | Hw2);
}

static int init_tx_dma(GDMANCTRL *reg, unsigned long fifo)
{
	if (reg)
	{
		BITCLR(reg->CHCTRL, Hw0);

		reg->ST_DADR = fifo;

		(reg->DPARAM)[0] = 0xFFFFE000 | 4; // fifo mask
		//(reg->DPARAM)[0] = 0; // fifo mask
		(reg->SPARAM)[0] = 0x0 | 4;

		reg->CHCTRL &= 0x00000001;

#if (__TX_TEST_DBTSR__ == 8)
		BITSET(reg->CHCTRL, (Hw3 | Hw5 | Hw6 | Hw7 | Hw8 | Hw9 | Hw12));
#elif (__TX_TEST_DBTSR__ == 4)
		BITSET(reg->CHCTRL, (Hw3 | Hw5 | Hw7 | Hw8 | Hw9 | Hw12));
#elif (__TX_TEST_DBTSR__ == 2)
		BITSET(reg->CHCTRL, (Hw3 | Hw5 | Hw6 | Hw8 | Hw9 | Hw12));
#else
		BITSET(reg->CHCTRL, (Hw3 | Hw5 | Hw8 | Hw9 | Hw12));
#endif
		/* clear DRI at RPTCTRL */
		BITCLR(reg->RPTCTRL, Hw31);

		BITSET(reg->EXTREQ, Hw14);

		return 0;
	}
	return -1;
}

static int init_rx_dma(GDMANCTRL *reg, unsigned long fifo)
{
	if (reg)
	{
		BITCLR(reg->CHCTRL, Hw0);

		reg->ST_SADR = fifo;

		(reg->SPARAM)[0] = 0xFFFFE000 | 4; // fifo mask
		//(reg->SPARAM)[0] = 0; // fifo mask
		(reg->DPARAM)[0] = 0x0 | 4;

		reg->CHCTRL &= 0x00000001;

#if (__RX_TEST_DBTSR__ == 8)
		BITSET(reg->CHCTRL, (Hw3 | Hw5 | Hw6 | Hw7 | Hw8 | Hw9));
#elif (__RX_TEST_DBTSR__ == 4)
		BITSET(reg->CHCTRL, (Hw3 | Hw5 | Hw7 | Hw8 | Hw9));
#elif (__RX_TEST_DBTSR__ == 2)
		BITSET(reg->CHCTRL, (Hw3 | Hw5 | Hw6 | Hw8 | Hw9));
#else
		BITSET(reg->CHCTRL, (Hw3 | Hw5 | Hw8 | Hw9));
#endif
		/* clear DRI at RPTCTRL */
		BITCLR(reg->RPTCTRL, Hw31);

		BITSET(reg->EXTREQ, Hw13);
		return 0;
	}
	return -1;
}

static void init_sata_irq(volatile PPIC p_pic)
{
	BITSET(p_pic->SEL1, 1 << (INT_SATA - 32));
	BITSET(p_pic->MODE1, 1 << (INT_SATA - 32));
}

static void init_dma_irq(volatile PPIC p_pic)
{
	BITSET(p_pic->MODE0, Hw29);	// level-trigger
	BITCLR(p_pic->POL0, Hw29);	// active-high
}


#if 0
static void set_tx_dma(GDMANCTRL *reg, unsigned long dma_addr, unsigned int length)
{
	reg->ST_SADR = dma_addr;
	reg->HCOUNT = length / (4 * __TX_TEST_DBTSR__);
}
#else
static void set_tx_dma(struct tcc_sata_dev *tcc_dev, struct tcc_sg_list * p_list)
{
	tcc_dev->is_tx_complete = 0;
	tcc_dev->is_last_tx = (p_list->flags & _TCC_DMA_END_) ? 1 : 0;

	tcc_dev->dma_tx_reg->ST_SADR = p_list->dma_addr;
	tcc_dev->dma_tx_reg->HCOUNT = p_list->length / (4 * __TX_TEST_DBTSR__);
}
#endif

#if 0
static void set_rx_dma(GDMANCTRL *reg, unsigned long dma_addr, unsigned int length)
{
	reg->ST_DADR = dma_addr;
	reg->HCOUNT = length / (4 * __RX_TEST_DBTSR__);
}
#else
static void set_rx_dma(struct tcc_sata_dev *tcc_dev, struct tcc_sg_list * p_list)
{
	tcc_dev->is_rx_complete = 0;
	tcc_dev->is_last_rx = (p_list->flags & _TCC_DMA_END_) ? 1 : 0;

	tcc_dev->dma_rx_reg->ST_DADR = p_list->dma_addr;
	tcc_dev->dma_rx_reg->HCOUNT = p_list->length / (4 * __RX_TEST_DBTSR__);
}
#endif

/*
 * success: return value => (ret == 0)
 * fail   : return value => (ret != 0)
 */
static int confirm_dma_job(struct tcc_dma_Q *h)
{
	if (h)
	{
		if (h->top != h->pre)
		{
			h->top = h->pre;
			return 1;
		}
		return 0;
	}
	return -1;
}

/*
 * success: return value => (ret == 0)
 * fail   : return value => (ret != 0)
 */
static int flush_dma_job(struct tcc_dma_Q *h)
{
	if (h)
	{
		h->pre = h->bottom = h->top = 0;
		return 0;
	}
	return -1;
}

/*
 * success: return value => (ret == 0)
 * fail   : return value => (ret != 0)
 */
static int push_dma_job(struct tcc_dma_Q *h, unsigned int dma_addr, unsigned int len, int flags)
{
	if (h)
	{
		h->sg_list[h->pre].dma_addr = dma_addr;
		h->sg_list[h->pre].length = len;
		h->sg_list[h->pre].flags = flags;

		if (h->pre + 1 >= TCC_SG_LIST_MAX)
		{
			h->pre = 0;
		}
		else
		{
			h->pre++;
		}

		if (h->pre == h->bottom)
		{
			printk("!!!!!!!!!!!! overflow !!!!!!!!!!!!!!!!!\n");
		}
		return 0;
	}
	return -1;
}

/*
 * success: return value => (ret > 0)
 * empty  : return value => (ret == 0)
 * fail   : return value => (ret < 0)
 */
static int pop_dma_job(struct tcc_dma_Q *h, struct tcc_sg_list *p)
{
	if (h && p)
	{
		if (h->top != h->bottom)
		{
			memcpy(p, &(h->sg_list[h->bottom]), sizeof(struct tcc_sg_list));
			if (h->bottom + 1 >= TCC_SG_LIST_MAX)
			{
				h->bottom = 0;
			}
			else
			{
				h->bottom++;
			}
			return 1;
		}
		return 0;
	}
	return -1;
}

#define MAX_RETRY_CNT_IN_BUSY 50
static void wait_for_bits(volatile PSATA pSata, int ret_on_rst, int timeout_val, int exp_val, int mask, int *matched)
{
	int timeout_cnt = 0, error = 0;
	int failed;
	int rdata = 0, i = 0;
	int phy_sig_det_w = 1;
	int retry_cnt = 0;

Retry:
	timeout_cnt = error = rdata = i = 0;
	phy_sig_det_w = 1;

	*matched = 0;

	// wait for busy, and Drq to be cleared.
	// or timeout or reset, if checking reset
	while ((timeout_cnt < timeout_val)
		   && !*matched && (error == 0)
		   && !(!phy_sig_det_w && ret_on_rst))
	{
		rdata = pSata->CDR7;
		failed = 0;

		for (i = 0; i <= 31; i = i + 1)
		{
			if ((((mask >> i) & 1) == 1) && (((rdata >> i) & 1) != ((exp_val >> i) & 1)))
			{
				failed = 1;
			}
		}

		if (failed == 0)
		{
			*matched = 1;
		}

		// After bits matched, check for error
		if (*matched)
		{
			if (((rdata & 0xff) != 0x7f) // we're not in Power on reset
				&& (((rdata >> CDR7_ERR) & 1) == 1))   // we do have error bit set
			{
				error = 1;
			}
		}
		else
		{
			timeout_cnt = timeout_cnt + 1;
		}

	}

	if ((ret_on_rst == 1) && !phy_sig_det_w)
	{
		printk("wait_for_bits: detected a Non recov err, while waiting for bits on reg 0x%x, actual='0x%x'\n",
			   0x1C, rdata);
	}
	else if (timeout_cnt >= timeout_val)
	{
		if ((rdata & 0xFF) == 0x80)
		{
			retry_cnt++;
			if (retry_cnt < MAX_RETRY_CNT_IN_BUSY)
			{
				printk("RETRY[%d]: Timeout! Device is so busy that we retry negotiation. actual='0x%x'\n",
					   retry_cnt, rdata);
				msleep(100);
				goto Retry;
			}
		}
		printk("wait_for_bits - RETRY[%d]: Timeout waiting reg at address '0x%x' to equal '0x%x', actual='0x%x'\n",
			   retry_cnt, 0x1C, exp_val, rdata);
	}
	else
	{
		printk("wait_for_bits: Got register at address '0x%x' reg expected_value='0x%x', actual='0x%x'\n",
			   0x1C, exp_val, rdata);
	}
}

static void wait_for_bsydrq(volatile PSATA pSata, int ret_on_rst, int timeout_val, int *matched)
{
	int exp_val = 0;
	int mask = 0x88;

	wait_for_bits(pSata, ret_on_rst, timeout_val, exp_val, mask, matched);
}

static int prog_host_speed(volatile PSATA pSata, int speed, int *spd_ok)
{
	int spd_select;
	int reset;
	int reg_data;
	//int i;
	int ret_on_rst;
	int timeout_val;
	int matched;

	// Initialize
	ret_on_rst  = 0;
	//timeout_val = 1000000; // Must Be Huge For Speed Negotiation
	timeout_val =   1000000; // Must Be Huge For Speed Negotiation

	// Set speed
	spd_select = (speed == 2) ? 0x2 : 0x1;

	// Reset Device with correct speed
	reset = 1;
	reg_data = (spd_select << 4) | reset;

	// Write SCR2 with speed and reset
	pSata->SCR2 = reg_data;

	// Need to wait a few clocks of slowest
	//mdelay(1);
	mdelay(10);

	// Set normal operation & correct speed
	reset = 0;
	reg_data = spd_select << 4 | reset;
	//HwSATA->nSCR2 = reg_data;
	pSata->SCR2 = reg_data;

	// Wait for BSY & DRQ to be cleared
	wait_for_bsydrq(pSata, ret_on_rst, timeout_val, &matched);

	if (matched)
	{
		printk("prog_host_speed: Host Speed selected [speed:%d] SCR0[0x%X]\n",
			   speed, pSata->SCR0);
		*spd_ok = 1;
		return 0;
	}
	else
	{
		printk("prog_host_speed: TIMEOUT - Host Speed timed out waiting for speed change! [speed:%d], SCR0[0x%X]\n",
			   speed, pSata->SCR0);
		*spd_ok = 0;
	}
	return -1;
}

static int tcc_sata_hw_init(unsigned long sata_base)
{
	int spd_ok = 0;
	volatile PSATA pSata = (volatile PSATA)sata_base;

	if (prog_host_speed(pSata, 2, &spd_ok) == 0)
	{
		return 0;
	}
	else if (prog_host_speed(pSata, 1, &spd_ok) == 0)
	{
		return 0;
	}
	return -1;
}

static int tcc_sata_check_atapi_dma(struct ata_queued_cmd *qc)
{
	u8 cmnd = qc->scsicmd->cmnd[0];

	switch (cmnd)
	{
	case READ_10:
	case READ_12:
	case READ_16:
	case WRITE_10:
	case WRITE_12:
	case WRITE_16:
		printk("ATAPI DMA OK ~~~[0x%X]\n", cmnd);
		return 0;

	default:
		printk("ATAPI DMA FAIL [0x%X] ~~~\n", cmnd);
		return -1;
	}
}

static irqreturn_t tcc_dma_isr(int irq, void *dev_id)
{
	struct ata_host *host = (struct ata_host *)dev_id;
	struct tcc_sata_dev *tcc_dev = host->private_data;

	struct tcc_sg_list list;
	if (irq == tcc_dev->dma_rx_irq)
	{
		tcc_dev->dma_rx_reg->CHCTRL |= Hw3;
		tcc_stop_dma(tcc_dev->dma_rx_reg);

		if (tcc_dev->is_last_rx)
		{
			tcc_dev->is_rx_complete = 1;
			wake_up(&(tcc_dev->rx_dma_wait_q));
		}
		else
		{
			if (pop_dma_job(&(tcc_dev->dma_q), &list) > 0)
			{
				if (list.flags & _TCC_DMA_RX_)
				{
					set_rx_dma(tcc_dev, &list);
					tcc_start_dma(tcc_dev->dma_rx_reg);
				}
			}
			else
			{
				printk("something wrong !!!!!!!!!!!!!!!!!!!! [%s:%d]\n", __func__, __LINE__);
				tcc_dev->is_rx_complete = 1;
				wake_up(&(tcc_dev->rx_dma_wait_q));
			}
		}
	}
	else if (irq == tcc_dev->dma_tx_irq)
	{
		tcc_dev->dma_tx_reg->CHCTRL |= Hw3;
		tcc_stop_dma(tcc_dev->dma_tx_reg);
		if (tcc_dev->is_last_tx)
		{
			writel(HSATA_DMACR_TXRX_CLEAR, tcc_dev->membase + HSATA_DMACR_REG);  /* rx/tx disable */
			tcc_dev->is_tx_complete = 1;
			wake_up(&(tcc_dev->tx_dma_wait_q));
		}
		else
		{
			if (pop_dma_job(&(tcc_dev->dma_q), &list) > 0)
			{
				if (list.flags & _TCC_DMA_TX_)
				{
					set_tx_dma(tcc_dev, &list);
					tcc_start_dma(tcc_dev->dma_tx_reg);
				}
			}
			else
			{
				printk("something wrong !!!!!!!!!!!!!!!!!!!! [%s:%d]\n", __func__, __LINE__);
				tcc_dev->is_tx_complete = 1;
				wake_up(&(tcc_dev->tx_dma_wait_q));
			}
		}
	}
	return IRQ_HANDLED;
}

static void tcc_port_stop(struct ata_port *ap)
{
	struct tcc_sata_dev *tcc_dev = ap->host->private_data;

	if (tcc_dev)
	{
		stop_sata_thread(tcc_dev);
		free_irq(tcc_dev->dma_tx_irq, ap->host);
		free_irq(tcc_dev->dma_rx_irq, ap->host);

		if (tcc_dev->membase)
		{
			iounmap(tcc_dev->membase);
			kfree(tcc_dev);
		}
	}
	ap->host->private_data = NULL;
}

static int fetch_dma_thread(void *arg)
{
	struct tcc_sata_dev *tcc_dev = (struct tcc_sata_dev *)arg;
	int ret = 0;

	sata_printk("### start kernel thread ###\n");
	if (tcc_dev)
	{
		struct tcc_sg_list list;
		do
		{
			ret = wait_event_interruptible_timeout(tcc_dev->job_wait_q,
												   (tcc_dev->dma_q.top != tcc_dev->dma_q.bottom) || !(tcc_dev->is_continue),
												   msecs_to_jiffies(1000));
			while (tcc_dev->is_continue)
			{
				if (pop_dma_job(&(tcc_dev->dma_q), &list) > 0)
				{
					sata_printk("--- DMA [%s] dma_addr[0x%X], len[%d], flags[0x%X]\n",
								list.flags & _TCC_DMA_RX_ ? "RX" : "TX", list.dma_addr, list.length, list.flags);

					//ret = readl(tcc_dev->membase + HSATA_DBTSR_REG);
					//printk("before Rx/Tx DBTSR_REG[0x%X], dma HCOUNT[0x%X]\n", ret, tcc_dev->dma_rx_reg->HCOUNT);

					if (list.flags & _TCC_DMA_START_)
					{
						sata_printk("--- start DMA [%s] dma_addr[0x%X], len[%d], flags[0x%X]\n",
									list.flags & _TCC_DMA_RX_ ? "RX" : "TX", list.dma_addr, list.length, list.flags);

						writel(HSATA_DMACR_TXRX_CLEAR, tcc_dev->membase + HSATA_DMACR_REG);  /* rx/tx disable */
						writel(HSATA_DMA_DBTSR_VAL, tcc_dev->membase + HSATA_DBTSR_REG);

						writel((list.flags & _TCC_DMA_RX_) ? HSATA_DMACR_RX_EN : HSATA_DMACR_TX_EN,
							   tcc_dev->membase + HSATA_DMACR_REG);
						if (list.flags & _TCC_DMA_RX_)
						{
							set_rx_dma(tcc_dev, &list);
							tcc_start_dma(tcc_dev->dma_rx_reg);
							if (wait_event_interruptible_timeout((tcc_dev->rx_dma_wait_q),
																 tcc_dev->is_rx_complete,
																 msecs_to_jiffies(WAIT_TIME_FOR_DMA_DONE)) == 0)
							{
								tcc_stop_dma(tcc_dev->dma_rx_reg);
								flush_dma_job(&(tcc_dev->dma_q));
								ret = readl(tcc_dev->membase + HSATA_DBTSR_REG);
								printk("[%s:%d] RX wait_event timeout  (%dms) !!!\n",
									   __func__, __LINE__, WAIT_TIME_FOR_DMA_DONE);
								printk("rx DBTSR_REG[0x%X], dma HCOUNT[0x%X]\n", ret, tcc_dev->dma_rx_reg->HCOUNT);
							}
						}
						else
						{
							set_tx_dma(tcc_dev, &list);
							tcc_start_dma(tcc_dev->dma_tx_reg);
							if (wait_event_interruptible_timeout((tcc_dev->tx_dma_wait_q),
																 tcc_dev->is_tx_complete,
																 msecs_to_jiffies(WAIT_TIME_FOR_DMA_DONE)) == 0)
							{
								tcc_stop_dma(tcc_dev->dma_tx_reg);
								flush_dma_job(&(tcc_dev->dma_q));
								ret = readl(tcc_dev->membase + HSATA_DBTSR_REG);
								printk("[%s:%d] TX wait_event timeout  (%dms) !!!\n",
									   __func__, __LINE__, WAIT_TIME_FOR_DMA_DONE);
								printk("tx DBTSR_REG[0x%X], dma HCOUNT[0x%X]\n", ret, tcc_dev->dma_tx_reg->HCOUNT);
							}
						}
					}
					else
					{
						flush_dma_job(&(tcc_dev->dma_q));
						printk("[%s:%d] mismatch q-list !!!\n", __func__, __LINE__);
					}
				}
				else
				{
					break;
				}
			}
		}
		while (!kthread_should_stop());
	}
	sata_printk("### end kernel thread ###\n");
	return 0;
}

static int tcc_port_start(struct ata_port *ap)
{
	volatile u32 val32;
	struct tcc_sata_dev *tcc_dev = NULL;
	volatile PPIC p_pic = (volatile PPIC)tcc_p2v(HwPIC_BASE);
	int ret = 0;
	unsigned long sata_base = 0;
	unsigned long tx_base = (unsigned long)tcc_p2v(HwGDMA2_BASE);
	unsigned long rx_base = (unsigned long)tcc_p2v(HwGDMA3_BASE);
	unsigned long mmio_base_addr = (unsigned long)(ap->private_data);

	tcc_dev = kmalloc(sizeof(struct tcc_sata_dev), GFP_KERNEL);
	if (tcc_dev)
	{
		memset(tcc_dev, 0, sizeof(struct tcc_sata_dev));
		tcc_dev->membase = ioremap_nocache(mmio_base_addr, 0x800);
		tcc_sata_setup_port(&ap->ioaddr, tcc_dev->membase);
		ap->host->private_data = tcc_dev;
	}
	else
	{
		printk("[%s:%d] cannot allocate 'tcc_dev struct' \n", __func__, __LINE__);
		return -1;
	}
	HSATA_ENABLE_INTERRUPTS(tcc_dev);

	tcc_dev->dma_tx_reg = (GDMANCTRL *)(tx_base + 0x60);
	tcc_dev->dma_rx_reg = (GDMANCTRL *)(rx_base + 0x60);

	tcc_dev->dma_tx_irq = INT_DMA2_CH2;
	tcc_dev->dma_rx_irq = INT_DMA3_CH2;

	init_dma_irq(p_pic);
	init_waitqueue_head(&(tcc_dev->job_wait_q));
	init_waitqueue_head(&(tcc_dev->tx_dma_wait_q));
	init_waitqueue_head(&(tcc_dev->rx_dma_wait_q));

	init_rx_dma(tcc_dev->dma_rx_reg, TCC_DMADR_REG_ADDR);
	init_tx_dma(tcc_dev->dma_tx_reg, TCC_DMADR_REG_ADDR);
	writel(HSATA_DMACR_TXMODE_BIT, tcc_dev->membase + HSATA_DMACR_REG);  /* disable */

	tcc_dev->fetch_task = start_sata_thread(fetch_dma_thread, tcc_dev, "tcc_sata_task");
	if (tcc_dev->fetch_task == NULL)
	{
		return -1;
	}

	ret = request_irq(tcc_dev->dma_rx_irq,
					  tcc_dma_isr,
					  IRQF_SHARED,              /* flags */
					  "HSATA-DMA-RX",    /* in /proc/interrupts */
					  ap->host); /* user data passed to ISR */
	if (!ret)
	{
		ret = request_irq(tcc_dev->dma_tx_irq,
						  tcc_dma_isr,
						  IRQF_SHARED,              /* flags */
						  "HSATA-DMA-TX",    /* in /proc/interrupts */
						  ap->host); /* user data passed to ISR */
		if (!ret)
		{
			sata_base = (unsigned long)tcc_p2v(HwSATA_BASE);
			tcc_sata_hw_init(sata_base);

			writel(HSATA_DMA_DBTSR_VAL, tcc_dev->membase + HSATA_DBTSR_REG);

			val32 = readl(tcc_dev->membase + HSATA_SCR0_REG);
			g_SataConnectionState = HSATA_SCR0_SPD_GET(val32);
			switch (g_SataConnectionState)
			{
			case 0x0:
				printk("Rx_DBTSR[%d] Tx_DBTSR[%d] **** NO NEGOTIATED SPEED!!! ****\n", __RX_TEST_DBTSR__, __TX_TEST_DBTSR__);
				break;
			case 0x1:
				printk("Rx_DBTSR[%d] Tx_DBTSR[%d] **** GEN I RATE NEGOTIATED ****\n", __RX_TEST_DBTSR__, __TX_TEST_DBTSR__);
				break;
			case 0x2:
				printk("Rx_DBTSR[%d] Tx_DBTSR[%d] **** GEN II RATE NEGOTIATED ****\n", __RX_TEST_DBTSR__, __TX_TEST_DBTSR__);
				break;
			}
			return 0;
		}
		else
		{
			printk("ERROR rx request_irq ret[%d]\n", ret);
			free_irq(tcc_dev->dma_rx_irq, ap->host);
		}
	}
	else
	{
		printk("ERROR tx request_irq ret[%d]\n", ret);
	}
	return -1;
}


static int tcc_scr_read(struct ata_link *link, unsigned int sc_reg, u32 *val)
{
	if (sc_reg > SCR_CONTROL)
	{
		return -EINVAL;
	}
	*val = readl((void *)link->ap->ioaddr.scr_addr + (sc_reg * 4));
	return 0;
}

static int tcc_scr_write(struct ata_link *link, unsigned int sc_reg, u32 val)
{
	if (sc_reg > SCR_CONTROL)
	{
		return -EINVAL;
	}
	writel(val, (void *)link->ap->ioaddr.scr_addr + (sc_reg * 4));
	return 0;
}

static void tcc_tf_load(struct ata_port *ap, const struct ata_taskfile *tf)
{
	unsigned int is_addr = tf->flags & ATA_TFLAG_ISADDR;

	sata_printk("TFLAGS: %s %s %s %s\n",
				(tf->flags & ATA_TFLAG_LBA48) ? "LBA48" : "NOT-LBA48",
				(tf->flags & ATA_TFLAG_ISADDR) ? "ISADDR" : "",
				(tf->flags & ATA_TFLAG_DEVICE) ? "DEVICE" : "",
				(tf->flags & ATA_TFLAG_WRITE) ? "WRITE" : "READ");
	sata_printk("CTL: %s %s %s\n",
				(tf->ctl & ATA_HOB) ? "HOB" : "NOT-HOB",
				(tf->ctl & ATA_SRST) ? "SRST" : "NOT-SRST",
				(tf->ctl & ATA_NIEN) ? "NIEN" : "INTERRUPTS-ENABLED");

	if (is_addr && (tf->flags & ATA_TFLAG_LBA48))
	{
		sata_printk("hob: feat=0x%X nsect=0x%X, lba:0x%X 0x%X 0x%X\n",
					tf->hob_feature,
					tf->hob_nsect,
					tf->hob_lbal,
					tf->hob_lbam,
					tf->hob_lbah);
	}

	if (is_addr)
	{
		sata_printk("feat=0x%X nsect=0x%X, lba:0x%X 0x%X 0x%X\n",
					tf->feature,
					tf->nsect,
					tf->lbal,
					tf->lbam,
					tf->lbah);
	}

	if (tf->flags & ATA_TFLAG_DEVICE)
	{
		sata_printk("ATA_TFLAG_DEVICE device=0x%X (%s)\n", tf->device,
					(tf->device & ATA_LBA) ? "LBA" : "CHS");
	}

	if (!is_addr)
	{
		sata_printk("is_addr=%d\n", is_addr);
	}

	ata_sff_tf_load(ap, tf);
}

static void tcc_tf_read(struct ata_port *ap, struct ata_taskfile *tf)
{
	jprintk("          nsect=0x%X, lba:0x%X 0x%X 0x%X device=0x%x\n",
			tf->nsect, tf->lbal, tf->lbam, tf->lbah, tf->device);

	if (tf->flags & ATA_TFLAG_LBA48)
	{
		jprintk("hob: feat=0x%X nsect=0x%X, lba:0x%X 0x%X 0x%X\n",
				tf->hob_feature, tf->hob_nsect, tf->hob_lbal,
				tf->hob_lbam, tf->hob_lbah);
	}

	ata_sff_tf_read(ap, tf);
}

static struct scsi_host_template tcc_sata_sht =
{
	ATA_BMDMA_SHT(DRV_NAME),
};

static u8 tcc_stat_check_status(struct ata_port *ap)
{
	return ioread8(ap->ioaddr.status_addr);
}

static void tcc_dma_setup_noexec_by_tag(struct ata_queued_cmd *qc, u8 tag)
{
	//struct tcc_sata_dev *tcc_dev = qc->ap->host->private_data;
	jprintk("DMA SETUP tag=%d id=%d [%s]\n", tag, qc->ap->print_id, (qc->dma_dir == DMA_TO_DEVICE) ? "WRITE" : "READ");
}

static void tcc_exec_command(struct ata_port *ap, const struct ata_taskfile *tf)
{
	tcc_exec_command_by_tag(ap, tf, 0);
}

static void tcc_dma_setup_by_tag(struct ata_queued_cmd *qc, u8 tag)
{
	tcc_dma_setup_noexec_by_tag(qc, tag);
	tcc_exec_command(qc->ap, &qc->tf);
}


void tcc_dma_setup(struct ata_queued_cmd *qc)
{
	//struct tcc_sata_dev *tcc_dev = qc->ap->host->private_data;
	tcc_dma_setup_by_tag(qc, 0);

}

void tcc_dma_start_by_tag(struct ata_queued_cmd *qc, u8 tag)
{
	struct tcc_sata_dev *tcc_dev = qc->ap->host->private_data;

	confirm_dma_job(&(tcc_dev->dma_q));
	wake_up(&(tcc_dev->job_wait_q));
}


static void tcc_dma_start(struct ata_queued_cmd *qc)
{
	tcc_exec_command(qc->ap, &qc->tf);
	tcc_dma_start_by_tag(qc, 0);
}

void tcc_dma_stop(struct ata_queued_cmd *qc)
{
	struct tcc_sata_dev *tcc_dev = qc->ap->host->private_data;
	if (qc->dma_dir != DMA_TO_DEVICE)
	{
		writel(HSATA_DMACR_TXRX_CLEAR, tcc_dev->membase + HSATA_DMACR_REG);  /* rx/tx disable */
	}
	else
	{
		//writel(HSATA_DMACR_TXRX_CLEAR, tcc_dev->membase + HSATA_DMACR_REG);  /* rx/tx disable */
	}
}


u8 tcc_dma_status(struct ata_port *ap)
{
	//struct tcc_sata_dev *tcc_dev = qc->ap->host->private_data;
	return ATA_DMA_INTR;
}

void tcc_irq_clear(struct ata_port *ap)
{
	/* read status reg to clear interrupt in controller */
	ata_sff_check_status(ap);
}

static void tcc_exec_command_by_tag(struct ata_port *ap, const struct ata_taskfile *tf, u8 tag)
{
	struct ata_host *host = ap->host;
	struct tcc_sata_dev *tcc_dev = host->private_data;
	volatile u32 val32;

	switch (tf->command)
	{
	case ATA_CMD_CHK_POWER:
		jprintk("ATA_CMD_CHK_POWER - tag=%d\n", tag); break;
	case ATA_CMD_EDD:
		jprintk("ATA_CMD_EDD - tag=%d\n", tag); break;
	case ATA_CMD_FLUSH:
		jprintk("ATA_CMD_FLUSH - tag=%d\n", tag); break;
	case ATA_CMD_FLUSH_EXT:
		jprintk("ATA_CMD_FLUSH_EXT - tag=%d\n", tag); break;
	case ATA_CMD_ID_ATA:
		jprintk("ATA_CMD_ID_ATA - tag=%d\n", tag); break;
	case ATA_CMD_ID_ATAPI:
		jprintk("ATA_CMD_ID_ATAPI - tag=%d\n", tag); break;
	case ATA_CMD_READ:
		jprintk("ATA_CMD_READ - tag=%d\n", tag); break;
	case ATA_CMD_READ_EXT:
		jprintk("ATA_CMD_READ_EXT - tag=%d\n", tag); break;
	case ATA_CMD_WRITE:
		jprintk("ATA_CMD_WRITE - tag=%d\n", tag); break;
	case ATA_CMD_WRITE_EXT:
		jprintk("ATA_CMD_WRITE_EXT - tag=%d\n", tag); break;
	case ATA_CMD_PIO_READ:
		jprintk("ATA_CMD_PIO_READ - tag=%d\n", tag); break;
	case ATA_CMD_PIO_READ_EXT:
		jprintk("ATA_CMD_PIO_READ_EXT - tag=%d\n", tag); break;
	case ATA_CMD_PIO_WRITE:
		jprintk("ATA_CMD_PIO_WRITE - tag=%d\n", tag); break;
	case ATA_CMD_PIO_WRITE_EXT:
		jprintk("ATA_CMD_PIO_WRITE_EXT - tag=%d\n", tag); break;
	case ATA_CMD_SET_FEATURES:
		jprintk("ATA_CMD_SET_FEATURES - tag=%d\n", tag); break;
	case ATA_CMD_PACKET:
		jprintk("ATA_CMD_PACKET - tag=%d\n", tag); break;
	case HSATA_CMD_QWRITE:
		jprintk("HSATA_CMD_QWRITE - tag=%d\n", tag); break;
	case HSATA_CMD_QREAD:
		jprintk("HSATA_CMD_QREAD - tag=%d\n", tag); break;
	default:
		jprintk("ATA_CMD_??? (0x%X)\n", tf->command); break;
	}

	val32 = readl(tcc_dev->membase + HSATA_SERROR_REG);
	jprintk("SERROR=0x%X\n", val32);
	writel(val32, tcc_dev->membase + HSATA_SERROR_REG);
	val32 = readl(tcc_dev->membase + HSATA_INTPR_REG);
	jprintk("INTPR=0x%x\n",    val32);

	ata_sff_exec_command(ap, tf);
}

void tcc_qc_prep_by_tag(struct ata_queued_cmd *qc, u8 tag)
{
	struct scatterlist *sg = qc->sg;  /* include/asm-arm/scatterlist.h */
	struct ata_port *ap = qc->ap;
	struct tcc_sata_dev *tcc_dev = ap->host->private_data;
	unsigned int nelem;
	int dir, flags = _TCC_DMA_START_;
	unsigned int fis_len = 0, dma_addr = 0, sg_len = 0, offset = 0, len = 0;

	if (!(qc->flags & ATA_QCFLAG_DMAMAP))
	{
		jprintk("NO DMA - exiting\n");
		return;
	}

#if 0
	assert(sg != NULL);
	assert(qc->n_elem > 0);
#endif
	/* rudals.debug.start */
	{
		PPMU p_pmu = (PPMU)tcc_p2v(HwPMU_BASE);
		BITCLR(p_pmu->PWROFF, Hw4); // SATA popwer on
		tca_ckc_setiobus(RB_SATAHCONTROLLER, 1);
	}
	/* rudals.debug.end */

	dir = qc->dma_dir;

	jprintk("QC PREP id=%d dma dir=%s n_elem=%d\n",
			qc->ap->print_id, (dir == DMA_FROM_DEVICE) ? "FROM_DEVICE" : "TO_DEVICE", qc->n_elem);

	for (nelem = qc->n_elem; nelem; nelem--)
	{
		dma_addr = (unsigned int)sg_dma_address(sg);
		sg_len = sg_dma_len(sg);
		while (sg_len)
		{

			flags |= ((dir == DMA_TO_DEVICE) ? _TCC_DMA_TX_ : _TCC_DMA_RX_);

			offset = dma_addr & 0xFFFF;
			len = sg_len;

			if ((offset + sg_len) > 0x10000)
			{
				printk("overflow !!!!!!!!!! offset[%u], sg_len[%u]\n", offset, sg_len);
				len = 0x10000 - offset;
			}

			if ((fis_len + len) > 8192)
			{
				sata_printk("SPLITTING: fis_len=%d/0x%x  len=%d/0x%x\n", fis_len, fis_len, len, len);
				len = (8192 - fis_len);
				fis_len = 0;
			}
			else
			{
				fis_len += len;
			}

			if (fis_len == 8192)
			{
				fis_len = 0;
			}

			sata_printk("+++ DMA sg nelem=%d dma_addr=0x%X sg_len=%d [%s]\n",
						qc->n_elem, dma_addr, sg_len, (dir == DMA_TO_DEVICE) ? "TX" : "RX");

			sg_len -= len;
			if ((sg_len == 0) && (nelem == 1))
			{
				flags |= _TCC_DMA_END_;
				sata_printk("********* LAST DMA LIST *********\n");
			}

			push_dma_job(&(tcc_dev->dma_q), dma_addr, len, flags);
			dma_addr += len;
			flags = 0;
		}
		sg++;
	}
}

static void tcc_qc_prep(struct ata_queued_cmd *qc)
{
	if (!(qc->flags & ATA_QCFLAG_DMAMAP))
	{
		return;
	}
	tcc_qc_prep_by_tag(qc, 0);
}


static unsigned int tcc_qc_issue_prot(struct ata_queued_cmd *qc)
{
	jprintk("id=%d\n", qc->ap->print_id);

	/* see promise driver also */
	switch (qc->tf.protocol)
	{
	case ATA_PROT_DMA:
		jprintk("ATA_PROT_DMA\n");
		break;
	case ATA_PROT_PIO:
		jprintk("ATA_PROT_PIO\n");
		break;
	}
	return ata_sff_qc_issue(qc);
}

static struct ata_port_operations tcc_sata_ops =
{
	//.inherits		= &ata_bmdma_port_ops,
	//.inherits		= &sata_pmp_port_ops,

	.inherits       		= &ata_sff_port_ops,
	.mode_filter        	= ata_bmdma_mode_filter,
	.bmdma_setup	= tcc_dma_setup,
	.bmdma_start        = tcc_dma_start,
	.bmdma_stop     	= tcc_dma_stop,
	.bmdma_status	= tcc_dma_status,

	.sff_exec_command     = tcc_exec_command,

	.sff_tf_load		= tcc_tf_load,
	.sff_tf_read		= tcc_tf_read,
	.sff_check_status	= tcc_stat_check_status,
	.check_atapi_dma	= tcc_sata_check_atapi_dma,

	.scr_read			= tcc_scr_read,
	.scr_write		= tcc_scr_write,

	.port_start     		= tcc_port_start,
	.port_stop      		= tcc_port_stop,
	.sff_irq_clear      	= tcc_irq_clear,

	.qc_prep        		= tcc_qc_prep,
	.qc_issue       		= tcc_qc_issue_prot,
};

static void tcc_sata_setup_port(struct ata_ioports *port, void __iomem *base)
{
	port->cmd_addr          = base + 0x00;
	port->data_addr         = base + 0x00;


	port->error_addr        = base + 0x04;
	port->feature_addr      = base + 0x04;

	port->nsect_addr        = base + 0x08;

	port->lbal_addr         = base + 0x0c;
	port->lbam_addr         = base + 0x10;
	port->lbah_addr         = base + 0x14;

	port->device_addr       = base + 0x18;

	port->command_addr      = base + 0x1c;
	port->status_addr       = base + 0x1c;

	port->altstatus_addr    = base + 0x20;
	port->ctl_addr          = base + 0x20;

	{
		unsigned long sata_base = (unsigned long)tcc_p2v(HwGDMA2_BASE);
		sata_base += 0x60;
		port->bmdma_addr        = (void __iomem *)(sata_base);  /* we better never use this */
	}
	port->scr_addr          = base + 0x24;

}

static const struct ata_port_info sata_tcc_port_info[] =
{
	{
	.flags = (ATA_FLAG_SATA /* loik ??? flags */
	| ATA_FLAG_NO_LEGACY /* no legacy mode check */
	//| ATA_FLAG_SRST /* use ATA SRST, not E.D.D. */
	| ATA_FLAG_MMIO), /* use MMIO, not PortIO */

	.pio_mask   = 0x1f, /* pio0-4  - IDENTIFY DEVICE word 63 */
	.udma_mask  = 0x7f, /* udma0-6 - IDENTIFY DEVICE word 88 */
	//.mwdma_mask = 0x07, /* mwdma0-2  - IDENTIFY DEVICE word 64 */
	.port_ops   = &tcc_sata_ops,
	},
};

static irqreturn_t tcc_sata_isr(int irq, void *dev_id)
{
	struct ata_host *host = (struct ata_host *)dev_id;
	struct tcc_sata_dev *tcc_dev = host->private_data;
	struct ata_port *ap;
	volatile u32 intpr;
	volatile u32 val32;
	unsigned int handled = 0;
	unsigned int i;
	u32 err_interrupt;
	u8  tag;
	volatile u32 sactive, tmp;
	struct ata_queued_cmd *qc;
	//unsigned long flags;

	//spin_lock_irqsave(&host->lock, flags);

	ap = host->ports[0];

	intpr = readl(tcc_dev->membase + HSATA_INTPR_REG);
	jprintk("INTPR=0x%x\n", intpr);

	if (intpr & HSATA_INTMR_ERRM_BIT)
	{
		val32 = readl(tcc_dev->membase + HSATA_SERROR_REG);
		sata_printk("============> SERROR=0x%08x INTPR=0x%x\n", val32, intpr);
		writel(val32, tcc_dev->membase + HSATA_SERROR_REG);  /* to clear */
		writel(HSATA_INTMR_ERRM_BIT, tcc_dev->membase + HSATA_INTPR_REG);  /* to clear */
		err_interrupt = 1;
		handled = 1;
		goto DONE;
	}
	else
	{
		err_interrupt = 0;
	}

	tmp = readl(tcc_dev->membase + HSATA_SACTIVE_REG);
	jprintk("SACTIVE=0x%x\n", tmp);

	if (intpr & HSATA_INTMR_NEWFP_BIT)
	{
		writel(HSATA_INTMR_NEWFP_BIT, tcc_dev->membase + HSATA_INTPR_REG);  /* to clear */
		tag = ((u8)readl(tcc_dev->membase + HSATA_FPTAGR_REG)) & 0x1f;
		sata_printk("============> NEWFP tag=%d\n", tag);
		handled = 1;
		goto DONE;
	}

	if (intpr & HSATA_INTMR_DMAT_BIT)
	{
		writel(HSATA_INTMR_DMAT_BIT, tcc_dev->membase + HSATA_INTPR_REG);  /* to clear */
		sata_printk("============> HSATA_INTMR_DMAT_BIT\n");
		handled = 1;
		goto DONE;
	}
	if (intpr & HSATA_INTMR_PMABORT)
	{
		writel(HSATA_INTMR_PMABORT, tcc_dev->membase + HSATA_INTPR_REG);  /* to clear */
		sata_printk("============> HSATA_INTMR_PMABORT\n");
		handled = 1;
		goto DONE;
	}
	if (intpr & HSATA_INTMR_NEWBIST_BIT)
	{
		writel(HSATA_INTMR_NEWBIST_BIT, tcc_dev->membase + HSATA_INTPR_REG);  /* to clear */
		sata_printk("============> HSATA_INTMR_NEWBIST_BIT\n");
		handled = 1;
		goto DONE;
	}
	if (intpr & HSATA_INTMR_PRIMERR_BIT)
	{
		writel(HSATA_INTMR_PRIMERR_BIT, tcc_dev->membase + HSATA_INTPR_REG);  /* to clear */
		sata_printk("============> HSATA_INTMR_PRIMERR_BIT\n");
		handled = 1;
		goto DONE;
	}
	if (intpr & HSATA_INTMR_CMDABORT_BIT)
	{
		writel(HSATA_INTMR_CMDABORT_BIT, tcc_dev->membase + HSATA_INTPR_REG);  /* to clear */
		sata_printk("============> HSATA_INTMR_CMDABORT_BIT\n");
		handled = 1;
		goto DONE;
	}
	if (intpr & HSATA_INTMR_CMDGOOD_BIT)
	{
		writel(HSATA_INTMR_CMDGOOD_BIT, tcc_dev->membase + HSATA_INTPR_REG);  /* to clear */
		jprintk("============> HSATA_INTMR_CMDGOOD_BIT\n");
		handled = 1;
		goto DONE;
	}

	sactive = readl(tcc_dev->membase + HSATA_SACTIVE_REG);  /* remaining pending */
	if (sactive)
	{
		sata_printk("UNEXPECTED SACTIVE???  sactive=0x%x\n", sactive);
	}

	for (i = 0; i < host->n_ports; i++)
	{
		struct ata_port *ap;

		ap = host->ports[i];
		if (ap &&
			!(ap->flags & ATA_FLAG_DISABLED))
		{

			qc = ata_qc_from_tag(ap, ap->link.active_tag);
			if (qc && (!(qc->tf.flags & ATA_TFLAG_POLLING)) &&
				(qc->flags & ATA_QCFLAG_ACTIVE))
			{
				handled |= ata_sff_host_intr(ap, qc);
			}
		}
	}

	sata_printk("====================> INTR DONE\n");
	if (handled == 0)
	{
		sata_printk("handled error !!!!!!!!!  \n");
		if ((qc = ata_qc_from_tag(ap, ap->link.active_tag)))
		{
			ata_qc_complete(qc);
		}
		ata_sff_check_status(ap);
		handled = 1;
	}

	//spin_unlock_irqrestore(&host->lock, flags);
DONE:
	return IRQ_RETVAL(handled);
}

static int tcc_sata_remove(struct platform_device *pdev)
{
	struct ata_host *host = platform_get_drvdata(pdev);
	//struct tcc_sata_dev *tcc_dev = NULL;

	printk("[sata_snps:%d] %s\n", __LINE__, __func__);

	if (host)
	{
		ata_host_detach(host);
	}
	return 0;
}

static int __init tcc_sata_probe(struct platform_device *pdev)
{
	struct clk *pSataClk;
	const struct ata_port_info *ppi[] = { &(sata_tcc_port_info[0]), NULL };
	struct ata_host *host;
	int n_ports;
	struct resource *res;
	struct ata_port *ap = NULL;
	int irq = -1;
	PPMU p_pmu = (PPMU)tcc_p2v(HwPMU_BASE);
	volatile PPIC p_pic = (volatile PPIC)tcc_p2v(HwPIC_BASE);

	printk("TCC SATA version " DRV_VERSION "\n");

	BITCLR(p_pmu->PWROFF, Hw4); // SATA popwer on

	if ((pSataClk = clk_get(NULL, "sata_hclk")) == NULL) 
	{
		printk("***** SATA Clock Enable Failed.*****\n");
		return -EINVAL;
	}	
	clk_enable(pSataClk);

	init_sata_irq(p_pic);
	res = platform_get_resource(pdev, IORESOURCE_MEM, 0);
	if (res == NULL)
	{
		return -EINVAL;
	}
	irq = platform_get_irq(pdev, 0);

	/* allocate host */
	n_ports = 1;
	host = ata_host_alloc_pinfo(&pdev->dev, ppi, n_ports);
	if (!host)
	{
		return -ENOMEM;
	}

	ap = host->ports[0];
	ap->private_data = (void *)(res->start);
	platform_set_drvdata(pdev, host);

	return ata_host_activate(host, irq, tcc_sata_isr, IRQF_SHARED, &tcc_sata_sht);
	//return ata_host_activate(host, irq, ata_sff_interrupt, IRQF_SHARED, &tcc_sata_sht);
}

#ifdef CONFIG_PM
static int tcc_sata_suspend(struct platform_device *pdev, pm_message_t mesg)
{
	struct clk *pSataClk;
	struct ata_host *host = platform_get_drvdata(pdev);
	PPMU p_pmu = (PPMU)tcc_p2v(HwPMU_BASE);

	ata_host_suspend(host, mesg);

	if ((pSataClk = clk_get(NULL, "sata_hclk")) == NULL) 
	{
		printk("***** SATA Clock Disable Failed.*****\n");
		return 0;
	}
	clk_disable(pSataClk);

	BITSET(p_pmu->PWROFF, Hw4); // SATA popwer off
	tcc_pca953x_setup(PCA9539_U2_SLAVE_ADDR, SATA_ON, OUTPUT, LOW, SET_DIRECTION | SET_VALUE);

	return 0;
}

static int tcc_sata_resume(struct platform_device *pdev)
{
	struct clk *pSataClk;
	struct ata_host *host = platform_get_drvdata(pdev);
	volatile PPMU p_pmu = (volatile PPMU)tcc_p2v(HwPMU_BASE);
	unsigned long sata_base = (unsigned long)tcc_p2v(HwSATA_BASE);
	struct tcc_sata_dev *tcc_dev = host->private_data;
	volatile PPIC p_pic = (volatile PPIC)tcc_p2v(HwPIC_BASE);

	tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, PWRGP4, OUTPUT, HIGH, SET_DIRECTION | SET_VALUE);
	tcc_pca953x_setup(PCA9539_U2_SLAVE_ADDR, SATA_ON, OUTPUT, HIGH, SET_DIRECTION | SET_VALUE);
	BITCLR(p_pmu->PWROFF, Hw4); // SATA popwer on

	if ((pSataClk = clk_get(NULL, "sata_hclk")) == NULL) 
	{
		printk("***** SATA Clock Enable Failed.*****\n");
		return 0;
	}	
	clk_enable(pSataClk);

	init_sata_irq(p_pic);
	init_dma_irq(p_pic);

	init_rx_dma(tcc_dev->dma_rx_reg, TCC_DMADR_REG_ADDR);
	init_tx_dma(tcc_dev->dma_tx_reg, TCC_DMADR_REG_ADDR);

	writel(HSATA_DMACR_TXMODE_BIT, tcc_dev->membase + HSATA_DMACR_REG);  /* disable */
	if(g_SataConnectionState){
		tcc_sata_hw_init(sata_base);
	}
	writel(HSATA_DMA_DBTSR_VAL, tcc_dev->membase + HSATA_DBTSR_REG);

	ata_host_resume(host);

	return 0;
}
#endif

static struct platform_driver tcc_sata_driver =
{
	.probe 	= tcc_sata_probe,
#ifdef CONFIG_PM
	.suspend	= tcc_sata_suspend,
	.resume	= tcc_sata_resume,
#endif
	.remove = tcc_sata_remove,
	.driver =
	{
		.name = "tcc-sata",
		.owner = THIS_MODULE,
	},
};

#ifdef CONFIG_SATA_TCC
static stpwrinfo sata_pwrinfo = { PWR_STATUS_ON };
/**********************************************************************
 * [The condition of SATA power control ]
 *  1 The Main-Board must be greater than version 1.0 or equal.
 *  2 And you need to fix the Main-Board. (refer to the schematic)
 * ********************************************************************/
static int sata_pwr_ctl(void *h_private, int cmd, void *p_out)
{
	int ret = -EINVAL;
	PPMU p_pmu = (PPMU)tcc_p2v(HwPMU_BASE);

	switch (cmd)
	{
	case PWR_CMD_OFF:
		sata_printk("PWR_CMD_OFF command ==> [%d]\n", cmd);
		if (sata_pwrinfo.status == PWR_STATUS_ON)
		{
			platform_driver_unregister(&tcc_sata_driver);
			tcc_pca953x_setup(PCA9539_U2_SLAVE_ADDR, SATA_ON, OUTPUT, LOW, SET_DIRECTION | SET_VALUE);
			BITSET(p_pmu->PWROFF, Hw4); // SATA popwer off
			tca_ckc_setiobus(RB_SATAHCONTROLLER, 0);
			sata_pwrinfo.status = PWR_STATUS_OFF;
			ret = 0;
		}
		else
		{
			printk("already SATA power is ON !!!\n");
			ret = 0;
		}
		break;
	case PWR_CMD_ON:
		sata_printk("PWR_CMD_ON command ==> [%d]\n", cmd);
		if (sata_pwrinfo.status == PWR_STATUS_OFF)
		{
			tcc_pca953x_setup(PCA9539_U2_SLAVE_ADDR, SATA_ON, OUTPUT, HIGH, SET_DIRECTION | SET_VALUE);
			if (platform_driver_register(&tcc_sata_driver) == 0)
			{
				sata_pwrinfo.status = PWR_STATUS_ON;
				ret = 0;
			}
			else
			{
				ret = -EIO;
				tcc_pca953x_setup(PCA9539_U2_SLAVE_ADDR, SATA_ON, OUTPUT, LOW, SET_DIRECTION | SET_VALUE);
				BITSET(p_pmu->PWROFF, Hw4); // SATA popwer off
				tca_ckc_setiobus(RB_SATAHCONTROLLER, 0);
				printk("cannot register a SATA driver !!!\n");
			}
		}
		else
		{
			printk("already SATA power is OFF !!!\n");
			ret = 0;
		}
		break;
	case PWR_CMD_GETSTATUS:
		sata_printk("PWR_CMD_GETSTATUS command ==> [%d]\n", cmd);
		memcpy(p_out, &sata_pwrinfo, sizeof(stpwrinfo));
		ret = 0;
		break;
		/*
		case PWR_CMD_MAX:
		printk("PWR_CMD_MAX command ==> [%d]\n", cmd);
		break;
		*/
	default:
		printk("unknown pwr command !!! ==> [%d]\n", cmd);
		break;
	}
	return ret;
}
#endif

static int __init tcc_sata_init(void)
{
	printk("[sata_snps:%d] %s\n", __LINE__, __func__);

	tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, PWRGP4, OUTPUT, HIGH, SET_DIRECTION | SET_VALUE);
	tcc_pca953x_setup(PCA9539_U2_SLAVE_ADDR, SATA_ON, OUTPUT, HIGH, SET_DIRECTION | SET_VALUE);

	return platform_driver_register(&tcc_sata_driver);
}

static void __exit tcc_sata_exit(void)
{
	PPMU p_pmu = (PPMU)tcc_p2v(HwPMU_BASE);

	printk("[sata_snps:%d] %s\n", __LINE__, __func__);

	platform_driver_unregister(&tcc_sata_driver);

	BITSET(p_pmu->PWROFF, Hw4); // SATA popwer off
	tca_ckc_setiobus(RB_SATAHCONTROLLER, 0);
	tcc_pca953x_setup(PCA9539_U2_SLAVE_ADDR, SATA_ON, OUTPUT, LOW, SET_DIRECTION | SET_VALUE);
}

MODULE_AUTHOR("Telechips");
MODULE_DESCRIPTION("TCC SATA controller");
MODULE_LICENSE("GPL");
MODULE_VERSION(DRV_VERSION);

module_init(tcc_sata_init);
module_exit(tcc_sata_exit);

